/* JavaGame is a game engine used to teach computer programming.
 * There is a corresponding JavaScript version to use in a web page.
 *
 * JavaGame copyright 2020 Itty Bitty Computers and released at this time
 * to the public as open source. There are no warranties of any kind.
 */                                               // 2021 July 15

import java.io.File;               // these all used by the mechanism..
import java.io.FileReader;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.io.BufferedWriter;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowListener;
import java.awt.event.MouseListener;
import java.awt.event.KeyListener;
import java.awt.event.WindowEvent;
import java.awt.event.MouseEvent;
import java.awt.event.KeyEvent;
import java.awt.MouseInfo;
import java.awt.Graphics;
import java.awt.Insets; // not available at win creation
import java.awt.image.BufferedImage;
import java.awt.image.DataBufferInt;
import javax.swing.JFrame;
import javax.swing.SwingUtilities;
import javax.swing.JOptionPane;
import javax.swing.Timer;
import java.util.Date;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.net.HttpURLConnection;
import java.net.URL;

public class JavaGame extends JFrame implements MouseListener, KeyListener,
    WindowListener {
  private static final long serialVersionUID = 1L; // unneed but Java insists

 /**
  * All GameEngine (non-widget) control is done in this class.
  */

  // First some public data...

// [Shared] GameEngine constants..
  final static String GameTitle = "GameEngine",       RevDate = "21/7/10";

  final static boolean Logging = false, NoSeeTxLn = true, Do_JS = false,
      GoodCirc = false, BitPoker = false, GoodTool = false, GoodDiag = false,
      GoodImex = false, GoodMore = false, GoodStart = false, GrafLog = false,
      GoodOptn = false, GoodFont = false, GoodEnd = false, GoodNew = false,
      GoodTst = false, GoodRip = false, GoodKchn = false, GoodPix = false,
      GoodImp = false, GoodAngle = false, GoodJS = false, GoodClok = false,
      GoodQuiet = false, GoodScro = false, LockStart = false,
      SRchanged = false, Dev_Log = false,
      StopSprites = true, ScrnSee = true; // MiniLog = false,

  final static int Qlog = 0, y2020 = -304*60, FrameTime = 100,
      MaxWgts = 4444, Max_Log = 333, Fust_Log = 222, Win_Hi = 640, Win_Wi = 800,
      MaxCircle = 32, myPL = 1,  WgtNameHi = 16,
      WflgBump = 0x100, WflgKey = 0x200, WflgClik = 0x400, WflgDrag = 0x800,
      WflgRoll = 0x1000, Clik2time = 999;

// [Predefined] GameEngine Macros & Data..              -- 2020 October 28
  public final int[] Wgt_Macros = {253,4, 0,3,1,12,12,0xFFFFFF,0, // checkbox..
        2,3,1,12,12,0x0,-7, 2,1,2,16,12,0xFF0000,-6, 4,4,16,16,24,0x0,-66,
      254,3, 4,2,0,12,16,0x0,-66, // labelled text entry (t=20, w=40 colo=-1)..
        0,0,8,20,32,0xFFFFFF,0x42616B46,  // = 'BakF'
        1540,3,12,12,28,0x0,-77,
      252,9, 4,5,8,16,24,0x0,-66, // pushbutton (tall=22,color=0xCCCCCC)..
        0,1,1,2,30,0xFFFFFF,0, 0,1,1,18,2,0xFFFFFF,0,
        0,17,2,2,30,0x999999,0, 0,2,28,17,2,0x999999,0,
        0,0,0,1,32,0x0,0, 0,0,30,20,1,0x0,0, 0,0,0,20,1,0x0,0,
        0,19,0,1,32,0x0,0,
      251,5, 2,0,0,16,16,0x0,-5, 0,0,0,1,16,0x0,0, // color swatch (t/w=16)..
        0,0,15,16,1,0x0,0, 0,0,0,16,1,0x0,0, 0,15,0,1,16,0x0,0,
      250,28, 0,0,0,1,1024,0x10000,0,  0,0,0,256,1,0x10000,0,  // ants border..
        0,31,0,3,1024,0x10000,0,  0,0,31,256,3,0x10000,0, // 256(tall) x 1K(wi)
        2,15,0,3,256,0xFEFFFF,-2, 2,0,15,32,3,0xFEFFFF,-2,
        2,0,0,1,256,0xFEFFFF,-2,  2,0,0,32,1,0xFEFFFF,-2,
        2,31,256,3,256,0xFEFFFF,-2, 2,31,512,3,256,0xFEFFFF,-2,
        2,31,768,3,256,0xFEFFFF,-2, 2,32,31,32,3,0xFEFFFF,-2,
        2,64,31,32,3,0xFEFFFF,-2, 2,96,31,32,3,0xFEFFFF,-2,
        2,0,256,1,256,0xFEFFFF,-2, 2,0,512,1,256,0xFEFFFF,-2,
        2,0,768,1,256,0xFEFFFF,-2, 2,32,0,32,1,0xFEFFFF,-2,
        2,64,0,32,1,0xFEFFFF,-2, 2,96,0,32,1,0xFEFFFF,-2,
        2,128,31,32,3,0xFEFFFF,-2, 2,160,31,32,3,0xFEFFFF,-2,
        2,192,31,32,3,0xFEFFFF,-2, 2,224,31,32,3,0xFEFFFF,-2,
        2,128,0,32,1,0xFEFFFF,-2, 2,160,0,32,1,0xFEFFFF,-2,
        2,192,0,32,1,0xFEFFFF,-2, 2,224,0,32,1,0xFEFFFF,-2,
      248,1, 5,0,0,32,32,-1,0,                 // tool wgt template (stub) **
      247,3, 5,0,0,64,64,-1,0,                 // V+H-scro'd panel (s=64/64)
        0x28F4,0,52,64,12,0x999999,0, 0x28F6,52,0,12,64,0x999999,0,
      246,1, 0,0,0,12,64,0xFFFFFF,0,     // H-scrollbar thumb (in 12/64 box)
      244,1, 0,0,0,64,12,0xFFFFFF,0,     // V-scrollbar thumb (in 64/12 box)
      245,2, 5,0,0,64,64,-1,0,                 // V-scrolled panel (s=64/78)
        0x28F4,0,64,64,12,0x999999,0,
      241,7, 4,10,8,16,252,0x0,-66,               // alert box (t=64,w=240)..
        1276,36,192,20,40,0xCCCCCC,1, 1276,36,408,20,60,0xCCCCCC,2,
        0,0,0,2,240,0x0,0, 0,0,238,64,2,0x0,0,
        0,0,0,64,2,0x0,0, 0,62,0,2,240,0x0,0,
      240,8, 254,8,8,40,208,-1,0,  2,9,408,16,16,0x0,-4, // InpDlg (t=64,c=wh)..
        1276,8,192,20,40,0xCCCCCC,1, 1276,36,172,20,60,0xCCCCCC,2, // (w=240)
        0,0,0,2,240,0x0,0, 0,0,238,64,2,0x0,0,
        0,0,0,64,2,0x0,0, 0,62,0,2,240,0x0,0,
      249,3, 2,3,1,12,12,0xFFFFFF,-8,  // radio button (tall=16 color=-1)..
        2,3,1,12,12,0x0,-8,  4,4,16,16,44,0x0,-66,
      255,0,  0,0};
  public final int[] DragArrows = {256,896,1984,256,256, // enb by +2048
      0x2108,0x610C,0xFFFE,0x610C,0x2108,256,256,1984,896,256,0};
  public final int[] NoCoGr = {0x55555555,0xAAAAAAAA, 0x55555555,0xAAAAAAAA,
        0x55555555,0xAAAAAAAA, 0x55555555,0xAAAAAAAA, 0x55555555,0xAAAAAAAA,
        0x55555555,0xAAAAAAAA, 0x55555555,0xAAAAAAAA, 0x55555555,0xAAAAAAAA,
        0,0};
  public final static int[] ChkBoxz = {
      4095,4095,3075,3075,3075,3075,3075,3075,3075,3075,4095,4095};
  public final static int[] ChkBitz = {
      0x380,0x700,0x1E00,0x3C00,0x1E00,0x780,0x1E0,0x70,0x30,24,4,0, 0,0,0,0};
  public final static int[] RadBBk = {0x0F0,0x3FC,0x7FE,0x7FE,0xFFF,0xFFF,
      0xFFF,0xFFF,0x7FE,0x7FE,0x3FC,0x0F0, 0};
  public final static int[] RadBtz = {0x0F0,0x30C,0x402,0x402,0x801,0x801,
      0x801,0x801,0x402,0x402,0x30C,0x0F0,  0x0F0,0x30C,0x402,0x4F2,0x9D9,0x9E9,
      0x9F9,0x9F9,0x4F2,0x402,0x30C,0x0F0, 0};

  final int[] AimIndex = {4,5,1,13,12,15,3,7}; // 1=+h,3=-h,4=+v,12=-v
  final int[] SmallAnts = {0,0,0, 0x3603, // off-bits:15, +/-n:4, patln:4
      0xCC04, 0x39C05, 0xE3806, 0x387007, 0xF0F008, 0x3C1E009, 0xF83E00A,
      0x3F7E00B, 0xE3806, 0xE3816, 0x387007, 0xF0F0F8, 0xF0F008, 0xF0F018,
      0x3C1E009, 0xF83E0FA, 0xF83E00A, 0xF0F008, 0,0, 0,0,0,0, 0,0,0,0};
  final int[] CirclePerims = {8,8,12, 8,12,16,16, 20,24,24,32, 32,36,36,40,
      44,52,48,52, 56,60,56,64, 64,68,72,76, 76,80,84,84, 88};
  final int[] ArcQsteps = {1,1,2,  0x11,0x12,0x12,0x112,         // 1..
      0x112,0x113,0x1112,0x1113,  0x1122,0x1123,0x11113,0x11123, // 8..
      0x11123,0x11223,0x111123,0x111124,  0x111223,             // 16..
      0x1111124,0x1111133,0x1111224,      0x11111133,           // 21..
      0x11111215,0x11111224,0x11112134,   0x11111224,           // 25..
      0x11111234,0x11112134,0x11111234,   0x11111234};          // 29..

  public final int[] Antz = { // max tall = 32, max wide = 256..
      0x0F0F0F0F,0x87878787, 0xC3C3C3C3,0xE1E1E1E1,
      0xF0F0F0F0,0x78787878, 0x3C3C3C3C,0x1E1E1E1E,
      0x0F0F0F0F,0x87878787, 0xC3C3C3C3,0xE1E1E1E1,
      0xF0F0F0F0,0x78787878, 0x3C3C3C3C,0x1E1E1E1E,
      0x0F0F0F0F,0x87878787, 0xC3C3C3C3,0xE1E1E1E1,
      0xF0F0F0F0,0x78787878, 0x3C3C3C3C,0x1E1E1E1E,
      0x0F0F0F0F,0x87878787, 0xC3C3C3C3,0xE1E1E1E1,
      0xF0F0F0F0,0x78787878, 0x3C3C3C3C,0x1E1E1E1E,
      0x0F0F0F0F,0x87878787, 0xC3C3C3C3,0xE1E1E1E1,
      0xF0F0F0F0,0x78787878, 0x3C3C3C3C,0x1E1E1E1E,
      0x0F0F0F0F,0x87878787, 0xC3C3C3C3,0xE1E1E1E1,
      0xF0F0F0F0,0x78787878, 0x3C3C3C3C,0x1E1E1E1E,
      0x0F0F0F0F,0x87878787, 0xC3C3C3C3,0xE1E1E1E1,
      0xF0F0F0F0,0x78787878, 0x3C3C3C3C,0x1E1E1E1E,
      0x0F0F0F0F,0x87878787, 0xC3C3C3C3,0xE1E1E1E1,
      0xF0F0F0F0,0x78787878, 0x3C3C3C3C,0x1E1E1E1E, // [63]
      1,1,1,1,0,0,0,0, 1,1,1,1,0,0,0,0, 1,1,1,1,0,0,0,0, 1,1,1,1,0,0,0,0,
      1,1,1,1,0,0,0,0, 1,1,1,1,0,0,0,0, 1,1,1,1,0,0,0,0, 1,1,1,1,0,0,0,0,
      1,1,1,1,0,0,0,0, 1,1,1,1,0,0,0,0, 1,1,1,1,0,0,0,0, 1,1,1,1,0,0,0,0,
      1,1,1,1,0,0,0,0, 1,1,1,1,0,0,0,0, 1,1,1,1,0,0,0,0, 1,1,1,1,0,0,0,0,
      1,1,1,1,0,0,0,0, 1,1,1,1,0,0,0,0, 1,1,1,1,0,0,0,0, 1,1,1,1,0,0,0,0,
      1,1,1,1,0,0,0,0, 1,1,1,1,0,0,0,0, 1,1,1,1,0,0,0,0, 1,1,1,1,0,0,0,0,
      1,1,1,1,0,0,0,0, 1,1,1,1,0,0,0,0}; // (+8 needed for animation)

  public static boolean UserMode = false, WuzChanged = false, // (Java only..)
      NameChanged = false, unSavedWgts = false, NoReload = false;

  // Default Monaco-like font +0
  private final int[] MonoFnt = { 0x4D6F6E6F, 0xC0209, 1, 6, 0, 6, // 'Mono'
                               0, 0, 190, 0, 0,              // !
     0, 14, 0, 14, 0,          40, 124, 40, 124, 40,
     76, 146, 511, 146, 100,   66, 37, 146, 328, 132,
     96, 150, 137, 118, 208,   0, 0, 14, 0, 0,
     0, 124, 130, 257, 0,      0, 257, 130, 124, 0,
     8, 42, 28, 42, 8,         16, 16, 124, 16, 16,
     0, 512, 448, 192, 0,      16, 16, 16, 16, 16,
     0, 192, 192, 0, 0,        0, 384, 96, 24, 6,
     56, 68, 130, 68, 56,      0, 4, 254, 0, 0,              // 0
     132, 194, 162, 146, 140,  68, 130, 146, 146, 108,
     48, 40, 36, 254, 32,      78, 138, 138, 138, 114,
     120, 148, 146, 146, 96,   2, 2, 226, 26, 6,
     108, 146, 146, 146, 108,  12, 146, 146, 82, 60,
     0, 204, 204, 0, 0,        0, 716, 460, 0, 0,
     0, 16, 40, 68, 130,       40, 40, 40, 40, 40,
     0, 130, 68, 40, 16,       4, 2, 162, 18, 12,
     120, 132, 306, 330, 60,   224, 56, 38, 56, 224,         // @A
     130, 254, 146, 146, 108,  124, 130, 130, 130, 68,
     130, 254, 130, 68, 56,    254, 146, 146, 130, 130,
     254, 18, 18, 18, 2,       124, 130, 130, 146, 244,
     254, 16, 16, 16, 254,     0, 130, 254, 130, 0,
     96, 128, 128, 128, 126,   254, 16, 40, 68, 130,
     254, 128, 128, 128, 128,  254, 4, 24, 4, 254,
     254, 4, 24, 32, 254,      124, 130, 130, 130, 124,
     254, 18, 18, 18, 12,      124, 130, 130, 386, 636,      // P
     254, 18, 50, 82, 140,     76, 146, 146, 146, 100,
     2, 2, 254, 2, 2,          126, 128, 128, 128, 126,
     14, 48, 192, 48, 14,      30, 224, 24, 224, 30,
     198, 40, 16, 40, 198,     6, 8, 240, 8, 6,
     194, 162, 146, 138, 134,  0, 0, 511, 257, 0,
     0, 6, 24, 96, 384,        0, 257, 511, 0, 0,
     4, 2, 1, 2, 4,            128, 128, 128, 128, 128,
     0, 2, 4, 0, 0,            64, 168, 168, 168, 240,       // `a
     254, 136, 136, 136, 112,  112, 136, 136, 136, 136,
     112, 136, 136, 136, 254,  112, 168, 168, 168, 48,
     16, 252, 18, 2, 4,        112, 648, 648, 648, 504,
     254, 8, 8, 8, 240,        0, 136, 250, 128, 0,
     512, 520, 506, 0, 0,      254, 16, 48, 72, 128,
     0, 2, 254, 0, 0,          248, 8, 240, 8, 240,
     248, 16, 8, 8, 240,       112, 136, 136, 136, 112,
     1016, 136, 136, 136, 112, 112, 136, 136, 136, 1016,     // p
     248, 16, 8, 8, 16,        144, 168, 168, 168, 72,
     0, 8, 126, 136, 128,      120, 128, 128, 128, 248,
     8, 48, 192, 48, 8,        56, 192, 48, 192, 56,
     136, 80, 32, 80, 136,     536, 608, 384, 96, 24,
     136, 200, 168, 152, 136,  0, 16, 238, 257, 0,
     0, 0, 511, 0, 0,          0, 257, 238, 16, 0,
     4, 2, 2, 4, 2 }; //~MonoFnt

  private final int[] TextFnt = { 0x54657874, 0xC0309, 1, 3, // 'Text'
      100,100,101,104,109,114,124,132, 134,137,140,145,150,152,155,157,
      159,164,169,174,179,184,189,194, 199,204,209,210,212,218,224,230,
      234,244,253,259,266,273,279,285, 293,301,304,309,317,323,332,340,
      347,353,360,367,372,379,387,394, 405,413,420,426,428,430,432,437,
      443,445,450,455,459,464,468,472, 477,483,486,488,494,497,506,512,
      517,522,527,532,536,539,545,552, 561,566,572,576,579,580,583,588,
      0x017E,0x000E,0x0000,0x000E,0x0048,0x01FE,0x0048,0x01FE, // 100..
      0x0048,0x008C,0x0112,0x03FF,0x0112,0x00E4,0x0018,0x0124,
      0x00A2,0x0052,0x002C,0x00D2,0x012A,0x0116,0x0092,0x0060,
      0x00C0,0x012C,0x0112,0x00AA,0x0044,0x00B0,0x0110,0x0100,
      0x0008,0x0006,0x00F0,0x030C,0x0402,0x0402,0x030C,0x00F0,
      0x0014,0x0008,0x003E,0x0008,0x0014,0x0020,0x0020,0x00F8,
      0x0020,0x0020,0x0400,0x0300,0x0020,0x0020,0x0020,0x0100,
      0x0000,0x01E0,0x001E,0x00FC,0x0102,0x0102,0x0102,0x00FC,
      0x0000,0x0104,0x01FE,0x0100,0x0000,0x0184,0x0142,0x0122,
      0x0112,0x008C,0x0084,0x0102,0x0112,0x0112,0x00EC,0x0060,
      0x0050,0x0048,0x0044,0x01FE,0x009E,0x0112,0x0112,0x0112, // 180..
      0x00E2,0x00F8,0x0114,0x0112,0x0112,0x00E0,0x0006,0x0002,
      0x01C2,0x0032,0x000E,0x00EC,0x0112,0x0112,0x0112,0x00EC,
      0x001C,0x0122,0x0122,0x00A2,0x007C,0x0108,0x0400,0x0308,
      0x0020,0x0020,0x0050,0x0050,0x0088,0x0088,0x0050,0x0050,
      0x0050,0x0050,0x0050,0x0050,0x0088,0x0088,0x0050,0x0050,
      0x0020,0x0020,0x0004,0x0162,0x0012,0x000C,0x01F0,0x020C,
      0x04E4,0x0512,0x050A,0x048A,0x01CA,0x0132,0x0084,0x0078,
      0x0100,0x0180,0x0160,0x0058,0x0046,0x0058,0x0160,0x0180,
      0x0100,0x0102,0x01FE,0x0112,0x0112,0x0112,0x00EC,0x0078,
      0x0084,0x0102,0x0102,0x0102,0x0084,0x004E,0x0102,0x01FE, // 260..
      0x0102,0x0102,0x0102,0x0084,0x0078,0x0102,0x01FE,0x0112,
      0x0112,0x013A,0x0186,0x0102,0x01FE,0x0112,0x0012,0x003A,
      0x0006,0x0078,0x0084,0x0102,0x0102,0x0102,0x0124,0x00EE,
      0x0020,0x0102,0x01FE,0x0112,0x0010,0x0010,0x0112,0x01FE,
      0x0102,0x0102,0x01FE,0x0102,0x0080,0x0100,0x0102,0x00FE,
      0x0002,0x0102,0x01FE,0x0112,0x0028,0x0046,0x0182,0x0102,
      0x0100,0x0102,0x01FE,0x0102,0x0100,0x0100,0x0180,0x0102,
      0x01FE,0x0106,0x0018,0x0060,0x0018,0x0106,0x01FE,0x0102,
      0x0102,0x01FE,0x0108,0x0010,0x0020,0x0042,0x01FE,0x0002,
      0x0078,0x0084,0x0102,0x0102,0x0102,0x0084,0x0078,0x0102, // 340..
      0x01FE,0x0122,0x0022,0x0022,0x001C,0x0078,0x0084,0x0102,
      0x0302,0x0502,0x0484,0x0078,0x0102,0x01FE,0x0122,0x0022,
      0x0062,0x01A2,0x011C,0x01CC,0x0092,0x0112,0x0124,0x00CE,
      0x0006,0x0002,0x0102,0x01FE,0x0102,0x0002,0x0006,0x0002,
      0x00FE,0x0102,0x0100,0x0100,0x0102,0x00FE,0x0002,0x0002,
      0x000E,0x0032,0x01C0,0x0032,0x000E,0x0002,0x0002,0x000E,
      0x0032,0x01C0,0x0032,0x000E,0x0032,0x01C0,0x0032,0x000E,
      0x0002,0x0102,0x0186,0x014A,0x0030,0x0030,0x014A,0x0186,
      0x0102,0x0002,0x0006,0x011A,0x01E0,0x011A,0x0006,0x0002,
      0x0186,0x0142,0x0122,0x0112,0x010A,0x0186,0x07FE,0x0402, // 420..
      0x001E,0x01E0,0x0402,0x07FE,0x0008,0x0004,0x0002,0x0004,
      0x0008,0x0400,0x0400,0x0400,0x0400,0x0400,0x0400,0x000C,
      0x0002,0x00C0,0x0128,0x0128,0x00F0,0x0100,0x0002,0x00FE,
      0x0108,0x0108,0x00F0,0x00F0,0x0108,0x0108,0x0098,0x00F0,
      0x0108,0x010A,0x01FE,0x0100,0x00F0,0x0128,0x0128,0x00B0,
      0x0108,0x01FC,0x010A,0x0002,0x02B0,0x0548,0x0548,0x0530,
      0x0208,0x0102,0x01FE,0x0008,0x0108,0x01F0,0x0100,0x0108,
      0x01FA,0x0100,0x0408,0x03FA,0x0102,0x01FE,0x0020,0x0158,
      0x0188,0x0100,0x0102,0x01FE,0x0100,0x0108,0x01F0,0x0008,
      0x0108,0x01F0,0x0008,0x0108,0x01F0,0x0100,0x0108,0x01F0, // 500..
      0x0108,0x0008,0x01F0,0x0100,0x00F0,0x0108,0x0108,0x0108,
      0x00F0,0x0408,0x07F8,0x0508,0x0108,0x00F0,0x00F0,0x0108,
      0x0508,0x07F8,0x0400,0x0108,0x01F0,0x0108,0x0008,0x0010,
      0x0190,0x0128,0x0128,0x00C8,0x0008,0x00FC,0x0108,0x0008,
      0x00F8,0x0100,0x0108,0x00F8,0x0100,0x0008,0x0018,0x0068,
      0x0180,0x0068,0x0018,0x0008,0x0008,0x0018,0x0068,0x0198,
      0x0068,0x0180,0x0068,0x0018,0x0008,0x0108,0x0198,0x0060,
      0x0198,0x0108,0x0408,0x0418,0x0268,0x0180,0x0068,0x0018,
      0x0198,0x0148,0x0128,0x0198,0x0020,0x03DC,0x0402,0x07FE, // 572..
      0x0402,0x03DC,0x0020,0x0010,0x0008,0x0018,0x0010,0x0008 }; //~TextFnt

  private final int[] ItalFnt = { 0x4974616C, 0xC0309, 0, 4, // 'Ital'
      100,100,105,109,116,123,135,143,145,151,157,163,168,170,173,174,
      180,187,193,201,208,216,224,231,238,245,252,256,261,268,275,282,
      288,301,309,318,328,338,348,358,368,380,387,395,406,413,426,438,
      447,456,465,475,484,493,503,511,523,535,544,554,561,565,572,577,
      583,585,592,598,603,610,615,623,630,637,640,646,652,655,663,669,
      675,682,689,695,699,703,709,715,724,730,736,741,747,748,753,758,
      0x0100,0x0040,0x0030,0x000C,0x0002,0x000C,0x0002,0x000C, // 100..
      0x0002,0x0140,0x00C0,0x0178,0x00CC,0x007A,0x000C,0x000A,
      0x0180,0x0300,0x01DC,0x0132,0x00DE,0x0023,0x0006,0x0100,
      0x0110,0x00A8,0x0064,0x0054,0x00EA,0x0152,0x012A,0x00AA,
      0x0046,0x0002,0x0002,0x01C0,0x0120,0x0120,0x009C,0x0072,
      0x01CA,0x0136,0x0010,0x0008,0x0006,0x07C0,0x0030,0x0008,
      0x0004,0x0002,0x0002,0x0400,0x0400,0x0200,0x0100,0x00C0,
      0x003E,0x0010,0x0014,0x0038,0x000C,0x001A,0x0004,0x0020,
      0x00E0,0x0030,0x0028,0x0020,0x0400,0x0300,0x0020,0x0020,
      0x0020,0x0100,0x0100,0x00C0,0x0020,0x0010,0x000C,0x0002,
      0x01C0,0x0130,0x010C,0x0102,0x00C2,0x0032,0x000E,0x0100, // 180..
      0x0100,0x01C0,0x0034,0x000C,0x0002,0x0100,0x0180,0x0140,
      0x0124,0x0122,0x0092,0x0012,0x000E,0x0180,0x0100,0x0104,
      0x0112,0x00F2,0x0012,0x000E,0x0040,0x0060,0x0050,0x0148,
      0x00C8,0x0034,0x000C,0x0002,0x0180,0x0110,0x011C,0x0112,
      0x00D2,0x0022,0x0002,0x0002,0x01C0,0x0130,0x0118,0x0114,
      0x00D2,0x0022,0x0002,0x0100,0x0084,0x0042,0x0022,0x0012,
      0x000A,0x0006,0x01C0,0x0120,0x012C,0x0112,0x00F2,0x0012,
      0x000E,0x0100,0x0110,0x012C,0x00A2,0x0062,0x0032,0x000E,
      0x0100,0x0000,0x0000,0x0008,0x0400,0x0300,0x0000,0x0000,
      0x0008,0x0020,0x0020,0x0050,0x0090,0x0088,0x0008,0x0008, // 260..
      0x0040,0x0050,0x0050,0x0050,0x0050,0x0050,0x0010,0x0080,
      0x0080,0x0048,0x0048,0x0050,0x0020,0x0020,0x0100,0x0044,
      0x0022,0x0022,0x0012,0x000E,0x0100,0x06C0,0x0430,0x05C8,
      0x0524,0x0014,0x0192,0x014A,0x012A,0x009A,0x0042,0x0036,
      0x0008,0x0100,0x0180,0x0040,0x0060,0x0050,0x0148,0x0144,
      0x01FE,0x0100,0x0100,0x01C0,0x0130,0x011A,0x0116,0x00D2,
      0x0032,0x000E,0x0040,0x01B0,0x0108,0x0104,0x0102,0x0082,
      0x0042,0x0006,0x000C,0x0002,0x0100,0x0100,0x01C0,0x0130,
      0x010A,0x0106,0x0082,0x0042,0x0036,0x0008,0x0100,0x0100,
      0x01C0,0x0130,0x011A,0x0116,0x00B2,0x000A,0x0006,0x0002, // 340..
      0x0100,0x0100,0x01C0,0x0030,0x001A,0x0016,0x0032,0x0002,
      0x0006,0x0002,0x0040,0x01B0,0x0108,0x0104,0x0102,0x0102,
      0x00E2,0x0026,0x002C,0x0002,0x0100,0x0100,0x01C0,0x0030,
      0x001E,0x0112,0x0112,0x01D0,0x0030,0x000E,0x0002,0x0002,
      0x0100,0x0100,0x01C0,0x0030,0x000E,0x0002,0x0002,0x0180,
      0x0100,0x0100,0x00C0,0x0030,0x000E,0x0002,0x0002,0x0100,
      0x0100,0x01C0,0x0030,0x001E,0x0162,0x0192,0x0108,0x0006,
      0x0002,0x0002,0x0100,0x0100,0x01C0,0x0130,0x010E,0x0102,
      0x0082,0x0100,0x0100,0x01C0,0x0030,0x000A,0x007E,0x0122,
      0x0110,0x01C8,0x0034,0x000E,0x0002,0x0002,0x0100,0x0100, // 420..
      0x01C0,0x0030,0x000E,0x001A,0x0160,0x00C0,0x0030,0x000E,
      0x0002,0x0002,0x0040,0x01B0,0x0108,0x0104,0x0102,0x0082,
      0x0042,0x0036,0x0008,0x0100,0x0100,0x01C0,0x0030,0x002A,
      0x0026,0x0022,0x0012,0x000E,0x0040,0x01B0,0x0708,0x0504,
      0x0102,0x0082,0x0042,0x0036,0x0008,0x0100,0x0100,0x01C0,
      0x0030,0x002A,0x0166,0x01A2,0x0022,0x0012,0x000E,0x0100,
      0x00C0,0x0100,0x011C,0x0112,0x00E2,0x0006,0x000C,0x0002,
      0x0100,0x0104,0x01C2,0x0032,0x000E,0x0002,0x0002,0x0006,
      0x0002,0x01C0,0x0130,0x010A,0x0106,0x0102,0x00C0,0x0030,
      0x000E,0x0002,0x0002,0x0100,0x00FA,0x0046,0x0022,0x0010, // 500..
      0x000A,0x0006,0x0002,0x0100,0x00FA,0x0006,0x001A,0x0120,
      0x00F2,0x000E,0x0032,0x0010,0x000E,0x0002,0x0002,0x0100,
      0x0100,0x0180,0x0040,0x0026,0x013A,0x0162,0x0190,0x0008,
      0x0006,0x0002,0x0002,0x0100,0x0100,0x01DA,0x0026,0x0012,
      0x0008,0x0006,0x0002,0x0002,0x0100,0x0180,0x0140,0x0144,
      0x0122,0x0112,0x008A,0x000A,0x0006,0x0002,0x0600,0x0500,
      0x00C0,0x0030,0x000C,0x0002,0x0002,0x0180,0x0060,0x0018,
      0x0006,0x0400,0x0400,0x0300,0x00C0,0x0030,0x000A,0x0006,
      0x0008,0x0004,0x0002,0x0006,0x0008,0x0400,0x0400,0x0400,
      0x0400,0x0400,0x0400,0x000C,0x0002,0x01C0,0x0130,0x0110, // 580..
      0x0088,0x01E8,0x0118,0x0080,0x0080,0x0170,0x011E,0x010A,
      0x00C8,0x0038,0x00E0,0x0110,0x0108,0x0088,0x0018,0x01E0,
      0x0110,0x0108,0x0088,0x0170,0x011A,0x0006,0x01E0,0x0130,
      0x0128,0x00A8,0x0018,0x0400,0x0400,0x0388,0x0078,0x000C,
      0x000A,0x0002,0x0002,0x0300,0x04B0,0x04C8,0x0548,0x0348,
      0x0030,0x0010,0x0180,0x0070,0x002E,0x0012,0x01C8,0x0138,
      0x0080,0x0188,0x0178,0x008A,0x0600,0x0400,0x0388,0x0078,
      0x0008,0x0002,0x0180,0x0070,0x002E,0x00E2,0x0118,0x0088,
      0x01E0,0x011A,0x0086,0x0100,0x00E8,0x0018,0x0190,0x0078,
      0x0010,0x01D0,0x0138,0x0188,0x0078,0x0028,0x0010,0x01E8, // 660..
      0x0118,0x00E0,0x0110,0x0108,0x0108,0x0088,0x0070,0x0400,
      0x0780,0x0568,0x0118,0x0108,0x0088,0x0078,0x01C0,0x0130,
      0x0510,0x0688,0x05C8,0x0070,0x0008,0x0180,0x0068,0x0018,
      0x0008,0x0008,0x0010,0x0180,0x0138,0x01C8,0x0018,0x0188,
      0x0178,0x008E,0x0008,0x0188,0x0178,0x0088,0x0040,0x01E0,
      0x0098,0x0008,0x0038,0x01C0,0x0080,0x0060,0x0018,0x0008,
      0x01F8,0x0080,0x0060,0x0010,0x01F8,0x0080,0x0060,0x0018,
      0x0100,0x0088,0x0088,0x00F0,0x0110,0x0088,0x0408,0x0418,
      0x02F0,0x0100,0x00C0,0x0038,0x0100,0x0190,0x0168,0x0218,
      0x0308,0x0020,0x03E0,0x0418,0x0404,0x0002,0x0002,0x01FE, // 740..
      0x0400,0x0600,0x0382,0x005E,0x0024,0x0060,0x0020,0x0020,
      0x0020,0x0030 }; //~ItalFnt

  private final int[] TinyFnt = { 0x54696E79, 0x90207, 0, 2, // 'Tiny'
      100,100,102,105,110,114,120,126,128,131,134,137,142,144,147,148,
      150,154,157,161,165,169,173,177,181,185,189,190,192,195,199,202,
      205,212,219,224,228,233,238,243,248,254,257,261,267,272,279,285,
      290,295,300,305,309,314,320,325,332,337,342,346,349,351,354,357,
      362,364,367,371,374,377,380,383,386,390,392,394,398,400,406,410,
      413,417,420,424,427,430,433,436,441,444,447,450,453,455,458,462,
      0x0000,0x005E,0x0006,0x0000,0x0006,0x0024,0x007E,0x0024, // 100..
      0x007E,0x0024,0x0024,0x004A,0x00FF,0x0032,0x000C,0x0052,
      0x002C,0x0038,0x004C,0x0032,0x0030,0x004C,0x005A,0x0036,
      0x0020,0x0050,0x0008,0x0006,0x0000,0x00FC,0x0102,0x0102,
      0x00FC,0x0000,0x000C,0x001E,0x000C,0x0010,0x0010,0x007C,
      0x0010,0x0010,0x0100,0x00C0,0x0010,0x0010,0x0010,0x0040,
      0x0070,0x000E,0x003C,0x0042,0x0042,0x003C,0x0044,0x007E,
      0x0040,0x0044,0x0062,0x0052,0x004C,0x0024,0x0042,0x004A,
      0x0034,0x0018,0x0014,0x007E,0x0010,0x002E,0x004A,0x004A,
      0x0032,0x003C,0x004A,0x004A,0x0030,0x0006,0x0062,0x001A,
      0x0006,0x0034,0x004A,0x004A,0x0034,0x000C,0x0052,0x0052, // 180..
      0x003C,0x0048,0x0100,0x00C8,0x0010,0x0028,0x0044,0x0028,
      0x0028,0x0028,0x0028,0x0044,0x0028,0x0010,0x0004,0x0052,
      0x000C,0x0038,0x0044,0x00B2,0x00AA,0x009A,0x0024,0x0018,
      0x0040,0x0060,0x0058,0x0016,0x0058,0x0060,0x0040,0x0042,
      0x007E,0x004A,0x004A,0x0034,0x003C,0x0042,0x0042,0x0026,
      0x0042,0x007E,0x0042,0x0042,0x003C,0x0042,0x007E,0x004A,
      0x0042,0x0066,0x0042,0x007E,0x004A,0x0002,0x0006,0x0018,
      0x0024,0x0042,0x0052,0x0072,0x0042,0x007E,0x0008,0x0008,
      0x007E,0x0042,0x0042,0x007E,0x0042,0x0020,0x0042,0x003E,
      0x0002,0x0042,0x007E,0x004A,0x0014,0x0062,0x0042,0x0042, // 260..
      0x007E,0x0042,0x0040,0x0060,0x0042,0x007E,0x0044,0x0008,
      0x0044,0x007E,0x0042,0x0042,0x007E,0x0048,0x0012,0x007E,
      0x0002,0x003C,0x0042,0x0042,0x0042,0x003C,0x0042,0x007E,
      0x0052,0x0012,0x000C,0x003C,0x0042,0x00C2,0x0142,0x003C,
      0x0042,0x007E,0x0052,0x0032,0x004C,0x0064,0x004A,0x0052,
      0x0026,0x0006,0x0042,0x007E,0x0042,0x0006,0x0002,0x003E,
      0x0040,0x0040,0x003E,0x0002,0x0006,0x001A,0x0060,0x001A,
      0x0006,0x0002,0x001E,0x0062,0x001C,0x0062,0x001E,0x0002,
      0x0042,0x0066,0x0018,0x0066,0x0042,0x0006,0x0048,0x0070,
      0x0048,0x0006,0x0062,0x0052,0x004A,0x0046,0x0000,0x01FE, // 340..
      0x0102,0x000E,0x0070,0x0102,0x01FE,0x0000,0x0004,0x0002,
      0x0004,0x0100,0x0100,0x0100,0x0100,0x0100,0x000C,0x0002,
      0x0068,0x0058,0x0070,0x0002,0x007E,0x0048,0x0030,0x0030,
      0x0048,0x0048,0x0030,0x004A,0x007E,0x0030,0x0068,0x0058,
      0x0008,0x007C,0x000A,0x0030,0x0148,0x00F8,0x0002,0x007E,
      0x0008,0x0070,0x0008,0x007A,0x0108,0x00FA,0x0002,0x007E,
      0x0010,0x0068,0x0002,0x007E,0x0008,0x0078,0x0008,0x0078,
      0x0008,0x0070,0x0008,0x0078,0x0008,0x0070,0x0030,0x0048,
      0x0030,0x0108,0x01F8,0x0048,0x0030,0x0030,0x0048,0x01F8,
      0x0000,0x0078,0x0010,0x0008,0x0050,0x0058,0x0028,0x0008, // 420..
      0x003C,0x0048,0x0038,0x0040,0x0078,0x0018,0x0060,0x0018,
      0x0018,0x0060,0x0018,0x0060,0x0018,0x0048,0x0030,0x0048,
      0x0138,0x00C0,0x0038,0x0068,0x0058,0x0048,0x0010,0x00EC,
      0x0102,0x0000,0x01FE,0x0102,0x00EC,0x0010,0x0010,0x0008,
      0x0010,0x0008 }; //~TinyFnt

  private final int[] NanoFnt = { 0x4E616E6F, 0x70105, 1, 2, // 'Nano'
      100,100,101,104,107,110,113,116,118,120,122,125,128,130,133,134,
      137,140,142,145,148,151,154,157,160,163,166,167,169,172,175,178,
      181,184,187,190,193,196,199,202,205,208,211,214,217,220,223,226,
      229,232,235,238,241,244,247,250,253,256,259,262,264,267,269,272,
      276,278,281,284,287,290,293,296,299,302,303,305,308,310,313,316,
      319,322,325,328,331,334,337,340,343,346,349,352,355,356,359,362,
      0x002E,0x0006,0x0000,0x0006,0x003E,0x0014,0x003E,0x002C, // 100..
      0x007E,0x0034,0x0012,0x0008,0x0024,0x0038,0x003E,0x0028,
      0x0004,0x0002,0x003E,0x0041,0x0041,0x003E,0x000A,0x0004,
      0x000A,0x0008,0x001C,0x0008,0x0040,0x0020,0x0004,0x0004,
      0x0004,0x0020,0x0060,0x0018,0x0006,0x001C,0x0022,0x001C,
      0x0004,0x003E,0x0032,0x002A,0x0024,0x0022,0x002A,0x0014,
      0x001C,0x0010,0x003E,0x002E,0x002A,0x0012,0x001C,0x002A,
      0x0012,0x0032,0x000A,0x0006,0x0014,0x002A,0x0014,0x0004,
      0x002A,0x001C,0x0028,0x0040,0x0028,0x0008,0x0014,0x0022,
      0x0014,0x0014,0x0014,0x0022,0x0014,0x0008,0x0002,0x002A,
      0x0004,0x003C,0x0042,0x005C,0x003C,0x0012,0x003C,0x003E, // 180..
      0x002A,0x0014,0x001C,0x0022,0x0014,0x003E,0x0022,0x001C,
      0x003E,0x002A,0x0022,0x003E,0x000A,0x0002,0x001C,0x0022,
      0x0032,0x003E,0x0008,0x003E,0x0022,0x003E,0x0022,0x0020,
      0x0020,0x001E,0x003E,0x0008,0x0036,0x003E,0x0020,0x0020,
      0x003E,0x0004,0x003E,0x003E,0x0002,0x003C,0x003E,0x0022,
      0x003E,0x003E,0x000A,0x000E,0x003E,0x0062,0x007E,0x003E,
      0x000A,0x0034,0x002C,0x002A,0x001A,0x0002,0x003E,0x0002,
      0x003E,0x0020,0x003E,0x001E,0x0020,0x001E,0x003E,0x0010,
      0x003E,0x0036,0x0008,0x0036,0x0006,0x0038,0x0006,0x0032,
      0x002A,0x0026,0x007F,0x0041,0x0006,0x0018,0x0060,0x0041, // 260..
      0x007F,0x0004,0x0002,0x0004,0x0040,0x0040,0x0040,0x0040,
      0x0002,0x0004,0x0010,0x0028,0x0038,0x003E,0x0028,0x0038,
      0x0010,0x0028,0x0028,0x0038,0x0028,0x003E,0x0038,0x0028,
      0x0018,0x0008,0x003C,0x000A,0x0050,0x0068,0x0038,0x003E,
      0x0008,0x0030,0x003A,0x0040,0x003A,0x003E,0x0010,0x0028,
      0x0002,0x003E,0x0038,0x0018,0x0038,0x0038,0x0008,0x0030,
      0x0010,0x0028,0x0010,0x0078,0x0028,0x0038,0x0038,0x0028,
      0x0078,0x0038,0x0008,0x0008,0x0020,0x0038,0x0008,0x0008,
      0x003E,0x0028,0x0038,0x0020,0x0038,0x0018,0x0020,0x0018,
      0x0038,0x0030,0x0038,0x0028,0x0010,0x0028,0x0058,0x0020, // 340..
      0x0018,0x0028,0x0038,0x0028,0x0008,0x0036,0x0041,0x007F,
      0x0041,0x0036,0x0008,0x0002,0x0002,0x0002 }; //~NanoFnt

  private final int[] BoldFnt = { 0x426F6C64, 0xC0309, 1, 4, // 'Bold'
      100,100,102,107,113,118,127,133,135,138,141,147,153,155,159,161,
      165,170,175,180,185,190,195,200,206,211,216,218,220,226,232,238,
      243,252,260,267,274,281,287,293,300,306,308,313,320,326,334,340,
      348,354,362,368,374,380,386,393,403,410,418,424,427,431,434,439,
      445,447,452,458,463,469,475,480,486,491,493,495,501,503,511,516,
      522,528,534,538,543,549,554,560,568,574,580,585,589,591,595,601,
      0x01BE,0x01BE,0x000E,0x000E,0x0000,0x000E,0x000E,0x0040, // 100..
      0x01E8,0x005E,0x01E8,0x005E,0x0008,0x009C,0x01B6,0x03FF,
      0x01B6,0x00E6,0x000C,0x001E,0x0312,0x00DE,0x00FC,0x01F8,
      0x0126,0x01E0,0x00C0,0x00EC,0x01FE,0x0132,0x01FE,0x00CC,
      0x0160,0x0016,0x000E,0x01F8,0x03FC,0x0402,0x0402,0x03FC,
      0x01F8,0x006C,0x0028,0x00FE,0x00FE,0x0028,0x006C,0x0060,
      0x0060,0x01F8,0x01F8,0x0060,0x0060,0x0580,0x0380,0x0060,
      0x0060,0x0060,0x0060,0x0180,0x0180,0x0180,0x00F0,0x003C,
      0x0006,0x00FC,0x01FE,0x0102,0x01FE,0x00FC,0x0000,0x0008,
      0x01FC,0x01FE,0x0000,0x018C,0x01C6,0x0162,0x013E,0x011C,
      0x008C,0x0186,0x0132,0x01FE,0x00EC,0x0060,0x0058,0x004C, // 180..
      0x01FE,0x01FE,0x009E,0x011E,0x0112,0x01F2,0x00E2,0x00FC,
      0x01FE,0x0112,0x01F2,0x00E4,0x0002,0x0182,0x01E2,0x007A,
      0x001E,0x0006,0x00EC,0x01FE,0x0112,0x01FE,0x00EC,0x009C,
      0x013E,0x0122,0x01FE,0x00FC,0x0198,0x0198,0x0598,0x0398,
      0x0060,0x0060,0x0090,0x0090,0x0198,0x0108,0x0090,0x0090,
      0x0090,0x0090,0x0090,0x0090,0x0108,0x0198,0x0090,0x0090,
      0x0060,0x0060,0x000C,0x0002,0x0162,0x013E,0x001C,0x0078,
      0x0084,0x0172,0x014A,0x014A,0x017A,0x0142,0x0044,0x0038,
      0x0180,0x01E0,0x0078,0x005E,0x004E,0x0078,0x01E0,0x0180,
      0x01FE,0x01FE,0x0112,0x0112,0x011E,0x01FC,0x00E0,0x0078, // 260..
      0x00FC,0x0186,0x0102,0x0102,0x01CE,0x00CC,0x01FE,0x01FE,
      0x0102,0x0102,0x0186,0x00FC,0x0078,0x01FE,0x01FE,0x0112,
      0x0112,0x0112,0x0102,0x01FE,0x01FE,0x0012,0x0012,0x0012,
      0x0002,0x0078,0x00FC,0x0186,0x0102,0x0122,0x01A6,0x01EC,
      0x01FE,0x01FE,0x0010,0x0010,0x01FE,0x01FE,0x01FE,0x01FE,
      0x00C0,0x01C0,0x0100,0x01FE,0x00FE,0x01FE,0x01FE,0x0070,
      0x007C,0x00EE,0x01C6,0x0102,0x01FE,0x01FE,0x0100,0x0100,
      0x0100,0x0100,0x01FE,0x01FE,0x0038,0x01E0,0x01E0,0x0038,
      0x01FE,0x01FE,0x01FE,0x01FE,0x0038,0x00E0,0x01FE,0x01FE,
      0x0078,0x00FC,0x0186,0x0102,0x0102,0x0186,0x00FC,0x0078, // 340..
      0x01FE,0x01FE,0x0022,0x0022,0x003E,0x001C,0x0078,0x00FC,
      0x0186,0x0102,0x0142,0x0186,0x03FC,0x0278,0x01FE,0x01FE,
      0x0022,0x0022,0x01FE,0x01DC,0x009C,0x019E,0x0132,0x0132,
      0x0136,0x00E4,0x0002,0x0002,0x01FE,0x01FE,0x0002,0x0002,
      0x00FE,0x01FE,0x0100,0x0100,0x01FE,0x00FE,0x0006,0x003E,
      0x00F0,0x0180,0x00F0,0x003E,0x0006,0x0006,0x003E,0x01F0,
      0x01F0,0x001E,0x001E,0x01F0,0x01F0,0x003E,0x0006,0x0102,
      0x0186,0x00FC,0x0030,0x00FC,0x0186,0x0102,0x0002,0x000E,
      0x001C,0x01F0,0x01F0,0x001C,0x000E,0x0002,0x0182,0x01C2,
      0x0172,0x013A,0x010E,0x0106,0x07FE,0x07FE,0x0402,0x0006, // 420..
      0x003C,0x00F0,0x0180,0x0402,0x07FE,0x07FE,0x0030,0x001C,
      0x0006,0x001C,0x0030,0x0400,0x0400,0x0400,0x0400,0x0400,
      0x0400,0x001C,0x001A,0x01D0,0x01E8,0x0128,0x01F8,0x01F0,
      0x01FE,0x01FE,0x0108,0x0108,0x01F8,0x00F0,0x00F0,0x01F8,
      0x0108,0x0108,0x0098,0x00F0,0x01F8,0x0108,0x0108,0x01FE,
      0x01FE,0x00F0,0x01F8,0x0128,0x0128,0x01B8,0x00B0,0x0010,
      0x01FC,0x01FE,0x0012,0x0002,0x00F0,0x05F8,0x0508,0x0508,
      0x07F8,0x03F8,0x01FE,0x01FE,0x0008,0x01F8,0x01F0,0x01FA,
      0x01FA,0x07FA,0x07FA,0x01FE,0x01FE,0x0060,0x01F8,0x0198,
      0x0100,0x01FE,0x01FE,0x01F8,0x01F8,0x0008,0x01F8,0x01F8, // 500..
      0x0008,0x01F8,0x01F0,0x01F8,0x01F8,0x0008,0x01F8,0x01F8,
      0x00F0,0x01F8,0x0108,0x0108,0x01F8,0x00F0,0x07F8,0x07F8,
      0x0108,0x0108,0x01F8,0x00F0,0x00F0,0x01F8,0x0108,0x0108,
      0x07F8,0x07F8,0x01F8,0x01F8,0x0008,0x0018,0x00B0,0x01B8,
      0x0168,0x01D8,0x00D0,0x0008,0x0008,0x00FE,0x01FE,0x0108,
      0x0008,0x00F8,0x01F8,0x0100,0x01F8,0x01F8,0x0018,0x0078,
      0x01C0,0x01C0,0x0078,0x0018,0x0018,0x00F8,0x01C0,0x0078,
      0x0078,0x01C0,0x00F8,0x0018,0x0108,0x0198,0x00F0,0x00F0,
      0x0198,0x0108,0x0018,0x0478,0x07C0,0x01C0,0x0078,0x0018,
      0x0188,0x01C8,0x0168,0x0138,0x0118,0x0060,0x03FC,0x079E, // 600..
      0x0402,0x01FE,0x01FE,0x0402,0x079E,0x03FC,0x0060,0x0040,
      0x0020,0x0020,0x0060,0x0040,0x0020 }; //~BoldFnt

  private final int[] HeadFnt = { 0x48656164, 0x12040E, 2, 5, // 'Head'
      100,100,103,109,119,128,143,154,157,161,166,172,182,185,190,193,
      200,209,215,224,233,242,251,260,269,278,287,290,293,303,313,323,
      332,346,358,369,380,391,401,410,422,432,435,443,454,463,476,487,
      499,509,521,532,543,553,563,574,590,601,612,622,626,633,637,645,
      655,658,667,676,685,694,703,709,718,727,730,733,742,745,759,768,
      778,787,796,802,811,816,825,834,848,857,866,874,880,883,889,900,
      0x030FE,0x033FE,0x030FE,0x0003E,0x0003E, // 100..
      0x00000,0x0003E,0x0003E,0x00006,0x00600,
      0x03E30,0x03F30,0x007F0,0x03E3E,0x03FB0,
      0x00FF0,0x0067E,0x00636,0x00030,0x00C78,
      0x03CFC,0x038FE,0x030C6,0x0FFFF,0x03186,
      0x03F9C,0x01F1C,0x00E10,0x00038,0x0007C,
      0x000C6,0x000C6,0x030C6,0x00C7C,0x00338,
      0x000C0,0x01E60,0x01E18,0x03306,0x03300,
      0x03300,0x01E00,0x01E00,0x00E00,0x01F80,
      0x03FBC,0x031FE,0x031E6,0x033E6,0x03F7E,
      0x01E3C,0x03F80,0x03780,0x02180,0x00026, // 150..
      0x0003E,0x0001E,0x01FC0,0x07FF0,0x1FFFC,
      0x3C01E,0x20002,0x3C01E,0x1FFFC,0x07FF0,
      0x01FC0,0x0000C,0x0006C,0x0003E,0x0003E,
      0x0007C,0x0002C,0x00300,0x00300,0x00300,
      0x00300,0x03FF0,0x03FF0,0x03FF0,0x00300,
      0x00300,0x00300,0x13800,0x0F800,0x07800,
      0x00300,0x00300,0x00300,0x00300,0x00300,
      0x03800,0x03800,0x03800,0x02000,0x03C00,
      0x01F00,0x003E0,0x000FC,0x0001E,0x00006,
      0x007F0,0x01FFC,0x03FFE,0x0380E,0x03006, // 200..
      0x0380E,0x03FFE,0x01FFC,0x007F0,0x00018,
      0x00018,0x00018,0x03FFC,0x03FFE,0x03FFE,
      0x03030,0x03C3C,0x03E3E,0x03F06,0x03706,
      0x033C6,0x031FE,0x030FC,0x03078,0x00C10,
      0x01C1C,0x0380E,0x030C6,0x030C6,0x039FE,
      0x03FFC,0x01FB8,0x00F00,0x00700,0x007C0,
      0x007E0,0x00678,0x0061E,0x03FFE,0x03FFE,
      0x03FFE,0x00600,0x00800,0x019FE,0x039FE,
      0x030C6,0x030C6,0x039C6,0x03FC6,0x01F86,
      0x00F00,0x007F0,0x01FFC,0x03FBC,0x038C6,
      0x030C6,0x030C6,0x03FCE,0x01F8C,0x00F00,
      0x00006,0x00006,0x03C06,0x03F06,0x01FC6,
      0x001F6,0x0007E,0x0001E,0x0000E,0x00F38,
      0x01FFC,0x03FFE,0x030C6,0x030C6,0x030FE,
      0x03FFC,0x01FB8,0x00F00,0x008F8,0x019FC,
      0x03BFE,0x0338E,0x03306,0x03B8E,0x01FFE,
      0x01FFC,0x007F0,0x03060,0x03060,0x03060,
      0x13060,0x1F060,0x0F060,0x00700,0x00700,
      0x00780,0x00F80,0x00CC0,0x00CC0,0x018C0,
      0x01860,0x01860,0x03030,0x00CC0,0x00CC0, // 300..
      0x00CC0,0x00CC0,0x00CC0,0x00CC0,0x00CC0,
      0x00CC0,0x00CC0,0x00CC0,0x03030,0x01860,
      0x01860,0x018E0,0x00CC0,0x00CC0,0x00D80,
      0x00780,0x00780,0x00700,0x00018,0x0001C,
      0x0001E,0x03306,0x03386,0x033C6,0x000FE,
      0x0007C,0x00038,0x003E0,0x00E30,0x01818,
      0x0180C,0x033CC,0x037E6,0x03676,0x03636,
      0x03336,0x037F6,0x01634,0x01E0C,0x01318,
      0x001F0,0x02000,0x03C00,0x03F80,0x01FF0,
      0x007FE,0x0067E,0x0061E,0x006FE,0x007F8,
      0x03FC0,0x03E00,0x03000,0x03FFE,0x03FFE,
      0x03FFE,0x030C6,0x030C6,0x030C6,0x030C6,
      0x038FE,0x03FFC,0x01FB8,0x00F00,0x003E0,
      0x00FF8,0x01FFC,0x03C1E,0x0380E,0x03006,
      0x03006,0x03006,0x0180E,0x01C1C,0x00C18,
      0x03FFE,0x03FFE,0x03FFE,0x03006,0x03006,
      0x03006,0x03006,0x03C0E,0x01FFC,0x00FF8,
      0x003F0,0x03FFE,0x03FFE,0x03FFE,0x030C6,
      0x030C6,0x030C6,0x030C6,0x030C6,0x030C6,
      0x03006,0x03FFE,0x03FFE,0x03FFE,0x000C6, // 400..
      0x000C6,0x000C6,0x000C6,0x000C6,0x00006,
      0x003E0,0x00FF8,0x01FFC,0x03C1C,0x0380E,
      0x03006,0x03006,0x03186,0x0198E,0x01F9C,
      0x03F9C,0x03F90,0x03FFE,0x03FFE,0x03FFE,
      0x000C0,0x000C0,0x000C0,0x000C0,0x03FFE,
      0x03FFE,0x03FFE,0x03FFE,0x03FFE,0x03FFE,
      0x00E00,0x01E00,0x03E00,0x03000,0x03000,
      0x03FFE,0x01FFE,0x00FFE,0x03FFE,0x03FFE,
      0x03FFE,0x003C0,0x001E0,0x007F8,0x00FFC,
      0x01F3E,0x03C1E,0x03806,0x03002,0x03FFE,
      0x03FFE,0x03FFE,0x03000,0x03000,0x03000,
      0x03000,0x03000,0x03000,0x03FFE,0x03FFE,
      0x03FFE,0x000FE,0x00FF8,0x03F80,0x03C00,
      0x03FE0,0x003FE,0x0001E,0x03FFE,0x03FFE,
      0x03FFE,0x03FFE,0x03FFE,0x03FFE,0x0003C,
      0x000F0,0x003C0,0x00780,0x01E00,0x03FFE,
      0x03FFE,0x03FFE,0x003E0,0x00FF8,0x01FFC,
      0x03C1E,0x03006,0x03006,0x03006,0x03006,
      0x03C1E,0x01FFC,0x00FF8,0x003E0,0x03FFE,
      0x03FFE,0x03FFE,0x00186,0x00186,0x00186, // 500..
      0x00186,0x001FE,0x000FC,0x00078,0x003E0,
      0x00FF8,0x01FFC,0x01C1E,0x03806,0x03006,
      0x03406,0x03E06,0x03C1E,0x01FFC,0x03FF8,
      0x067E0,0x03FFE,0x03FFE,0x03FFE,0x00186,
      0x00186,0x00186,0x00186,0x03F86,0x03FFE,
      0x03E7C,0x02038,0x00C38,0x01CFC,0x01CFE,
      0x030C6,0x031C6,0x031C6,0x031C6,0x0338E,
      0x01F9C,0x01F98,0x00600,0x00006,0x00006,
      0x00006,0x00006,0x03FFE,0x03FFE,0x03FFE,
      0x00006,0x00006,0x00006,0x00FFE,0x01FFE,
      0x03FFE,0x03800,0x03000,0x03000,0x03800,
      0x03FFE,0x01FFE,0x00FFE,0x00002,0x0001E,
      0x000FE,0x007F8,0x03F80,0x03C00,0x03F00,
      0x00FE0,0x001FE,0x0003E,0x0000E,0x00006,
      0x0003E,0x003FE,0x01FF0,0x03F00,0x03F00,
      0x01FF8,0x000FE,0x0001E,0x003FE,0x03FE0,
      0x03C00,0x03FC0,0x00FFC,0x000FE,0x0000E,
      0x02000,0x03006,0x03C1E,0x01F3E,0x00FF8,
      0x003E0,0x007F0,0x01F7C,0x03E1E,0x0380E,
      0x03002,0x00002,0x0000E,0x0003E,0x0007C, // 600..
      0x03FF0,0x03FC0,0x03FE0,0x001F8,0x0007E,
      0x0001E,0x00006,0x03806,0x03C06,0x03E06,
      0x03786,0x033C6,0x031E6,0x030F6,0x0303E,
      0x0301E,0x0300E,0x1FFFE,0x1FFFE,0x1FFFE,
      0x18006,0x00002,0x0001E,0x000FC,0x003E0,
      0x01F80,0x03C00,0x03000,0x18006,0x1FFFE,
      0x1FFFE,0x1FFFE,0x00100,0x001E0,0x000F8,
      0x0003E,0x0001E,0x0007C,0x001F0,0x00180,
      0x08000,0x08000,0x08000,0x08000,0x08000,
      0x08000,0x08000,0x08000,0x08000,0x08000,
      0x0003C,0x00036,0x00032,0x01E00,0x03F60,
      0x03F70,0x03330,0x03330,0x01FF0,0x03FF0,
      0x03FE0,0x02000,0x03FFE,0x03FFE,0x03FFE,
      0x01860,0x03030,0x03870,0x03FF0,0x01FE0,
      0x00FC0,0x00FC0,0x01FE0,0x03FF0,0x03870,
      0x03030,0x03030,0x038F0,0x018E0,0x008C0,
      0x00FC0,0x01FE0,0x03FF0,0x03870,0x03030,
      0x01860,0x03FFE,0x03FFE,0x03FFE,0x00F80,
      0x01FE0,0x03FF0,0x03B30,0x03330,0x03330,
      0x03BF0,0x01BE0,0x00BC0,0x00030,0x03FFC, // 700..
      0x03FFE,0x03FFE,0x00036,0x00030,0x08FC0,
      0x19FE0,0x3BFF0,0x33870,0x33030,0x39860,
      0x3FFF0,0x1FFF0,0x0FFF0,0x03FFE,0x03FFE,
      0x03FFE,0x00060,0x00030,0x00030,0x03FF0,
      0x03FF0,0x03FE0,0x03FF6,0x03FF6,0x03FF6,
      0x3FFF6,0x3FFF6,0x1FFF6,0x03FFE,0x03FFE,
      0x03FFE,0x00380,0x00FC0,0x01FE0,0x03E70,
      0x03810,0x02000,0x03FFE,0x03FFE,0x03FFE,
      0x03FF0,0x03FF0,0x03FF0,0x00020,0x00030,
      0x00030,0x03FF0,0x03FE0,0x03FE0,0x00030,
      0x00030,0x03FF0,0x03FF0,0x03FE0,0x03FF0,
      0x03FF0,0x03FF0,0x00060,0x00030,0x00030,
      0x03FF0,0x03FF0,0x03FE0,0x00780,0x01FE0,
      0x01FE0,0x03870,0x03030,0x03030,0x03870,
      0x01FE0,0x01FE0,0x00780,0x3FFF0,0x3FFF0,
      0x3FFF0,0x01860,0x03030,0x03870,0x03FF0,
      0x01FE0,0x00FC0,0x00FC0,0x01FE0,0x03FF0,
      0x03870,0x03030,0x01860,0x3FFF0,0x3FFF0,
      0x3FFF0,0x03FF0,0x03FF0,0x03FF0,0x00060,
      0x00070,0x00070,0x018C0,0x019E0,0x03BF0, // 800..
      0x03330,0x03330,0x03730,0x03F70,0x01E60,
      0x00C00,0x00030,0x01FFC,0x03FFC,0x03FFC,
      0x03030,0x01FF0,0x03FF0,0x03FF0,0x03000,
      0x03000,0x01800,0x03FF0,0x03FF0,0x03FF0,
      0x00010,0x000F0,0x007F0,0x03F80,0x03C00,
      0x03E00,0x01FE0,0x003F0,0x00070,0x00030,
      0x001F0,0x01FF0,0x03F00,0x03C00,0x03FF0,
      0x003F0,0x007F0,0x03FC0,0x03800,0x03F80,
      0x007F0,0x000F0,0x00010,0x02010,0x03030,
      0x03CF0,0x01FE0,0x00780,0x01FC0,0x03EF0, // 850..
      0x03870,0x03010,0x00010,0x300F0,0x307F0,
      0x3BF80,0x1FC00,0x0FE00,0x01FE0,0x003F0,
      0x00070,0x03070,0x03870,0x03C70,0x03F70,
      0x033F0,0x031F0,0x030F0,0x03070,0x00200,
      0x1FFF8,0x1FDFC,0x3F8FE,0x30006,0x20002,
      0x03FFE,0x03FFE,0x03FFE,0x30006,0x3F8FE,
      0x1FDFE,0x1FFFC,0x00700,0x00200,0x00600,
      0x00700,0x00180,0x00180,0x00380,0x00300,
      0x00700,0x00600,0x00600,0x00380,0x00080 }; //~HeadFnt

  public final int[] ScorFnt = { 0x53636F72, 0x1F041A, 2, 5, // 'Scor'
  // ! $ % ( ) * + , - . / 0 1 2 3 4 5 6 7 8 9 < = > ? A B C D E F O T X x
    100,100,104,104, 104,119,144,144, 144,151,159,171, 187,191,206,210,
    221,236,251,266, 281,296,311,326, 341,356,371,371, 371,388,404,421,
    435,435,455,473, 492,510,526,541, 541,541,541,541, 541,541,541,541,
    563,563,563,563, 563,580,580,580, 580,600,600,600, 600,600,600,600,
    600,600,600,600, 600,600,600,600, 600,600,600,600, 600,600,600,600,
    600,600,600,600, 600,600,600,600, 600,615,615,615, 615,615,615,615,
    0x03C07FF0,0x03CFFFF0,0x03CFFFF0,0x03C07FF0, 0x00301E00,0x00F07F80, // 100..
    0x01F07FC0,0x01F0FFC0, 0x03C0F1E0,0x0381E0E0,0x1FFFFFF8,0x1FFFFFF8,
    0x1FFFFFF8,0x0383C0E0,0x03C7C1E0,0x01FF87C0, 0x01FF8780,0x00FF0700,
    0x003E0000,0x1E00,0x7F80,0xFFC0,0xFFC0,0x0001F3E0,
    0x0001E1E0,0x0301E1E0,0x03C1F3E0,0x01E0FFC0, 0x0078FFC0,0x001E7F80,
    0x00079E00,0x0003C000, 0x007CF000,0x00FE3C00,0x01FF0F00,0x03FF8780, // 130..
    0x03EF81E0,0x03C78060,0x03C78000,0x03EF8000, 0x03FF8000,0x01FF0000,
    0x00FE0000,0x007C0000, 0x0007F800,0x007FFF00,0x01FFFFE0,0x07FFFFF0,
    0x0FE001FC,0x1F00003E,0x18000006,0x10000002, 0x1C00000E,0x1F00003E,
    0x0FF003FC,0x07FFFFF8, 0x01FFFFE0,0x007FFF80,0x0007FC00,0x00018000,
    0x00218000,0x00718000,0x003B8000,0x001F8000, 0x000FF000,0x000FF800, // 160..
    0x001FF000,0x003F8000, 0x00798000,0x00318000,0x00018000,0x000F0000,
    0x000F0000,0x000F0000,0x000F0000,0x000F0000, 0x000F0000,0x03FFFC00,
    0x03FFFC00,0x03FFFC00, 0x03FFFC00,0x000F0000,0x000F0000,0x000F0000,
    0x000F0000,0x000F0000,0x000F0000,0x13C00000, 0x1BC00000,0x0FC00000,
    0x07C00000,0x000F0000, 0x000F0000,0x000F0000,0x000F0000,0x000F0000, // 190..
    0x000F0000,0x000F0000,0x000F0000,0x000F0000, 0x000F0000,0x000F0000,
    0x000F0000,0x000F0000, 0x000F0000,0x000F0000,0x03C00000,0x03C00000,
    0x03C00000,0x03C00000,0x03C00000,0x03F80000, 0x03FE0000,0x01FFC000,
    0x007FF000,0x000FFE00, 0x0001FFC0,0x7FF0,0x0FF0,0x03F0, // ..219
    0x0070,0x000FF800,0x007FFF00,0x00FFFF80, 0x01FFFFC0,0x03F007C0, // 220..
    0x03E003E0,0x03C001E0, 0x03C001E0,0x03C001E0,0x03E003E0,0x03F007C0,
    0x01FFFFC0,0x00FFFF80,0x007FFF00,0x000FF800, 0,0,0,0x0E00,
    0x0E00,0x0E00,0x0F00,0x0F00, 0x03FFFFC0,0x03FFFFE0,0x03FFFFE0,0x03FFFFE0, // 240..
    0,0,0,0x03800C00, 0x03E00F00,0x03F00F80,0x03F80FC0,0x03FC0FE0,
    0x03FC03E0,0x03DE01E0,0x03CF01E0,0x03C701E0, 0x03C781E0,0x03C3C3E0, // 256..
    0x03C1FFC0,0x03C1FFC0, 0x03C0FF80,0x03C03E00,0x00380C00,0x00F80F00,
    0x01F80FC0,0x01F80FC0,0x03F003E0,0x03C001E0, 0x03C0C1E0,0x03C0C1E0,
    0x03C1C1E0,0x03C1E3E0, 0x03E3FFC0,0x01FFFFC0,0x00FFBF80,0x00FF1F00,
    0x003E0000,0x001E0000,0x001F8000,0x001FE000, 0x001FF000,0x001EFC00, // 280..
    0x001E7E00,0x001E1F80, 0x001E07E0,0x03FFFFE0,0x03FFFFE0,0x03FFFFE0,
    0x03FFFFE0,0x001E0000,0x001E0000,0x001E0000, 0x00300000,0x00F1FC00,
    0x01F1FFE0,0x01F1FFE0, 0x03F1FFE0,0x03C0E1E0,0x03C0F1E0,0x03C0F1E0,
    0x03C0F1E0,0x03C0F1E0,0x01E1F1E0,0x01FFE1E0, 0x00FFE1E0,0x007FC1E0,
    0x003F0000,0x000FF800, 0x007FFE00,0x00FFFF80,0x01FFFFC0,0x01E1CFC0, // 310..
    0x03E0E3E0,0x03C0F1E0,0x03C0F1E0,0x03C0F1E0, 0x03C0F1E0,0x03E1F3E0,
    0x01FFE3C0,0x00FFE380, 0x007FC300,0x003F0000,0x01E0,0x01E0,
    0x01E0,0x03C001E0,0x03F801E0,0x03FE01E0, 0x03FF81E0,0x00FFE1E0,
    0x000FF1E0,0x0001FDE0, 0x7FE0,0x1FE0,0x07E0,0x03E0, // ..339
    0x01E0,0x003E0000,0x00FF0F00,0x01FFBF80, 0x01FFBFC0,0x03E3FFC0, // 340..
    0x03C1F3E0,0x03C1E1E0, 0x03C1E1E0,0x03C1E1E0,0x03C1F3E0,0x03E3FFC0,
    0x01FFFFC0,0x01FFBF80,0x00FF1F00,0x003E0000, 0x00207E00,0x00E1FF00,
    0x01E3FF80,0x03E3FFC0, 0x07E7C3E0,0x078781E0,0x078781E0,0x078781E0,
    0x078781E0,0x07C381E0,0x03FDC3C0,0x03FFFFC0, 0x01FFFF80,0x007FFF00,
    0x000FF800,0x000F0000, 0x000F0000,0x001F8000,0x001F8000,0x003F8000, // 370..
    0x003FC000,0x003FC000,0x007FE000,0x0079E000, 0x0079E000,0x00F9F000,
    0x00F0F000,0x00F0F000, 0x01F0F800,0x01E07800,0x01E07800,0x03C07C00,
    0x0079E000,0x0079E000,0x0079E000,0x0079E000, 0x0079E000,0x0079E000,
    0x0079E000,0x0079E000, 0x0079E000,0x0079E000,0x0079E000,0x0079E000,
    0x0079E000,0x0079E000,0x0079E000,0x0079E000, 0x03C03C00,0x01E07800, // 400..
    0x01E07800,0x01E0F800, 0x00F0F000,0x00F0F000,0x00F9F000,0x0079E000,
    0x0079E000,0x007FE000,0x003FC000,0x003FC000, 0x003F8000,0x001F8000,
    0x001F8000,0x001F0000, 0x000F0000,0x0700,0x07C0,0x07E0, 0x07E0,0x01F0,
    0x03C780F0,0x03CFC0F0, 0x03CFE0F0,0x03C7E0F0,0x0001F1F0,0xFFE0, 0x7FE0,
    0x3FC0,0x1F00,0x02000000, 0x03C00000,0x03F80000,0x03FF0000,0x01FFE000,
    0x003FFC00,0x003FFF80,0x003DFFE0,0x003C3FF0, 0x003C07F0,0x003C03F0, // 440..
    0x003C1FF0,0x003DFFF0, 0x003FFFC0,0x00FFFE00,0x03FFF000,0x03FF8000,
    0x03FC0000,0x03F00000,0x03800000,0x03FFFFF0, 0x03FFFFF0,0x03FFFFF0,
    0x03FFFFF0,0x03C1E0F0, 0x03C1E0F0,0x03C1E0F0,0x03C1E0F0,0x03C1E0F0,
    0x03C1E0F0,0x03C1E0F0,0x03C1E0F0,0x03C1F1F0, 0x03E3FFF0,0x01FFFFE0,
    0x01FF9FC0,0x00FF8F80, 0x003E0000,0x0007F800,0x001FFE00,0x007FFF80, // 470..
    0x00FFFFC0,0x01FC0FE0,0x01F003E0,0x03E001E0, 0x03C000F0,0x03C000F0,
    0x03C000F0,0x03C000F0, 0x03C000F0,0x03E001F0,0x01F003E0,0x01F80FE0,
    0x00F80FE0,0x00F80FC0,0x00780F80,0x00180E00, 0x03FFFFF0,0x03FFFFF0,
    0x03FFFFF0,0x03FFFFF0, 0x03C000F0,0x03C000F0,0x03C000F0,0x03C000F0,
    0x03C000F0,0x03C000F0,0x03C000F0,0x03E001F0, 0x01F001E0,0x01FC07E0, // 500..
    0x00FFFFC0,0x007FFF80, 0x001FFF00,0x0007FC00,0x03FFFFF0,0x03FFFFF0,
    0x03FFFFF0,0x03FFFFF0,0x03C1E0F0,0x03C1E0F0, 0x03C1E0F0,0x03C1E0F0,
    0x03C1E0F0,0x03C1E0F0, 0x03C1E0F0,0x03C1E0F0,0x03C1E0F0,0x03C1E0F0,
    0x03C1E0F0,0x03C000F0,0x03FFFFF0,0x03FFFFF0, 0x03FFFFF0,0x03FFFFF0,
    0x0001E0F0,0x0001E0F0, 0x0001E0F0,0x0001E0F0,0x0001E0F0,0x0001E0F0, // 530..
    0x0001E0F0,0x0001E0F0,0x0001E0F0,0x0001E0F0, 0x00F0,
    0x7F800,0x1FFE00,0x7FFF80, 0xFFFFC0,0xFFFFC0,0x01FC0FE0,0x01F003E0, // 541..
    0x03E001F0,0x03E001F0,0x03C000F0,0x03C000F0, 0x03C000F0,0x03C000F0,
    0x03E001F0,0x03E001F0, 0x01F003E0,0x01FC0FE0,0x00FFFFC0,0x00FFFFC0,
    0x007FFF80,0x001FFE00,0x0007F800,    0x00F0, 0x00F0,0x00F0,  // 560..
    0x00F0,0x00F0, 0x00F0,0x00F0,0x03FFFFF0,0x03FFFFF0, 0x03FFFFF0,
    0x03FFFFF0,0x00F0,0x00F0, 0x00F0,0x00F0,0x00F0,0x00F0,
    0x02000000,0x03000030,0x03C000F0,0x03F001F0,0x03F807F0,0x03FE0FF0, // 580..
    0x03FF3FF0,0x00FFFFE0,0x003FFF80,0x000FFE00,0x000FFC00,0x003FFF00,
    0x00FFFFC0,0x03FFBFF0,0x03FE0FF0,0x03FC07F0,0x03F001F0,0x03C000F0,
    0x03800030,0x02000000,             0x02000000,0x03000C00,  // 598..
    0x03C01C00,0x03E07C00, 0x03F8FC00,0x01FFFC00,0x00FFF000,0x003FC000,
    0x003FC000,0x00FFF000, 0x03FFFC00,0x03F0FC00,0x03E07C00,0x03801C00,
    0x03000C00, 0}; //~ScorFnt // ->615

// [Shared] GameEngine working data (public state)..
public static int NextRefNo = 0, CallersID = 0,
    SharedRez = 0, DrawLogger = 0, MaxWinSize = 0, NuNmMode = 0,
    WinHi = Win_Hi, WinWi = Win_Wi;
public boolean EngBasic = false, BasicEvents = false;
public int MaxLogWgts = Max_Log, MaxLogAnim = Fust_Log>>1,
    FustFocLog = Fust_Log, FustKeyLog = Fust_Log;
public int[] PxLst; // scratchpad used to make ants
public int[] thePixels = null;

public static GameWgt[] AllWgts = null; // init'd in game const'r

// [Shared] GameEngine working data (static).. // (used by static method)
public static Timer TickTock = null;
public static JavaGame theWindow = null;

  public final static char Arro_LEFT = (char)28, // (internal, Java has none)
    Arro_RIGHT = (char)29, Arro_UP = (char)30, Arro_DOWN = (char)31,
    Esc_Key = (char)27;

  public int EdgeOffs = 0, TabIsAt = 0,
      SeenLog = -1; // bits log startup milestones (Jv)
  public int[] Tab2fields = new int[16];

  final static String Quot2 = "\"";

// These names are used in converting to JavaScript..
  public final static String JSwgtMeths = " GetRefNum GetAnimBits "
        + " GetInfo GetColor GetParent GetPosn GetSize GetSelim "
        + " SetInfo SetColor SetParent SetPosn SetSize SetSelim "
        + " GetVposn GetHposn GetTall GetWide GetDnVelo GetRtVelo "
        + " SetVposn SetHposn SetTall SetWide SetDnVelo SetRtVelo "
        + " GetIxData SetIxData GetTextLn PutTextLn AddPrntsPos "
        + " IsVisible GetType SetFlags HideSho GetData SetData "
        + " SetFocus SetBlinker FindElemt XchElemt LogMe toString ",
      OrideGameMeths = " GameList StartUp toString Collided GotEnterKey "
        + " GotEscKey GotTabKey GotDelKey GotArrowKey KeyFld FocusEv BlurEv "
        + " DoOften ClickEvt MouseDn MouseUp StartDrag Dragging CancelDrag "
        + "Dropped . \nfunction o_Collided(whom,hitt) {return true;}\n"
        + "function o_GotEnterKey(whom) {return GotEnterKey(whom);}\n"
        + "function o_GotEscKey(whom) {return GotEscKey(whom);}\n"
        + "function o_GotTabKey(whom) {}\nfunction o_GotDelKey(whom) {}\n"
        + "function o_GotArrowKey(whom,info) {GotArrowKey(whom,info);}\n"
        + "function o_KeyFld(whom,info) {KeyFld(whom,info);}\n"
        + "function o_FocusEv(whom,vert,horz) {FocusEv(whom,vert,horz);}\n"
        + "function o_BlurEv(whom) {BlurEv(whom);}\n"
        + "function o_Dropped(whom) {}\n" // + "function o_GameEnding() {}\n"
        + "function o_DoOften() {}\nfunction o_ClickEvt(whom,vert,horz)"
        + " {return ClickEvt(whom,vert,horz);}\n"
        + "function o_MouseDn(whom,vert,horz) {MouseDn(whom,vert,horz);}\n"
        + "function o_MouseUp(whom,vert,horz) {MouseUp(whom,vert,horz);}\n"
        + "function o_Dragging(whom,vert,horz) {Dragging(whom,vert,horz);}\n"
        + "function o_StartDrag(whom,vert,horz) {return 0;}\n"
        + "function o_CancelDrag(GameWgt whom) {CancelDrag(whom);}\n",
      CantDoMeths = " GameEnding GetTextWide SetBlinker DrawMe "
        + " IsValidIDtxt GetIDtxt FixHaff HexyMore "
        + " FmtInfo FmtFront SetFocus SetBlinker CheckIt RadioBtn "
        + " DeleElemt AddElemt toStrMash KeyData ",
      JavaTypes = " public private final static void int boolean String "
        + " double float char JavaGame GameEvent GameWgt GameWgt[] "
        + " String[] char[] int[] double[] boolean[] ";

 /**
  * JavaResWds is the list of names you can't use for widget names
  */
    final String JavaResWds = " abstract assert boolean break byte"
        + " case catch char class const continue default double do"
        + " else enum extends false final finally float for while"
        + " goto implements if import instanceof int interface long"
        + " native new null package private protected public return"
        + " short static strictfp super switch synchronized"
        + " this throw throws transient true try void volatile ";

 /**
  * BaseMkrWgts is the widget list exported by GameMaker.GameList()
  */
  String BaseMkrWgts = "1 5 0 0 512 640 0xCCCCFF 0 0 4 60 3 2 28 "
      + "'[List of item lists] in win' \"Build Game\" -- 2021 July 10\n"
    + "2 5 0 0 512 640 -1 0 1 12 14 15 0 0 0 0 0 0 0 0 0 0 '[Sprite list]'\n"
    + "3 5 0 512 512 128 0x999999 0 1 36 5 16 20 21 22 23 24 25 26 27 29 30 "
      + "31 32 33 34 35 36 37 38 39 40 49 57 58 99 104 "
      + "107 119 126 136 149 150 151 0 0 '[Tool panel]'\n"
    + "4 5 0 0 512 512 -1 0 1 12 0 0 0 0 0 0 0 0 0 0 0 0 '[User wgts list]'\n"
    + "5 5 240 56 256 68 0xFFFFFF 0 3 12 52 53 54 55 56 62 0 0 0 0 0 0 '"
      + "[User wgt name list]'\n"
    + "6 3 0 0 12 12 0x0 0 0 0 '[Font M] Mono font'\n"
    + "7 3 0 0 12 12 0x0 1 0 0 '[Font T] Text font'\n"
    + "8 3 0 0 12 12 0x0 4 0 0 '[Font B] Bold font'\n"
    + "9 3 0 0 12 12 0x0 5 0 0 '[Font H] Head font'\n"
    + "10 3 0 0 12 12 0x0 3 0 0 '[Font Y] tinY font'\n"
    + "11 3 0 0 12 12 0x0 6 0 0 '[Font N] Nano font'\n"
    + "12 3 0 0 12 12 0x0 2 0 0 '[Font I] Ital font'\n"
    + "13 3 0 0 12 12 0x0 7 0 0 '[Font S] Score font'\n"
    + "14 241 120 40 64 276 0xFFFFFF 0 2 0 'Hello world!' [Std Alert Dialog] "
      + "`AlDg`\n"
    + "15 240 250 40 64 276 0xFFFFFF 0 2 0 'Prompt:' '(..)' [Std Input Dialog] "
      + "`IpDg`\n"
    + "16 3077 240 116 256 9 0xCCCCCC 0 3 4 17 18 19 0 '[WgLst Scrollbar]'\n"
    + "17 0 8 0 8 9 0x666666 0 16 0 '[WgLst ScroThumb]'\n"
    + "18 2 0 0 8 9 0x0 -10 16 0 '[WgLst UpBtn]'\n"
    + "19 2 248 0 8 9 0x0 -10 16 0 '[WgLst DnBtn]'\n"
    + "20 1277 4 80 16 40 -1 0 3 0 'Grid' [Grid checkbox] `CkGr`\n"
    + "21 254 48 8 16 112 -1 0 3 0 'Name=' 'Untitled' Game/wgt "
      + "[name entry panel] `NmFb`\n"
    + "22 254 72 8 16 44 -1 0 3 0 'V=' '256' [Top entry field] `VtFd`\n"
    + "23 254 72 64 16 44 -1 0 3 0 'H=' '512' [Left entry field] `HzFe`\n"
    + "24 254 96 8 16 44 -1 0 3 0 'T=' '256' [Tall entry field] `TaFf`\n"
    + "25 254 96 64 16 48 -1 0 3 0 'W=' '512' [Wide entry field] `WiFg`\n"
    + "26 1275 26 8 16 16 0xCCCC00 0 3 0 '[Color Swatch]' `CoFa`\n"
    + "27 4 29 32 12 64 0x0 8 3 0 'MainGame' [Current wgt type name]\n"
    + "28 250 0 0 32 32 -1 0 0 0 '[Deployable Marching Ants]' `Antz`\n"
    + "29 9980 24 80 20 40 0xCCCCCC 3 3 0 ['noJS' button] `noJS`\n"
    + "30 9980 24 80 20 40 0xCCCCCC 3 3 0 ['->JS' button] `MkJS`\n"
    + "31 9980 24 80 20 40 0xCCCCCC 3 3 0 ['Build' button] `Bild`\n"
    + "32 9980 24 92 20 20 0xCCCCCC 1 3 0 ['OK' (accept) button] `OKOK`\n"
    + "33 1277 120 4 16 40 -1 0 3 0 'Key' [Typable checkbox] `CkTy`\n"
    + "34 1277 120 42 16 40 -1 0 3 0 'Clik' [Clickable checkbox] `CkKc`\n"
    + "35 1277 120 80 16 40 -1 0 3 0 'Drag' [Dragable checkbox] `CkDg`\n"
    + "36 1277 144 4 16 40 -1 0 3 0 'Bump' [Collidable checkbox] `CkBm`\n"
    + "37 1277 144 56 16 40 -1 0 3 0 'Rollover' [Rollable checkbox] `CkRo`\n"
    + "38 1277 168 4 16 88 -1 0 3 0 'Alert+InputDlog' [Alrt+InDlg checkbox] "
      + "`CkAI`\n"
    + "39 254 168 8 16 88 -1 0 3 0 'I=' '-1234567890' [Rect Ginfo entry field] "
      + "`RIFl`\n"
    + "40 5 168 0 60 127 -1 0 3 8 41 42 43 44 45 46 47 48 '"
      + "[Font selector radio group]' `RdB!`\n"
    + "41 1273 24 4 16 60 -1 0 40 0 'Large' [\"Head\" font radio btn] `Rd_H`\n"
    + "42 1273 24 64 16 60 -1 0 40 0 'MonoSp' [\"Mono\" font radio btn] "
      + "`Rd_M`\n"
    + "43 1273 4 4 16 60 -1 0 40 0 'Times' [\"Text\" font radio btn] `Rd_T`\n"
    + "44 1273 4 54 16 60 -1 0 40 0 'Bold' [\"Bold\" font radio btn] `Rd_B`\n"
    + "45 1273 44 4 16 44 -1 0 40 0 'Ital' [\"Ital\" font radio btn] `Rd_I`\n"
    + "46 1273 44 42 16 44 -1 0 40 0 'tinY' [\"tinY\" font radio btn] `Rd_Y`\n"
    + "47 1273 44 84 16 44 -1 0 40 0 'Nano' [\"Nano\" font radio btn] `Rd_N`\n"
    + "48 1273 4 96 16 44 -1 0 40 0 'Sc' [\"Scor\" font radio btn] `Rd_S`\n"
    + "49 5 168 0 24 127 -1 0 3 4 50 51 0 0 '[Velocity entry group]' `Velo`\n"
    + "50 254 0 8 16 52 -1 0 49 0 'D=' '00.00' [Vertical velocity entry field] "
      + "`VvFj`\n"
    + "51 254 0 68 16 52 -1 0 49 0 'R=' '00.00' "
      + "[Horizontal velocity entry field] `HvFk`\n"
    + "52 0 1 1 18 68 0xFF9900 0 5 0 '[Name list hilight]'\n"
    + "53 0 0 0 1 68 0x0 0 5 0 'name list outline (top)'\n"
    + "54 0 0 0 256 1 0x0 0 5 0 'name list outline (left)'\n"
    + "55 0 255 0 1 68 0x0 0 5 0 'name list outline (botm)'\n"
    + "56 0 0 67 268 1 0x0 0 5 0 'name list outline (right)'\n"
    + "57 4 6 16 20 80 0xFF 9 3 0 'BYOG' blue title over tool panel\n"
    + "58 5 240 0 256 48 -1 0 3 12 59 65 70 85 76 94 99 107 126 138 0 0 '"
      + "[Tool Palette]' `Tpal`\n"
    + "59 0 0 0 30 32 0xFF9900 0 58 0 '[Tool icon hilight]'\n"
    + "60 5 0 0 512 512 -1 0 1 2 61 0 'Game workspace' `MGWS`\n"
    + "61 3845 44 44 250 444 0x66FF66 0 60 8 62 63 64 0 0 0 0 0 '"
      + "[Main Game asset list]' `MGDW` *\n"
    + "62 1028 4 4 12 60 0x0 7 5 0 'GameBd' [Main Game actual name]\n"
    + "63 4 0 0 12 64 0x0 0 61 0 'MainGame' its wgt type name (TyNm)\n"
    + "64 0 0 5 1 1 -1 0 61 0 'Main Game flags' `Flgz`\n"
    + "65 5 0 0 24 24 -1 0 58 2 66 0 'Rect tool icon base' `RcBp`\n"
    + "66 3589 6 4 20 24 0xFF 0 65 4 67 68 69 0 '[Draggable Rect tool] "
      + "`RcDT` => 'RcDW' *\n"
    + "67 1028 4 4 12 60 0x0 7 5 0 'Rect.0' (placeholder for actual name)\n"
    + "68 4 0 0 12 64 0x0 0 66 0 'Rect' its wgt type name (TyNm)\n"
    + "69 0 0 0 1 1 -1 0 66 0 'Rect tool flags' `Flgz`\n"
    + "70 5 32 0 32 32 -1 0 58 2 71 0 'Text tool icon base' `TxBp`\n"
    + "71 3589 8 0 24 32 -1 0 70 5 72 73 74 75 0 '[Draggable Text tool]' "
      + "`TxDT` => 'TxDW' *\n"
    + "72 4 0 0 32 32 0x0 8 71 0 'Text' 'Text tool image' (TxTx)\n"
    + "73 1028 4 4 12 60 0x0 7 5 0 'Text.0' (placeholder for actual name)\n"
    + "74 4 0 0 12 64 0x0 0 71 0 'Text' its wgt type name (TyNm)\n"
    + "75 0 0 4 1 1 -1 0 71 0 'Text tool flags' `Flgz`\n"
    + "76 5 128 0 32 32 -1 0 58 2 77 0 'Circle tool icon base' `CrBp`\n"
    + "77 3589 0 0 32 32 -1 0 76 6 78 79 82 83 84 0 '[Draggable Circle tool] "
      + "`CrDT` => 'CrDW' *\n"
    + "78 2 0 0 32 32 0x6600CC -3 77 0 'Circle tool icon' (CrIc)\n"
    + "79 5 0 0 32 32 -1 0 77 2 80 81 '(hidable) [Circle tool icon ants]' "
      + "`AntI`\n"
    + "80 2 0 0 32 32 0x10000 -3 79 0 "
      + "'Circle tool icon hollow outline' (CrHo)\n"
    + "81 2 0 0 32 32 0xFEFFFF -3 79 0 'Circle tool icon ants' (CrAn)\n"
    + "82 1028 4 4 12 60 0x0 7 5 0 'Circ.0' (placeholder for actual name)\n"
    + "83 4 0 0 12 64 0x0 0 77 0 'Circle' its wgt type name (TyNm)\n"
    + "84 0 0 2 1 1 -1 0 77 0 'Circle tool flags' `Flgz`\n"
    + "85 5 64 0 32 32 -1 0 58 2 86 0 'Ball sprite tool icon base' `BsBp`\n"
    + "86 3589 0 0 32 32 -1 0 85 6 87 88 91 92 93 0 '[Draggable Ball tool] "
      + "`BsDT` => 'BsDW' *\n"
    + "87 2 0 0 32 32 0xFF0000 -3 86 0 'Ball tool icon' (BsIc)\n"
    + "88 5 0 0 32 32 -1 0 86 2 89 90 '(hidable) [Ball tool icon ants]' "
      + "`AntI`\n"
    + "89 2 0 0 32 32 0x10000 -3 88 0 'Ball tool icon hollow outline' (BsHo)\n"
    + "90 2 0 0 32 32 0xFEFFFF -3 88 0 'Ball tool icon ants' (BsAn)\n"
    + "91 1028 4 4 12 60 0x0 7 5 0 'Ball.0' (placeholder for actual name)\n"
    + "92 4 0 0 12 64 0x0 0 86 0 'Ball' its wgt type name (TyNm)\n"
    + "93 0 0 0 1 1 -1 0 86 0 'Ball tool flags' `Flgz`\n"
    + "94 5 96 0 24 24 -1 0 58 2 95 0 'SkyRec sprite tool icon base' `FSBp`\n"
    + "95 3589 4 4 24 24 0x99CCFF 0 94 4 96 97 98 0 '[Rect Sprite tool] "
      + "`FSDT` => 'FSDW' *\n"
    + "96 1028 4 4 12 60 0x0 7 5 0 'SkyRx.0' (placeholder for actual name)\n"
    + "97 4 0 0 12 64 0x0 0 95 0 'SkyRec' its wgt type name (TyNm)\n"
    + "98 0 0 0 1 1 -1 0 95 0 'SkyRec tool flags' `Flgz`\n"
    + "99 5 160 0 24 24 -1 0 58 2 100 0 'Diag tool icon base' `DcBp`\n"
    + "100 3589 4 6 24 21 0x996600 0 99 4 101 102 103 0 '[Diagonal line tool] "
      + "`DcDT` => 'DcDW' *\n"
    + "101 1028 4 4 12 60 0x0 7 5 0 'Diag.0' (placeholder for actual name)\n"
    + "102 4 0 0 12 64 0x0 0 100 0 'DiagLn' its wgt type name (TyNm)\n"
    + "103 0 0 0 1 1 -1 0 100 0 'Diag tool flags' `Flgz`\n"
    + "104 5 168 0 20 120 -1 0 3 4 105 106 0 0 '[Diagonal line group] `DlIG`\n"
    + "105 1277 0 4 16 40 -1 0 104 0 'Up' [Diagonal Up checkbox] `CkUp`\n"
    + "106 254 2 40 16 40 -1 0 104 0 'Th=' '3' "
      + "[Diagonal thickness entry field] `DtFm`\n"
    + "107 5 192 0 32 32 -1 0 58 2 108 0 'RocPapScis tool icon base' `QxBp`\n"
    + "108 3333 0 0 32 32 -1 0 107 6 109 117 118 124 0 0 '[Draggable RPS tool] "
      + "`QvDT` => 'QvDW' *\n"
    + "109 6 0 0 32 32 -1 0 108 6 110 111 112 113 114 0 "
      + "'RPS tool token sprite'\n"
    + "110 2 0 0 32 32 0xFFFFFF 0 109 0 'RPS tool paper icon' [QxPi]\n"
    + "111 2 0 0 32 32 0x666666 0 109 0 'RPS tool rock icon' [QxRi]\n"
    + "112 2 0 0 32 32 0x0 0 109 0 'RPS tool scissors icon' [QxSi]\n"
    + "113 2 0 0 32 32 0xDDDDDD 0 109 0 'RPS tool cloud icon' [QxQi]\n"
    + "114 5 0 0 32 32 -1 0 109 2 115 116 '(hidable) [RPS tool icon ants]' "
      + "`AntI`\n"
    + "115 2 0 0 32 32 0x10000 -9 114 0 'RPS tool icon hollow outline' (QxHo)\n"
    + "116 2 0 0 32 32 0xFEFFFF -9 114 0 'RPS tool icon ants' (QxAn)\n"
    + "117 1028 4 4 12 60 0x0 7 5 0 'RoPaSc.0' (placeholder for actual name)\n"
    + "118 4 0 0 12 64 0x0 0 108 0 'RocPapSci' its wgt type name (TyNm)\n"
    + "119 5 188 0 40 127 -1 0 3 4 120 121 122 123 '"
      + "[RPS selector radio group]' `RdB!`\n"
    + "120 1273 4 4 16 60 -1 0 119 0 'Rock' [Rock RPS radio btn] `Rd_2`\n"
    + "121 1273 4 64 16 60 -1 0 119 0 'Paper' [Paper RPS radio btn] `Rd_1`\n"
    + "122 1273 24 4 16 60 -1 0 119 0 'Scissor' [Scis RPS radio btn] `Rd_3`\n"
    + "123 1273 24 64 16 60 -1 0 119 0 'Cloud' [Cloud RPS radio btn] `Rd_4`\n"
    + "124 0 0 0 1 1 -1 0 108 0 'RPS tool flags' `Flgz`\n"
    + "125 5 0 0 512 640 -1 0 1 0 '[null widget]' `XxDT` => 'XxDW' *\n"
    + "126 5 224 0 32 32 -1 0 58 2 127 0 'Kitchen tool icon base' `KxBp`\n"
    + "127 3333 0 0 32 32 -1 0 126 6 128 134 135 137 0 0 '"
      + "[Draggable Kitchen tool] <xb> `K0DT` => 'K0DW' *\n"
    + "128 6 0 0 32 32 -1 0 127 6 129 130 131 0 0 0 "
      + "'Kitchen tool token sprite' (KF) <bx>\n"
    + "129 2 0 0 32 32 0xFFFFFF 0 128 0 'Kitchen tool 1st icon' [KxXi]\n"
    + "130 2 0 0 32 32 0xCC6600 0 128 0 'Kitchen tool 2nd icon' [KxBi]\n"
    + "131 5 0 0 32 32 -1 0 128 2 132 133 '(hidable) [Kitchen tool icon ants]' "
      + "`AntI`\n"
    + "132 2 0 0 32 32 0x10000 -9 131 0 "
      + "'Kitchen tool icon hollow outline' (KxHo)\n"
    + "133 2 0 0 32 32 0xFEFFFF -9 131 0 'Kitchen tool icon ants' (KxAn)\n"
    + "134 1028 4 4 12 60 0x0 7 5 0 'Kitch.0' (placeholder for actual name)\n"
    + "135 4 0 0 12 64 0x0 0 127 0 'Kitchen' its wgt type name (TyNm)\n"
    + "136 254 188 8 20 127 -66 0 3 0 'M=' 'BCDFJKLMPSTX' "
      + "[Kitchen inventory entry field] `KtFm`\n"
    + "137 0 0 0 1 1 -1 0 127 0 'Kitchen tool flags' `Flgz`\n"
    + "138 5 256 0 24 24 -1 0 58 2 139 0 'Sprite tool icon base' `SpBp`\n"
    + "139 3333 4 4 24 24 -1 0 138 6 140 146 147 148 0 0 '"
      + "[Draggable Sprite tool] `SpDT` => 'SpDW' *\n"
    + "140 6 0 0 24 24 -1 0 139 4 141 142 143 0 'Sprite tool token sprite' "
      + "`SpSp`\n"
    + "141 0 8 0 10 20 0xAA00 0 140 0 'Sprite tool green rect' (Spgr)\n"
    + "142 0 0 8 20 8 0xFF5500 0 140 0 'Sprite tool orange rect' (Spor)\n"
    + "143 5 0 0 32 32 -1 0 140 2 144 145 '(hidable) [Sprite tool icon ants]' "
      + "`AntI`\n"
    + "144 2 0 0 24 24 0x10000 -9 143 0 "
      + "'Sprite tool icon hollow outline' (SpHo)\n"
    + "145 2 0 0 24 24 0xFEFFFF -9 143 0 'Sprite tool icon ants' (SpAn)\n"
    + "146 1028 4 4 12 60 0x0 7 5 0 'Sprit.0' (placeholder for actual name)\n"
    + "147 4 0 0 12 64 0x0 0 139 0 'Sprite' its wgt type name (TyNm)\n"
    + "148 0 0 0 1 1 -1 0 139 0 'Sprite tool flags' `Flgz`\n"
    + "149 9980 216 4 20 120 0xCCCCCC 1 3 0 'Start Over (New)' "
      + "[Restart button] `Rnew`\n"
    + "150 4 500 80 9 36 0x6600 10 3 0 '21 / 7 / 10 ' [RevDate]\n"
    + "151 4 500 4 9 84 0x6600 10 3 0 ' ' [Status Report]\n";

 /**
  * UsrGameJv is the source for the subclass generated by the Build button
  */
  public final static String UsrGameJv = "\nimport javagame.GameWgt;\n"
  + "import javagame.GameEvent;\nimport javagame.JavaGame;\n\n/**\n"
  + " * Subclass to handle events when creating a new game. 2020 November 14\n"
  + " *\n * The name of this class is the same as the name of your Game. If\n"
  + " * you change the name in GameMaker, then the Build button will create\n"
  + " * a new (or modify an existing) Java file with the new name.\n *\n"
  + " * Three sections of this code (beginning and ending with \"$\" tags)\n"
  + " * are generated when you click the \"Build\" button. You should avoid\n"
  + " * modifying these lines because your modifications will be lost when\n"
  + " * you click Build again. If you modify or move the tag lines the Build\n"
  + " * process may be thwarted or unpredictable. Move or rename your Java\n"
  + " * file and it will create a new clean copy in its place.\n */\n"
  + "public class UserGame extends GameEvent {\n"
  + "  JavaGame myGame = null; // soon: reference to the GameEngine\n"
  + "  GameWgt myUserGame = null; // reference to the game board widget\n"
  + "  /// ($) Begin generated class variables (do not modify) ($..)\n"
  + "  /// (..$) End generated class variables ($) (do not modify this line)\n"
  + "\n  public String toString() {return \"(UserGame)\";} //~toString"
  + "\n\n  // These called regardless:\n\n /**\n"
  + "  * (Generated code) GameList returns a text descriptor of all your\n"
  + "  * specified widgets. This code is created when you click the \"Build\"\n"
  + "  * button in GameMaker. It is called once before StartUp, then builds\n"
  + "  * the widgets from your text descriptor(s). You must call the super\n"
  + "  * to preserve the reference to the host JavaGame.\n  *\n"
  + "  * @param  whom  A reference to the host JavaGame, kept for later use\n"
  + "  *\n  * @return       The generated text string\n  */\n"
  + "  public String GameList(JavaGame whom) {\n"
  + "    String info = super.GameList(whom), data\n"
  + "    /// <$> Begin generated widget list (do not modify) <$..>\n"
  + "    /// <..$> End generated widget list <$> (do not modify this line)\n"
  + "    myGame = whom; // save a local reference to the GameEngine\n"
  + "    return data;} //~GameList\n\n /**\n"
  + "  * (Partly generated code) After the initial widgets have been created,\n"
  + "  * StartUp is called to do any other initialization you may have.\n"
  + "  * If you gave Java-correct names to any widgets, those variables are\n"
  + "  * assigned values by the generated code.\n  */\n"
  + "  public void StartUp() {\n"
  + "    if (myGame != null) myUserGame = myGame.FindListWgt(\"{UserGame}\");\n"
  + "    /// [$] Begin generated StartUp code (do not modify) [$..]\n"
  + "    /// [..$] End generated StartUp code [$] (do not modify this line)\n"
  + "    } //~StartUp\n\n /**\n"
  + "  * DoOften is called once each frame to do whatever game processing\n"
  + "  * needs to be done often. You should get in and get out, so that the\n"
  + "  * animation flows smoothly. Omit this method if you do not need it.\n"
  + "  */\n  // public void DoOften() {} //~DoOften\n\n /**\n"
  + "  * GameEnding is called once when the game window is about to close,\n"
  + "  * so you can do whatever needs to be done at the end.\n"
  + "  * Omit this method if you do not need it.\n  */\n"
  + "  // public void GameEnding() {} //~GameEnding\n\n"
  + "  // This is enabled by Gtype&256 (Collidable):\n\n /**\n"
  + "  * Collided is called once for each Sprite widget that is flagged as\n"
  + "  * Collidable (\"Bump\") when it touches any other widget also flagged\n"
  + "  * Collidable.\n  *\n  * @param  whom  The Sprite widget\n"
  + "  * @param  hitt  The other widget in the collision\n  *\n"
  + "  * @return       False if this collision is not satisfactory, or if\n"
  + "  *               you want to consider other (simultaneous) collisions\n"
  + "  *               for whom; true to terminate processing this Sprite.\n"
  + "  *               Note that other Sprites may yet have collisons pending\n"
  + "  *               in this same frame, resulting in additional calls\n"
  + "  *               on Collided.\n  */\n"
  + "  // public boolean Collided(GameWgt whom, GameWgt hitt) {\n"
  + "  //   if (keepLooking) return false;\n"
  + "  //   return true;} //~Collided // return true to stop looking\n\n"
  + "  // These events are enabled by Gtype&512 (Focusable):\n\n /**\n"
  + "  * GotEnterKey is called when the user types the EnterKey,\n"
  + "  * if FocusWgt is not null, or else if DefaultFocus is not null,\n"
  + "  * and that widget accepts keystrokes. If your game can open\n"
  + "  * a standard alert or input dialog, you should call super so\n"
  + "  * it can get its OK clicked. Your subclass can return either\n"
  + "  * true or false, because the first caller doesn't care.\n  *\n"
  + "  * @param  whom  The widget that has focus\n  *\n"
  + "  * @return       true if taken (by alert or input dlog)\n  */\n"
  + "  // public boolean GotEnterKey(GameWgt whom) {\n"
  + "  //   if (super.GotEnterKey(whom)) HandleDlogOK();\n"
  + "  //     else DoWhatever();\n"
  + "  //   return false;} //~GotEnterKey\n\n /**\n"
  + "  * GotEscKey is called when the user types the EscapeKey,\n"
  + "  * if FocusWgt is not null, or else if DefaultFocus is not null,\n"
  + "  * and that widget accepts keystrokes. If your game can open\n"
  + "  * a standard alert or input dialog, you should call super so\n"
  + "  * it can get its Cancel clicked. Your subclass can return either\n"
  + "  * true or false, because the first caller doesn't care.\n  *\n"
  + "  * @param  whom  The widget that has focus\n  *\n"
  + "  * @return       true if taken (by alert or input dlog)\n  */\n"
  + "  // public boolean GotEscKey(GameWgt whom) { // o'ride to do more\n"
  + "  //   if (super.GotEscKey(whom)) return true;\n  //   DoWhatever();\n"
  + "  //   return false;} //~GotEscKey\n\n /**\n"
  + "  * GotTabKey is called when the user types the TabKey,\n"
  + "  * if FocusWgt is not null, or else if DefaultFocus is not null,\n"
  + "  * and that widget accepts keystrokes.\n  *\n"
  + "  * @param  whom  The widget that has focus\n  */\n"
  + "  // public void GotTabKey(GameWgt whom) {} //~GotTabKey\n\n /**\n"
  + "  * GotArrowKey is called for any widget that has focus when the user\n"
  + "  * types one of the arrow keys. If FocusWgt is null and DefaultFocus\n"
  + "  * is null, or if that widget does not accept keystrokes, then\n"
  + "  * GotArrowKey is not called. The base class uses GotArrowKey to move\n"
  + "  * a dragable widget one pixel at time, so you should call super if\n"
  + "  * this is not yours.\n  *\n"
  + "  * @param  whom  The widget that has focus\n"
  + "  * @param  info  The arrow key that was pressed, one of the constant\n"
  + "  *               values JavaGame.Arro_UP, JavaGame.Arro_DOWN,\n"
  + "  *               JavaGame.Arro_LEFT, or JavaGame.Arro_RIGHT.\n  */\n"
  + "  // public void GotArrowKey(GameWgt whom, char info) {\n"
  + "  //   if (didSomethingWithIt) return;       // but call super..\n"
  + "  //   super.GotArrowKey(whom,info);} //~GotArrowKey\n\n /**\n"
  + "  * KeyFld is called for any widget that has focus when the user\n"
  + "  * types any other key. If FocusWgt is null and DefaultFocus is null,\n"
  + "  * or if that widget does not accept keystrokes, then KeyFld is not called.\n"
  + "  * The base class uses KeyFld to insert keystrokes in an entry text field,\n"
  + "  * so you should call super if this is not yours.\n  *\n"
  + "  * @param  whom  The widget that has focus\n"
  + "  * @param  info  The key that was typed\n  */\n"
  + "  // public void KeyFld(GameWgt whom, char info) {\n"
  + "  //   if (didSomethingWithIt) return;       // but call super..\n"
  + "  //   super.KeyFld(whom,info);} //~KeyFld\n\n /**\n"
  + "  * FocusEv is called for any widget that is \"Focusable\" when the user\n"
  + "  * clicks on it. Your code may also call FocusEv to give focus to a\n"
  + "  * widget for any other reason. You must call super to do the default\n"
  + "  * focus processing unless you choose to cancel it. The base class\n"
  + "  * calls the whom.SetFocus method for this widget, and the widget also\n"
  + "  * has the opportunity to cancel the focus.\n  *\n"
  + "  * Only one widget at a time can have focus, so BlurEv is called for\n"
  + "  * any previous widget with focus before any other widget can get it.\n"
  + "  *\n  * @param  whom  The widget to be given focus\n"
  + "  * @param  vert  The vertical offset into this widget where\n"
  + "  *                 it was clicked\n"
  + "  * @param  horz  The horizontal offset into this widget\n  */\n"
  + "  // public void FocusEv(GameWgt whom, int vert, int horz) {\n"
  + "  //   if (didSomethingWithIt) return;\n"
  + "  //   super.FocusEv(whom,vert,horz);} //~FocusEv\n\n /**\n"
  + "  * BlurEv is called for a widget when it loses focus. Your code may\n"
  + "  * also call BlurEv to unconditionally terminate focus for any other\n"
  + "  * reason. You should call super to do the default defocussing\n"
  + "  * (including telling the widget that it is losing focus) unless you\n"
  + "  * want to cancel the blur. If you cancel a blur that is called before\n"
  + "  * another widget is about to gain focus, then that focus is also\n"
  + "  * cancelled.\n  *\n"
  + "  * @param  whom  The widget losing focus, or null if unknown by caller\n"
  + "  */\n  // public void BlurEv(GameWgt whom) {\n"
  + "  //   if (cancellingThisBlur) return;\n  //   super.BlurEv(whom);\n"
  + "  //   // (do additional processing)\n  //   } //~BlurEv\n\n"
  + "  // This event is enabled by Gtype&1024 (Clickable):\n\n /**\n"
  + "  * ClickEvt is called for a widget that is \"Clickable\" when the user\n"
  + "  * clicks on it (mouse down followed by mouse up on the same widget).\n"
  + "  * FocusEv has already been called for this widget. The base class uses\n"
  + "  * ClickEvt to detect and process double-click events, so you should call\n"
  + "  * super if you want that to happen and this is not yours.\n  *\n"
  + "  * You would normally override ClickEvt to process button clicks.\n  *\n"
  + "  * The vertical and horizontal offsets are relative to the top-left\n"
  + "  * corner of this widget. Add whom.AddPrntsPos(whom.GetPosn()) to get\n"
  + "  * the position relative to the window.\n  *\n"
  + "  * @param  whom  The widget that was clicked\n"
  + "  * @param  vert  The vertical offset into this widget where\n"
  + "  *                 it was clicked\n"
  + "  * @param  horz  The horizontal offset into this widget\n  *\n"
  + "  * @return       true if taken (by 2-click or alert or input dlog)\n"
  + "  */\n  // public boolean ClickEvt(GameWgt whom, int vert, int horz) {\n"
  + "  //   if (super.ClickEvt(whom,vert,horz)) return true; // already did\n"
  + "  //   // DoWhatever();\n"
  + "  //   return false;} //~ClickEvt // (or true: nobody else cares)\n\n"
  + "  // These events are enabled by Gtype&2048 (Dragable):\n\n"
  + "  // DragonDrop (drag and drop) consists of the sequence of pressing the\n"
  + "  //   mouse button over a widget, then while holding the mouse button\n"
  + "  //   down, moving the mouse to some other place on the screen, then\n"
  + "  //   releasing the mouse button at that location. The default action\n"
  + "  //   in the base class for dragable widgets is to drag (move) the\n"
  + "  //   widget that was clicked to the new location. After a widget that\n"
  + "  //   is both dragable and focusable has been dragged with the mouse,\n"
  + "  //   you can fine-tune its final position using the arrow keys. You\n"
  + "  //   can specify other behavior for these events by overriding these\n"
  + "  //   methods.\n\n /**\n"
  + "  * MouseDn is called for a widget that is \"Dragable\" when the user\n"
  + "  * presses the (right) mouse button down on some widget. If the button\n"
  + "  * is released without moving the mouse, then it is just a ClickEvt\n"
  + "  * and no drag is attempted.\n  *\n"
  + "  * The vertical and horizontal offsets are relative to the top-left\n"
  + "  * corner of this widget. Add whom.AddPrntsPos(whom.GetPosn()) to get\n"
  + "  * the position relative to the window.\n  *\n"
  + "  * @param  whom  The widget that was clicked\n"
  + "  * @param  vert  The vertical offset into this widget where it\n"
  + "  *                 was clicked\n"
  + "  * @param  horz  The horizontal offset into this widget\n  */\n"
  + "  // public void MouseDn(GameWgt whom, int vert, int horz) {\n"
  + "  //   if (didSomethingWithIt) return;\n"
  + "  //   super.MouseDn(whom,vert,horz);} //~MouseDn\n\n /**\n"
  + "  * After a MouseDn is called for a widget that is \"Dragable\" and the\n"
  + "  * user moves the mouse at least eight pixels with the button still\n"
  + "  * pressed, then it becomes a drag and StartDrag is called. If you\n"
  + "  * override this method, you can either cancel the drag by returning\n"
  + "  * null (without calling super), or else substitute another (possibly\n"
  + "  * new) widget for (in the same location as) the clicked widget when you\n"
  + "  * call super, and the replacement will be what is dragged. If you do\n"
  + "  * this, there is some additional code needed in your MouseUp override\n"
  + "  * so that the Dropped event works correctly.\n  *\n"
  + "  * The vertical and horizontal offsets are relative to the top-left\n"
  + "  * corner of this widget. Add whom.AddPrntsPos(whom.GetPosn()) to get\n"
  + "  * the position relative to the window.\n  *\n"
  + "  * @param  whom  The widget that was clicked\n"
  + "  * @param  vert  The vertical offset into this widget where it\n"
  + "  *                 was clicked\n"
  + "  * @param  horz  The horizontal offset into this widget\n  *\n"
  + "  * @return       The widget being dragged, or null if cancelled\n  */\n"
  + "  // public GameWgt StartDrag(GameWgt whom, int vert, int horz) {\n"
  + "  //   // subclass is allowed to substitute another wgt for whom,\n"
  + "  //   //   then call super\n"
  + "  //   if (dontWantToStartDrag) return null;\n"
  + "  //   return super.StartDrag(whom,vert,horz);} //~StartDrag\n\n /**\n"
  + "  * After a StartDrag has been called for a Dragable widget and the user\n"
  + "  * continues to move the mouse, Dragging is called for the same widget.\n"
  + "  * You can override this method to do something with the dragged widget's\n"
  + "  * new position. Dragging in the base class updates the dragged widget's\n"
  + "  * position, so you must call super before you try to use that position.\n"
  + "  *\n"
  + "  * The vertical and horizontal offsets are relative to the top-left corner\n"
  + "  * of this widget before its position has been adjusted by super.\n  *\n"
  + "  * To get the position relative to the window, you should calculate\n"
  + "  * whom.AddPrntsPos(whom.GetPosn()) before calling super.\n  *\n"
  + "  * @param  whom  The widget that is being dragged\n"
  + "  * @param  vert  The vertical offset into this widget before it moves\n"
  + "  * @param  horz  The horizontal offset into this widget\n  */\n"
  + "  // public void Dragging(GameWgt whom, int vert, int horz) {\n"
  + "  //   super.Dragging(whom,vert,horz);\n  //   // DoWhatever();\n"
  + "  //   } //~Dragging\n\n /**\n"
  + "  * After a StartDrag has been called for a Dragable widget and the user\n"
  + "  * releases the mouse button, Dropped is called for the same widget.\n"
  + "  * If you override this method and do something with the dropped widget\n"
  + "  * that could invalidate further drags on it using the arrow keys, you\n"
  + "  * should also call BlurEv for it, or else make sure it cannot take\n"
  + "  * focus. The base class does nothing with Dropped, so it is not\n"
  + "  * necessary to call super.\n  *\n"
  + "  * @param  whom  The widget that was dragged\n  */\n"
  + "  // public void Dropped(GameWgt whom) {} //~Dropped\n\n /**\n"
  + "  * MouseUp is called for a widget that is \"Dragable\" when the user\n"
  + "  * releases the mouse button over the same widget that was clicked or\n"
  + "  * dragged. If the mouse is moved away from the clicked widget (and\n"
  + "  * there is no drag in process) then MouseUp is not called (and there\n"
  + "  * is no consequent ClickEvt).\n  *\n"
  + "  * The vertical and horizontal offsets are relative to the top-left\n"
  + "  * corner of this widget. Add whom.AddPrntsPos(whom.GetPosn()) to get\n"
  + "  * the position relative to the window.\n  *\n"
  + "  * @param  whom  The widget that was clicked\n"
  + "  * @param  vert  The vertical offset into this widget where it\n"
  + "  *                 was clicked\n"
  + "  * @param  horz  The horizontal offset into this widget\n  */\n"
  + "  // public void MouseUp(GameWgt whom, int vert, int horz) {\n"
  + "  //   if (didSomethingWithIt) return;\n"
  + "  //   super.MouseUp(whom,vert,horz);} //~MouseUp\n\n /**\n"
  + "  * This is the main() you start your game in.\n"
  + "  * DO NOT MODIFY THIS CODE.\n  */\n"
  + "  public static void main() { // (in UserGame)\n"
  + "    JavaGame whom = new JavaGame();\n"
  + "    GameEvent evh = new UserGame();\n"
  + "    String title = \"UserGame\", data = \"\";\n"
  + "    int tall = 0, wide = 0, colo = 0x0;\n"
  + "    if (whom != null) if (evh != null) {\n"
  + "      data = evh.GameList(whom);\n"
  + "      tall = JavaGame.SafeParseInt(JavaGame.NthItemOf(' ',5,data));\n"
  + "      wide = JavaGame.SafeParseInt(JavaGame.NthItemOf(' ',6,data));\n"
  + "      colo = JavaGame.SafeParseInt(JavaGame.NthItemOf(' ',7,data));\n"
  + "      whom.StartGame(tall,wide,colo,title,evh);\n"
  + "      if (false) whom.LogAlWgts(\"[main] \");}\n"
  + "    if (false) System.out.println(\"{main} \" + tall + \"x\" + wide"
  + " + \" '\" + title + \"'\");}\n\n /**\n  * Constructor // RevDate ="
  + " \"21/7/10\"\n  */\n  public UserGame() {\n"
  + "    if (false) System.out.println(\"(new UserGame)\");}} //~UserGame\n";

 /**
  * This main should be called to run JavaGame in GameMaker mode. To run
  * in user game mode, use the main() code in the generated Java file.
  */
  public static void main() { // (in JavaGame) (Java only)
    JavaGame whom = new JavaGame(); // alt: int main(String args[]) + return 0;
    GameEvent evh = null;
    String data = "", title = "";
    int tall = 0, wide = 0, nx, zx, colo = 0xFFFFFF;
    while (true) {
      if (whom == null) break;
      title = "(Oops!)";
      evh = whom.NewMaker();
      if (evh == null) break;
      // theWindow = whom; // now in const'r
      data = evh.GameList(whom);
      zx = NthOfset(0,"\n",data);
      nx = NthOfset(1,Quot2,data);
      if (nx<zx) if (nx>0) title = NthItemOf('\"',2,data);
      if (StrLength(title)==0) while (true) {
        nx = NthOfset(0,"}",data);
        if (nx<0) break;
        if (nx>zx) break;
        zx = NthOfset(0,"{",data);
        if (zx<0) break;
        if (zx>nx) break;
        title = NthItemOf('{',2,NthItemOf('}',1,data));
        if (StrLength(title)==0) title = "Game Engine";
        break;} //~while
      tall = SafeParseInt(NthItemOf(' ',5,data));
      wide = SafeParseInt(NthItemOf(' ',6,data));
      colo = SafeParseInt(NthItemOf(' ',7,data));
      whom.StartGame(tall,wide,colo,title,evh);
      if (Logging) LogAlWgts("[main] ");
      break;} //~while
    if (Logging)
      System.out.println("{main} " + tall + "x" + wide + " '" + title + "'");}

 /**
  * Return the class name for this object.
  *
  * @return     The reference String for this object
  */
  public String toString() {return "(JavaGame)";} //~toString

  // Handy (public utility) methods..

/**
  * A safe (no exceptions) non-OOPS way to extract a character from a string.
  *
  * @param  here  The (0-base) character position
  * @param  aStr  The string from which to extract that character
  *
  * @return       The character extracted, or '\0' if out of bounds
  */
  public static char CharAt(int here, String aStr)  { // Java version..
    if (here<0) return '\0';
    if (aStr == null) return '\0';
    try {
      if (here >= aStr.length()) return '\0';
      return aStr.charAt(here);}
    catch (Exception ex)  {return '\0';}} //~CharAt

 /**
  * A safe (no exceptions) non-OOPS way to get the length of a string.
  *
  * @param  aStr  The string to get the length of
  *
  * @return       Its length, or 0 if null
  */
  public static int StrLength(String aStr)  { // Java version..
    if (aStr == null) return 0;
    return aStr.length();} //~StrLength

 /**
  * Strongly-typed (not overloaded) string selector.
  * This is part of a collection of tools to build debug print lines.
  *
  * @param  whom  The selector
  * @param  tru   Returned if whom=true
  * @param  fls   Returned if whom=false
  *
  * @return       The selected string, tru if whom=true, else fls
  */
  public static String IffyStr(boolean whom, String tru, String fls)  {
    if (whom) return tru;
    return fls;} //~IffyStr

 /**
  * A safe (no exceptions) way to get the rest of a string.
  * Text past the end is considered empty.
  *
  * @param  here  The (0-base) position to start
  * @param  aStr  The string from which to extract that substring
  *
  * @return       The rest of the string, after here
  */
  public static String RestOf(int here, String aStr)  {
    int lxx;
    try {
    if (here<0) return "";
    lxx = StrLength(aStr);
    if (lxx==0) return "";
    if (here >= lxx) return "";
    return aStr.substring(here);
    } catch (Exception ex)  {}
    return "";  } //~RestOf

  /**
  * A safe (no exceptions) way to extract a substring.
  * Text past the end is empty.
  *
  * @param  here  The (0-base) position to start
  * @param  lxx   The desired length (the result could be less)
  * @param  aStr  The string from which to extract that substring
  *
  * @return       The substring
  */
  public static String Substring(int here, int lxx, String aStr)  {
    int nx = StrLength(aStr);
    try {
    if (here<0) return "";
    if (lxx <= 0) return "";
    if (nx==0) return "";
    if (here >= nx) return "";
    lxx = here+lxx;
    if (lxx >= nx) return RestOf(here,aStr);
    return aStr.substring(here,lxx);
    } catch (Exception ex)  {}
    return "";  } //~Substring

  /**
  * Formats an integer as hexadecimal and returns it with a prefix and suffix.
  * This is useful when printing numbers that you want to see the bits of.
  *
  * @param  before The prefix
  * @param  whom   The number to be formatted
  * @param  after  The suffix
  *
  * @return       The combined string
  */
  public static String SeeHex(String before, int whom, String after)  {
    int shft = 28, lxx = StrLength(before); // special-case URL encoding..
    if (lxx==1) if ((whom&-256)==0) if (StrLength(after)==0)
        if ((int)(CharAt(0,before)) == 37)  { // '%'
      if (whom>15)  {
        before = before + SeeHex("",whom>>4,"");
        whom = whom&15;} //~if
      else before = before + "0";} //~if
    if (whom<0)  {
      before = SeeHex(before,(whom>>28)&15,"");
      if (whom+0x00100000 >= 0)  { // remove leading 'F's..
        shft = 16;
        while ((1<<shft)+whom >= 0)  {
          shft = shft-4;
          if (shft==0) break;} //~while
        while (shft>0)  { // skip over (internal) leading zeros..
          if (((15<<shft)&whom) !=0) break;
          shft = shft-4;} //~while
        whom = whom&((16<<shft)-1);
        before = before + "-F";} //~if
      else whom = whom&0x0FFFFFFF;} //~if // (whom<0)
    if (whom>15) before = SeeHex(before,whom>>4,"");
    return before + Substring(whom&15,1,"0123456789ABCDEF")
        + after;} //~SeeHex

  /**
  * Extracts and sign-extends the second of a pair of half-integers.
  *
  * @param  here  The pair
  *
  * @return       The low half, sign-extended to a full int
  */
  public static int SignExtend(int here)  { // low 16 bits signed
    return (here&0x7FFF)-(here&0x8000);} //~SignExtend

  /**
  * Formats a 32-bit integer as two 16-bit values and returns it with a prefix
  * and suffix. This is useful when printing numbers that pack two smaller
  * integers (like board coordinates) into a single number.
  *
  * @param  before The prefix
  * @param  whom   The number to be formatted
  * @param  after  The suffix
  *
  * @return       The combined string
  */
  public static String LogVH(String before, int whom, String after)  {
    return before + (whom>>16) + "/" + SignExtend(whom) + after;} //~LogVH

/**
  * A safe (non-OOPS, no exceptions) way to replace every instance of
  * a particular string within a larger string.
  * An empty string cannot be replaced.
  *
  * @param  nuly    The replacement string
  * @param  prio    The string to replace
  * @param  theText The string in which to make the substitutions
  *
  * @return         theText as changed
  */
  public static String ReplacAll(String nuly, String prio, String theText)  {
    int whar, thar = 0, lxx = StrLength(theText),
        nby = StrLength(prio),
        more = StrLength(nuly);
    if (nby>0) if (lxx >= nby) try {  while (true)  {
      whar = theText.indexOf(prio,thar); // JS=Java
      if (whar<0) break;
      theText = Substring(0,whar,theText) + nuly + RestOf(whar+nby,theText);
      thar = whar+more;} //~while
    } catch (Exception ex)  {}
    return theText;} //~ReplacAll

  /**
  * A safe (no exceptions) way to find the (0-based) offset of a
  * particular string within a larger string, while ignoring zero
  * or more initial (non-overlapping) occurrences. An empty string
  * cannot be found.
  *
  * @param  whom  The number of occurrences to skip over
  * @param  aWord The string to look for
  * @param  aStr  The string in which to find it
  *
  * @return       The (indexOf) offset, or -1 if not found
  */
  public static int NthOfset(int whom, String aWord, String aStr)  {
    // skip over whom instances, 0-based offset; rtn = -2: can't find ""
    int lxx = StrLength(aWord), thar = 0, whar = -2;
    try {
     if (StrLength(aStr)>0) if (lxx>0) if (whom >= 0) while (true)  {
        whar = aStr.indexOf(aWord,thar);
        if (whar<0) break;
        whom--;
        if (whom<0) break;
        thar = whar+lxx;} //~while
    } catch (Exception ex)  {}
    return whar;} //~NthOfset

/**
  * Adds two pairs of half-integers in a single int.
  *
  * @param  here  The first pair
  * @param  thar  The second pair
  *
  * @return       The pair of sums
  */
  public static int AddPair(int here, int thar)  {
    return (here&-0x10000)+(thar&-0x10000)
        + ((here+thar)&0xFFFF);} //~AddPair

  /**
  * Returns in a single int the negatives of a pair of half-integers.
  *
  * @param  here  The pair
  *
  * @return       Its negative
  */
  public static int PairNeg(int here)  {
    return ((-here)&0xFFFF)-(here&-0x10000);} //~PairNeg

/**
  * A safe (no exceptions) way to get the size of a (possibly null) array.
  *
  * @param  data  The array
  *
  * @return       Its length, or =0 if null
  */
  public static int SafeArySz(int[] data)  {
    if (data == null) return 0;
    return data.length;} //~SafeArySz

  /**
  * A safe (no exceptions) way to get a (possibly null) array element.
  * Returns a specified default value if the array is null or the given
  * index is out of bounds.
  *
  * @param  ix      The index into the array
  * @param  data    The array
  * @param  defalt  A default value to return if can't access
  *
  * @return       Its length, or =0 if null
  */
  public static int SafeAryElt(int ix, int[] data, int defalt)  {
    while (true)  {
      if (data == null) break;
      if (ix<0) break;
      if (ix >= data.length) break;
      defalt = data[ix];
      break;} //~while
    return defalt;} //~SafeAryElt

  /**
  * Formats a 16-bit integer as 8.8 fixed-point and returns it with a prefix.
  *
  * @param  before The prefix
  * @param  whom   The number to be formatted
  *
  * @return       The result string
  */
  public static String FormFixt(String before, int whom)  {
    int frac = whom&255, zx = frac*10+6;   // assumes 8-bit fraction
    String info = "";                     // ..and signed 8+bit integer part
    boolean shorten = ((whom>25599)||(StrLength(before)==1));
    if (false) if ((whom&-0x8000) !=0) if (whom+0x10000 != (whom&0xFFFF))
      whom = SignExtend(whom); // caller did
    if (whom<0) return FormFixt(before + "-",-whom);
    before = before + (whom>>8);
    if (frac>0)  {
      before = before + ".";
      if (frac==128) info = "5";
      else if (frac==64) info = "25";
      else if (frac==192) info = "75";
      else if (frac==1)  {
        if (shorten) info = "01"; else
          info = "004";} //~if
      else if ((zx&255)<13) info = info + (zx>>8); // rounds to 1 digit
      else if (shorten)  { // round to 2 fractional digits..
        frac = (frac*100+127)>>8;
        if (frac<10) info = "0";
        else if (frac>99) frac = 99;
        info = info + frac;} //~if
      else if ((frac&31) !=0)  { // round to 3 fractional digits..
        frac = (frac*1000+127)>>8;
        if (frac<10) info = "00";
        else if (frac<100) info = "0";
        else if (frac>999) frac = 999;
        info = info + frac;} //~if
      else info = info + ((frac>>5)*125);} //~if
    return before + info;} //~FormFixt

  /**
  * Formats an integer as hexadecimal if larger than 32K and returns it
  * with a prefix and suffix, for easier reading.
  *
  * @param  before The prefix
  * @param  whom   The number to be formatted
  * @param  after  The suffix
  *
  * @return       The result string
  */
  public static String HexIfMore(String before, int whom, String after)  {
    if (SignExtend(whom)==whom) return before + whom + after;
    return SeeHex(before + "0x",whom,after); } //~HexIfMore

private static String ArrayDumpLine(int[] data, int sho, int sofar) { // Jv stub
  return " [...]";} //~ArrayDumpLine

 /**
  * A safe (no exceptions) way to get the height of the game window.
  *
  * @return       Its height
  */
  public static int GameWinTall() { // rtns window height
    return WinHi;} //~GameWinTall

 /**
  * A safe (no exceptions) way to get the width of the game window.
  *
  * @return       Its width
  */
  public static int GameWinWide() { // rtns window width
    return WinWi;} //~GameWinWide

 /**
  * A safe (no exceptions) way to get the size of the game window.
  *
  * @return       Its (packed) (height<<16)+width
  */
  public static int GameWinSize() { // rtns packed window size
    return (WinHi<<16)+WinWi;} //~GameWinSize

 /**
  * A diagnostic tool to make an ASCII plot of the pixel array of an icon.
  *
  * @param  pix   the the pixels
  * @param  msg   a short label to put on the header line
  */
  public void PlotIcn(int[] pix, String msg) {
    int nx, kx;
    for (nx=0; nx <= 32; nx++) {
      if ((nx&31)==0) System.out.println(" __  0 . . . 4 . . . 8 . . .12 . . ."
          + "16 . . .20 . . .24 . . .28 . . . " + msg);
      msg = ":";
      if (nx==32) break;
      for (kx=0; kx <= 31; kx++) if (pix != null)
        if (kx < pix.length)
          msg = msg + IffyStr(((pix[kx]>>nx)&1)==0,"  "," x");
      System.out.println(IffyStr(nx>9," ","  ") + nx + msg);
      if (nx>30) msg = "";}} //~PlotIcn

 /**
  * Safely pick out a single pixel from an icon pixel array.
  *
  * @param  rx     the pixel row, 0 is top
  * @param  cx     the pixel column, 0 is left edge
  * @param  pix    the pixel array
  *
  * @return        true if the pixel is opaque
  */
  public static boolean PeekIconPix(int rx, int cx, int[] pix) {
    if (rx<0) return false;
    if (cx<0) return false;
    if (rx>31) return false;
    if (pix == null) return false;
    if (cx >= pix.length) return false;
    return ((pix[cx]>>rx)&1) !=0;} //~PeekIconPix

 /**
  * Safely insert a single pixel into an icon pixel array.
  *
  * @param  aBit   true to set the pixel opaque, false transparent
  * @param  rx     the pixel row, 0 is top
  * @param  cx     the pixel column, 0 is left edge
  * @param  pix    the pixel array
  */
  public static void PokeIconPix(int why, boolean aBit, int rx, int cx,
      int[] pix, boolean logy) {
    int zx = 0, mask;
    if (rx<0) return;
    if (cx<0) return;
    if (rx>31) return;
    if (pix == null) return;
    if (cx >= pix.length) return;
    if (aBit) zx--;
    mask = 1<<rx;
    pix[cx] = pix[cx]&~mask|zx&mask;
    if (Logging) if (logy) if (StartupDone) if (GoodPix)
      System.out.println("  (Poke/" + why + ") " + aBit + " @ " + rx + "/" + cx
          + SeeHex(" & x",mask,SeeHex(" => x",pix[cx],"")));} //~PokeIconPix

 /**
  * Copy a rectangular portion from one icon pixel array to another.
  * Negative source offset copies zeroes, also height or width beyond end.
  * Overlapped rects on same array replicates horizontally unless wide<0
  *   (=right-to-left) does not match the overlap.
  *
  * @param  stop   the source vertical offset, 0 is top
  * @param  sleft  the source horizontal offset, 0 is left edge
  * @param  dtop   the destination vertical offset
  * @param  dleft  the destination horizontal offset
  * @param  tall   the destination height, =0 means all (32)
  * @param  wide   the destination width, =0 means all, however wide it is
  * @param  src    the source pixel array
  * @param  pix    the destination pixel array
  */
  public void IconPixCopy(int stop, int sleft, int dtop, int dleft,
      int tall, int wide, int[] src, int[] pix) {
    int info = 0, why = 0, rx = 0, cx = dleft, kx = dtop, sx = sleft,
        mask = -1, shift = dtop-stop, dright = cx+wide-1, dbotm = dtop,
        size = SafeArySz(src), lxx = SafeArySz(pix), zx;
    boolean back = wide<0;
    while (true) {
      if (tall==0) tall = 32-dtop;
      dbotm = dtop+tall-1;
      if (wide==0) {
        wide = lxx-dleft;
        dright = dleft+wide-1;} //~if
      why--; // why = -1
      if (lxx <= 0) break; // includes pix=null
      why--; // why = -2
      if (size <= 0) break;
      if (back) {
        wide = -wide;
        sx = sx+wide-1;
        if (sx >= size) sx = size-1;
        cx = cx+wide-1;
        if (cx >= lxx) cx = lxx-1;
        dright = cx;} //~if
      if (dleft+wide>lxx) wide = lxx-dleft;
      if (dright >= lxx) dright = lxx-1;
      if (dtop<0) {
        stop = stop-dtop;
        tall = tall+dtop;
        dtop = 0;} //~if
      rx = dtop+tall;
      if (rx>32) {
        tall = 32-dtop;
        rx = 32;} //~if
      if (dleft<0) {
        sleft = sleft-dleft;
        wide = wide+dleft;
        dleft = 0;} //~if
      why--; // why = -3
      if (tall <= 0) break;
      why--; // why = -4
      if (wide <= 0) break;
      why--; // why = -5
      if (dtop>0) mask = mask<<dtop;
      if (rx>0) if (rx<32) {
        zx = 1<<rx;
        mask = (zx-1)&mask;} //~if
      if (mask==0) break; // shoulda got caught above
      why = 0;
      break;} //~while
    if (Logging) if (BitPoker) if (StartupDone)
      System.out.println("  (PixCpy) = " + why + " sh=" + shift + " " + back
          + SeeHex(" & ",mask," (") + rx + ") " + stop + "/" + sleft
          + " + " + tall + "/" + wide + " => " + kx + ".." + dbotm
          + " / " + dleft + ".." + dright);
    if (why<0) return;
    mask = ~mask;
    if (back) for (cx=cx; cx >= dleft; cx += -1) {
      if (sx < src.length)
        if (sx >= 0) info = src[sx];
      if (cx >= 0) if (cx < pix.length) {
        if ((mask|shift) !=0) {
          if (shift<0) info = info>>(-shift);
          else if (shift>0) info = info<<shift;
          info = pix[cx]&mask|info&~mask;} //~if
        pix[cx] = info;} //~if
      info = 0;
      sx--;} //~for
    else for (cx=cx; cx <= dright; cx++) {
      if (sx < src.length)
        if (sx >= 0) info = src[sx];
      if (cx >= 0) if (cx < pix.length) {
        if ((mask|shift) !=0) {
          if (shift<0) info = info>>(-shift);
          else if (shift>0) info = info<<shift;
          info = pix[cx]&mask|info&~mask;} //~if
        pix[cx] = info;} //~if
      info = 0;
      sx++;} //~for
    if (Logging) if (BitPoker) if (StartupDone)
      System.out.println("  ==> " + ArrayDumpLine(pix,0,6));} //~IconPixCopy

  public void IconFlip(int doit, int tall, int wide, int[] pix) {
    int rx, cx, lxx = SafeArySz(pix), why = 0;
    boolean aBit, zBit;
    while (true) {
      why--; // why = -1
      if (lxx <= 0) return; // includes pix=null
      // if (pix == null) return;
      why--; // why = -1
      if (doit<1) return;         // 1: R<->C   2: T<->B   3: ccw90   4: L<->R
      why--; // why = -1
      if (doit>7) return;         // 5: cw+90   6: ro180   7: TL/BR (flip)
      why--; // why = -1
      if (tall<2) return;
      why--; // why = -1
      if (wide<2) return;
      why--; // why = -1
      if (doit !=4) if (tall>32) return;
      why--; // why = -1
      if (doit !=2) if (wide>lxx) return;
      why = 0;
      break;} //~while
    if (Logging) if (BitPoker) if (StartupDone)
      System.out.println("  (IcoFlip/" + doit + ") = "
          + why + " + " + lxx + " " + tall + "/" + wide);
    if (why<0) return;
    // flip row/col (across down diagonal)..
    if ((doit&1) !=0) {
      if (tall != wide) return;
      for (rx=tall-2; rx >= 0; rx += -1) for (cx=rx; cx >= 0; cx += -1) {
        aBit = PeekIconPix(rx,cx,pix);
        zBit = PeekIconPix(cx,rx,pix);
        PokeIconPix(2,aBit,cx,rx,pix,BitPoker);
        PokeIconPix(3,zBit,rx,cx,pix,BitPoker);}} //~if //~for
    // flip top/bottom..
    if ((doit&2) !=0) for (rx=(tall>>1)-1; rx >= 0; rx += -1) for (cx=wide-1; cx >= 0; cx += -1) {
      aBit = PeekIconPix(rx,cx,pix);
      zBit = PeekIconPix(tall-1-rx,cx,pix);
      PokeIconPix(4,aBit,tall-1-rx,cx,pix,BitPoker);
      PokeIconPix(5,zBit,rx,cx,pix,BitPoker);} //~for
    // flip left/right..
    if ((doit&4) !=0) for (rx=tall-1; rx >= 0; rx += -1) for (cx=(wide>>1)-1; cx >= 0; cx += -1) {
      aBit = PeekIconPix(rx,cx,pix);
      zBit = PeekIconPix(rx,wide-1-cx,pix);
      PokeIconPix(6,aBit,rx,wide-1-cx,pix,BitPoker);
      PokeIconPix(7,zBit,rx,cx,pix,BitPoker);} //~for
    if (Logging) if (BitPoker) if (StartupDone)
      System.out.println("  ==> " + ArrayDumpLine(pix,0,6));} //~IconFlip

  public void DoArcQ(int rad2, int voff, int[] pix) { // caller cleared pix
    // create 1-quadrant arc in pix, centered 0,0 (top-left), radius rad2/2
    int nx = rad2-1, rx = 0, cx = nx>>1, bitz = ArcQsteps[nx&31];
    if (Logging) if (BitPoker) System.out.println("  (ArcQ) " + rad2 + " +" + voff
        + SeeHex(" x",bitz,".."));
    if (pix != null) if (rad2>0) if (rad2<33) while (cx >= rx) {
      if (bitz==0) bitz++; // (unneed for 1..32)
      for (nx=(bitz&15)-1; nx >= 0; nx += -1) {
        if (Logging) if (BitPoker) if (StartupDone)
          if (GoodPix) System.out.println("  (...) " + rx + "/" + cx + " +" + nx);
        PokeIconPix(8,true,rx,cx,pix,BitPoker);
        PokeIconPix(9,true,cx,rx,pix,BitPoker);
        rx++;} //~for
      bitz = bitz>>4;
      cx--;} //~while
    if (Logging) if (BitPoker) if (StartupDone)
      System.out.println("  ==> " + ArrayDumpLine(pix,0,6));} //~DoArcQ

  private int FillBits(int data, int whom) {
    // retns the bits for whom to bucket-fill data
    int mask = whom&~data, aBit = mask&-mask, prio = 0;
    if (mask==0) return 0; // no fillable pixels
    if (data==0) return -1;
    if (mask==aBit) { // filling from a single pixel..
      if ((data&aBit) !=0) return 0; // already filled
      mask = aBit-1;
      prio = data&-aBit; // =0 if hi bit to fill is bit31
      prio = prio&-prio; // = hi bit boundary (=0 if none)
      prio = prio-1-mask; // fill these bits, up to hi bit boundary
      while ((mask&data) !=0) mask = mask-(mask&-mask);
      return mask|aBit|prio;} //~if // fill these bits
    while (whom !=0) {
      if (prio !=0) {
        data = data|prio;
        whom = whom&~data;
        if (whom==0) break;
        aBit = whom&-whom;} //~if
      prio = FillBits(data,aBit)|prio;
      whom = whom&~aBit;} //~while
    return prio;} //~FillBits

  public int BucketFill(int rx, int cx, int[] pix) {
    int nuly, here, aBit = 1<<(rx&31),  // rtns the new pix[cx], =0 if can't
        lxx = SafeArySz(pix), rez = 0, deep = 0, mask = 0, prio = 0, base = 0;
    int[] data;
    if (lxx <= 0) return 0; // no pixels
    if (cx<0) return 0; // off left
    if (cx >= lxx) return 0; // off right
    if ((rx&-32) !=0) return 0; // off top or bottom
    if (pix==null) return 0;
    data = new int[lxx*2];
    if (cx < pix.length)
      base = pix[cx];
    mask = FillBits(base,aBit);
    base = base|mask;
    if (cx < pix.length)
      pix[cx] = base;
    rez = base;
    prio = mask;
    aBit = ~base; // possible back-fill pix (recursion needed)
    if (cx>0) for (here=cx-1; here >= 0; here += -1) {
      nuly = pix[here];
      if ((prio&~nuly)==0) break;
      prio = FillBits(nuly,prio);
      nuly = nuly|prio;
      pix[here] = nuly;
      aBit = aBit&prio;
      if (aBit !=0) if (data != null) if (deep >= 0)
          if (deep < data.length-2) {
        data[deep] = here+1;
        data[deep+1] = aBit;
        deep = deep+2;} //~if
      aBit = ~nuly;} //~for
    lxx--;
    prio = mask;
    aBit = ~base;
    if (cx<lxx) for (here=cx+1; here <= lxx; here++) {
      nuly = pix[here];
      if ((prio&~nuly)==0) break;
      prio = FillBits(nuly,prio);
      nuly = nuly|prio;
      pix[here] = nuly;
      aBit = aBit&prio;
      if (aBit !=0) if (data != null) if (deep >= 0)
          if (deep < data.length-2) {
        data[deep] = here-1;
        data[deep+1] = aBit;
        deep = deep+2;} //~if
      aBit = ~nuly;} //~for
    for (deep=deep-2; deep >= 0; deep += -2)
        if (deep < data.length-2) {
      prio = data[deep+1]; // =1 in bits that can be filled
      here = data[deep];
      if (here<0) break;
      if (here >= pix.length) break;
      nuly = prio&~pix[here];
      if (nuly==0) continue; // already filled some other way
      for (rx=0; rx <= 31; rx++) {
        if ((nuly&(1<<rx))==0) continue;
        aBit = BucketFill(rx,here,pix);
        if (aBit==0) continue; // failed (shouldn't)
        nuly = prio&~aBit;}} //~if //~for (rx) (deep)
    return rez;} //~BucketFill

 /**
  * A private utility for finding another block of pixels when building ants
  * around disconnected blocks of an icon in MkSpriteAnts.
  *
  * @param  wide  the nominal icon width
  * @param  pix   the pixel array for this icon
  *
  * @return       the row,column packed (rx<<16)+cx, or -1 if none found
  */
  private int FindIconPixel(int wide, int[] pix) { // frm DoAntTr
    int info = 0, rx = 0, cx;
    for (cx=0; cx <= wide-1; cx++) {
      if (cx >= 0) if (cx < pix.length)
        info = pix[cx];
      if (info==0) continue;
      while ((info&1)==0) {
        info = info>>1;
        rx = rx+0x10000;} //~while
      return rx+cx;} //~for
    return -1;} //~FindIconPixel

 /**
  * A private utility for tracing the edge of a block of pixels when
  * building ants around an icon in MkSpriteAnts. It looks at the next pixel
  * in the specified aim direction (0..7, *45 degrees ccw from south)
  *
  * @param  aim   the current travel direction, /45 degrees ccw from south
  * @param  rx    the current pixel row
  * @param  rx    the current pixel column
  * @param  pix   the pixel array for this icon
  *
  * @return       true if there's an opaque pixel there, else false
  */
  private boolean PeekPlusAim(int aim, int rx, int cx, int[] pix) {
    int zx = AimIndex[aim&7];
    cx = (zx&1)-(zx&2)+cx;
    zx = zx>>2;
    rx = (zx&1)-(zx&2)+rx;
    return PeekIconPix(rx,cx,pix);} //~PeekPlusAim

 /**
  * Trace an icon outline to turn off the transparent segments of an ant
  * trail for MkSpriteAnts. This builds a sequence of icons in the array pix
  * (which the caller must ensure is lare enough), and returns the animation
  * info for this sequence of icons.
  *
  * @param  wide  the nominal icon width +1 = size of one animation frame
  * @param  pix   the pixel array for the ants, sized for animation (wide*10)
  * @param  data  another pixel array, size=wide, becomes solid outline
  *
  * @return       the animation spec for this icon sequence (also in SharedRez)
  */
  public int DoAnTrail(int wide, int[] pix, int[] data, boolean logy) {
    /// if (lxx>5) while (true) { // we can do this..
        // 21:4+3-4+3-4+3-, 22:4+3-4+4-4+3-, 23:4+4-4+4-4+3-, 24:4+4-4+4-4+4-,
        // 25:5+4-4+4-4+4-, 26:5+4-5+4-4+4-, 27:5+4-5+4-5+4-,
        // 28:4+3-4+3-4+3-4+3-, 29:4+3-4+3-4+4-4+3-, 30:4+4-4+3-4+4-4+3-,
        // 31:4+4-4+4-4+4-4+3-, 32:4+4-4+4-4+4-4+4-, 33:5+4-4+4-4+4-4+4-,
      // for lxx>49, let n>6 be such that lxx<=n*8, p=n*8-lxx<8; distribute p
      //   pairs of (4+3-) evenly among n-p (4+4-) pairs (total n pairs).
      // for lxx>20, ditto, except if p>n, distribute instead p-n pairs
      //   of (5+4-) among (4+4-), total n-1 pairs
      // for lxx<21 use: 3:2+1-, 4:2+2-, 5:3+2-, 6:3+3-, 7:4+3-, 8:4+4-, 9:5+4-,
      //   10:5+5-, 11:5+6-, 12:3+3-3+3-, 13:4+3-3+3-, 14:4+3-4+3-, 15:4+4-4+3-,
      //   16:4+4-4+4-, 17:5+4-4+4-, 18:5+4-5+4-, 19:5+5-5+4-, 20:5+5-5+5-,
      // Can't do that. Use same sizing algorithm, but choose majority
      //   pair size, and +/- deviations replicate or drop a pixel there,
      //   regardless of what part of the pattern goes there.
      // Use Peek/PokeIconPix(aBit,rx,cx,pix) to walk around the border.
    ///  yx = (bitz+7)>>3; // candidate + 7/8/9-pix black-white pairs
    /// else if (bitz>5) zx = SmallAnts[lxx&31]&15;  final int[] SmallAnts =
    ///  {0,0,0, 0x3603, 0xCC04, 0x39C05, 0xE3806, 0x387007, 0xF0F008,
    //.. 0x3C1E009, 0xF83E00A, 0x3F7E00B, 0xE3806, 0xE3816, 0x387007, 0xF0F0F8,
    //.. 0xF0F008, 0xF0F018, 0x3C1E009, 0xF83E0FA, 0xF83E00A, 0xF0F008, 0,0, };
    //..   AimIndex = {4,5,1,13,12,15,3,7}; // 1=+h,3=-h,4=+v,12=-v
    ///
    // We are not solving the general case!! Assume (length of outline) lxx<256,
    //   no separate blobs, no 1-wide isthmus or extension
    /// initial aim =0: pointing down (white always on right), +1 cwise 45 deg.
    int[] epath = PxLst;
    boolean pp, aBit, drop;
    int whar, mask, logz = 0, fust = -1,
        sho = (logz>>8)&255, kx = logz, px, yx, nx = 0, rx = 0, cx = 0,
        zx = 0, pmx = 0, lxx = 0, doing = 0, info = 0, sum = 0, bitz = 0,
        aim = 0, step = 0, fno = 0;
    String msg = "";
    SharedRez = 0; // rtns animation info in ShRez
    if (!Logging) {
      logz = 0;
      sho = 0;
      zx--;} //~if
    else if (!GoodRip) {
      logz = 0;
      sho = 0;
      zx = -2;} //~if
    else if (!GoodKchn) {
      logz = 0;
      sho = 0;
      zx = -2;} //~if
    else if (!logy) {
      logz = 0;
      sho = 0;
      zx = -3;} //~if
    else if (logz<0) {
      yx = logz&255;
      if ((logz&0x40000000) !=0) {
        logz = 0;
        sho = 0;
        zx++;} //~if
      else if (yx-2>wide) {
        logz = 0;
        sho = 0;
        zx = 2;} //~if
      else if (yx+2<wide) {
        logz = 0;
        sho = 0;
        zx = 3;} //~if
      if (logz<0) {
        if ((kx&0xFF0000)>0) kx = kx-0x10000;
        if ((kx&0xFF0000)==0) kx = 0;
        fust = 0;}} //~if
    if (Logging) System.out.println(SeeHex("  {DAT} ",logz,SeeHex("/",kx," = "))
        + sho + "/" + zx + " " + logy + "/" + GoodRip + "/" + GoodKchn
        + " + " + wide);
    if (Logging) if (logz<0)
      PlotIcn(pix,"(AnTr)");
    zx = 0;
    if (epath == null) { // create path list..
      epath = new int[256]; // = int[lxx+2];
      if (epath == null) return 0;
      PxLst = epath;} //~if
    for (nx=0; nx <= 255; nx++) {
      if (nx >= 0) if (nx < epath.length)
        epath[nx] = 0;
      if (nx >= wide) continue;
      if (nx<0) break;
      if (data == null) break;
      if (nx >= data.length) break;
      data[nx] = 0;
      if (pix == null) break;
      if (nx >= pix.length) break;
      info = pix[nx];
      if (info==0) continue;
      if (zx !=0) continue;
      cx = nx;
      zx = info;} //~for // (nx)
    if (zx==0) return 0; // no starting pixel
    while ((zx&1)==0) { // convert start pixel to row+..
      zx = zx>>1;
      rx++;} //~while
    while (doing<256) { // initially (doing=0) rx/cx is already at 1st pixel
      // AimIndex = {4,5,1,13,12,15,3,7}; // 1=+h,3=-h,4=+v,12=-v
      // initial aim =0: pointing down (white always on right), +1 ccwise 45 deg.
      // No longer trying to square off inside corners.
      if (doing>0) { // heuristic to find next adjacent pixel to rx,cx..
        pp = true; // F: saw white on right; aim+/-4 is came-from (sb ink)
        for (fno=aim-3; fno <= aim+4; fno++) { // search c-cwise from back-right..
          aBit = PeekPlusAim(fno,rx,cx,pix);
          if (aBit) { // saw ink..
            if (pp) continue; // not seen white yet
            zx = AimIndex[fno&7]; // go there
            cx = (zx&1)-(zx&2)+cx;
            zx = zx>>2;
            rx = (zx&1)-(zx&2)+rx;
            aim = fno&7;
            break;} //~if
          pp = false;} //~for // (fno: search c-cwise from back-right)
        if (pp) break;} //~if // (heuristic to find next adjacent pixel)
      if (PeekIconPix(rx,cx,data)) break; // reached start
      kx = (rx<<16)+cx;
      if (doing >= 0) if (doing < epath.length)
        epath[doing] = kx;
      PokeIconPix(10,true,rx,cx,data,logy); // data is the outline being formed
      if (doing==255) break; // too long a path, stop here
      doing++;} //~while
    if (Logging) if (logy) if ((logz<0) || StartupDone) {
      if (fust<33) System.out.println("epath = (" + doing + ")"
        );
      if (logz<0) PlotIcn(data,"(path)");} //~if
    if (doing<4) return 0; // not enough ink to make ants
    lxx = doing;
    if (lxx<32) {
      info = SmallAnts[lxx&31];
      sum = info>>4;
      pmx = (sum&7)-(sum&8);
      nx = lxx/12+1;} //~if
    if (lxx>20) {      // lxx is the total edge length in pixels
      info = 0xF0F008; // info>>8 is black/white pattern; info&15 is pattern ln
      nx = (lxx+7)>>3; // nx is the + ants = black/white pairs
      pmx = lxx-(nx<<3); // pmx is the +/- number of extra/missing pix in trail
      if (pmx+nx<0) {
        pmx = pmx+8;
        info--;
        nx--;}} //~if
    if (pix != null) bitz = pix.length;
    zx = info&15; // zx is + frames = len of 1 ant(B+W pair); bitz = zx*wide+2
    kx = zx*wide; // wide is actual pixel width, incl 1 blank col on right end
    step = Math.abs(pmx);  // 'DoAnTr +78 8 w=22 n=10 p=-2 xF0F008 2 true +178 : 176'
    if (Logging) if ((logz<0) || StartupDone)
      System.out.println("(DoAnTr) +" + lxx + " w=" + wide // + " " + base
          + " n=" + nx + " p=" + pmx + SeeHex(" x",info," ") + step + " "
          + (pix != null) + " +" + bitz + " : " + kx + " " + logy + "="
          + BitPoker + "|" + GoodRip + "|" + GoodKchn + SeeHex(" x",logz,
          " ") + fust);
    if (nx==0) return 0;
    if (zx<6) return 0; // (caller knows) we don't do ants shorter than 3+3 // ***
    if (bitz<kx) return 0; // caller shoulda set bitz=kx+2
    if (pix == null) return 0; // last early exit, but for 3 errs
    // kx = 0; // replicate the border pattern (unneed, fill 0 then poke 1)
    zx = pix.length;
    for (zx=zx-1; zx >= 0; zx += -1)
      if (zx >= 0) pix[zx] = 0;
    sum = 0;
    if (step >= nx) step = 256; // step/256 is how often to add/drop a pixel
    else if (step>0) {
      sum = (lxx/step)>>1;
      step = ((step<<8)+(lxx>>1))/lxx;
      sum = sum*step;} //~if
    info--;
    doing = info&15;
    bitz = info;
    for (nx=0; nx <= lxx-1; nx++) { // each pixel in the border path..
      fno = info&15; // info was decremented above
      mask = 256<<fno; // so ants march clockwise (=256; for ccw)
      whar = 0;        // (or we could step whar backward from fno*wide)
      zx = sum+step;
      drop = (zx>255);
      sum = zx&255;
      kx = 0;
      if (nx < epath.length)
        kx = epath[nx];
      rx = kx>>16;
      cx = kx&0xFFFF;
      if (Logging) if (logy) if ((logz<0) || StartupDone) if (fust==0)
         System.out.println("(DAT) " + nx + " @ " + rx + "/" + cx + " (" + fno
            + FormFixt(") s=",zx) + IffyStr(drop," -1:"," +0:") );
      for (fno=fno; fno >= 0; fno += -1) { // next pix in patn, next image in icn
        aBit = ((bitz&mask)==0);
        if (Logging) if (logy) if ((logz<0) || StartupDone)
          if ((logz<0) || GoodPix) System.out.println("  .. " + fno + " #"
              + doing + " @ " + rx + "/" + cx + " +" + whar
              + SeeHex(" x",bitz,SeeHex(" & x",mask,IffyStr(aBit," w",""))));
        if (aBit) PokeIconPix(11,true,rx,cx+whar,pix,logy);
        mask = mask>>1; // (cwise) // mask = mask<<1; // (ccw)
        whar = whar+wide;} //~for // (fno: next pix in patn, next image in icn)
      if (drop) {
        if (pmx<0) { // skip a pixel in ant..
          doing--; if (doing<0) { // start new ant..
            doing = info&15;
            bitz = info;} //~if
          else bitz = bitz>>1;} //~if
        else continue;} //~if
      doing--; if (doing<0) { // start new ant..
        doing = info&15;
        bitz = info;} //~if
      else bitz = bitz>>1;} //~for // (nx: each pixel in the border)
    zx = info&15; // 6b(fr.frate),6b(+frs-1),6b(stpsz-1),14b(im.of)..
    zx = (((zx-1)*64+wide-1)<<14)+0x40000000; // (c32: x40778000
    if (Logging) if (logz<0)
      PlotIcn(pix,"(Antz)");
    SharedRez = zx; // the animation info for these ants
    if (Logging) if (logy) if ((logz<0) || StartupDone)
      System.out.println(SeeHex(" ==> x",zx,"")
    );
    return zx;} //~DoAnTrail

 /**
  * A safe (no exceptions) non-OOPS way to convert a text number into
  * a fixed-point value with eights bits to the right of the binary point.
  *
  * @param  whom  a string containing a number
  *
  * @return       the same number as a fixed-point integer, or =0 if fails
  */
  public static int ParsFixPt(String whom) {
    int rez = 0, lxx = 0, sgn = 0, whar; // rtns 8-bit each, int+fraction
    String msg = whom; // for log
    whom = NthItemOf(' ',1,whom);
    if (Logging) if (BitPoker) if (StartupDone)
      msg = "  (ParsFxPt) '" + msg
          + IffyStr(StrLength(whom)==StrLength(msg),"'","' => " + whom);
    whar = NthOfset(0,"-",whom);
    if (whar >= 0) {
      whom = RestOf(whar+1,whom);
      if (Logging) if (BitPoker) if (StartupDone)
        msg = msg + " -" + whar + "/" + whom;
      sgn--;} //~if
    whar = NthOfset(0,".",whom);
    if (whar >= 0) {
      if (whar>0) rez = SafeParseInt(Substring(0,whar,whom))<<8;
      whom = RestOf(whar+1,whom);
      lxx = StrLength(whom);
      if (Logging) if (BitPoker) if (StartupDone)
        msg = msg + " ." + whar + "/" + whom + " =" + rez + " +" + lxx;
      if (lxx>4) {
        whom = Substring(0,4,whom);
        lxx = 4;} //~if
      whar = SafeParseInt(whom)<<8;
      if (Logging) if (BitPoker) if (StartupDone)
        msg = msg + SeeHex(" x",whar," ");
      if (lxx==4) whar = (whar+5000)/10000;
      else if (lxx==3) whar = (whar+500)/1000;
      else if (lxx==2) whar = (whar+50)/100;
        else whar = (whar+5)/10;
      if (Logging) if (BitPoker) if (StartupDone)
        msg = msg + " => " + whar;
      rez = rez+whar;} //~if
    else rez = SafeParseInt(whom)<<8;
    if (rez>0x8000) rez = rez&0x7FFF;
    else if (sgn==0) if (rez==0x8000) rez--;
    if (Logging) if (BitPoker) if (StartupDone)
      System.out.println(msg + SeeHex(IffyStr(sgn<0," = -x"," = x"),rez," "));
    if (sgn<0) rez = -rez;
    return rez;} //~ParsFixPt

 /**
  * A safe (no exceptions) way get the reference number from a (possibly null)
  * widget. If it is null, SafeWgtNum returns zero.
  *
  * @param  whom  the widget, or null if none
  *
  * @return       that widget's RefNum, or =0 if none
  */
  public static int SafeWgtNum(GameWgt whom) { // returns RefNum or 0
    if (whom==null) return 0;
    return whom.GetRefNum();} //~SafeWgtNum

 /**
  * LogWgtTree prints out to the console log an (indented) tree representation
  * of a specified widget with all its sub-widgets. It calls itself recursively
  * with deeper indentations for the sub-widgets.
  *
  * @param  whom   the top-level widget to log
  * @param  deep   the initial indentation, usually 0 or 1
  */
  public static void LogWgtTree(GameWgt whom, int deep) {
    int nx, tye;                               // caller ensures if (Logging)
    String why;
    GameWgt nuly;
    if (whom==null) return;
    tye = whom.GetType();
    why = Substring(0,deep+deep,". . . . . . . . . . . . . . . . . . . . ");
    whom.LogMe(why);
    deep++;
    if (deep<20) if (((tye-5)&255)<3) for (nx=0; nx <= 99; nx++) {
      nuly = GetIxWgt(whom.GetIxData(nx,0));
      if (nuly==null) break;
      LogWgtTree(nuly,deep);}} //~LogWgtTree

 /**
  * A search tool to find uniquely identified widgets built from the list
  * returned by the client GameList, so to assign references to them in
  * variables usable in the game.
  *
  * If the found widget is a sprite, the line is saved in TooTmpStr
  *
  * @param  msg   the unique string to search for
  *
  * @return       the first widget specified in theImpoList with that string
  */
  public GameWgt FindListWgt(String msg) { // in theImpoList
    int here, thar, info = 0;
    String aLine = "", data = theImpoList; // '\n' guaranteed at front
    GameWgt whom = null;
    here = NthOfset(0,msg,data);
    thar = StrLength(msg)+here;
    // TooTmpStr = "";
    while (here >= 0) {
      if ((int)(CharAt(here,data))<32) {
        aLine = NthItemOf('\n',2,Substring(here,thar-here,data));
        info = SafeParseInt(aLine); // = lino, except for additions
        whom = GetIxWgt(info);
        if (whom==null) break; // didn't find it
        if ((whom.GetType()&255)==6) TooTmpStr = aLine;
        break;} //~if
      else here--;} //~while
    if (Logging) {
      data = IffyStr(whom==null,"(NOT) found #","(!) Found #")
          + info + " '" + msg + "' @ " + here;
      here = StrLength(aLine);
      info = StrLength(data);
      if (info<69) aLine = ": '" + Substring(0,76-info,aLine)
          + IffyStr(info+here<77,"'","..");
      else aLine = "";
      System.out.println(data + aLine);} //~if
    return whom;} //~FindListWgt

  // Now some private data...

private String FindLog = "", myTmpStr = "";
    // myTmpSt is used internally in NewGameWg, FindSubWg, Antsify,
    //   MouseDisp, Every100m, KeyDisp, & FordEvt;
    // .. and to rtn data from EncodeID, BuildThisMac

// [BlueJ only] GameWgt subclasses here..

/**
 * Single color icon.
 *
 * Ginfo is used for animation info, bit fields, high to low:
 *     (frame rate)*16:6, (+ frames)-1:6, (step size)-1:6, (pixel offset):14
 *   The frame rate is fixed-point, 2i.4f with a range [0 .. 3.935] where
 *     0.0 never changes the image, and 2.0 skips every other image.
 *   The number of frames determines how many frames show before the image
 *     repeats (starts over).
 *   The step size is the number of ints to advance when changing the image.
 *   The pixel offset is an offset into the image array to start the image.
 *
 *   Each frame (drawing cycle = 100ms) the fractional frame rate is added
 *     to the current value in the upper half of Gtype, then the integer part
 *     of the sum is subtracted off and multiplied by the step size and the
 *     pixel offset is reduced by that many units. If it goes negative, it
 *     is restarted at an offset = ((+ frames)-1)*(step size).
 *   It is not necessary for the step size to equal the image size. For
 *     example, the marching ants icon image used for rectangles is up to
 *     256 pixels wide, but the step size =1, so that the image shifts over
 *     one pixel each frame; since the pattern is diagonal bars, but only
 *     a single pixel (wide or tall) the pattern appears to be moving down
 *     or right one pixel per frame.
 */
private class GameIcon extends GameWgt  { // Single color icon..

 /**
  * Return the class name and RefNum for this widget, plus its ID code if any.
  *   See toStrMash for formatting.
  *
  * @return     The reference String for this widget
  */
  public String toString()  {                                     // (GameIcon)
    return toStrMash(super.toString(),"(GameIcon #");} //~toString

 /**
  * This is an internal utility for formatting a widget's information value,
  * GameIcon overrides this to append the formatted animation parts if !=0.
  *
  * @param  before  a prefix to add to the front
  * @param  tye     the widget's type
  *
  * @return         the composite display value
  */
  public String FmtInfo(String before, int tye)  {                // (GameIcon)
    before = super.FmtInfo(before,tye);
    if ((Ginfo&-0x4000) !=0) before = before + "+" + ((Ginfo>>14)&63)
        + "#" + ((Ginfo>>20)&63) + FixHaff("/",(Ginfo>>22)&0x3F0);
    return before;} //~FmtInfo

  /**
  * This draws the GameIcon pixels on the window after advancing the offset
  * according to the animation specification. The boundaries of a masking
  * rectangle limit the drawing to (usually) the containing widget's dimensions.
  *
  * Ginfo is used for animation info, bit fields, high to low:
  *     (frame rate)*16:6, (+ frames)-1:6, (step size)-1:6, (pixel offset):14
  *   The frame rate is fixed-point, 2i.4f with a range [0 .. 3.935] where
  *     (for examples) 0.0 never changes the image, 0.5 shows each image
  *     for 2 frames, and 2.0 skips every other image. The slowest rate shows
  *     each each image for 16 frames, but you can make it slower by repeating
  *     images in the Gdata array.
  *   The number of frames determines how many frames show before the image
  *     repeats (starts over). The max value 63 signals a 1-shot animation,
  *     from the current pixel offset down to zero, and then stops.
  *   The step size is the number of ints to advance when changing the image.
  *   The pixel offset is an offset into the image array to start the image.
  *
  *   Each frame (drawing cycle = 100ms) the fractional frame rate is added
  *     to the current value in the upper half of Gtype, then the integer part
  *     of the sum is subtracted off and multiplied by the step size and the
  *     pixel offset is reduced by that many units. If it goes negative, it
  *     is restarted at an offset = ((+ frames)-1)*(step size).
  *
  * @param  top    do not draw above this position
  * @param  left   do not draw to the left of this position
  * @param  tall   do not draw below this position +top
  * @param  wide   do not draw to the right of this position +left
  */
  public void DrawMe(int top, int left, int tall, int wide)  {    // (GameIcon)
    int colo = Gcolor, RealPrent = WhoCalled(0), // used only to verify hierarchy
        nx = AddPrntsPos(GetPosn()), myTop = nx>>16, myLft = SignExtend(nx),
        zx = 0, rx = myTop+Gtall, cx = myLft+Gwide, far = SafeArySz(Gdata),
        lxx = Gtall-1, info = Ginfo>>14, thar, here, logy = 0, why = 0;
    String msg = "";
    // CallersID = 0; whar = myLft-left,
    while (true)  {
      if (Logging)  { // tye=2: "wIcon"
        if (logy>0)  {}
        else if (MaxLogWgts>0)  {
          logy = MaxLogWgts;
          MaxLogWgts--;} //~if
        else if (info !=0)  {
          nx = ((info&0x3F000)<<4)+Gtype; // factional counter
          if (MaxLogAnim>8) logy = MaxLogAnim;
          else if ((nx&-0x100000) !=0) logy = MaxLogAnim;
          if (logy>0) MaxLogAnim--;}} //~if
      if ((SeenLog&4)==0) if ((Gtype&1)==0) logy = 1234; // tye=2
      if (logy>0) msg = FmtFront("  (Dr.",logy>999,IffyStr(logy>999," "
          + top + "/" + left + " +" + tall + "/" + wide,"") + " + ");
      why++; // why = 1
      if (Gtop>0x3FFF) break; // hidden
      why++; // why = 2
      if (Parent==0) break;
      if (RealPrent !=0) if (RealPrent != Parent) break;
      why++; // why = 3
      if ((Gtype&254) !=2) break; // we also draw font chars +3..
      why++; // why = 4
      nx = 0;
      why++; // why = 5
      if (Gcolor<0) break; // single-layer hit rgn for sprite (undrawn)
      why++; // why = 6
      if (Gdata == null) break;
      why++; // why = 7
      if (lxx<0) break;
      if (lxx>31) lxx = 31;
      if (logy==1234) SeenLog = SeenLog|4;
      if (rx <= top+tall) tall = rx-1; // now it's edge, not size..
        else tall = top+tall-1;
      if (cx <= left+wide) wide = cx-1; // cx is win-rel right edge
        else wide = left+wide-1;
      cx = Ginfo&0x3FFF; // 6b(fr.frate),6b(+frs-1),6b(stpsz),14b(im.of)
      if (info !=0)  { // animated..
        nx = ((info&0x3F000)<<4)+Gtype; // add fractional counter
        rx = (info&63)+1; // = step size
        while ((nx&-0x100000) !=0)  { // got integer part, go next image..
          nx = nx-0x100000; // shouldn't, but could skip an image
          if (cx<rx)  {
            thar = (info>>6)&63;
            if (thar==63)  { // one-shot ran out, kill anim..
              nx = nx&0xFFFF;
              info = 0;
              cx = 0;} //~if
            else cx = thar*rx;} //~if // restart at right end
          else cx = cx-rx;} //~while // otherwise step down toward front
        Ginfo = (info<<14)+cx;
        Gtype = nx; } //~if
      if (logy>0)  {
        msg = msg + cx;
        if (nx !=0)
          msg = msg + SeeHex(" a=x",nx, "");}
      why++; // why = 8
      if ((Gtype&255)==3)  { // somebody else set this up..
        if (Gwide <= 0) break;
        why++; // why = 9
        if (cx+Gwide>far) break;} //~if
      if (myTop>top) top = myTop; // myTop/myLft are Pixels(=win)-relative
      if (myLft>left) left = myLft; // ..for drawing this icon/char
      if (top<0) top = 0;
      if (left<0) left = 0;
      if (logy>999) msg = msg + " => " + top + "/" + left
          + " +" + tall + "/" + wide;
      for (nx=0; nx <= Gwide-1; nx++) {
        info = SafeAryElt(nx+cx,Gdata,0);
        if (info==0) continue;
        here = nx+myLft;
        if (here<left) continue;
        if (here>wide) continue;
        for (rx=0; rx <= lxx; rx++) {
          thar = rx+myTop;
          if ((info&1) !=0)  { if (false)
            if (zx>0) if (Logging) if (logy>999)
              System.out.println("   +++" + rx + "/" + nx + " +" + cx
                  + IffyStr((thar >= top) && (thar <= tall)," +V"," -v")
                  + IffyStr((here<left) || (here>wide),"-h ","+H ")
                  + thar + "/" + here + SeeHex(" => ",colo,""));
            if (thar >= top) if (thar <= tall)  {
              PokePixel(colo,thar,here);
              zx--;}} //~if
          info = info>>1;
          if (info==0) break;}} //~for
      why = -why;
      break;} //~while // why = -8/-9
    if (false) if (logy <= 0) if (why==2) logy = 1001;
    if (myPL==0) if (Logging||(logy==1234)) if (logy>0)  {
      here = Ginfo&0x3FFF;
      System.out.println(msg + " = " + why         // (Dr.wFont + / (Dr.wIcon +
          + IffyStr(why==2," !prnt=" + Parent + "=" + RealPrent + " (RP) "," ")
          + logy  + " " + (here<far));
      if (logy<999) return;
      LogMe("  (DrMe) ");
      if (here==0) return; // @ front, LogMe already did it
      // far = SafeArySz(Gdata);
      if (here >= far) return; // off the end = 'invisible'
      zx = (RefNum<<16)+here;
      nx = Gwide;
      lxx = 78;
      msg = "  [+" + here + ":";
      if (nx>32) nx = 32;
      for (nx=nx-1; nx >= 0; nx += -1) {
        msg = msg + SeeHex(" ",SafeAryElt(here,Gdata,0),"");
        zx = StrLength(msg);
        if (zx>lxx)  {
          msg = msg + "\n    ";
          lxx = zx+78;} //~if
        here++;} //~for
      System.out.println(msg + " ]");}} //~DrawMe

 /**
  * GameIcon constructor.
  */
  public GameIcon()  {
    if (Logging)
      System.out.println("(new GmIcon #" + RefNum + ")");
    Gtype = 2;}} //~GameIcon

/**
 * Font specification, mostly using GameIcon's drawing code.
 * Each character in the font is set up by CharSetup to be drawn as if
 * it were an icon.
 */
private class GameFont extends GameIcon  { // Font islike icon..

 /**
  * Return the class name and RefNum for this widget.
  *
  * @return     The reference String for this widget
  */
  public String toString()  {return "(GameFont #" + RefNum + ")";} //~toString

  /**
  * CharSetup takes a position, the ASCII (numeric code of a) character,
  * a color and Parent RefNum to alter this widget to resemble an icon
  * positioned within the parent container, so when it is drawn, the
  * character appears there. This is repeated for each character in a
  * GameTxLn's line of text.
  *
  * @param  top    the vertical position for this character
  * @param  left   the horizontal position for this character
  * @param  xCh    the character to be drawn there
  * @param  color  its color, 0x00RRGGBB
  * @param  whom   the parent (group) widget to insert this icon under
  * @param  logy   true to spit out many lines of console text
  *
  * @return        the pixel position ended = left+Gwide, or else err<0
  */
  public int CharSetup(int top, int left, int xCh, int color,
      int whom, boolean logy)  { // (this is noisy, avoid logging if posible)
    int rez = 0;
    int tall = 0, info = 0, why = 0, here, thar, nx,
        lxx = SafeArySz(Gdata);
    if (!GoodFont) logy = false;
    else if (Logging) if (MaxLogWgts+77>Max_Log) logy = true;
    while (true)  { // top/left are relative to Parent=whom
      why--; // why = -1
      if ((Gtype&255) !=3) break;
      why--; // why = -2
      if (Gdata==null) break;
      why--; // why = -3
      if (lxx<8) break;    // thar=0 if mono, >0 is end of index..
      thar = GetIxData( 4,-1);
      why--; // why = -4
      if (thar<0) break; // not a valid font table
      here = GetIxData( 1,0); // lnHi/desc/asc here
      why--; // why = -5
      if (here <= 0) break; // not a valid font table
      why = -why; // why = +5
      rez = ((here>>8)+here)&31;
      if (xCh<0) break; // caller wants line height
      why++; // why = +6
      rez = left;
      if (xCh<32) break;
      why++; // why = +7
      if (xCh>126) break;
      why++; // why = +8
      if (xCh==32)  {
        rez = GetIxData( 3,-1)+left;
        Gcolor = -1; // so it won't try to draw something
        Ginfo = xCh;
        break;} //~if
      tall = (here>>16)&63;
      here = GetIxData( 2,-1); // letter spacing
      why = -why; // why = -8
      if (here<0) break; // not a valid font table
      why--; // why = -9
      if (here>32) break; // assume we can't do that
      if (thar>0)  { // proportional..
        nx = xCh-28;
        why--; // why = -10
        if (nx>thar) break; // past end of short font, assume invalid
        thar = GetIxData( nx+1,0);
        nx = GetIxData( nx,0); // = offset to this char
        why--; // why = -11
        if (nx<100) break; // not in pixel rgn of tbl, assume invalid
        thar = thar-nx;} //~if // now it's the active pixel width
      else { // monospaced..
        thar = GetIxData( 3,0);
        thar = thar-GetIxData( 2,0);
        nx = GetIxData( 5,0);
        nx = (xCh-33)*thar+nx;} //~else
      why = -12;
      if (thar<0) break; // we don't do kerning, assume invalid
      why--; // why = -13
      if (nx+thar>lxx) break; // ran off end, ditto
      why = -why; // why = +13
      rez = left;
      if (thar==0) break;
      rez = left+thar+here;
      why = 0;
      if (color<0) break; // just measuring
      Parent = whom; // (temp'ly set inside the font wgt, so it draws)
      Gtop = top; // top/left are relative to Parent=whom
      Gleft = left;
      Gtall = tall;
      Gwide = thar;
      Gcolor = color;
      Ginfo = nx;
      break;} //~while
    if (why<0)  { // rtns end or -err..
      logy = GoodFont;
      rez = why;} //~if
    if (Logging) if (logy)  {
      System.out.println("  -ChrSet- #" + RefNum + ") " + xCh
          + " (~" + MaxLogWgts + ") = " + rez + "/" + why);
      if (false) LogMe("  (ChSt) ");
      MaxLogWgts--;} //~if
    return rez;} //~CharSetup

 /**
  * GameFont constructor.
  */
  public GameFont()  {
    if (Logging)
      System.out.println("(new GmFont #" + RefNum + ")");
    Gtype = 3;}} //~GameFont

/**
 * One line of text.
 *
 * Ginfo is used for cursor blinking, its bit fields, high to low:
 *     (cursor H-posn):12, (cursor height):5, (ref+ of the font):14
 *   The horizontal position is a pixel offset from the left edge of the
 *     widget to the cursor bar; the width of the cursor is one pixel.
 *   The top of the cursor is the top of the text widget; adding its height
 *     gives the bottom of the cursor bar.
 *   Fonts are generally instantiated early in the game definition process,
 *     so the RefNum of the font widget to be used is usually a small number;
 *     however, it could be anywhere in the load, and if there are more than
 *     10,000 widgets, the performance of GameEngine will be intolerably slow.
 *     14 bits is therefore quite adequate.
 *
 *   The cursor blinks at a fixed rate, but the upper bits of Gtype are still
 *     used to count the fractional frame rate.
 *
 *   To display the text, GameTxLn's DrawMe steps through the text line, one
 *     character at a time, then calls on the selected font's CharSetup method
 *     to prepare that character for display, then calls its DrawMe (inherited
 *     from GameIcon) to draw it and return a horizontal pixel position to
 *     draw the next character.
 *
 * Gselim (GameTxLn/GameScro only) = current selection
 */
private class GameTxLn extends GameWgt  { // one line of text..
  // int Gselim; // = current selection

 /**
  * Returns the selection of this GameTxLn. The selection will eventually
  * encompass a block of characters, but for now it is only an insertion
  * point, a single character offset.
  *
  * @return       The selection character offset = Gselim
  */
  // public int GetSelim()  {return Gselim;} //~GetSelim

 /**
  * Set the selection of a GameTxLn. The selection will eventually
  * encompass a block of characters, but for now it is only an insertion
  * point, a single character offset.
  *
  * @param  whar   The new selection to set Gselim
  */
  // public void SetSelim(int whar)  {Gselim = whar;} //~SetSelim

 /**
  * Return the class name and RefNum for this widget, plus its ID code if any.
  *   See toStrMash for formatting.
  *
  * @return     The reference String for this widget
  */
  public String toString()  {                                     // (GameTxLn)
    return toStrMash(super.toString(),"(GameTxLn #");} //~toString

 /**
  * Set the cursor in a GameTxLn's text, either as a pixel offset, or else
  * as calculated from a character offset. The pixel offset is used when
  * the user clicked in the text field; the character offset is used when
  * the text has been edited (characters added or deleted, or else an arrow
  * key is used to move the cursor left or right). If whar specifies a
  * valid cursor position, both Gselim and Ginfo are set.
  *
  * @param  pix    True if the cursor position is a pixel offset
  * @param  whar   The new cursor position as a pixel or character offset
  */
  public void SetBlinker(boolean pix, int whar)  { // sets blink
    // pix=T: whar is +pix to left (in wgt); pix=F: whar is +chars to left
    GameWgt funt = null; // (GameFont)
    int cx, kx, nx, info = Ginfo&0x3FFF, lxx = SafeArySz(Gdata)-1,
        posn = 0, tall = 0, here = 0, thar = 0, why = 0;
    boolean logy = false;
    funt = GetIxWgt(info);
    if (funt != null)
      tall = funt.CharSetup(0,0,-1,-1,0,logy)&31; // gets line height
    if (tall>0) for (nx=0; nx <= lxx; nx++) { // why = +20 for each completed int
      info = SafeAryElt(nx,Gdata,0);
      if (info==0) break;
      for (cx=0; cx <= 3; cx++) {
        why++; // why = +1
        if (!pix) if (posn >= whar) break; // normal exit at end of whar chars
        kx = info&255;
        why++; // why = +2
        if (kx==0) break; // normal exit for partial int (end of text)
        if (kx<32) continue;
        if (kx>126) continue;
        why++; // why = +3
        info = info>>8;
        thar = funt.CharSetup(0,here,kx,-1,0,logy); // adds char width to here
        if (thar<0) break; // (err) font table probably broken
        why++; // why = +4 for each completed char
        if (pix) if (thar >= whar) break; // normal exit at end of whar pixels
        here = thar;
        posn++;} //~for // why = +16 for each completed int
      why = why+4;
      if (thar<0) break;
      if (pix)  {if (thar >= whar) break;} //~if
      else if (posn >= whar) break;} //~for // (nx: each int)
    if (thar >= 0)  {
      why = -why;
      if (thar>here) if (here+thar<whar+whar)  { // past halfway, advance to next
        here = thar; // relative to wgt
        posn++;} //~if
      here = (((here<<5)+tall)<<14)+(Ginfo&0x3FFF);
      Gselim = posn;
      Ginfo = here;} //~if
    if (Logging) if (logy)
      System.out.println("  (SetBlnk/" + IffyStr(pix,"T","F")
          + "+" + whar + ") => " + posn + SeeHex("/x",here," ") + thar
          + " " + tall + " = " + why);} //~SetBlinker

 /**
  * Give this widget focus, or else take focus away.
  * A widget that is not typeable, clickable, or dragable cannot accept focus.
  *
  * When a GameTxLn widget accepts focus, it used the horiozntal click position
  * to determine where to set the insertion point (cursor), and then it starts
  * it blinking. When it gives up focus, it turns the animation off.
  *
  * @param  nuly   true to give focus, false to take it
  * @param  vert   the vertical position of a mouse-click giving focus
  * @param  horz   the horizontal position of the mouse-click
  *
  * @return        true if focus accepted, false if can't or losing it
  */
  public boolean SetFocus(boolean nuly, int vert, int horz)  {    // (GameTxLn)
    String msg = "";   // (vert,horz are wgt-relative; rtn F if refused/off)
    int why = 0, tye = Gtype&255, thar = Ginfo&0x3FFF, blnk = Ginfo>>14;
    GameWgt funt = GetIxWgt(thar);
    boolean logy = (MaxLogWgts>0);
    Gtype = Gtype&0xFFFF;
    if (!nuly) while (true)  { // F: turn off the focus/selection..
      why--; // why = -1
      if (tye !=4) break;
      Ginfo = thar;
      Gselim = -1;
      why--; // why = -2
      break;} //~while
    else while (true)  {
      if (FustFocLog>0) if (MaxLogWgts<FustFocLog)  {
        MaxLogWgts = FustFocLog;
        FustFocLog = 0;
        logy = true;} //~if
      why++; // why = 1
      if (tye !=4) break;
      Ginfo = thar;
      why++; // why = 2
      if (Gdata == null) break;
      why++; // why = 3
      if (funt==null) break; // unspecified/non-existing font
      why++; // why = 4
      if ((funt.GetType()&255) !=3) break; // gotta be a font
      why++; // why = 5
      if (vert<0) if (horz<0) horz = blnk>>5; // *** sb right end
      if (Logging) msg = "  (SeFocus #" + RefNum + ") t=" + (Gtype&255)
          + " " + vert + "/" + horz + " " + blnk;
      SetBlinker(true,horz);
      blnk = Ginfo>>14;
      why = 0;
      break;} //~while
    if (Logging) if (logy)  {
      if (StrLength(msg)==0) msg = "  (sFoc #" + RefNum + ")";
      System.out.println(msg + " => " + blnk + " = " + why);
      MaxLogWgts--;} //~if
    return why==0;} //~SetFocus

 /**
  * This is an internal utility for formatting a widget's information value,
  * GameTxLn overrides this to append the formatted cursor parts if !=0.
  *
  * @param  before  a prefix to add to the front
  * @param  tye     the widget's type
  *
  * @return         the composite display value
  */
  public String FmtInfo(String before, int tye)  {                // (GameTxLn)
    before = super.FmtInfo(before,tye);
    tye = Ginfo>>14;
    if ((Ginfo&(-0x4000)) !=0)
      before = before + "+" + (tye&31) + "/" + ((tye>>5)&0xFFF);
    return before;} //~FmtInfo // '" s=" + Gselim' now incl'd in FmtInf

  /**
  * This draws the GameTxLn pixels on the window as a sequence of character
  * icons. The boundaries of a masking rectangle (usually) limit the drawing
  * to the containing widget's dimensions.
  *
  * Ginfo is used for cursor blinking, its bit fields, high to low:
  *     (cursor H-posn):12, (cursor height):5, (ref+ of the font):14
  *   The horizontal position is a pixel offset from the left edge of the
  *     widget to the cursor bar; the width of the cursor is one pixel.
  *   The top of the cursor is the top of the text widget; adding its height
  *     gives the bottom of the cursor bar.
  *
  *   The cursor blinks at a fixed rate, but the upper bits of Gtype are still
  *     used to count the fractional frame rate.
  *
  *   To display the text, GameTxLn's DrawMe steps through the text line, one
  *     character at a time, then calls on the selected font's CharSetup method
  *     to prepare that character for display, then calls its DrawMe (inherited
  *     from GameIcon) to draw it and return a horizontal pixel position to
  *     draw the next character.
  *
  * @param  top    do not draw above this position
  * @param  left   do not draw to the left of this position
  * @param  tall   do not draw below this position +top
  * @param  wide   do not draw to the right of this position +left
  */
  public void DrawMe(int top, int left, int tall, int wide)  {    // (GameTxLn)
    int RealPrent = WhoCalled(0), // used only to verify hierarchy
        nx = AddPrntsPos(GetPosn()), myTop = nx>>16, myLft = SignExtend(nx),
        here, thar = Ginfo&0x3FFF, rx = myTop+Gtall, cx = myLft+Gwide, zx,
        lxx = SafeArySz(Gdata), blnk = Ginfo>>14, colo = Gcolor,
        logy = 0, info = 0, xCh = 0, saw = 0, why = 0;
    String msg = "";
    GameWgt funt = GetIxWgt(thar);
    int[] data = Gdata;
    // CallersID = 0;
    while (true)  {
      if (Logging)  { // tye=4: "wTxLn"  (Dr.wTxLn +
        if (logy <= 0) if (GoodTxt)  {
          if (MaxLogWgts>0)  {
            logy = MaxLogWgts;
            MaxLogWgts--;} //~if
          else if (blnk !=0)  {
            if (MaxLogAnim>8) logy = MaxLogAnim;
            else if (((Gtype+0x10000)&0x30000)==0) logy = MaxLogAnim;
            if (logy>0) MaxLogAnim--;}}} //~if
      if ((SeenLog&16)==0) logy = 1234;
      if (logy>0) msg = FmtFront("  (Dr.",false," +" + lxx
          + " " + blnk + "/" + (Gtype>>16));
      why++; // why = 1
      if (Gtop>0x3FFF) break; // hidden
      why++; // why = 2
      if (Parent==0) break;
      if (RealPrent !=0) if (RealPrent != Parent) break;
      why++; // why = 3
      if ((Gtype&255) !=4) break;
      why++; // why = 4
      if (data == null) break;
      why++; // why = 5
      if (funt==null) break;
      why++; // why = 6
      if ((funt.GetType()&255) !=3) break; // gotta be a font
      why++; // why = 7
      if (lxx <= 0) break;
      why++; // why = 8
      if (logy==1234) SeenLog = SeenLog|16;
      if (rx <= top+tall) tall = rx-1; // now it's edge, not size..
        else tall = top+tall-1;
      if (cx <= left+wide) wide = cx-1;
        else wide = left+wide-1;
      if (myTop>top) top = myTop;
      if (myLft>left) left = myLft;
      if (top<0) top = 0;
      if (left<0) left = 0;
      if (logy>999) msg = msg + " => " + top + "/" + left
          + " +" + tall + "/" + wide;
      if (blnk !=0) if (Gcolor >= 0) while (true)  {
        here = myTop-1;
        Gtype = Gtype+0x10000;
        if ((Gtype&0x40000) !=0) break; // off phase of blink
        rx = (blnk&31)+here; // = btm of blinker
        // if (rx <= top) break;
        // if (myTop>tall) break;
        cx = (((blnk>>5)+nx)&0xFFF)-1; // H-posn of blinker
        if (here<top) nx = top;
          else nx = here;
        if (rx>tall) rx = tall; // nx..rx is V-coord
        if (logy>0) msg = msg + " {" + nx + IffyStr(nx<rx,"_","^") + rx
            + "/" + cx + IffyStr(cx+1<left,"-}",IffyStr(cx+2>wide,"+}","}"));
        if (cx<0) break;
        if (cx+1<left) break; // clipped (off-screen)..
        if (cx+2>wide) break;
        if (nx<rx) RectFill(0,nx,cx,rx,cx);
        break;} //~while
      here = 0; // can't use Gleft/myLft, cuz Dr.wFon adds Parent(=RefNum) loc
      if (logy>0) msg = msg + " '";
      rx = RefNum;
      for (nx=0; nx <= lxx-1; nx++) {
        if (here+myLft >= wide) break;
        info = GetIxData( nx,0);
        if (info==0) break;
        for (cx=0; cx <= 3; cx++) {
          if (here >= wide) break;
          xCh = info&255;
          info = info>>8;
          if (xCh==0) break;
          if (xCh<32) continue;
          if (xCh>126) continue;
           if (logy>0) msg = msg + (char)xCh;

          zx = funt.  Gtype&255;
          if ((zx&255) != 3)  {
            xCh = 0;
            break;} //~if
          thar = funt.  CharSetup(0,here,xCh,colo,rx,logy>999);
          if (thar<0) break; // (err) font table probably broken  // ..not myTop
          if (xCh>32)  {
            if (thar+myLft>left)  {
              zx = WhoCalled(rx);
              funt.  DrawMe(top,left,tall-top,wide-left); } //~if
            else saw = (thar<<16)+myLft;} //~if
          here = thar;} //~for
        if (xCh==0) break;} //~for
      if (logy>0) msg = msg + "'";
      why = 0;
      break;} //~while
    if (false) if (logy <= 0) if (why==2) logy = 1001;
    if (Logging||(logy==1234)) if (logy>0)  {                // (Dr.wTxLn +
      System.out.println(msg + " = " + why + IffyStr(saw==0,"",LogVH(" <",saw,">"))
          + IffyStr(why==2," !prnt=" + Parent + "=" + RealPrent + " (RP)",""));
      if (logy>999) LogMe("  (DrMe) ");} //~if
    zx = WhoCalled(0);} //~DrawMe

 /**
  * GetTextWide returns the pixel width of this widget's text line.
  *
  * @return         the pixel width of the text line
  */
  public int GetTextWide()  { // now frm tbl..
    int posn = Gselim, info = Ginfo, logz = 0,
        zx = MaxLogWgts, ix, kx, nx, here, thar = info&0x3FFF, rez = 0, why = 0;
    String msg = "";
    GameWgt fnt = GetIxWgt(thar); // (GameFont)
    String aLine = GetTextLn( );
    boolean logy = zx>0;
    while (true)  {
      here = StrLength(aLine);
      why++; // why = 1
      if (fnt==null) break;
      why++; // why = 2
      if (here==0) break;    // thar=0 if mono, >0 is end of index..
      thar = fnt.  GetIxData( 4,-1);
      kx = fnt.  GetIxData( 3,-1);
      why++; // why = 3
      if (kx<0) break; // invalid tbl
      why = -why; // why = -3
      if (thar==0)  {
        rez = kx*here;
        break;} //~if
      zx = fnt.  GetIxData( 2,0);
      why++; // why = -2
      if (logz>0)  {msg = "\n   '" + aLine + "' " + kx + "+" + zx
          + "<" + thar;  logy = true;}
      for (here=here-1; here >= 0; here += -1) {
        nx = (int)(CharAt(here,aLine))-28;
        if (logz>0) msg = msg + " " + here + ":" + nx;
        if (nx==4) rez = rez+kx;
        if (nx<5) continue;
        if (nx+2>thar) continue;
        ix = nx+1;
        ix = fnt.  GetIxData( ix,0)
            - fnt.  GetIxData( nx,0);
        rez = rez+ix+zx;
        if (logz>0) msg = msg + "+" + ix + "=" + rez;  } //~for
      break;} //~while
    if (Logging) if (GoodTxt) if (logy)
      System.out.println("  (GetTxtWi #" + RefNum + ") = " + rez + "/" + why + msg);
    return rez;} //~GetTextWide // only works in GameTx

 /**
  * Returns this widget's text line.
  *
  * @return       The text line = Gdata as a String
  */
  public String GetTextLn()  { // only works in GameTx            // (GameTxLn)
    int lxx = SafeArySz(Gdata), info, cx, nx, kx = 0;
    String aLine = "";
    int[] data = Gdata;
    if (lxx>0) for (nx=0; nx <= lxx-1; nx++) {
      if (data == null) break;
      if (nx >= data.length) break;
      info = data[nx];
      aLine = aLine + UnPak4i(info,GoodEnd);
      if ((info&255)==0) break;} //~for
    return aLine;} //~GetTextLn

 /**
  * Set this widget's text line.
  *
  * @param  aLine    The text to set Gdata to
  *
  * @return         the pixel width of the text line
  */
  public int PutTextLn(String aLine)  { // only works in GameTx   // (GameTxLn)
    boolean logy = (MaxLogWgts>0);
    int lxx = StrLength(aLine), kx = (lxx+3)>>2, nx = SafeArySz(Gdata),
        px, info = 0, why = 0;
    int[] data = null;
    while (true)  {
      why++; // why = 1
      if ((Gtype&255) !=4) break;
      why++; // why = 2
      if (lxx==0) if (GetIxData(0,0)==0) break; // no text = no data, done
      if (nx==0) SetData(new int[kx+1]); // no data, create it
      else if (kx>nx) SetData(new int[kx+4]); // enlarge existing data
      why++; // why = 3
      px = SafeArySz(Gdata);
      if (lxx>px*4) break; // failed to enlarge
      data = Gdata;
      px--;
      nx = 0;
      if (kx>0) nx = kx-1;
      for (nx=nx; nx <= px; nx++)
        if (data == null) break;
        else if (nx >= data.length) break;
        else if (nx >= 0) data[nx] = 0;
      why++; // why = 4
      for (nx=0; nx <= lxx; nx += 4) {
        info = Pack42i(Substring(nx,4,aLine),GoodEnd,false);
        kx = nx>>2;
        if (kx<0) break;
        if (data != null) if (kx < data.length)
          data[kx] = info;} //~for
      why = 0;
      break;} //~while
    kx = GetTextWide();
    if (Logging) if (GoodTxt) if (logy)  {
      if (data == null) nx = 0;
        else nx = SafeArySz(data);
      System.out.println("  (Put.TxLn #" + RefNum + ") *" + nx + "/" + lxx + " => '"
          + GetTextLn() + "' w=" + kx + "/" + Gwide + " = " + why);} //~if
    MaxLogWgts--;
    return kx;} //~PutTextLn

  public GameTxLn()  {
    Gselim = -1; // no selection
    if (Logging)
      System.out.println("(new GmTxLn #" + RefNum + ")");
    Gtype = 4;}} //~GameTxLn

/**
 * Animated group head for a complex sprite.
 *
 * Ginfo is the packed (signed) velocity vector,
 *      (vertical*256):16, (horizontal*256):16
 *   Each velocity component is a 16-bit fixed-point number, 8i.8f,
 *     with eight bits of signed integer part and 8 bits fraction part,
 *     for a value range [-128.0 .. +127.996], where +2.0 means the sprite
 *     is moving down or right at two pixels per frame, about 20 pix/sec
 *     given a nominal frame rate of 10fps, and -0.1 means the sprite is
 *     moving up or left one pixel every ten frames, or 1 pix/sec.
 *
 *   Otherwise this is like a group (type=5), except that the components
 *     are expected to be one or more layers of simple widgets, rectangle or
 *     icon or text, each with their own color, (and the icons possibly with
 *     their own animation), all contained within the dimenstions of the
 *     sprite.
 *
 *   Curves or other acceleration must be programmed by periodically changing
 *     the velocity vector. For example, gravity is programmed by adding a
 *     constant positive value +32 feet/s/s = +10m/s/s to the vertical
 *     component each frame. If your game models vertical distance at 20 pix/m,
 *     then you want to add 10m/s every second, which is 1 m/s = 20 pix/s each
 *     frame (= 0.1 secs, use DoOften) to the vertical velocity, which is
 *     (20<<8)<<16 = Ginfo+0x14000000, or more simply, if your widget reference
 *     is named 'heavy' in Java:
 *
 *         heavy.SetInfo(heavy.GetInfo()+((20<<8)<<16));
 *
 *     In practice you may need to tweak these numbers for it to look right.
 *     Note that the (20<<8) shifts the integer 20 over eight places (the size
 *     of the fractional part) and then you shift it another 16 places into the
 *     upper half of the information value (because the horizontal component is
 *     in the low 16 bits).
 *
 *     For (very light) asteroid gravity, say 0.1 pix/s, you need to calculate
 *     the value of 0.1*256 = 25.6 rounded to 26, which is the fractional part
 *     (do not shift <<8): ...SetInfo(heavy.GetInfo()+(26<<16));
 *
 *   Of course your sprite does not need to move at all, or only get a new
 *     position on explicit call to SetPosn, so in such a case just leave
 *     Ginfo = 0.
 *
 *   If you want your sprite to have various colors, you would layer several
 *     icons of different colors, but the icon editor to do that in GameEngine
 *     does not yet exist. Real Soon Now.
 */
private class GameSprit extends GameWgt  { // Moving sprite

 /**
  * Return the class name and RefNum for this widget, plus its ID code if any.
  *   See toStrMash for formatting.
  *
  * @return     The reference String for this widget
  */
  public String toString()  {                                    // (GameSprit)
    return toStrMash(super.toString(),"(GameSprit #");} //~toString

 /**
  * Returns the specified color for this widget, = 0x00RRGGBB or -1 if none.
  * Not the color of a sprite root, but reach in and get it from
  * whatever rect or icon or text is last (top) child.
  *
  * @return       The specified color, or -1 if none
  */
  public int GetColor()  {                                       // (GameSprit)
    int inx, info, tye, colo = Gcolor, logy = 0;
    GameWgt nuly = null;
    if (false) for (inx=0; inx <= 99; inx++) {
      info = GetIxData(inx,0);
      if (info <= 0) break; // normal exit
      nuly = GetIxWgt(info);
      if (nuly==null) break;
      tye = nuly.GetType()&255;
      info = nuly.GetColor();
      if (Logging)
        System.out.println("(Sp.GetColo #" + RefNum + ") " + inx
            + ": t=" + tye + SeeHex(" c=",info,""));
      if (tye>4) continue;
      if ((tye&1) !=0) continue;
      colo = info;} //~for // save it if it's 0/2/4
    return colo;} //~GetColor

 /**
  * Set the color for this widget, = 0x00RRGGBB or -1 if none.
  * Never change the color of a sprite root, but reach in and
  * do it to whatever rect or icon or text is last (top) child,
  * but only if the color is opaque.
  *
  * @param  color    The color to set Gcolor to, <0 is no color
  */
  public void SetColor(int color)  {                             // (GameSprit)
    int inx, info, tye, logy = 0;
    GameWgt whom = null, nuly = null;
    if (color >= 0) for (inx=0; inx <= 99; inx++) {
      info = GetIxData(inx,0);
      if (info <= 0) break; // normal exit
      nuly = GetIxWgt(info);
      if (nuly==null) break;
      tye = nuly.GetType()&255;
      if (Logging)
        System.out.println("(Sp.SetColo #" + RefNum + ") " + inx
            + ": t=" + tye + SeeHex(" c=",nuly.GetColor(),""));
      if (tye>4) continue;
      if ((tye&1) !=0) continue;
      whom = nuly;} //~for // save it if it's 0/2/4
    if (whom != null) whom.SetColor(color); // set its color
      else Gcolor = color;} //~SetColor

 /**
  * This is an internal utility for formatting a widget's information value,
  * GameSprit overrides this to format it as two fixed-point numbers.
  *
  * @param  before  a prefix to add to the front
  * @param  tye     the widget's type
  *
  * @return         the composite display value
  */
  public String FmtInfo(String before, int tye)  {               // (GameSprit)
    return FixHaff(before,Ginfo>>16) + FixHaff("/",SignExtend(Ginfo));} //~FmtInfo

  /**
  * This draws the GameSprit subwidgets on the window as just like the group
  * (type=5), except that the velocity is applied to the group position before
  * calling super to draw it.
  *
  * Ginfo is a velocity vector, each component a 16-bit fixed-point number,
  *     8i.8f, with eight bits of signed integer part and 8 bits fraction part,
  *     for a value range [-128.0 .. +127.996], where +2.0 means the sprite
  *     is moving down or right at two pixels per frame, about 20 pix/sec
  *     given a nominal frame rate of 10fps, and -0.1 means the sprite is
  *     moving up or left one pixel every ten frames, or 1 pix/sec.
  *
  * @param  top    do not draw above this position
  * @param  left   do not draw to the left of this position
  * @param  tall   do not draw below this position +top
  * @param  wide   do not draw to the right of this position +left
  */
  public void DrawMe(int top, int left, int tall, int wide)  {   // (GameSprit)
    int frax = Gtype>>16, RealPrent = WhoCalled(0), zx = RealPrent, kx,
        whar, thar = 0, logy = 0; // RP used only to verify hierarchy
    // int[] data = Gdata;
    String msg = "";
    // CallersID = 0;          // this only works properly w/parent origin @ 0,0
    if ((Gtype&255) !=6) return;
    if (RealPrent !=0) if (Parent !=0) if (RealPrent != Parent) return;
    if (Gtop>0x3FFF) return; // hidden
    if (Gdata == null) if (Gcolor<0) return;
    if ( JavaGame.SpritePause()) {
      super.DrawMe(top,left,tall,wide);
      return;} //~if
    if ((SeenLog&64)==0) logy = 12345;
    else if (Logging) if (logy>0)
      if ((logy&0xFFFF) != RefNum) if ((logy>>16) != RefNum) logy = 0;
    if (logy>0) msg = FmtInfo(" ",6) + FixHaff(" f=",(frax>>8)&255)
        + FixHaff("/",frax&255) + msg + LogVH(" (",MaxWinSize,
        ")");
    // frax = Gtype>>16;
    if (Ginfo !=0)  {
      thar = 0; // we are moving, do it (fix.pt, 8-frac)..
      whar = (Ginfo>>16)+(Gtop<<8)+((frax>>8)&255); // new signed top
      if (logy>0) msg = msg + FixHaff(" => ",whar);
      frax = frax&255|((whar&255)<<8);
      whar = whar>>8;
      zx = whar-Gtop; // the diff (supe cares)
      if (whar<0) thar++; // off top
      thar = thar<<1;
      Gtop = whar;
      if (Gtop+Gtall > WinHi) thar++; // off botm
      thar = thar<<1;
      whar = SignExtend(Ginfo)+(Gleft<<8)+(frax&255); // new signed left
      if (logy>0) msg = msg + FixHaff("/",whar);
      frax = frax&0xFF00|whar&255;
      whar = whar>>8;
      kx = whar-Gleft; // the diff (supe cares)
      if (whar<0) thar++; // off left edge
      thar = thar<<1;
      Gleft = whar;
      if (Gleft+Gwide > WinWi) thar++; // off right
      if ((Gtype&256) !=0) if (thar>0)
        Gtype = Gtype|0x8000; // flag that we ran off the edge
      if (logy>0) msg = msg + " (" + thar + FixHaff(") f=",(frax>>8)&255)
          + FixHaff("/",frax&255);
      Gtype = (frax<<16)|Gtype&0xFFFF;  } //~if
    if (Logging||(logy==12345)) if (logy>0)
      System.out.println("   (Sp.Dr #" + RefNum + ")" + msg
          + " => " + Gtop + "/" + Gleft);
    whar = WhoCalled(Parent);
    super.DrawMe(top,left,tall,wide);
    if (logy==12345) SeenLog = SeenLog|64;} //~DrawMe

  public GameSprit()  {
    if (Logging)
      System.out.println("(new GmSprt #" + RefNum + ")");
    Gtype = 6;}} //~GameSprit

/**
 * Scrolled panel with one or two scrollbars. [Not yet fully implemented]
 *
 * The scrolled panel is typically a group, the first widget indexed in the
 * Gdata array. The second indexed widget is the vertical scrollbar (details
 * TBD), and the third is the horizontal scrollbar. If a scrollbar is omitted,
 * its position in the array is zero.
 *
 * Gselim (GameTxLn/GameScro only) = scroll limits
 */
private class GameScro extends GameWgt  { // scrolling field..
  // int Gselim; // = current scroll limits

 /**
  * Returns the scroll limits of a GameScro.
  *
  * @return       The scroll limits = Gselim
  */
  // public int GetSelim()  {return Gselim;} //~GetSelim

 /**
  * Set the scroll limits of a GameScro.
  *
  * @param  whar   The new scroll limits for Gselim
  */
  // public void SetSelim(int whar)  {Gselim = whar;} //~SetSelim

 /**
  * Return the class name and RefNum for this widget, plus its ID code if any.
  *   See toStrMash for formatting.
  *
  * @return     The reference String for this widget
  */
  public String toString()  {                                     // (GameScro)
    return toStrMash(super.toString(),"(GameScro #");} //~toString

 /**
  * This draws the GameScro subwidgets on the window as just like the group
  * (type=5), except that the scrollbar values are applied to the panel group
  * position before calling super to draw everything.
  *
  * @param  top    do not draw above this position
  * @param  left   do not draw to the left of this position
  * @param  tall   do not draw below this position +top
  * @param  wide   do not draw to the right of this position +left
  */
  public void DrawMe(int top, int left, int tall, int wide)  { // (GameScr)
    int RealPrent = WhoCalled(0), // used only to verify hierarchy
        nx = AddPrntsPos(GetPosn()), myTop = nx>>16, myLft = SignExtend(nx),
        here, thar, rx = myTop+Gtall, cx = myLft+Gwide, lxx = SafeArySz(Gdata),
        logy = 0, info = 0, why = 0;
    String msg = "";
    // CallersID = 0;
    while (true)  {
      why++; // why = 1
      if (Gtop>0x3FFF) break; // hidden
      why++; // why = 2
      if (RealPrent !=0) if (Parent !=0) if (RealPrent != Parent) break;
      why++; // why = 3
      if ((Gtype&255) !=7) break;
      why++; // why = 4
      why++; // why = 5
      if (Gcolor<0) break; // single-layer hit rgn for sprite (undrawn)
      why++; // why = 6
      if (Gdata == null) break;
      why++; // why = 7
      if (lxx <= 0) break;
      if (Logging) if (MaxLogWgts>0)
        msg = FmtFront("  (Dr.",false," +" + lxx);
      if (rx <= top+tall) tall = rx-1; // now it's edge, not size..
        else tall = top+tall-1;
      if (cx <= left+wide) wide = cx-1;
        else wide = left+wide-1;
      if (myTop>top) top = myTop;
      if (myLft>left) left = myLft;
      if (top<0) top = 0;
      if (left<0) left = 0;
      why++; // why = 8
      for (nx=0; nx <= Gwide-1; nx++) {
        info = SafeAryElt(nx+Ginfo,Gdata,0);
        if (info==0) continue;
        here = nx+myLft;
        if (here<left) continue;
        if (here>wide) continue;
        for (rx=0; rx <= lxx; rx++) {
          thar = rx+myTop;
          if ((info&1) !=0) if (thar >= top) if (thar <= tall)
            PokePixel(Gcolor,thar,here);
          info = info>>1;
          if (info==0) break;}} //~for
      break;} //~while // tye: "wScro"
    if (Logging) if (MaxLogWgts>0)
      System.out.println(msg + " = " + why);
    MaxLogWgts--;} //~DrawMe

  public GameScro()  {
    Gselim = 0; // no limits
    if (Logging)
      System.out.println("(new GmScro #" + RefNum + ")");
    Gtype = 7;}} //~GameScro

  private static int[][] KnownCirclz = new int[100][]; // +0 sld, +1 hol, +2 ant
  private static int[][] RipAnts = new int[32][]; // +0 hol, +16 ant

// Rock-Paper-Scissor (RPS) icon pixels..
  private final int[] QxPiPix = {0,0,0,0,0, 0x03C00000, // folded paper..
      0x03FF8000,0x03FFFF00, 0x07FFFFF0,0x07FFFFF0,0x07FFFFF0,0x07FFFFF0,
      0x07FFFFF0,0x0FFFFFF0,0x0FFFFFF8,0x0FFFFFF8, 0x0FFFFFF8,0x0FFFFFF8,
      0x0FFFFFF8,0x1FFFFFF8,0x1FFFFFF8,0x1FFFFFFC, 0x1FFFFFFC,0x1FFFFFFC,
      0x01FFFF8C,0x0003FF80,0x0780, 0,0,0,0,0, 0};
  private final int[] QxRiPix = {0,0, 0x001F0000, // rock..
      0x007FC000,0x00FFF000, 0x00FFF800,0x01FFF800,0x01FFFC00,0x01FFFC00,
      0x03FFFC00,0x03FFFE00,0x03FFFF00,0x03FFFF80, 0x03FFFF80,0x03FFFF80,
      0x03FFFF80,0x03FFFF80, 0x03FFFF80,0x01FFFF00,0x01FFFF00,0x00FFFE00,
      0x00FFFC00,0x007FF800,0x001FE000, 0,0,0, 0,0,0,0,0, 0};
  private final int[] QxSiPix = {0,0,0,0x00400100, // scissors, op+clo..
      0x00C00300,0x00E00300,0x00600700,0x00700600, 0x00300E00,0x00380C00,
      0x00181C00,0x00181800, 0x001C1800,0x000C3800,0x000E3000,0x00067000,
      0x00076000,0x0003E000,0x003BCE00,0x00FFFF80, 0x00C7F180,0x018360C0,
      0x018360C0,0x018360C0, 0x018360C0,0x018360C0,0x00C63180,0x00FE3F80,
      0x00380E00,0,0,0,    0,0, 0x8000,0x0001C000, 0x0001C000,0x0001C000,
      0x0001C000,0x0001C000, 0x0001C000,0x0001C000,0x0001C000,0x0001C000,
      0x0001C000,0x0001C000,0x0001C000,0x0001C000, 0x0001C000,0x0001C000,
      0x0001C000,0x001DDC00, 0x007FFF00,0x0063E300,0x00C1C180,0x00C1C180,
      0x00C1C180,0x00C1C180, 0x00C1C180,0x00636300,0x007F7F00,0x001C1C00,
      0,0, 0};
  private final int[] QxQiPix = {0,0,0,0,0,0,0,0, // explode, 6 frs..
      0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0, // (ending: none)
      0x00FE0000,0x03FF8000,0x07FFC000,0x0FFFE000, 0x0FFFE000,0x1FFFF000,
      0x1FFFF000,0x3FFFFF00, 0x3FFFFFC0,0x3FFFFFE0,0x3FFFFFF0,0x3FFFFFF8,
      0x3FFFFFFC,0x1FFFFFFC,0x1FFFFFFE,0x3FFFFFFE, 0x3FFFFFFE,0x7FFFFFFE,
      0x7FFFFFFE,0x7FFFFFFE, 0x7FFFFFFE,0x7FFFFFFC,0x7FFFFFFC,0x7FFFFFF8,
      0x7FFFFFF0,0x3FFFFFE0,0x3FFFFFC0,0x1FFF3F00, 0x1FFF0000,0x0FFE0000,
      0x07FC0000,0x01F00000,              0,0,0,0, 0x00780000,0x00FC0000,
      0x01FE0000,0x03FF0000, 0x03FF8000,0x07FF8000,0x07FFBE00,0x07FFFF80,
      0x07FFFFC0,0x07FFFFE0,0x0FFFFFE0,0x0FFFFFF0, 0x0FFFFFF0,0x0FFFFFF0,
      0x0FFFFFF0,0x0FFFFFF0, 0x0FFFFFE0,0x0FFFFFE0,0x0FFFFFC0,0x07FFFF80,
      0x07FF3E00,0x03FF0000,0x03FE0000,0x01FC0000, 0x00F80000, 0,0,0,
              0,0,0,0,0,0,0, 0x001C0000,0x003E0000,0x003F0000,0x007F8000,
      0x007F8000,0x007FF000,0x007FFC00,0x003FFE00, 0x003FFF00,0x007FFF00,
      0x007FFF00,0x007FFF00, 0x007FFE00,0x007FFC00,0x003FC000,0x003F8000,
      0x000F0000, 0, 0,0,0,0,0,0,0,      0,0,0,0,0,0,0,0, 0,0,
      0x0003C000,0x000FF000, 0x001FF800,0x001FF800,0x003FFC00,0x003FFC00,
      0x003FFC00,0x003FFC00,0x001FF800,0x001FF800, 0x000FF000,0x0003C000,
      0,0, 0,0,0,0,0,0,0,0,       0,0,0,0,0,0,0,0, 0,0,0,0,
      0x0001C000,0x0003E000,0x0007F000,0x0007F000, 0x0007F000,0x0003E000,
      0x0001C000,0, 0,0,0,0, 0,0,0,0,0,0,0,0, 0};

  static GameWgt UserGameBd = null; // (unused)

  public String DefaltGmName = "GameBd", SavedGameData = "";

  private static final int[] BtnAros8 = {1,33,49,57,61,57,49,33,1,0,
      255,223,207,199,195,199,207,223,255,0,  128,132,140,156,188,
      156,140,132,128,0,  255,223,207,199,195,199,207,223,255,0,0};

 /**
  * WgtMacros is the (internal) data used to build complex widget structures
  * from simple user specifications. Normally we use a predefined array, but
  * this can be overridden by a specification included in the user data.
  */
  private int[] WgtMacros = Wgt_Macros;

 /**
  * UserEvHandler is set when your main() program calls StartGame, and used
  * everywhere to call your overridden methods.
  */
  private GameEvent UserEvHandler = null; // call InstallMyEvts to set this

 /**
  * FindSubWgt is not particularly speedy, but sometimes different places
  * need the same subwidget, so a successful find is cached so it can be
  * returned immediately if asked for again.
  *
  * GameEngine allows for multiple lists of widgets under the general game
  * board; SpriteList is defined to be a the first of them, and for speed
  * only its contents are considered for collision testing.
  *
  * Only one widget or group can have focus at a time, and "marching ants"
  * around it give a visual indication of focus. In the case of (rectangular)
  * groups, mAntsHost remembers who is currently showing ants, and mAntsWgt
  * is a reference to the ants group that is moved around from host to host.
  * Sprites each have their own ants data, customized to the shape of the
  * respective sprite.
  *
  * When your game is specified to use alert and input dialogs, theAlertDlg
  * and theInputDlg are references to the structures that display these
  * dialogs in your game window; DlgInputData is a reference to to the
  * actual input GameTxLn widget that accepts the user typed-in text, for
  * easy access by the code that processes that input.
  *
  * GameWgtsList is the top-level group of groups of widgets. When not
  * otherwise active, the marching ants group mAntsWgt is stored there.
  * Its first subwidget should be the SpriteList.
  */
  private GameWgt CashFindWgt = null, CashFoundWgt = null,
      SpriteList = null, mAntsHost = null, mAntsWgt = null,
      theAlertDlg = null, theInputDlg = null, DlgInputData = null,
      GameWgtsList = null, GameRoot = null, DontBump = null;

   /**
  * Dragging and rollover processing take time even if not currently
  * active, so if your game does not need one or the other of these,
  * the extra processing can be eliminated. The booleans WantsRollover
  * and WantsDrag remember that.
  *
  * The boolean flags ValidMouse and MouseIsDown remember the current
  * state of the mouse from frame to frame.
  */
  private boolean WantsRollover = false, WantsDrag = false,
      ValidMouse = false, MouseIsDown = false, KillDrag = false,
      PendingDraw = false, Bumpt = false;

   /**
  * MouseVert and MouseHorz are window-relative coordinates of the mouse
  * as of the most recent MouseDispatch.
  *
  * MonoFontNo,TextFontNo,TinyFontNo,BoldFontNo,ItalFontNo,HeadFontNo,
  * and NanoFontNo are the widget reference numbers for the respective
  * fonts in the current game environment. You can get them in GameMaker
  * using:
  *
  *      ChoseFont(null,code)
  *
  * where code is the ASCII code for the first letter of the name (except
  * TinyFontNo is 'Y' = 89).
  */
  private static int MouseVert = 0, MouseHorz = 0, LocalVert = 0,
      MonoFontNo = 0, TextFontNo = 0, TinyFontNo = 0, BoldFontNo = 0,
      ItalFontNo = 0, HeadFontNo = 0, NanoFontNo = 0, ScorFontNo = 0,
      TimerTix = 0, TimeBase = 0,  DlogIsOpen = 0,
      InputDlgNo = 0, CashFindWhy = 0, Floatem = 0, InputWhy = 0;

   /**
  * These strings are used in translating your game file to JavaScript.
  */
  private String JSinfo = "", RecentJava = "", UserDate = "",
      Game_Title = GameTitle;

  /**
  * SpritePause is called by sprite widgets to see if they should pause
  * their motion (like when an alert or input dialog is open).
  *
  * @return         true to pause
  */
  public static boolean SpritePause() {
    if (!StopSprites) return false;
    return (DlogIsOpen>0);} //~SpritePause

  /**
  * GetGameValue is an access method to get one of the private data items
  * from outside the JavaGame game instance.
  *
  * @param  whom    A selector, 0..10
  *
  * @return         the selected value
  */
  public static int GetGameValue(int whom) {
    switch (whom) {
    case 0:
      whom = MonoFontNo;
      break;
    case 1:
      whom = TextFontNo;
      break;
    case 2:
      whom = TinyFontNo;
      break;
    case 3:
      whom = BoldFontNo;
      break;
    case 4:
      whom = ItalFontNo;
      break;
    case 5:
      whom = HeadFontNo;
      break;
    case 6:
      whom = NanoFontNo;
      break;
    case 7:
      whom = LocalVert;
      break;
    case 8:
      whom = MouseVert;
      break;
    case 9:
      whom = MouseHorz;
      break;
    case 10:
      whom = DlogIsOpen;
      break;
    case 11:
      whom = InputDlgNo;
      break;} //~switch
    return whom;} //~GetGameValue

 /**
  * Returns a (language-neutral) new int array of the specified length.
  * (Because the syntax is different in Java vs JavaScript)
  * Java = JS syntax for learning the length: myarray.length
  *
  * @param lxx  The length desired
  *
  * @return     The array new int[lxx]
  */
  public static int[] NewArray(int lxx) {
    return new  int[lxx];} //~NewArray

  /**
  * Converts a mouse click position from window-relative coordinates to
  * be relative to the widget clicked in. The horizontal offset is returned
  * directly, the vertical offset is cached in LocalVert for later recovery.
  *
  * @param  whom  The widget that was clicked
  * @param  vert  The vertical offset relative to the window
  * @param  horz  The horizontal offset relative to the window
  *
  * @return       The horizontal offset into this widget
  */
  public int LocalizePos(GameWgt whom, int vert, int horz) { // frm MouseDisp
    int zx, whar = (vert<<16)+(horz&0xFFFF);
    LocalVert = 0;
    if (whom==null) return 0;
    whar = AddPair(whar,PairNeg(whom.AddPrntsPos(whom.GetPosn())));
    LocalVert = whar>>16;
    return SignExtend(whar);} //~LocalizePos

/**
  * GetIxWgt converts a widget reference number into a reference to that widget.
  *
  * @param  ix    an index into the AllWgts array = RefNum
  *
  * @return       the widget with that RefNum
  */
  public static GameWgt GetIxWgt(int ix)  {
    if (AllWgts != null)
    if (ix<MaxWgts) if (ix>0) return AllWgts[ix];
    return null;} //~GetIxWgt

/**
  * A language-neutral way to get a (32-bit) time in milliseconds.
  * The first time this is called the caller sets TimeBase (initially =0),
  * so that all subsequent calls are based off that start value,
  * despite being less precision than normal Java.
  *
  * @return       The number of milliseconds since game start
  */
  public static int GetMilliSecs()  {
    return ((int)System.currentTimeMillis())  -TimeBase;} //~GetMilliSecs

/**
  * A safe (no exceptions) way to count the number of (non-overlapping)
  * occurrences of a particular string within a larger string.
  * An empty string cannot be counted.
  *
  * @param  aWord The string to look for
  * @param  aStr  The string in which to find and count it
  *
  * @return       The number of occurrences found, or 0 if none
  */
  public static int Countem(String aWord, String aStr)  {
    int lxx = StrLength(aWord), whar, thar = 0, nby = 0;
    if (lxx==0) return 0;
    try {  while (true)  {
        whar = aStr.indexOf(aWord,thar); // JS=Java
        if (whar<0) break;
        thar = whar+lxx;
        nby++;} //~while
    } catch (Exception ex)  {}
    return nby;} //~Countem

  /**
  * A safe (no exceptions) way to extract an integer number from a string.
  * If the number begins with "0x" then it is assumed to be hexadecimal.
  * Leading white space is ignored, and the first character that is not
  * part of the number ends the scan.
  *
  * @param  aStr  The string from which to extract that character
  *
  * @return       The number parsed, or 0 if none
  */
  public static int SafeParseInt(String aStr)  {
    int nCh, here, lxx = StrLength(aStr), valu = 0, mode = 0, seen = 0;
    for (here=0; here <= lxx-1; here++) {
      nCh = (int)(CharAt(here,aStr));
      if (nCh <= 0) break;
      if (valu==0)  {
        if (nCh <= 32)  {
          if (seen>0) break;
          continue;} //~if
        if (nCh==45)  { // '-'
          if (seen>0) break;
          if (mode !=0) break;
          mode++;
          continue;} //~if
        if ((nCh|0x20)==120)  { // 'x'
          if (mode<0) break;
          if (seen !=1) break;
          mode = mode-4;
          continue;}} //~if
      if (nCh<48) break; // '0'
      if (mode<0)  {
        if (nCh>96) nCh = nCh&0x5F; // convert lower case to caps
        if (nCh>70) break; // 'F'
        if (nCh>57) if (nCh<65) break; // '9'..'A'
        valu = valu*16+(nCh&15);
        if (nCh>57) valu = valu+9;} //~if // '9'
      else if (nCh <= 57)  {
        valu = valu*10+(nCh&15);
        seen++;} //~if
      else break;} //~for
    if ((mode&3)>0) valu = -valu;
    return valu;} //~SafeParseInt

  /**
  * Replace a substring.
  *
  * @param  nuly     The replacement string
  * @param  ofx      The starting offset to put it at
  * @param  sz       The length of text to replace
  * @param  theText  The string in which to make the substitutions
  *
  * @return          theText as changed
  */
  public static String Replace(String nuly, int ofx, int sz, String theText)  {
    return Substring(0,ofx,theText) + nuly + RestOf(ofx+sz,theText);} //~Replace

  /**
  * A safe (no exceptions) way to extract an item (word/line) from a string.
  * When getting words, excess white space is ignored, so the returned
  * string is empty only when requesting a word beyond the last. Otherwise
  * consecutive delimiters result in empty items being returned, so that
  * you always get the nth item of a multi-item string. Items past the end
  * or before the front (0 or less) are empty.
  *
  * @param  delim  Use '\n' to get a whole line, ' ' to get a word
  * @param  whom   The (1-base) item number to get
  * @param  aStr   The string from which to extract that item
  *
  * @return       The extracted string
  */
  public static String NthItemOf(char delim, int whom, String aStr)  {
    char xCh;
    int nCh = (int)(delim), lxx = StrLength(aStr), here, aCh;
    boolean xact = true, wordy = false, seen = false;
    String aWord = "";
    if (nCh >= 10) if (nCh <= 32)  {
      if (nCh == 32) wordy = true;
      xact = false;} //~if
    whom--; // comes in 1-based, now 0-base
    try {  if (whom >= 0) for (here=0; here <= lxx-1; here++) {
      xCh = CharAt(here,aStr);
      aCh = (int)(xCh);
      if (aCh <= 0) break;
      if (xact)  {
        if (aCh == nCh)  {
          whom--;
          if (whom<0) break;
          continue;}} //~if
      else if (aCh <= 32)  {
        if (wordy)  { // one or more spaces..
          if (seen) whom--;
          seen = false;
          if (whom<0) break;
          continue;} //~if
        else if (aCh < 32)  { // any ctl counts as \n or \r
          whom--;
          if (whom<0) break;
          continue;}} //~if
      else seen = true;
      if (whom <= 0) aWord = aWord + xCh;} //~for
    } catch (Exception ex)  {}
    return aWord;} //~NthItemOf

 /**
  * Converts an integer to String (for JS, Java not always auto'ly coerces).
  *
  * @param  whom  The integer
  *
  * @return       The same number as String
  */
  public static String CvInt2Str(int whom)  { // 'cuz JS is untyped
    return "" + whom;} //~CvInt2Str

/**
  * A safe way to read a whole text file.
  * The line-ends in the input file are converted to '\n' regardless of
  * how they are on the file.
  *
  * @param  filename The name of the text file
  *
  * @return          The text as read
  */
  public static String ReadWholeTextFile(String filename)  {
    char aCh;
    int nx, lxx = 0, nuly = 0; // adapted from what I found on StackOverflow..
    String content = "";
    File myFile = new File(filename); // for ex foo.txt
    FileReader reader = null;
    try {
      reader = new FileReader(myFile);
      lxx = (int)myFile.length();
      if (lxx <= 0) return "";
      char[] chars = new char[lxx];
      reader.read(chars);
      for (nx=0; nx <= lxx-1; nx++) { // convert Windoze line ends to internal..
        aCh = chars[nx];
        if (aCh == '\r')  {
          if (nx+1<lxx) if (chars[nx+1] == '\n') continue;
          aCh = '\n';}
        chars[nuly] = aCh;
        nuly++;} //~for
      content = new String(chars);
      if (nuly<lxx) content = content.substring(0,nuly);
      reader.close();
      reader = null;} //~try
    catch (Exception ex)  {}
    finally {
      try {if (reader !=null) reader.close();}
      catch (Exception ex)  {}} //~finally
    return content;} //~ReadWholeTextFile

 /**
  * A safe way to write a whole text file.
  *
  * @param  filename The name of the text file
  * @param  content  The text to be written
  *
  */
  public static void WriteWholeTextFile(String filename, String data)  {
    int lino, nx, lxx = data.length(), sofar = 0;
    String aLine;
    FileWriter myFile = null;
    BufferedWriter whom = null;
    PrintWriter doit = null;
    try {
      // data = ReplacAll("\r\n","\n",data); // unneeded
      myFile = new FileWriter(filename);
      whom = new BufferedWriter(myFile);
      doit = new PrintWriter(whom);
      for (lino=1; lino <= 999999; lino++) { // not the most efficient way to do it, but..
        if (sofar >= lxx) break; // normal exit at end
        aLine = NthItemOf('\n',lino,data);
        nx = aLine.length();
        if (nx>0) doit.print(aLine);
        sofar = sofar+nx;
        if (sofar >= lxx) break; // no \n on final line? OK
        doit.println();
        sofar++;} //~for // (lino)
      doit.close();
      doit = null;} //~try
    catch (Exception ex)  {}
    finally {
      try {if (doit !=null) doit.close();} //~try
      catch (Exception ex)  {}}} //~finally //~WriteWholeTextFile

 /**
  * Find the most significant non-zero bit in a 32-bit integer.
  * The least significant bit is easily extracted by (whom&-whom) but there
  * is no similarly trivial way to get the other end.
  *
  * @param  whom  an integer
  *
  * @return       the most significant bit of that number, or =0 if none
  */
  public static int TopBit(int whom)  { // rtns most significant non-zero bit
    int zx;
    while (true)  { // exit when found it
      if (whom==0) break; // none
      if (whom<0) whom = 0x80000000; // high bit
      zx = whom&-whom; // = its low bit, done when =
      if (zx==whom) break; // got it
      whom = whom-zx;} //~while
    return whom;} //~TopBit

 /**
  * A simple way to get a date+time string.
  *
  * @param  noDate   True: omit date
  *
  * @return          The date+time string
  */
  public static String TimeStamp(boolean noDate)  {
    String why = IffyStr(noDate,"HH:mm:ss","yy MMM dd, HH:mm:ss");
    DateFormat sdf = new SimpleDateFormat(why);
    Date now = new Date();
    return sdf.format(now);} //~TimeStamp

 /**
  * A safe (no exceptions) way to compare two strings. (Jv/JS)
  * Works like compareTo but also works properly with null strings.
  *
  * @param  aStr  The first string to compare
  * @param  xStr  The other string to compare it to
  *
  * @return       -1/0/+1 as aStr<xStr or = or >
  */
  public static int SafeCompare(String aStr, String xStr)  {
    int whom = 0, az = 0, xz = 0;
    try {
    if (xStr != "") xz++;  if (aStr != "")  az++;
    if (az==0)  {if (xz>0) whom--;}
    else if (xz==0) whom++;
      else whom = aStr.compareTo(xStr);
    } catch (Exception ex)  {whom = 0;}
    return whom;} //~SafeCompare

 /**
  * theImpoList is saved from the list of widgets returned by your GameList
  * method. It is used in FindListWgt to search for widgets by descriptor.
  */
  public String theImpoList = "";

// Tool Panel Update Management (fake class)..           -- 2021 June 11
//  Also student game tests: RPS, SeaMan, TTT, Calc, etc.

  public static final int UpdtItNow = 0x0200, UpdtIfDrag = 0x00F0,
      Tab2name = 1, Tab2top = 2, Tab2left = 3, Tab2tall = 4, Tab2wide = 5,
      Tab2dn = 6, Tab2rt = 7, Tab2thk = 8, Tab2info = 9, Tab2invnt = 10,
      OmitIfTool = 0x0030; // (unused yet)
  public int ChgdBitSet = 0, TxBitSel = 0, // TBS>0: chgd, <0: sel'd but unchgd
    // CBS bits: b0=file, b1=color, b2=name, b3=flags, b4=V, b5=H, b6=T, b7=Wi
    // .. b8=AIDck, b9=[font], b10=D,b11=R, b12=I, b13=up/thik, b14=icty,
    // .. b15=txt -- (could merge 4+5, 6+7, 10+11 if need more)
    // ...(but be sure to fix `XxF*` field names to match '*' to ChgdBitSet bit+)
      KnownFonts = 0, CashTooInfo = 0, CashTooMo = 0, TooTmpInt = 0;
  public static boolean ToolActive = false, GamesAIdlg = false,
      StartupDone = GoodStart, RefreshPan = false;
  public GameWgt GameBoard = null, GameNameWgt = null, WgtNamEntry = null,
      TopField = null, LeftField = null, TallField = null, WideField = null,
      TypChkBx = null, ClikChkBx = null, DragChkBx = null, BumpChkBx = null,
      RollChkBx = null, NullWgt = null, AIdlgChkBx = null, UwgtNamList = null,
      ListofWgts = null, ActivWgt = null, VeloEntryG = null, FontRad = null,
      RcInfoFld = null, RPS_ToolWgt = null, RPS_Rad = null, RestartBtn = null,
      KitchenToolWgt = null, KitchenInv = null,
      Rev_Date = null, StatReport = null, CashTooWgt = null;

// GameTool base class specification (by StartupTool line +)..
    // 0:GameTool,   1:BoardTool, 2:RecTool,  3:DiagTool,  4:TextTool,
    // 5:CircleTool, 6:SkyRxTool, 7:BallTool, 8:RPS_Tool, 9:SpriteTool

  public int LastTool = 0, ActvGmTooInf = 0, xToolTyx = 0, xToolInfx = 0;
  public boolean TooTmpBoo = false; // (unused)
  public String TooTmpStr = "", xTooLinex = "", xToolInit = "";
  final public String StartupTools = "0x8000 M [Main Game asset list]\n"
      + "2048 R [Draggable Rect tool]\n2048 D [Diagonal line tool]\n"
      + "0x900 F [Rect Sprite tool]\n256 Q [Draggable RPS tool]\n"
      + "512 T [Draggable Text tool]\n1024 C [Draggable Circle tool]\n"
      + "0x500 B [Draggable Ball tool]\n256 S [Draggable Sprite tool]\n"
      + "256 K <bx> [Draggable Kitchen tool]";
  public GameTool ToolList = null;
  public GameTool ActivGmTool = null;
  public int[] ToolTypTb = new int[32];
  public GameTool[] WgTooTb = new GameTool[32];

 /**
  * Class GameTool, the base class for GameMaker tools.
  * These are indexed in ToolTypTb/WgTooTb and linked into a list in ToolList,
  * with their identification strings in separate lines of StartupTool.
  * Each tool has a unique tool ID starting with a different alphabet letter:
  *
  * Used: 'BsDW' 'CrDW' 'DgDW' 'FSDW' 'KtDW' 'MGDW' 'QxDW' 'RcDW' 'SpDW' 'TxDW'
  */
  private class GameTool { // . . . . . . . . . . . (GameTool)
    int ToolNum,   // sequentially assigned in order created
        ToolType,  // top 16 bits of 'Xxxx' tool/draggable ID, +ToolNum,
                   // .. +256 if sprite, +512 if text, +1K if circle, +2K rect
        ToolWgtNo, // =RefNum of draggable tool wgt (theTool)
        TpalPosn,  // tool palette position (from wgt list)
        ToolMacroBase, nToolMacros, ActiveMacro; // AcMac=ToolNum when active
    String ToolName; // shows in tool panel when selected
    GameWgt theTool; // reference to the draggable tool in the palette
    GameTool PreTool; // -> next link in ToolList

 /**
  * Return the class name for this object.
  * No override is necessary.
  *
  * @return     The reference String for this object
  */
  public String toString() { // . . . . . . . . . . . . . . . . . . (GameTool)
    // subclass: return super.toString() + " (particular info)";
    return "(GameTool #" + ToolWgtNo + "/" + ToolNum + " '" + ToolName
        + SeeHex("') ",ToolType," Mc=") + ToolMacroBase + "+"
        + nToolMacros;} //~toString

 /**
  * NewTool is called from GmMkr.StartUp for each new game tool.
  * Any override should call super.
  *
  * @param  whom    the tool name, for ToolName
  * @param  info    for ToolType
  * @param  base    for ToolMacroBase
  * @param  lxx     for nToolMacros
  * @param  theWgt  for theTool
  * @param  rx      for ToolWgtNo
  */
    public int NewTool(String whom, int info, int bx, GameWgt theWgt, int rx) {
      ToolName = whom; // called from GmMkr.StartUp->NewGmTool
      theTool = theWgt;
      if (theTool != null) rx = theTool.GetRefNum();
      ToolWgtNo = rx;
      ToolType = (info&-256)+ToolNum;
      // ToolTypTb[nx&31] = ToolType; // caller does it..
      // WgTooTb[nx&31] = this;
      ToolMacroBase = bx&255;
      nToolMacros = bx>>8;
      return ToolType;} //~NewTool

 /**
  * SetSubColor sets widget's first child's color, but only
  * if it's an icon or text. Called only from another tool.
  *
  * @param   colo    the color swatch color to be copied to the widget
  * @param   theWgt  the widget to copy the color to its child
  */
    public void SetSubColor(int colo, GameWgt theWgt) {
      GameWgt nuly = null;
      int tye = 0;
      if (theWgt != null) nuly = GetIxWgt(theWgt.GetIxData(0,0));
      if (nuly != null) if (colo+2>0) {
        tye = nuly.GetType()&255;
        if ((tye==2)||(tye==4)) nuly.SetColor(colo);}} //~SetSubColor

 /**
  * GetSubColor returns widget's first child's color if it's an icon
  * or text. Otherwise it returns -2. Called only from another tool.
  *
  * @param   theWgt  the widget to get its child's color from
  *
  * @return          the child's color, or -2
  */
    public int GetSubColor(GameWgt theWgt) {
      GameWgt nuly = null;
      int tye = 0;
      if (theWgt != null) nuly = GetIxWgt(theWgt.GetIxData(0,0));
      if (nuly != null) {
        tye = nuly.GetType()&255;
        if ((tye==2)||(tye==4)) return nuly.GetColor();} //~if
      return -2;} //~GetSubColor

 /**
  * SetWgtsDat sets widget's color and optional data from the tool panel.
  * SetTooledDat should call the correct subclass implementation,
  * but each subclass should recognize their own, or else do nothing.
  * This base class provides reasonable default color behavior.
  *
  * @param   info    ToolType previously returned by GetToolInfo
  * @param   colo    the color swatch color to be copied to the widget
  * @param   theWgt  the widget to copy the color and data to
  */
    public void SetWgtsDat(int info, int colo, GameWgt theWgt) { // (GameTool)
      if (theWgt != null) if ((info&0x0707FF00) !=0)
        if (colo+2>0) theWgt.SetColor(colo);} //~SetWgtsDat

 /**
  * SetWgtsSize sets the widget's size as defined by the tool.
  * This default sets the size of the draggable widget.
  *
  * @param   whom    the widget to set its size
  * @param   tall    the height to set it to
  * @param   wide    the width to set it to
  */
    public void SetWgtsSize(GameWgt whom, int tall, int wide) {  // (GameTool)
      if (whom==null) return;
      if ((whom.GetInfo()&0xFFFF) == 0x4457) // 'DW' only draggable widgets..
        whom.SetSize(tall,wide);} //~SetWgtsSize

 /**
  * SetChildSize sets the first child widget's size,
  * as a convenience for subclasses who need it.
  *
  * @param   whom    the widget to set its size
  * @param   tall    the height to set it to
  * @param   wide    the width to set it to
  */
    public void SetChildSize(GameWgt whom, int tall, int wide) { // (GameTool)
      GameWgt nuly;
      if (whom==null) return;
      nuly = GetIxWgt(whom.GetIxData(0,0));
      if (nuly != null) nuly.SetSize(tall,wide);} //~SetChildSize

 /**
  * GetWgtsSize returns the widget's size as defined by the tool.
  * This default gets the size from the draggable widget.
  *
  * @param   whom    the widget to get its size
  *
  * @return          its (packed) size
  */
    public int GetWgtsSize(GameWgt whom) {
      if (whom==null) return 0;
      return whom.GetSize();} //~GetWgtsSize

 /**
  * GetChildSize returns the first child widget's size.
  *
  * @param   whom    the widget to get its child's size
  *
  * @return          its (packed) size
  */
    public int GetChildSize(GameWgt whom) {
      GameWgt nuly;
      if (whom==null) return 0;
      nuly = GetIxWgt(whom.GetIxData(0,0));
      if (nuly==null) return 0;
      return nuly.GetSize();} //~GetChildSize

 /**
  * ShoWgtsDat displays the widget's optional data in the tool panel, and
  * returns the proper color for the color swatch. TooledShoDat should
  * call the correct subclass implementation, but each subclass should
  * recognize their own, or else do nothing and return -2.
  * This base class provides reasonable default color behavior.
  *
  * @param   info    ToolType previously returned by GetToolInfo
  * @param   whom    the widget to show its data
  * @param   FlgWg   its 'Flgz' child widget
  *
  * @return          the widget's color, or -2
  */
    public int ShoWgtsDat(int info, GameWgt whom, GameWgt FlgWg) { // (GmTool)
      if (whom != null) return whom.GetColor();
      return -2;} //~ShoWgtsDat

 /**
  * HidWgtsDat hides widget's optional data in the tool panel.
  * TooledShoDat should call the correct subclass implementation when
  * some other widget is selected, but each subclass should recognize
  * their own, or else do nothing.
  */
    public void HidWgtsDat() { // . . . . . . . . . . . . . . . . . (GameTool)
      // subclass: if (ItsMine) // hide this tool's optional data
      } //~HidWgtsDat

 /**
  * ClikTooWgt accepts a click in this tool's checkbox or radio button.
  * ToolClick should call the correct subclass implementation, but
  * each subclass should recognize their own, or else return false.
  *
  * @param   whom    the widget that was clicked
  * @param   info    the ToolInfo of the ActivWgt
  * @param   tuly    true if this is a tool widget, false if a game wgt
  *
  * @return          true if handled
  */
    public boolean ClikTooWgt(GameWgt whom, int info, boolean tuly) { // (GmT)
      return false;} //~ClikTooWgt

 /**
  * DropTooWgt decides if the widget being dragged can be dropped on this
  * container widget, and if doit=true, does the drop (if it can).
  *
  * @param   whom    the widget being dragged
  * @param   whar    the widget to drop it on
  * @param   tuly    true if whar is a tool widget, false if a game wgt
  * @param   doit    =0 when rollover, -1 to turn off prio ro ants, +1 to drop
  *
  * @return          true if can/did drop
  */
    public boolean DropTooWgt(GameWgt whom, int info, boolean tuly) { // (GmT)
      return false;} //~DropTooWgt

 /**
  * ImporTool returns the proper theTool widget for a given import line.
  * TooledImport sequentially calls each subclass implementation, so
  * each subclass should recognize their own, or else return null.
  *
  * @param   tye    the type code parsed from word 2 of the line
  * @param   aLine  the import line
  *
  * @return          ToolType, +0x8000 if it's a draggable widget, not tool
  */
    public GameWgt ImporTool(int tye, String aLine) { // . .  . . . (GameTool)
      // subclass: if (ItsMine) return theTool;
      return null;} //~ImporTool

 /**
  * SetWgtsInfo stores any additional information (including size) from
  * the import line into the new widget, as appropriate for this widget's
  * tool type. The widget color may be set here, or else return true to
  * let the caller set it. The base class (default) sets only the size.
  *
  * @param   whom    the new widget
  * @param   info    the info parameter from the import line
  * @param   size    the new widget size (tall,wide) from the import line
  * @param   colo    the new widget color from the import line
  * @param   aLine   the import line, in case something else is needed
  * @param   data    the full imported text, in case needed
  * @param   FlgWg   the 'Flgz' child widget, in case needed
  *
  * @return          false if color has been set, true for caller to set it
  */
    public boolean SetWgtsInfo(GameWgt whom, int info, int size, // (GameTool)
        int colo, String aLine, String data, GameWgt FlgWg) {
      // subclass: if (ItsMine) do whatever
      SetWgtsSize(whom,size>>16,size&0xFFFF);
      return true;} //~SetWgtsInfo

 /**
  * ExporTool constructs the specific text for an exported widget line.
  * TooledExport should call the correct subclass implementation,
  * each subclass should process their own, or do nothing (return 0).
  *
  *   xToolTyx   should be the exported widget type
  *   xToolInfx  should be the exported widget's Ginfo, preset to whom's
  *   xTooLinex  should be "the rest of the line"
  *   xToolInit  may have a variable name; if it needs initialization, it
  *                should be replaced with Java code starting with "\n    "
  *
  * @param   info  the ToolType previously returned by GetToolInfo
  * @param   whom  the widget to export
  * @param   tye   its type
  *
  * @return         =1 to use color from child widget, otherwise whom's
  *                 =2 to use velocity from 'Flgz' widget as Ginfo
  *                 4+ expect n/4 child widgets, otherwise =info
  */
    public int ExporTool(int info, GameWgt whom, int tye) { //  . . (GameTool)
      // subclass: if (ItsMine) return info; // set up export in xTooLinex...
      if (Logging) if (GoodImex||GoodTool) TooTmpStr = " (GameTool)";
      return 0;} //~ExporTool

 /**
  * ToolMacro is called for widget macros to support game tools.
  * See the JavaGame documentation for a complete explanation of macros.
  * [untested, docs still not there]
  *
  * @param  why   control number
  * @param  mID   macro ID code from source line
  * @param  whom  reference first to the parent group widget,
  *               then to each successor widget as created
  *
  * @return       the number of widgets to allocate, then info about each wgt
  */
    public int ToolMacro(int why, int mID, GameWgt whom) {
      // called for tool macros, whom is init'ly container, then new wgt;
      // mID is `xxxx` from data line (use w/why<0 to distinguish), or else =0
      // why<0: rtn number n of subwgts to allocate (must be 0<n<256)+x*256
      //   where 0<=x<256 is used in subsequent calls to distinguish;
      // for each wgt k=1..n, why=(x*256+k)*8+i, i=0 rtn type, >0 rtn param i;
      //   i=7 after wgt is created, so you can modify it as needed
      // subclass: if (PreTool != null) return PreTool.ToolMacro(...);
      return 0;} //~ToolMacro

 /**
  * A local utility for getting the 'Flgz' child widget.
  */
    public GameWgt FnFlgWgt(GameWgt wg) {
      int nx = 0x466C677A;
      if (wg==null) return null;
      return FindSubWgt(wg,nx,0);}

 /**
  * The GameTool and base constructor.
  */
    public GameTool() { //. . . . . . . . . . . . . . . . . . . . . (GameTool)
      LastTool++;
      ToolNum = LastTool;
      PreTool = ToolList;
      ToolList = this;
      ToolMacroBase = 0;
      nToolMacros = 0;
      ActiveMacro = 0;
      ToolWgtNo = 0;
      theTool = null;
      if (Logging) if (StartupDone)
        System.out.println("new GameTool #" + ToolNum);}} //~GameTool

  private class BoardTool extends GameTool { //    (BoardTool)

 /**
  * NewTool is called from GmMkr.StartUp for each new game tool.
  * This override initializes my panel widget(s).
  */
    public int NewTool(String whom, int info, int bx, GameWgt theWgt, int rx) {
      rx = super.NewTool(whom,info,bx,theWgt,rx);
      AIdlgChkBx = FindListWgt("[Alrt+InDlg checkbox]");
      if (AIdlgChkBx != null) AIdlgChkBx.HideSho(false);
      RestartBtn = FindListWgt("[Restart button]");
      if (RestartBtn != null) RestartBtn.HideSho(false);
      return rx;} //~NewTool

 /**
  * ShoWgtsDat displays the widget's optional data in the tool panel, and
  * returns the proper color for the color swatch. TooledShoDat should
  * call the correct subclass implementation, but each subclass should
  * recognize their own, or else do nothing and return -2.
  *
  * WuzChanged=true when the user clicks OK in GameMaker, basically
  * anything other than a clean board.
  *
  * NameChanged=true when the user changed the name of the game,
  * or a changed name came in GameMaker.txt.
  *
  * @param   info    ToolType previously returned by GetToolInfo
  * @param   whom    the widget to show its data
  * @param   FlgWg   its 'Flgz' child widget
  *
  * @return          the widget's color, or -2
  */
    public int ShoWgtsDat(int info, GameWgt whom, GameWgt FlgWg) { // (BdTool)
      boolean aBit;
      if (AIdlgChkBx != null) {
        aBit = (AIdlgChkBx.CheckIt(false) != GamesAIdlg);
        aBit = AIdlgChkBx.CheckIt(aBit);
        AIdlgChkBx.HideSho(true);} //~if
      aBit = (WuzChanged || NameChanged || unSavedWgts);
      if (Logging) if (GoodNew) System.out.println("  (ShoWgD/GmBd) = " + aBit
          + " = " + WuzChanged + "/" + NameChanged + "/" + unSavedWgts);
      if (RestartBtn != null) RestartBtn.HideSho(aBit);
      if (whom != null) return whom.GetColor();
      return -2;} //~ShoWgtsDat

 /**
  * HidWgtsDat hides widget's optional data in the tool panel.
  * TooledShoDat should call the correct subclass implementation when
  * some other widget is selected, but each subclass should recognize
  * their own, or else do nothing.
  */
    public void HidWgtsDat() { //  . . . . . . . . . . . . . . . . (BoardTool)
      if (RestartBtn != null) RestartBtn.HideSho(false);
      if (AIdlgChkBx != null) AIdlgChkBx.HideSho(false);} //~HidWgtsDat

 /**
  * ClikTooWgt accepts a click in this tool's checkbox or radio button.
  * ToolClick should call the correct subclass implementation, but
  * each subclass should recognize their own, or else return false.
  *
  * @param   whom    the widget that was clicked
  * @param   info    the ToolInfo of the ActivWgt
  * @param   tuly    true if this is a tool widget, false if a game wgt
  *
  * @return          true if handled
  */
    public boolean ClikTooWgt(GameWgt whom, int info, boolean tuly) { // (BdT)
      if (whom != AIdlgChkBx) return false;
      if (whom==null) return false;
      GamesAIdlg = whom.CheckIt(true); // = new value
      ChgdBitSet = ChgdBitSet|256; // b8=AIDck
      return true;} //~ClikTooWgt

 /**
  * The BoardTool constructor.
  */
    public BoardTool() { //. . . . . . . . . . . . . . . . . . . . (BoardTool)
      if (Logging) if (GoodStart)
        System.out.println("new BoardTool #" + ToolNum);}} //~BoardTool

  private class RecTool extends GameTool { //        (RecTool)

 /**
  * NewTool is called from GmMkr.StartUp for each new game tool.
  * This override initializes my panel widget(s).
  */
    public int NewTool(String whom, int info, int bx, GameWgt theWgt, int rx) {
      rx = super.NewTool(whom,info,bx,theWgt,rx);
      RcInfoFld = FindListWgt("[Rect Ginfo entry field]");
      if (RcInfoFld != null) {
        if (Tab2info>0) Tab2fields[Tab2info] = RcInfoFld.GetIxData(0,0);
        RcInfoFld.HideSho(false);} //~if
      return rx;} //~NewTool

 /**
  * ImporTool returns the proper theTool widget for a given import line.
  * TooledImport sequentially calls each subclass implementation, so
  * each subclass should recognize their own, or else return null.
  *
  * @param   tye    the type code parsed from word 2 of the line
  * @param   aLine  the import line
  *
  * @return          ToolType, +0x8000 if it's a draggable widget, not tool
  */
    public GameWgt ImporTool(int tye, String aLine) { // . . . . . . (RecTool)
      if (tye==0) if (NthOfset(0,"(D)",aLine)<0) return theTool;
      return null;} //~ImporTool

 /**
  * SetWgtsDat sets widget's color and optional data from the tool panel.
  * SetTooledDat should call the correct subclass implementation,
  * but each subclass should recognize their own, or else do nothing.
  *
  * @param   info    ToolType previously returned by GetToolInfo
  * @param   colo    the color swatch color to be copied to the widget
  * @param   theWgt  the widget to copy the color and data to
  */
    public void SetWgtsDat(int info, int colo, GameWgt theWgt) {  // (RecTool)
      GameWgt FlgWg = FnFlgWgt(theWgt); // 'Flgz'
      char xCh;
      boolean lft = false;
      String msg = "";
      if (theWgt==null) return;
      info = 0;
      if (colo+2>0) // vfy it's not Diagonal line..
          if ((theWgt.GetInfo()&0xFFE0FFFC) != 0x44604454) {
        if (RcInfoFld != null)
          msg = GetInputTxt(RcInfoFld);
        if (FlgWg != null) if (StrLength(msg)>0) {
          xCh = CharAt(0,msg);
          if (xCh != '\'') {
            msg = NthItemOf(' ',1,msg);
            xCh = CharAt(0,msg);} //~if
          if (xCh == '\'') {
            msg = NthItemOf('\'',2,msg);
            if (StrLength(msg)>1) lft = true;
            info = GameWgt. Pack42i(msg,lft,lft);
            xCh = ' ';} //~if
          else if (xCh >= '-') if (xCh <= '9')
              if ((xCh == '-') || (xCh >= '0')) {
            info = SafeParseInt(msg);
            xCh = ' ';} //~if
          if (xCh > ' ') info = GameWgt. Pack42i(msg,true,false);
          if ((info&-0x01000000)==0x44000000) // disallow 'D***'..
            info = info|0x20000000;
          FlgWg.SetPosn(info>>16,SignExtend(info)); // set info
          if ((info&0x8000)==0) ChgdBitSet = ChgdBitSet&~0x10000000; // b12=I
            else ChgdBitSet = ChgdBitSet&~0x1000;} //~if // (TxBitSel turns it on)
        theWgt.SetColor(colo);}} //~SetWgtsDat

 /**
  * ShoWgtsDat displays the widget's optional data in the tool panel, and
  * returns the proper color for the color swatch. TooledShoDat should
  * call the correct subclass implementation, but each subclass should
  * recognize their own, or else do nothing and return -2.
  *
  * @param   info    ToolType previously returned by GetToolInfo
  * @param   whom    the widget to show its data
  * @param   FlgWg   its 'Flgz' child widget
  *
  * @return          the widget's color, or -2
  */
    public int ShoWgtsDat(int info, GameWgt whom, GameWgt FlgWg) { // (RecToo)
      String msg = "";
      if (whom==null) return -2;
      while (true) {
        if ((info&0xFFE0FFFC) == 0x44604454) break; // don't want diag
        if (FlgWg==null) FlgWg = FnFlgWgt(whom); // 'Flgz'
        if (FlgWg==null) break;
        if (RcInfoFld==null) break;
        info = FlgWg.GetPosn();
        msg = HexIfMore("",info,"");
        if (info==0) msg = " ";
        else if ((info&0x80808080)==0) if (((info+0x01010101)&0x80808080)==0)
          if ((((info&info&0x60606060)+0xE0E0E0E0)&0x80808080)==0)
            msg = "'" + GameWgt. UnPak4i(info,true) + "'";
        if (SetInputText
            (RcInfoFld,"I=",msg,true)) RcInfoFld.HideSho(true);
        break;} //~while
      return whom.GetColor();} //~ShoWgtsDat

 /**
  * HidWgtsDat hides widget's optional data in the tool panel.
  * TooledShoDat should call the correct subclass implementation when
  * some other widget is selected, but each subclass should recognize
  * their own, or else do nothing.
  */
    public void HidWgtsDat() { //  . . . . . . . . . . . . . . . . . (RecTool)
      if (RcInfoFld != null) RcInfoFld.HideSho(false);} //~HidWgtsDat

 /**
  * SetWgtsInfo stores any additional information (including size) from
  * the import line into the new widget, as appropriate for this widget's
  * tool type. The widget color may be set here, or else return true to
  * let the caller set it. The base class (default) sets only the size.
  *
  * @param   whom    the new widget
  * @param   info    the info parameter from the import line
  * @param   size    the new widget size (tall,wide) from the import line
  * @param   colo    the new widget color from the import line
  * @param   aLine   the import line, in case something else is needed
  * @param   data    the full imported text, in case needed
  * @param   FlgWg   the 'Flgz' child widget, in case needed
  *
  * @return          false if color has been set, true for caller to set it
  */
    public boolean SetWgtsInfo(GameWgt whom, int info, int size,  // (RecTool)
        int colo, String aLine, String data, GameWgt FlgWg) {
      if (FlgWg != null) FlgWg.SetPosn(info>>16,SignExtend(info)); // set info
      return super.SetWgtsInfo(whom,info,size,colo,
          aLine,data,FlgWg);} //~SetWgtsInfo

 /**
  * ExporTool constructs the specific text for an exported widget line.
  * TooledExport should call the correct subclass implementation,
  * each subclass should process their own, or do nothing (return 0).
  *
  *   xToolTyx   should be the exported widget type
  *   xToolInfx  should be the exported widget's Ginfo, preset to whom's
  *   xTooLinex  should be "the rest of the line"
  *   xToolInit  may have a variable name; if it needs initialization, it
  *                should be replaced with Java code starting with "\n    "
  *
  * @param   info  the ToolType previously returned by GetToolInfo
  * @param   whom  the widget to export
  * @param   tye   its type
  *
  * @return         <0 use as exported info
  *                 =2 to use velocity from 'Flgz' widget as info
  *                 =1 to use info from child widget, otherwise from drug
  *                 =3 to get color from child widget, otherwise from drug
  */
    public int ExporTool(int info, GameWgt whom, int tye) { // . . . (RecTool)
      GameWgt FlgWg = FnFlgWgt(whom); // 'Flgz'
      if ((info&0xD00)==0x800) { // set up export in xTooLinex...
        if (Logging) if (GoodImex||GoodTool) TooTmpStr = " (RecTool)";
        xToolTyx = xToolTyx&-256;
        info = 0;
        if (FlgWg != null) info = FlgWg.GetPosn();
        xTooLinex = "";
        if ((info&0x80808080)==0) if (((info+0x01010101)&0x80808080)==0)
            if ((((info&info&0x60606060)+0xE0E0E0E0)&0x80808080)==0) {
          xTooLinex = " `" + GameWgt. UnPak4i(info,true) + "`";
          info = 0;} //~if
        xToolInfx = info;
        xToolInit = "";} //~if // can't have other inside rect
      return 0;} //~ExporTool

 /**
  * The RecTool constructor.
  */
    public RecTool() { //. . . . . . . . . . . . . . . . . . . . . . (RecTool)
      if (Logging) if (GoodStart)
        System.out.println("new RecTool #" + ToolNum);}} //~RecTool

  private class CircleTool extends GameTool { //  (CircleTool)

 /**
  * SetWgtsDat sets widget's color and optional data from the tool panel.
  * SetTooledDat should call the correct subclass implementation,
  * but each subclass should recognize their own, or else do nothing.
  *
  * @param   info    ToolType previously returned by GetToolInfo
  * @param   colo    the color swatch color to be copied to the widget
  * @param   theWgt  the widget to copy the color and data to
  */
    public void SetWgtsDat(int info, int colo, GameWgt theWgt) { // (CircTool)
      if (colo+2>0) SetSubColor(colo,theWgt);} //~SetWgtsDat

 /**
  * SetWgtsSize sets the widget's size as defined by the tool.
  * Circles must reshape the contained icon.
  *
  * @param   whom    the widget to set its size
  * @param   tall    the height to set it to
  * @param   wide    the width to set it to (ignored for circles)
  */
    public void SetWgtsSize(GameWgt whom, int tall, int wide) { //  (CircTool)
      if (whom==null) return;
      if (tall>32) tall = 32;
      if (tall <= 0) tall = 1; // CircSz calls Diam2wID..
      CircleSize(whom,tall);
      SetChildSize(whom,tall,tall);
      super.SetWgtsSize(whom,tall,tall);} //~SetWgtsSize

 /**
  * GetWgtsSize returns the widget's size as defined by the tool.
  * Circles get the size from the 2nd letter of the widget's ID.
  *
  * @param   whom    the widget to get its size
  */
    public int GetWgtsSize(GameWgt whom) {                     // (CircleTool)
      int info = 0;
      if (whom==null) return 0;
      info = GetChildSize(whom); // info = ID2diam(whom.GetInfo()); (info<<16)+
      return info;} //~GetWgtsSize

 /**
  * ShoWgtsDat displays the widget's optional data in the tool panel, and
  * returns the proper color for the color swatch. TooledShoDat should
  * call the correct subclass implementation, but each subclass should
  * recognize their own, or else do nothing and return -2.
  *
  * @param   info    ToolType previously returned by GetToolInfo
  * @param   whom    the widget to show its data
  * @param   FlgWg   its 'Flgz' child widget
  *
  * @return          the widget's color, or -2
  */
    public int ShoWgtsDat(int info, GameWgt whom, GameWgt FlgWg) { // (CircTo)
      return GetSubColor(whom);} //~ShoWgtsDat

 /**
  * ImporTool returns the proper theTool widget for a given import line.
  * TooledImport sequentially calls each subclass implementation, so
  * each subclass should recognize their own, or else return null.
  *
  * @param   tye    the type code parsed from word 2 of the line
  * @param   aLine  the import line
  *
  * @return          ToolType, +0x8000 if it's a draggable widget, not tool
  */
    public GameWgt ImporTool(int tye, String aLine) { //  . . . . (CircleTool)
      if (tye==2) return theTool;
      return null;} //~ImporTool

 /**
  * SetWgtsInfo stores any additional information (including size) from
  * the import line into the new widget, as appropriate for this widget's
  * tool type. The widget color may be set here, or else return true to
  * let the caller set it. The base class (default) sets only the size.
  *
  * @param   whom    the new widget
  * @param   info    the info parameter from the import line
  * @param   size    the new widget size (tall,wide) from the import line
  * @param   colo    the new widget color from the import line
  * @param   aLine   the import line, in case something else is needed
  * @param   data    the full imported text, in case needed
  * @param   FlgWg   the 'Flgz' child widget, in case needed
  *
  * @return          false if color has been set, true for caller to set it
  */
    public boolean SetWgtsInfo(GameWgt whom, int info, int size, // (CircTool)
        int colo, String aLine, String data, GameWgt FlgWg) {
      SetWgtsSize(whom,size>>16,size&0xFFFF);
      SetSubColor(colo,whom);
      return false;} //~SetWgtsInfo

 /**
  * ExporTool constructs the specific text for an exported widget line.
  * TooledExport should call the correct subclass implementation,
  * each subclass should process their own, or do nothing (return 0).
  *
  *   xToolTyx   should be the exported widget type
  *   xToolInfx  should be the exported widget's Ginfo, preset to whom's
  *   xTooLinex  should be "the rest of the line"
  *   xToolInit  may have a variable name; if it needs initialization, it
  *                should be replaced with Java code starting with "\n    "
  *
  * @param   info  the ToolType previously returned by GetToolInfo
  * @param   whom  the widget to export
  * @param   tye   its type
  *
  * @return         <0 use as exported info
  *                 =2 to use velocity from 'Flgz' widget as info
  *                 =1 to use info from child widget, otherwise from drug
  *                 =3 to get color from child widget, otherwise from drug
  */
    public int ExporTool(int info, GameWgt whom, int tye) { //  . (CircleTool)
      if ((info&0xD00)==0x400) { // set up export in xTooLinex...
        if (Logging) if (GoodImex||GoodTool) TooTmpStr = " (CircleTool)";
        xToolTyx = (xToolTyx&-256)+2;
        xToolInfx = -3;
        xTooLinex = " (C)";
        if (StrLength(xToolInit)>0) xToolInit = "\n    if (" + xToolInit
            + " != null) myGame.CircleSize(" + xToolInit + ",0);";
        return 3;} //~if // can't have other inside circle
      return 0;} //~ExporTool

 /**
  * The CircleTool constructor.
  */
    public CircleTool() { //. . . . . . . . . . . . . . . . . . . (CircleTool)
      if (Logging) if (GoodStart)
        System.out.println("new CircleTool #" + ToolNum);}} //~CircleTool

  private class TextTool extends GameTool { //      (TextTool)

 /**
  * NewTool is called from GmMkr.StartUp for each new game tool.
  * This override initializes my panel widget(s).
  */
    public int NewTool(String whom, int info, int bx, GameWgt theWgt, int rx) {
      rx = super.NewTool(whom,info,bx,theWgt,rx);
      if (FontRad==null) // should be done in StartUp..
        FontRad = FindListWgt("[Font selector radio group]");
      if (FontRad != null) FontRad.HideSho(false);
      return rx;} //~NewTool

 /**
  * SetWgtsDat sets widget's color and optional data from the tool panel.
  * SetTooledDat should call the correct subclass implementation,
  * but each subclass should recognize their own, or else do nothing.
  *
  * @param   info    ToolType previously returned by GetToolInfo
  * @param   colo    the color swatch color to be copied to the widget
  * @param   theWgt  the widget to copy the color and data to
  */
    public void SetWgtsDat(int info, int colo, GameWgt theWgt) { // (TextTool)
      int zx;
      if (theWgt==null) return;
      if (FontRad != null) { // gets radio's font, sets wgt's font to it..
        zx = ChoseFont(FontRad,-1);
        zx = zx&255;
        zx = ChoseFont(theWgt,zx);
        if ((info&0x8000) !=0) ChgdBitSet = ChgdBitSet|1;} //~if // b0=file
      if (colo+2>0) SetSubColor(colo,theWgt);} //~SetWgtsDat

 /**
  * SetWgtsSize sets the widget's size as defined by the tool.
  * Text resizes both the draggable widget and its contained text field.
  *
  * @param   whom    the widget to set its size
  * @param   tall    the height to set it to
  * @param   wide    the width to set it to
  */
    public void SetWgtsSize(GameWgt whom, int tall, int wide) { //  (TextTool)
      if (whom==null) return;
      SetChildSize(whom,tall,wide);
      super.SetWgtsSize(whom,tall,wide);} //~SetWgtsSize

 /**
  * GetWgtsSize returns the widget's size as defined by the tool.
  * Text gets the size from the first child (text) widget.
  *
  * @param   whom    the widget to get its size
  */
    public int GetWgtsSize(GameWgt whom) {
      return GetChildSize(whom);} //~GetWgtsSize

 /**
  * ShoWgtsDat displays the widget's optional data in the tool panel, and
  * returns the proper color for the color swatch. TooledShoDat should
  * call the correct subclass implementation, but each subclass should
  * recognize their own, or else do nothing and return -2.
  *
  * @param   info    ToolType previously returned by GetToolInfo
  * @param   whom    the widget to show its data
  * @param   FlgWg   its 'Flgz' child widget
  *
  * @return          the widget's color, or -2
  */
    public int ShoWgtsDat(int info, GameWgt whom, GameWgt FlgWg) { // (TxTool)
      if (whom==null) return -2; // gets the wgt's font
      info = ChoseFont(whom,-1);
      if (FontRad != null) { // sets the radio to the wgt's font
        info = ChoseFont(FontRad,info);
        FontRad.HideSho(true);} //~if
      return GetSubColor(whom);} //~ShoWgtsDat

 /**
  * HidWgtsDat hides widget's optional data in the tool panel.
  * TooledShoDat should call the correct subclass implementation when
  * some other widget is selected, but each subclass should recognize
  * their own, or else do nothing.
  */
    public void HidWgtsDat() { // . . . . . . . . . . . . . . . . . (TextTool)
      if (FontRad != null) FontRad.HideSho(false);} //~HidWgtsDat

 /**
  * ClikTooWgt accepts a click in this tool's checkbox or radio button.
  * ToolClick should call the correct subclass implementation, but
  * each subclass should recognize their own, or else return false.
  *
  * @param   whom    the widget that was clicked
  * @param   info    the ToolInfo of the ActivWgt
  * @param   tuly    true if this is a tool widget, false if a game wgt
  *
  * @return          true if handled
  */
    public boolean ClikTooWgt(GameWgt whom, int info, boolean tuly) { // (TxT)
      int zx;
      GameWgt nuly = ActivWgt;
      if (whom==null) return false;
      if ((info&512)==0) return false;
      if (FontRad==null) return false; // otherwise it's a text wgt..
      info = whom.GetInfo(); // 'Rd_*' is a radio button..
      zx = info&31;
      if (info>0x52645F40) if (info <= 0x52645F6A) // 'Rd_A'..'Rd_Z'
          if (((KnownFonts>>zx)&1) !=0) { // it's a known font..
        zx = zx|64;
        info = ChoseFont(FontRad,zx);
        if (nuly != null) {
          info = ChoseFont(nuly,zx);
          if (nuly == theTool) if ((zx==72)||(zx==83)) if (nuly.GetPosn() !=0) {
            nuly.SetPosn(0,0);
            nuly.SetTall(32);}} //~if
        if (!tuly) ChgdBitSet = ChgdBitSet|1;} //~if // b0=file
        // SetTooledDat(-2,ActivWgt); // unneeded, already set zbove
      return zx>64;} //~ClikTooWgt

 /**
  * ImporTool returns the proper theTool widget for a given import line.
  * TooledImport sequentially calls each subclass implementation, so
  * each subclass should recognize their own, or else return null.
  *
  * @param   tye    the type code parsed from word 2 of the line
  * @param   aLine  the import line
  *
  * @return          ToolType, +0x8000 if it's a draggable widget, not tool
  */
    public GameWgt ImporTool(int tye, String aLine) { //  . . . . . (TextTool)
      if (tye==4) return theTool;
      return null;} //~ImporTool

 /**
  * SetWgtsInfo stores any additional information (including size) from
  * the import line into the new widget, as appropriate for this widget's
  * tool type. The widget color may be set here, or else return true to
  * let the caller set it. The base class (default) sets only the size.
  *
  * @param   whom    the new widget
  * @param   info    the info parameter from the import line
  * @param   size    the new widget size (tall,wide) from the import line
  * @param   colo    the new widget color from the import line
  * @param   aLine   the import line, in case something else is needed
  * @param   data    the full imported text, in case needed
  * @param   FlgWg   the 'Flgz' child widget, in case needed
  *
  * @return          false if color has been set, true for caller to set it
  */
    public boolean SetWgtsInfo(GameWgt whom, int info, int size, // (TextTool)
        int colo, String aLine, String data, GameWgt FlgWg) {
      GameWgt nuly = null;
      if (whom != null) nuly = GetIxWgt(whom.GetIxData(0,0));
      if (info !=0) if (nuly != null) if ((nuly.GetType()&255)==4) {
        if (info>0) {
          data = NthItemOf('\n',info,data); // get/set font..
          if (SafeParseInt(NthItemOf(' ',2,data))==3) {
            info = NthOfset(0,"[Font ",data);   // prefer name letter..
            if (info>0) info = (int)(CharAt(info+6,data));
            if (info>90) info = 0;
            else if (info<65) info = 0;
            else if (((KnownFonts>>(info&31))&1)==0) info = 0;
            if (info==0) {
              info = SafeParseInt(NthItemOf(' ',8,data)); // MOS font+
              if (info>7) info = 0;}}} //~if
        else if (info+90<0) info = 0;
        else if (info+65>0) info = 0;
          else info = -info;
        if (info>0)
          info = ChoseFont(nuly,info);
        aLine = ReplacAll("'","`",NthItemOf('\'',2,aLine)); // gets text
        info = nuly.PutTextLn(aLine);} //~if
      ChgdBitSet = ChgdBitSet|0x8200; // b9=[font], b15=txt
      SetWgtsSize(whom,size>>16,size&0xFFFF);
      SetSubColor(colo,whom);
      return false;} //~SetWgtsInfo

 /**
  * ExporTool constructs the specific text for an exported widget line.
  * TooledExport should call the correct subclass implementation,
  * each subclass should process their own, or do nothing (return 0).
  *
  *   xToolTyx   should be the exported widget type
  *   xToolInfx  should be the exported widget's Ginfo, preset to whom's
  *   xTooLinex  should be "the rest of the line"
  *   xToolInit  may have a variable name; if it needs initialization, it
  *                should be replaced with Java code starting with "\n    "
  *
  * @param   info  the ToolType previously returned by GetToolInfo
  * @param   whom  the widget to export
  * @param   tye   its type
  *
  * @return         <0 use as exported info
  *                 =2 to use velocity from 'Flgz' widget as info
  *                 =1 to use info from child widget, otherwise from drug
  *                 =3 to get color from child widget, otherwise from drug
  */
    public int ExporTool(int info, GameWgt whom, int tye) { //  . . (TextTool)
      GameWgt nuly = null;
      if ((info&512) !=0) { // set up export in xTooLinex...
        if (whom != null) nuly = GetIxWgt(whom.GetIxData(0,0));
        if (nuly != null) {
          xTooLinex = " '" + ReplacAll("`","'",nuly.GetTextLn()) + "'";
          info = nuly.GetInfo(); // convert wgt+ to font name..
          info = ChoseFont(null,-info);
          xToolInfx = -info;} //~if
        if (Logging) if (GoodImex||GoodTool) TooTmpStr = " (TextTool)";
        xToolTyx = (xToolTyx&-256)+4;
        return 3;} //~if // can't have other inside text
      return 0;} //~ExporTool

 /**
  * The TextTool constructor.
  */
    public TextTool() { //. . . . . . . . . . . . . . . . . . . . . (TextTool)
      if (Logging) if (GoodStart)
        System.out.println("new TextTool #" + ToolNum);}} //~TextTool

  private class SkyRxTool extends GameTool { //    (SkyRxTool)

 /**
  * NewTool is called from GmMkr.StartUp for each new game tool.
  * This override initializes my panel widget(s).
  */
    public int NewTool(String whom, int info, int bx, GameWgt theWgt, int rx) {
      rx = super.NewTool(whom,info,bx,theWgt,rx);
      if (VeloEntryG==null)
        VeloEntryG = FindListWgt("[Velocity entry group]");
      if (VeloEntryG != null) {
        theWgt = FindListWgt("[Vertical velocity entry field]");
        if (theWgt != null) if (Tab2dn>0)
          Tab2fields[Tab2dn] = theWgt.GetIxData(0,0);
        theWgt = FindListWgt("[Horizontal velocity entry field]");
        if (theWgt != null) if (Tab2rt>0)
          Tab2fields[Tab2rt] = theWgt.GetIxData(0,0);
        VeloEntryG.HideSho(false);} //~if
      return rx;} //~NewTool

 /**
  * SetWgtsDat sets widget's color and optional data from the tool panel.
  * SetTooledDat should call the correct subclass implementation,
  * but each subclass should recognize their own, or else do nothing.
  *
  * @param   info    ToolType previously returned by GetToolInfo
  * @param   colo    the color swatch color to be copied to the widget
  * @param   theWgt  the widget to copy the color and data to
  */
    public void SetWgtsDat(int info, int colo, GameWgt theWgt) { // . (SkTool)
      GameWgt more = null, FlgWg = null;
      int zx = 0;
      boolean base = (info&2048) !=0;
     if (theWgt==null) return;
      if ((info&256)==0) return; // not a sprite
      FlgWg = FnFlgWgt(theWgt); // 'Flgz'
      if (FlgWg != null) if (VeloEntryG != null) { // get & display velo..
        info = 0;
        more = GetIxWgt(VeloEntryG.GetIxData(0,0)); // ..vert
        if (more != null)
          info = ParsFixPt( GetInputTxt(more));
        more = GetIxWgt(VeloEntryG.GetIxData(1,0)); // ..horz
        if (more != null)
          zx = ParsFixPt( GetInputTxt(more));
        FlgWg.SetPosn(info,zx);
        if ((info&0x8000) !=0) ChgdBitSet = (ChgdBitSet&~0xC00)|1;
          else ChgdBitSet = ChgdBitSet&~0xC000000;} //~if // b10=D, b11=R
      if (base) if (colo+2>0) theWgt.SetColor(colo);} //~SetWgtsDat

 /**
  * ShoWgtsDat displays the widget's optional data in the tool panel, and
  * returns the proper color for the color swatch. TooledShoDat should
  * call the correct subclass implementation, but each subclass should
  * recognize their own, or else do nothing and return -2.
  *
  * @param   info    ToolType previously returned by GetToolInfo
  * @param   whom    the widget to show its data
  * @param   FlgWg   its 'Flgz' child widget
  *
  * @return          the widget's color, or -2
  */
    public int ShoWgtsDat(int info, GameWgt whom, GameWgt FlgWg) { // (SkTool)
      GameWgt more = null;
      String msg = "";
      int rez = -2;
      while (true) {
        if (whom==null) break;
        if ((info&256)==0) break; // not a sprite
        if (FlgWg==null) FlgWg = FnFlgWgt(whom); // 'Flgz'
        if (FlgWg==null) break;
        if (VeloEntryG==null) break;
        info = FlgWg.GetPosn();
        msg = FormFixt("",info>>16);
        more = GetIxWgt(VeloEntryG.GetIxData(0,0));
        if (more != null) if (!SetInputText
            (more,"D=",msg,true)) break;
        msg = FormFixt("",SignExtend(info));
        more = GetIxWgt(VeloEntryG.GetIxData(1,0));
        if (more != null) if (!SetInputText
            (more,"R=",msg,true)) break;
        VeloEntryG.HideSho(true);
        rez = whom.GetColor();
        break;} //~while
      return rez;} //~ShoWgtsDat

 /**
  * HidWgtsDat hides widget's optional data in the tool panel.
  * TooledShoDat should call the correct subclass implementation when
  * some other widget is selected, but each subclass should recognize
  * their own, or else do nothing.
  */
    public void HidWgtsDat() { //  . . . . . . . . . . . . . . . . (SkyRxTool)
      if (VeloEntryG != null) VeloEntryG.HideSho(false);} //~HidWgtsDat

 /**
  * ImporTool returns the proper theTool widget for a given import line.
  * TooledImport sequentially calls each subclass implementation, so
  * each subclass should recognize their own, or else return null.
  *
  * @param   tye    the type code parsed from word 2 of the line
  * @param   aLine  the import line
  *
  * @return          ToolType, +0x8000 if it's a draggable widget, not tool
  */
    public GameWgt ImporTool(int tye, String aLine) { // . . . . . (SkyRxTool)
      if (tye==6) if (NthOfset(0,"(F)",aLine)>0) return theTool;
      return null;} //~ImporTool

 /**
  * SetWgtsInfo stores any additional information (including size)
  * from the import line into the new widget, as appropriate for this
  * widget tool type. Sprites also set the velocity from info.
  *
  * @param   whom    the new widget
  * @param   info    the info parameter from the import line
  * @param   size    the new widget size (tall,wide) from the import line
  * @param   colo    the new widget color from the import line
  * @param   aLine   the import line, in case something else is needed
  * @param   data    the full imported text, in case needed
  * @param   FlgWg   the 'Flgz' child widget, in case needed
  *
  * @return          false if color has been set, true for caller to set it
  */
    public boolean SetWgtsInfo(GameWgt whom, int info, int size,  // (SkyTool)
        int colo, String aLine, String data, GameWgt FlgWg) {
      if (FlgWg != null) FlgWg.SetPosn(info>>16,SignExtend(info)); // set velo
      ChgdBitSet = ChgdBitSet|0xC00; // b10=D, b11=R
      SetWgtsSize(whom,size>>16,size&0xFFFF);
      return true;} //~SetWgtsInfo

 /**
  * ExporTool constructs the specific text for an exported widget line.
  * TooledExport should call the correct subclass implementation,
  * each subclass should process their own, or do nothing (return 0).
  *
  *   xToolTyx   should be the exported widget type
  *   xToolInfx  should be the exported widget's Ginfo, preset to whom's
  *   xTooLinex  should be "the rest of the line"
  *   xToolInit  may have a variable name; if it needs initialization, it
  *                should be replaced with Java code starting with "\n    "
  *
  * @param   info  the ToolType previously returned by GetToolInfo
  * @param   whom  the widget to export
  * @param   tye   its type
  *
  * @return         <0 use as exported info
  *                 =2 to use velocity from 'Flgz' widget as info
  *                 =1 to use info from child widget, otherwise from drug
  *                 =3 to get color from child widget, otherwise from drug
  */
    public int ExporTool(int info, GameWgt whom, int tye) { // . . (SkyRxTool)
      if ((info&0x8D00)==0x8900) { // set up export in xTooLinex...
        if (Logging) if (GoodImex||GoodTool) TooTmpStr = " (SkyRxTool)";
        xToolTyx = (xToolTyx&-256)+6;
        xTooLinex = " (F)";
        return 2;} //~if // can't have other inside skyrect
      return 0;} //~ExporTool

 /**
  * The SkyRxTool and sprite constructor.
  */
    public SkyRxTool() { //. . . . . . . . . . . . . . . . . . . . (SkyRxTool)
      if (Logging) if (GoodStart)
        System.out.println("new SkyRxTool #" + ToolNum);}} //~SkyRxTool

  private class BallTool extends SkyRxTool { // . . (BallTool)

 /**
  * SetWgtsDat sets widget's color and optional data from the tool panel.
  * SetTooledDat should call the correct subclass implementation,
  * but each subclass should recognize their own, or else do nothing.
  *
  * @param   info    ToolType previously returned by GetToolInfo
  * @param   colo    the color swatch color to be copied to the widget
  * @param   theWgt  the widget to copy the color and data to
  */
    public void SetWgtsDat(int info, int colo, GameWgt theWgt) { // (BallTool)
      if (theWgt==null) return;
      super.SetWgtsDat(info,colo,theWgt);
      if (colo+2>0) SetSubColor(colo,theWgt);} //~SetWgtsDat

 /**
  * SetWgtsSize sets the widget's size as defined by the tool.
  * Balls must reshape the contained icon.
  *
  * @param   whom    the widget to set its size
  * @param   tall    the height to set it to
  * @param   wide    the width to set it to (ignored for circles)
  */
    public void SetWgtsSize(GameWgt whom, int tall, int wide) { //  (BallTool)
      if (whom==null) return;
      if (tall>32) tall = 32;
      if (tall <= 0) tall = 1; // CircSz calls Diam2wID..
      CircleSize(whom,tall);
      SetChildSize(whom,tall,tall);
      super.SetWgtsSize(whom,tall,tall);} //~SetWgtsSize

 /**
  * GetWgtsSize returns the widget's size as defined by the tool.
  * Balls get the size from the 2nd letter of the widget's ID.
  *
  * @param   whom    the widget to get its size
  */
    public int GetWgtsSize(GameWgt whom) {                       // (BallTool)
      int info = 0;
      if (whom==null) return 0;
      info = GetChildSize(whom); // info = ID2diam(whom.GetInfo()); (info<<16)+
      return info;} //~GetWgtsSize

 /**
  * ShoWgtsDat displays the widget's optional data in the tool panel, and
  * returns the proper color for the color swatch. TooledShoDat should
  * call the correct subclass implementation, but each subclass should
  * recognize their own, or else do nothing and return -2.
  *
  * @param   info    ToolType previously returned by GetToolInfo
  * @param   whom    the widget to show its data
  * @param   FlgWg   its 'Flgz' child widget
  *
  * @return          the widget's color, or -2
  */
    public int ShoWgtsDat(int info, GameWgt whom, GameWgt FlgWg) { // (BaTool)
      if (whom==null) return -1;
      info = super.ShoWgtsDat(info,whom,FlgWg);
      return GetSubColor(whom);} //~ShoWgtsDat

 /**
  * ImporTool returns the proper theTool widget for a given import line.
  * TooledImport sequentially calls each subclass implementation, so
  * each subclass should recognize their own, or else return null.
  *
  * @param   tye    the type code parsed from word 2 of the line
  * @param   aLine  the import line
  *
  * @return          ToolType, +0x8000 if it's a draggable widget, not tool
  */
    public GameWgt ImporTool(int tye, String aLine) { //  . . . . . (BallTool)
      if (tye==6) if (NthOfset(0,"(B)",aLine)>0) return theTool;
      return null;} //~ImporTool

 /**
  * SetWgtsInfo stores any additional information (including size)
  * from the import line into the new widget, as appropriate for this
  * widget tool type. Sprites also set the velocity from info.
  *
  * @param   whom    the new widget
  * @param   info    the info parameter from the import line
  * @param   size    the new widget size (tall,wide) from the import line
  * @param   colo    the new widget color from the import line
  * @param   aLine   the import line, in case something else is needed
  * @param   data    the full imported text, in case needed
  * @param   FlgWg   the 'Flgz' child widget, in case needed
  *
  * @return          false if color has been set, true for caller to set it
  */
    public boolean SetWgtsInfo(GameWgt whom, int info, int size, // (BallTool)
        int colo, String aLine, String data, GameWgt FlgWg) {
      int lino = SafeParseInt(NthItemOf(' ',11,aLine));
      boolean done = super.SetWgtsInfo(whom,info,size,-1,aLine,data,FlgWg);
      if (lino>0) { // get color of child..
        aLine = NthItemOf('\n',lino,data); // child's input line
        colo = SafeParseInt(NthItemOf(' ',7,aLine));
        SetSubColor(colo,whom);} //~if
      return false;} //~SetWgtsInfo

 /**
  * ExporTool constructs the specific text for an exported widget line.
  * TooledExport should call the correct subclass implementation,
  * each subclass should process their own, or do nothing (return 0).
  *
  *   xToolTyx   should be the exported widget type
  *   xToolInfx  should be the exported widget's Ginfo, preset to whom's
  *   xTooLinex  should be "the rest of the line"
  *   xToolInit  may have a variable name; if it needs initialization, it
  *                should be replaced with Java code starting with "\n    "
  *
  * @param   info  the ToolType previously returned by GetToolInfo
  * @param   whom  the widget to export
  * @param   tye   its type
  *
  * @return         <0 use as exported info
  *                 =2 to use velocity from 'Flgz' widget as info
  *                 =1 to use info from child widget, otherwise from drug
  *                 =3 to get color from child widget, otherwise from drug
  */
    public int ExporTool(int info, GameWgt whom, int tye) { //  . . (BallTool)
      if ((info&0x8D00)==0x8500) { // set up export in xTooLinex...
        if (Logging) if (GoodImex||GoodTool) TooTmpStr = " (BallTool)";
        xToolTyx = (xToolTyx&-256)+6;
        xToolInfx = -3;
        xTooLinex = " (B)";
        if (StrLength(xToolInit)>0) xToolInit = "\n    if (" + xToolInit
            + " != null) myGame.CircleSize(" + xToolInit + ",0);";
        return 6;} //~if // get one child, +velo
      else if (Logging) if (GoodImex||GoodTool) { // got the circle icon!?..
        TooTmpStr = " (BallTool) #" + SafeWgtNum(whom) + " ty=" + tye;
        xToolInfx = -3;} //~if
      return 0;} //~ExporTool

 /**
  * The BallTool (sprite) constructor.
  */
    public BallTool() { //  . . . . . . . . . . . . . . . . . . . . (BallTool)
      if (Logging) if (GoodStart)
        System.out.println("new BallTool #" + ToolNum);}} //~BallTool

  private class RPS_Tool extends SkyRxTool { // . . (RPS_Tool)

 /**
  * NewTool is called from GmMkr.StartUp for each new game tool.
  * This override initializes my panel widget(s).
  */
    public int NewTool(String whom, int info, int bx, GameWgt theWgt, int rx) {
      rx = super.NewTool(whom,info,bx,theWgt,rx);
      if (RPS_Rad==null) // should be done in StartUp (but not)..
        RPS_Rad = FindListWgt("[RPS selector radio group]");
      if (theWgt != null) if (RPS_Rad != null) {
        RPS_ToolWgt = theWgt;
        info = ChoseRPS(theWgt,6);
        info = ChoseRPS(RPS_Rad,7);
        if (Logging) if (GoodRip) bx = ChoseRPS(theWgt,-1);
        RPS_Rad.HideSho(false);} //~if
      return rx;} //~NewTool

 /**
  * SetWgtsDat sets widget's color and optional data from the tool panel.
  * SetTooledDat should call the correct subclass implementation,
  * but each subclass should recognize their own, or else do nothing.
  *
  * RPS color is fixed, but super does velocity.
  *
  * @param   info    ToolType previously returned by GetToolInfo
  * @param   colo    the color swatch color to be copied to the widget
  * @param   theWgt  the widget to copy the color and data to
  */
    public void SetWgtsDat(int info, int colo, GameWgt theWgt) { // (RPS_Tool)
      int nx, zx;
      if (theWgt==null) return;
      if (RPS_Rad != null) { // get radio, set wgt's icons to it..
        if (Logging) if (GoodRip) {
          System.out.println("  (RPS panel->Wgt #" + RPS_Rad.GetRefNum() + ")");} //~if
        nx = ChoseRPS(RPS_Rad,-2)&7;
        zx = ChoseRPS(theWgt,nx);
        if (nx==4) zx = ChoseRPS(theWgt,0x044); // show small cloud frame
        if (Logging) if (GoodRip) zx = ChoseRPS(theWgt,-4);
        if ((info&0x8000) !=0) ChgdBitSet = ChgdBitSet|1;} //~if // b0=file
      super.SetWgtsDat(info,-3,theWgt);} //~SetWgtsDat // never set color

 /**
  * SetWgtsSize sets the widget's size as defined by the tool.
  *
  * RPS size is fixed, cannot be set.
  *
  * @param   whom    the widget to set its size
  * @param   tall    the height to set it to
  * @param   wide    the width to set it to (ignored for circles)
  */
    public void SetWgtsSize(GameWgt whom, int tall, int wide) { //  (RPS_Tool)
      if (whom==null) return;} //~SetWgtsSize

 /**
  * ShoWgtsDat displays the widget's optional data in the tool panel, and
  * returns the proper color for the color swatch. TooledShoDat should
  * call the correct subclass implementation, but each subclass should
  * recognize their own, or else do nothing and return -2.
  *
  * @param   info    ToolType previously returned by GetToolInfo
  * @param   whom    the widget to show its data
  * @param   FlgWg   its 'Flgz' child widget
  *
  * @return          the widget's color, or -2
  */
    public int ShoWgtsDat(int info, GameWgt whom, GameWgt FlgWg) { // (RPSToo)
      int zx;
      if (whom==null) return -1;
      zx = ChoseRPS(whom,-3)&7; // (works like ChoseFont)
      if (RPS_Rad != null) { // sets the radio to the wgt's sel'd icon
        zx = ChoseRPS(RPS_Rad,zx);
        RPS_Rad.HideSho(true);} //~if
      zx = super.ShoWgtsDat(info,whom,FlgWg);
      return -2;} //~ShoWgtsDat

 /**
  * HidWgtsDat hides widget's optional data in the tool panel.
  * TooledShoDat should call the correct subclass implementation when
  * some other widget is selected, but each subclass should recognize
  * their own, or else do nothing.
  */
    public void HidWgtsDat() { // . . . . . . . . . . . . . . . . . (RPS_Tool)
      super.HidWgtsDat();
      if (RPS_Rad != null) RPS_Rad.HideSho(false);} //~HidWgtsDat

 /**
  * ClikTooWgt accepts a click in this tool's checkbox or radio button.
  * ToolClick should call the correct subclass implementation, but
  * each subclass should recognize their own, or else return false.
  *
  * @param   whom    the widget that was clicked
  * @param   info    the ToolInfo of the ActivWgt
  * @param   tuly    true if this is a tool widget, false if a game wgt
  *
  * @return          true if handled
  */
    public boolean ClikTooWgt(GameWgt whom, int info, boolean tuly) { // (RPS)
      GameWgt nuly = ActivWgt, more;
      int[] data = null;  int[] pix;  int[] tmp;
      int kx = 0, nx = 0, zx, colo;
      if (whom==null) return false;
      if ((info&256)==0) return false;
      if (RPS_Rad==null) return false; // otherwise it's a text wgt..
      info = whom.GetInfo(); // 'Rd_*' is a radio button..
      zx = info&7;
      if (nuly != null) nx = nuly.GetInfo();
      if (Logging) if (GoodRip) System.out.println("  (RPS clik rbtn #"
          + RPS_Rad.GetRefNum() + SeeHex(") x",nx,SeeHex(" => x",info,"")));
      if (info>0x52645F30) if (info <= 0x52645F34) { // 'Rd_1'..'Rd_4'
        info = ChoseRPS(RPS_Rad,zx|48);
        info = ChoseRPS(ActivWgt,zx); // show it immediately
        nx = (nx>>16)&7;                                         // 'Q'..
        if (nx != zx) if (nuly != null) if ((GetToolInfo(nuly)>>24) == 81) {
          nuly = GetIxWgt(nuly.GetIxData(0,0)); // = token sprite
          if (nuly != null) for (nx=0; nx <= 99; nx++) {
            more = GetIxWgt(nuly.GetIxData(nx,0));
            if (more==null) break;
            info = more.GetInfo();
            colo = more.GetColor();
            if (colo >= 0) {
              if (colo==0) {if (zx != 3) continue;} //~if
              else if (colo==0xFFFFFF) {if (zx != 1) continue;} //~if
              else if (colo==0x666666) {if (zx != 2) continue;} //~if
                else if (zx != 4) continue;
              data = more.GetData();
              kx = info&0x3FFF; // its image offset (frex, cloud)
              continue;} //~if // (colo >= 0)
            if (info == 0x416E7449) { // 'AntI'
              nuly = GetIxWgt(more.GetIxData(0,0)); // the outline
              more = GetIxWgt(more.GetIxData(1,0)); // the ants
              if (nuly != null) if (more != null) if (data != null) { // gottem
                pix = more.GetData();
                if (pix != null) if (data != null) {
                  IconPixCopy(0,kx,0,0,32,33,data,pix); // copies data->pix
                  if (Logging) if (GoodRip)
                    PlotIcn(pix,"(clk" + zx + ")");} //~if // (copies data->pix)
                data = nuly.GetData();
                if (pix != null) if (data != null) { // now make ants..
                  info = DoAnTrail(33,pix,data,GoodRip);
                  if (Logging) if (GoodRip)
                    PlotIcn(data,"(Aclk)");
                  more.SetInfo(info);}} //~if // (now make ants) (gottem)
              break;}}} //~if // ('AntI') ~for (nx) ('Q')
        // might also want to recalc ants (cache in RipAnts)
        if (Logging) if (GoodRip) if (ActivWgt != null)
          zx = ChoseRPS(ActivWgt,-5);      // ('Rd_1'..'Rd_4')..
        if (!tuly) ChgdBitSet = ChgdBitSet|1;} //~if // b0=file
        // SetTooledDat(-2,ActivWgt); // unneeded, already set above
      return true;} //~ClikTooWgt

 /**
  * ImporTool returns the proper theTool widget for a given import line.
  * TooledImport sequentially calls each subclass implementation, so
  * each subclass should recognize their own, or else return null.
  *
  * @param   tye    the type code parsed from word 2 of the line
  * @param   aLine  the import line
  *
  * @return          ToolType, +0x8000 if it's a draggable widget, not tool
  */
    public GameWgt ImporTool(int tye, String aLine) { //  . . . . . (RPS_Tool)
      if (tye==6) if (NthOfset(0,"(Q",aLine)>0) return theTool;
      return null;} //~ImporTool

 /**
  * SetWgtsInfo stores any additional information (including size)
  * from the import line into the new widget, as appropriate for this
  * widget tool type. Sprites also set the velocity from info.
  *
  * @param   whom    the new widget
  * @param   info    the info parameter from the import line
  * @param   size    the new widget size (tall,wide) from the import line
  * @param   colo    the new widget color from the import line
  * @param   aLine   the import line, in case something else is needed
  * @param   data    the full imported text, in case needed
  * @param   FlgWg   the 'Flgz' child widget, in case needed
  *
  * @return          false if color has been set, true for caller to set it
  */
    public boolean SetWgtsInfo(GameWgt whom, int info, int size, // (RPS_Tool)
        int colo, String aLine, String data, GameWgt FlgWg) { // frm NewDragWg
      int lino = NthOfset(0,"(Q",aLine);
      boolean done = super.SetWgtsInfo(whom,info,size,-1,aLine,data,FlgWg);
      if (lino>0) if (whom != null) { // radio setting..
        lino = (int)(CharAt(lino+2,aLine));
        if (Logging) if (GoodRip) System.out.println("  (RPS imp->Wgt #"
            + RPS_Rad.GetRefNum() + ") = " + lino);
        if (lino==112) lino++;
        if (lino>95) if (lino<122) {
          lino = ChoseRPS(whom,lino&7);
          if ((lino&7)<5) done = MkSpriteAnts
              (4,whom,false,GoodRip);} //~if
        if (Logging) if (GoodRip) lino = ChoseRPS(whom,-6);} //~if // logs res
      return false;} //~SetWgtsInfo

 /**
  * ExporTool constructs the specific text for an exported widget line.
  * TooledExport should call the correct subclass implementation,
  * each subclass should process their own, or do nothing (return 0).
  *
  *   xToolTyx   should be the exported widget type
  *   xToolInfx  should be the exported widget's Ginfo, preset to whom's
  *   xTooLinex  should be "the rest of the line"
  *   xToolInit  may have a variable name; if it needs initialization, it
  *                should be replaced with Java code starting with "\n    "
  *
  * @param   info  the ToolType previously returned by GetToolInfo
  * @param   whom  the widget to export
  * @param   tye   its type
  *
  * @return         <0 use as exported info
  *                 =2 to use velocity from 'Flgz' widget as info
  *                 =1 to use info from child widget, otherwise from drug
  *                 =3 to get color from child widget, otherwise from drug
  */
    public int ExporTool(int info, GameWgt whom, int tye) { //  . . (RPS_Tool)
      String msg = "";
      int nx = 0;
      if (whom==null) return 0;
      if ((info&0x8F00)==0x8100) { // set up export in xTooLinex...
        msg = whom.GetIDtxt();
        if (CharAt(0,msg) != 'Q') return 0;
        nx = (whom.GetInfo()>>16)&7;
        if (Logging) if (GoodImex||GoodTool||GoodRip) TooTmpStr = " (RPS_Tool)";
        xToolTyx = (xToolTyx&-256)+6;
        xToolInfx = 0;
        xTooLinex = " (" + Substring(0,2,msg) + ")";
        if (StrLength(xToolInit)>0) xToolInit = "\n    if (" + xToolInit
            + " != null) myGame.LoadRPSpix(" + xToolInit + "," + nx + ");";
        return 0x4012;} //~if // get four grand-children, +velo
      else if (Logging) if (GoodImex||GoodTool||GoodRip) { // got an icon..
        if (whom != null) nx = whom.GetColor();
        TooTmpStr = " (RPS_Tool) #" + SafeWgtNum(whom) + " ty=" + tye
            + SeeHex(" c=",nx,"");} //~if
      switch (nx&3) {
      case 0: // color black = scissors..
        xTooLinex = " 'RPS tool scissors icon' [QxSi]";
        break;
      case 1: // color lite gray = cloud (explosion)..
        xTooLinex = " 'RPS tool cloud icon' [QxQi]";
        break;
      case 2: // color dark gray = rock..
        xTooLinex = " 'RPS tool rock icon' [QxRi]";
        break;
      case 3: // color white = paper..
        xTooLinex = " 'RPS tool paper icon' [QxPi]";
        break;} //~switch
      xToolInfx = 0;
      return 0;} //~ExporTool

 /**
  * The RPS_Tool (sprite) constructor.
  */
    public RPS_Tool() { //  . . . . . . . . . . . . . . . . . . . . (RPS_Tool)
      if (Logging) if (GoodStart)
        System.out.println("new RPS_Tool #" + ToolNum);}} //~RPS_Tool

  private class SpriteTool extends SkyRxTool { // (SpriteTool)

 /**
  * SetWgtsDat sets widget's color and optional data from the tool panel.
  * SetTooledDat should call the correct subclass implementation,
  * but each subclass should recognize their own, or else do nothing.
  *
  * @param   info    ToolType previously returned by GetToolInfo
  * @param   colo    the color swatch color to be copied to the widget
  * @param   theWgt  the widget to copy the color and data to
  */
    public void SetWgtsDat(int info, int colo, GameWgt theWgt) { // (SpriTool)
      if (theWgt==null) return;
      if (colo+2>0) super.SetWgtsDat(info,colo,theWgt);} //~SetWgtsDat

 /**
  * ShoWgtsDat displays the widget's optional data in the tool panel, and
  * returns the proper color for the color swatch. TooledShoDat should
  * call the correct subclass implementation, but each subclass should
  * recognize their own, or else do nothing and return -2.
  *
  * @param   info    ToolType previously returned by GetToolInfo
  * @param   whom    the widget to show its data
  * @param   FlgWg   its 'Flgz' child widget
  *
  * @return          the widget's color, or -2
  */
    public int ShoWgtsDat(int info, GameWgt whom, GameWgt FlgWg) { // (SprToo)
      if (whom==null) return -1;
      info = super.ShoWgtsDat(info,whom,FlgWg);
      return -1;} //~ShoWgtsDat

 /**
  * ImporTool returns the proper theTool widget for a given import line.
  * TooledImport sequentially calls each subclass implementation, so
  * each subclass should recognize their own, or else return null.
  *
  * @param   tye    the type code parsed from word 2 of the line
  * @param   aLine  the import line
  *
  * @return          ToolType, +0x8000 if it's a draggable widget, not tool
  */
    public GameWgt ImporTool(int tye, String aLine) { //  . . . . (SpriteTool)
      if (tye==6) if (NthOfset(0,"(S)",aLine)>0) return theTool;
      return null;} //~ImporTool

 /**
  * SetWgtsInfo stores any additional information (including size) from
  * the import line into the new widget, as appropriate for this widget's
  * tool type. The widget color may be set here, or else return true to
  * let the caller set it. The base class (default) sets only the size.
  *
  * @param   whom    the new widget
  * @param   info    the info parameter from the import line
  * @param   size    the new widget size (tall,wide) from the import line
  * @param   colo    the new widget color from the import line
  * @param   aLine   the import line, in case something else is needed
  * @param   data    the full imported text, in case needed
  * @param   FlgWg   the 'Flgz' child widget, in case needed
  *
  * @return          false if color has been set, true for caller to set it
  */
    public boolean SetWgtsInfo(GameWgt whom, int info, int size,  // (SprTool)
        int colo, String aLine, String data, GameWgt FlgWg) {
      boolean done;
      ///** gotta build data array..
      // done = MkSpriteAnts(5,whom,false); // NG, assumes subs all in
      done = super.SetWgtsInfo(whom,info,size,colo,aLine,data,FlgWg);
      return false;} //~SetWgtsInfo

 /**
  * ExporTool constructs the specific text for an exported widget line.
  * TooledExport should call the correct subclass implementation,
  * each subclass should process their own, or do nothing (return 0).
  *
  *   xToolTyx   should be the exported widget type
  *   xToolInfx  should be the exported widget's Ginfo, preset to whom's
  *   xTooLinex  should be "the rest of the line"
  *   xToolInit  may have a variable name; if it needs initialization, it
  *                should be replaced with Java code starting with "\n    "
  *
  * @param   info  the ToolType previously returned by GetToolInfo
  * @param   whom  the widget to export
  * @param   tye   its type
  *
  * @return         <0 use as exported info
  *                 =2 to use velocity from 'Flgz' widget as info
  *                 =1 to use info from child widget, otherwise from drug
  *                 =3 to get color from child widget, otherwise from drug
  */
    public int ExporTool(int info, GameWgt whom, int tye) { //  . (SpriteTool)
      int nx, kx, rx, zx = 0;
      GameWgt nuly;
      if ((info&0xD00)==256) { // set up export in xTooLinex...
        if (Logging) if (GoodImex||GoodTool) TooTmpStr = " (SpriteTool)";
        xToolTyx = (xToolTyx&-256)+6;
        xTooLinex = " (S)";
        if (whom != null) whom = GetIxWgt(whom.GetIxData(0,0));
        if (whom != null) for (nx=0; nx <= 99; nx++) {
          if (whom==null) break;
          rx = whom.GetIxData(nx,0);
          kx = 0;
          nuly = GetIxWgt(rx);
          if (nuly != null) {
            kx = nuly.GetType()&255;
            if (kx==4) if (nuly.GetParent()==0) kx = -4;} //~if
          if (Logging) if (GoodImex||GoodTool)
            System.out.println("    .. " + nx + ": #" + rx + " " + kx);
          if (nuly==null) break;
          if (kx<0) break; // tool name comes after all children
          if (kx>4) break; // visible children must be first
          if ((kx&1) !=0) break;// 0/2/4 can be inside sprite
          zx++;} //~for
        return (zx<<2)+2;} //~if // the number of child wgts, *4, +velo
      else if (Logging) if (GoodImex||GoodTool) { // got an icon..
        if (whom != null) nx = whom.GetColor();
        TooTmpStr = " (SpriteTool) #" + SafeWgtNum(whom) + " ty=" + tye
            + SeeHex(" c=",nx,"");
        xToolInfx = 0;} //~if
      return 0;} //~ExporTool

 /**
  * The SpriteTool constructor.
  */
    public SpriteTool() { //. . . . . . . . . . . . . . . . . . . (SpriteTool)
      if (Logging) if (GoodStart)
        System.out.println("new SpriteTool #" + ToolNum);}} //~SpriteTool

  private class DiagTool extends GameTool { //  . . (DiagTool)
    GameWgt myDiagG, UpChkBx, ThikFld;

 /**
  * NewTool is called from GmMkr.StartUp for each new game tool.
  * This override initializes my panel widget(s).
  */
    public int NewTool(String whom, int info, int bx, GameWgt theWgt, int rx) {
      rx = super.NewTool(whom,info,bx,theWgt,rx);
      myDiagG = FindListWgt("[Diagonal line group]");
      UpChkBx = FindListWgt("[Diagonal Up checkbox]");
      ThikFld = FindListWgt("[Diagonal thickness entry field]");
      if (ThikFld != null) if (Tab2thk>0)
        Tab2fields[Tab2thk] = ThikFld.GetIxData(0,0);
      if (myDiagG != null) myDiagG.HideSho(false);
      return rx;} //~NewTool

 /**
  * SetWgtsDat sets widget's color and optional data from the tool panel.
  * SetTooledDat should call the correct subclass implementation,
  * but each subclass should recognize their own, or else do nothing.
  *
  * @param   info    ToolType previously returned by GetToolInfo
  * @param   colo    the color swatch color to be copied to the widget
  * @param   theWgt  the widget to copy the color and data to
  */
    public void SetWgtsDat(int info, int colo, GameWgt theWgt) { // (DiagTool)
      boolean aBit = false;
      int rx = 0, zx = 0, more = 0, bitz = 0, prio = 0, why = 0;
      String msg = "";
      if (theWgt==null) return;
      rx = theWgt.GetRefNum();
      bitz = theWgt.GetInfo();
      prio = bitz;
      if ((bitz&0xFFE0FFFC) != 0x44604454) return;
      if (UpChkBx != null) {
        if (UpChkBx.CheckIt(false)) more--;
        if (((bitz^more)&0x00100000) !=0) {
          aBit = true;
          why = 2;} //~if
        bitz = more&0x00100000 | bitz&~0x00100000;} //~if
      if (ThikFld != null) {
        msg = GetInputTxt(ThikFld);
        msg = NthItemOf(' ',1,msg);
        more = (int)(CharAt(0,msg));
        zx = more&15;
        if (more<48) more = 1; // '0'
        else if (more<58) {
          more = SafeParseInt(msg);
          if (more<1) more = 1;
          if (more>9) more = 9;} //~if
        else if (zx>13) more = zx;
          else more = 1;
        more = more<<16;
        if (((bitz^more)&0x000F0000) !=0) {
          aBit = true;
          why++;} //~if
        bitz = bitz&~0x000F0000|more;} //~if
      if (aBit) {
        if ((info&0x8000) !=0) ChgdBitSet = (ChgdBitSet&~0x2000)|1; // b0=file
          else ChgdBitSet = ChgdBitSet&~0x20000000; // b13=up/thik
        theWgt.SetInfo(bitz);} //~if
      if (Logging) if (GoodDiag) System.out.println("  (Diag #" + rx
          + SeeHex(") => x",bitz,SeeHex("/x",prio," = " + why)));
      if (colo >= 0) theWgt.SetColor(colo);} //~SetWgtsDat

 /**
  * ShoWgtsDat displays the widget's optional data in the tool panel, and
  * returns the proper color for the color swatch. TooledShoDat should
  * call the correct subclass implementation, but each subclass should
  * recognize their own, or else do nothing and return -2.
  *
  * @param   info    ToolType previously returned by GetToolInfo
  * @param   whom    the widget to show its data
  * @param   FlgWg   its 'Flgz' child widget
  *
  * @return          the widget's color, or -2
  */
    public int ShoWgtsDat(int info, GameWgt whom, GameWgt FlgWg) { // (DgTool)
      boolean aBit;
      String msg;
      if (whom==null) return -2;
      info = whom.GetInfo();
      if ((info&0xFFE0FFFC) != 0x44604454) return -2;
      aBit = (info&0x00100000) !=0; // T: Du/Dp (up) not Dd/Dg/Dn (down)
      if (UpChkBx != null) {
        aBit = (UpChkBx.CheckIt(false) != aBit);
        aBit = UpChkBx.CheckIt(aBit);
        UpChkBx.HideSho(true);} //~if
      if (ThikFld != null) {
        info = (info>>16)&15;
        if (info<1) info = 1;
        if (info>9) {
          if (info==15) msg = "o";
            else msg = "n";
          aBit = SetInputText
              (ThikFld,"Th=",msg,false);} //~if
        else aBit = FormInFld3(ThikFld,"Th=",info);
        ThikFld.HideSho(true);} //~if
      if (myDiagG != null) myDiagG.HideSho(true);
      if (Logging) if (GoodDiag)
        LogWgtTree(myDiagG,1);
      return whom.GetColor();} //~ShoWgtsDat

 /**
  * HidWgtsDat hides widget's optional data in the tool panel.
  * TooledShoDat should call the correct subclass implementation when
  * some other widget is selected, but each subclass should recognize
  * their own, or else do nothing.
  */
    public void HidWgtsDat() { // . . . . . . . . . . . . . . . . . (DiagTool)
      if (myDiagG != null) myDiagG.HideSho(false);} //~HidWgtsDat

 /**
  * ClikTooWgt accepts a click in this tool's checkbox or radio button.
  * ToolClick should call the correct subclass implementation, but
  * each subclass should recognize their own, or else return false.
  *
  * @param   whom    the widget that was clicked
  * @param   info    the ToolInfo of the ActivWgt
  * @param   tuly    true if this is a tool widget, false if a game wgt
  *
  * @return          true if handled
  */
    public boolean ClikTooWgt(GameWgt whom, int info, boolean tuly) { // (DgT)
      boolean rez = false;
      if (whom != UpChkBx) return false;
      if (whom==null) return false;
      rez = whom.CheckIt(true);
      if (tuly) ChgdBitSet = ChgdBitSet|0x80000; // b3=flags ??
        else ChgdBitSet = ChgdBitSet|8;
      info = ShoWgtsDat(0,whom,null); // always refresh immediately
      return true;} //~ClikTooWgt

 /**
  * ImporTool returns the proper theTool widget for a given import line.
  * TooledImport sequentially calls each subclass implementation, so
  * each subclass should recognize their own, or else return null.
  *
  * @param   tye    the type code parsed from word 2 of the line
  * @param   aLine  the import line
  *
  * @return          ToolType, +0x8000 if it's a draggable widget, not tool
  */
    public GameWgt ImporTool(int tye, String aLine) { //  . . . . . (DiagTool)
      if (tye==0) if (NthOfset(0,"(D)",aLine)>0) return theTool;
      return null;} //~ImporTool

 /**
  * SetWgtsInfo stores any additional information (including size) from
  * the import line into the new widget, as appropriate for this widget's
  * tool type. The widget color may be set here, or else return true to
  * let the caller set it. The base class (default) sets only the size.
  *
  * @param   whom    the new widget
  * @param   info    the info parameter from the import line
  * @param   size    the new widget size (tall,wide) from the import line
  * @param   colo    the new widget color from the import line
  * @param   aLine   the import line, in case something else is needed
  * @param   data    the full imported text, in case needed
  * @param   FlgWg   the 'Flgz' child widget, in case needed
  *
  * @return          false if color has been set, true for caller to set it
  */
    public boolean SetWgtsInfo(GameWgt whom, int info, int size, // (DiagTool)
        int colo, String aLine, String data, GameWgt FlgWg) {
      ChgdBitSet = ChgdBitSet|0x2000; // b13=up/thik
      info = info&0x1F0000|0x44604457;
      if (whom != null) whom.SetInfo(info);
      SetWgtsSize(whom,size>>16,size&0xFFFF);
      return true;} //~SetWgtsInfo

 /**
  * ExporTool constructs the specific text for an exported widget line.
  * TooledExport should call the correct subclass implementation,
  * each subclass should process their own, or do nothing (return 0).
  *
  *   xToolTyx   should be the exported widget type
  *   xToolInfx  should be the exported widget's Ginfo, preset to whom's
  *   xTooLinex  should be "the rest of the line"
  *   xToolInit  may have a variable name; if it needs initialization, it
  *                should be replaced with Java code starting with "\n    "
  *
  * @param   info  the ToolType previously returned by GetToolInfo
  * @param   whom  the widget to export
  * @param   tye   its type
  *
  * @return         <0 use as exported info
  *                 =2 to use velocity from 'Flgz' widget as info
  *                 =1 to use info from child widget, otherwise from drug
  *                 =3 to get color from child widget, otherwise from drug
  */
    public int ExporTool(int info, GameWgt whom, int tye) { //  . . (DiagTool)
      if (whom != null) if ((info&0xFF008D00)==0x44008800) { // 'D'
        xToolTyx = xToolTyx&-256; // set up export in xTooLinex...
        info = whom.GetInfo()&(-0x10000) | 0x446C; // 'D*Dl'
        if ((info&0x1F0000) == 0x1F0000) info = info&~0x200000;
        xToolInfx = info;
        xTooLinex = " (D) `" + GameWgt. UnPak4i(info,true) + "`";
        if (Logging) if (GoodImex||GoodTool) TooTmpStr = " (DiagTool)";
        return 0;} //~if // can't have other inside diag
      return 0;} //~ExporTool

 /**
  * The DiagTool constructor.
  */
    public DiagTool() { //. . . . . . . . . . . . . . . . . . . . . (DiagTool)
      myDiagG = null;
      UpChkBx = null;
      ThikFld = null;
      if (Logging) if (GoodStart||GoodDiag)
        System.out.println("new DiagTool #" + ToolNum);}} //~DiagTool

// Safe methods to call tool methods for the appropriate widget..

 /**
  * Report Status at bottom of tool panel.
  *
  * @param  msg  the text to display
  */
  public void ReportStatus(String msg) {
    int zx = 0;
    GameWgt whom = StatReport;
    if (whom != null) zx = whom.PutTextLn(msg);
    if (Logging) System.out.println("StatReport = '" + msg + "'"
        + IffyStr(zx>80," +" + zx,""));} //~ReportStatus

 /**
  * Report ChgdBitSet in StatReport at bottom of tool panel.
  */
  public void ReportChBitS() {
    if (SRchanged||GoodNew)
      ReportStatus(SeeHex("U=x ",ChgdBitSet,""));} //~ReportChBitS

 /**
  * ID2diam extracts the circle diameter from the widget ID.
  * d = 1..9 <= '1'..'9', 10..32 <= 'A'..'W'
  *
  * @param   whom    the widget ID
  *
  * @return          the diameter, =0 if invalid
  */
  public int ID2diam(int whom) {
    int info = ((whom>>16)&255)-48, rez = 0, why = 0;
    while (true) {
      why--; // why = -1
      if ((info&0xFE00FFFC) != 0x42004454) break;
      why--; // why = -2
      if (info <= 0) break;
      why--; // why = -3
      if (info>9) {
        info = info-7;
        if (info>32) break;} //~if
      rez = info;
      why = 0;
      break;} //~while
    if (Logging) if (GoodTool||BitPoker)
      System.out.println(SeeHex("(ID2dim) x",whom," => ") + rez + "/" + why);
    return rez;} //~ID2diam

 /**
  * Diam2wID encodes the circle diameter in 2nd position of the widget ID.
  * d = 1..9 => '1'..'9', 10..32 => 'A'..'W'
  *
  * @param   info    the diameter
  * @param   whom    the widget ID
  *
  * @return          the encoded ID, =0 if invalid
  */
  public int Diam2wID(int info, int whom) {
    int rez = 0, why = 0;
    while (true) {
      why--; // why = -1
      if (info>32) break;
      why--; // why = -2
      if (info <= 0) break;
      why--; // why = -3
      if ((whom&0xFE00FFFC) != 0x42004454) break;
      rez = info;
      if (rez>9) rez = rez+7;
      rez = rez+48;
      rez = (rez<<16)|whom&0xFF00FFFF;
      why = 0;
      break;} //~while
    if (Logging) if (GoodTool||BitPoker) System.out.println("(Dim2wID) " + info
        + SeeHex(" x",whom,SeeHex(" => x",rez,"/")) + why);
    return rez;} //~Diam2wID

 /**
  * GameMaker sizes and positions are limited to three decimal digits.
  * This tries to format whatever number you give it into three characters.
  *
  * @param  theWgt  the input field to format this number into
  * @param  label   its label
  * @param  whom    the number to be formatted
  *
  * @return         true if succeeded
  */
  private boolean FormInFld3(GameWgt theWgt, String label, int whom) {
    String msg = "   " + whom;
    int lxx = StrLength(msg);
    if (theWgt==null) return false;
    msg = RestOf(lxx-3,msg);
    return SetInputText
        (theWgt,label,msg,false);} //~FormInFld3

 /**
  * Sho2flds displays a packed number in 2 fields in the tool panel.
  *
  * @param   info    the number
  * @param   msg     the two labels, on separate lines
  * @param   vert    the widget to show its hi half
  * @param   horz    the widget to show its lo half
  */
  public void Sho2flds(int info, String msg, GameWgt vert, GameWgt horz) {
    boolean rez;
    if (horz != null)
      rez = FormInFld3(horz,NthItemOf('\n',2,msg),SignExtend(info));
    if (vert != null)
      rez = FormInFld3(vert,NthItemOf('\n',1,msg),info>>16);} //~Sho2flds

 /**
  * NewGmTool is called from GmMkr.StartUp for each new game tool.
  * It calls NewTool
  *
  * @param  tuly    the new GameTool, created by caller
  * @param  whom    the tool name, for ToolName
  * @param  info    for ToolType
  * @param  base    for ToolMacroBase
  * @param  lxx     for nToolMacros
  * @param  theWgt  for theTool
  */
  public void NewGmTool(GameTool tuly, String whom, int info, GameWgt theWgt) {
    int nx = (info>>24)&31, zx, rx = 0;
    if (tuly == null) return;
    if (theWgt != null) rx = theWgt.GetRefNum();
    zx = tuly.NewTool(whom,info,0,theWgt,rx); // it does:
      // ToolName = whom; // called from subclass const'r or GmMkr.StartUp..
      // theTool = theWgt;
      // ToolWgtNo = rx;
      // ToolType = info&-256|ToolNum; // <-- returned
      // ToolMacroBase = base&255; // (base=0)
      // nToolMacros = base>>8;
    ToolTypTb[nx&31] = zx;
    WgTooTb[nx&31] = tuly;
    if (Logging) if (GoodTool) System.out.println("(NuGmTool #" + rx + ") [" + nx
        + SeeHex("] x",info,SeeHex(" => x",zx,"")));} //~NewGmTool

 /**
  * GetToolInfo returns ToolType for a specified widget, or =0 if neither
  * tool nor draggable widget derived from tool (which adds +0x8000).
  * Mostly this returns ToolType, which already has these bits:
  *   +256: sprite (velo), +512: text (font), +1K: circle, +2K: rect (no child)
  *
  * @param   whom   the widget to determine what it is
  *
  * @return         ToolType, +0x8000 if it's a draggable widget, not tool
  */
  public int GetToolInfo(GameWgt whom) { // . . . . . . . . . . . . (GameTool)
    int logy = 0, info = 0, nx = 0, rx = 0, rez = 0,
        why = 0;
    while (true) {
      why++; // why = 1
      if (whom==null) break;
      if (whom==CashTooWgt) {
        rez = CashTooInfo;
        if (Logging) {
          rx = CashTooMo>>6;
          nx = CashTooMo&31;} //~if
        why = -why;
        break;} //~if
      rx = whom.GetRefNum();
      info = whom.GetInfo();
      nx = (info>>24)&31;
      why++; // why = 2
      if ((info&0xE000FFFC) != 0x40004454) break;
      rez = ToolTypTb[nx&31];
      why++; // why = 3
      if (rez==0) {
        break;} //~if
      rez = ((info&1)<<15)|rez;
      CashTooInfo = rez;
      CashTooWgt = whom;
      CashTooMo = (rx<<6)+nx;
      why = 0;
      break;} //~while
    if (Logging) if (GoodTool) if ((why>0)
        || (rx>0) && (((logy&0xFFFF) == rx) || ((logy>>16) == rx)))
      System.out.println("(GeTooInf #" + rx + ") [" + nx + SeeHex("] x",info,
          SeeHex(" => x",rez," = "))
          + why );
    return rez;} //~GetToolInfo

 /**
  * GetWgtsTool returns the GameTool for a specified widget, or null if
  * not derived from a tool.
  *
  * @param   theWgt  the widget to determine what it is
  *
  * @return          the GameTool for this widget
  */
  public GameTool GetWgtsTool(GameWgt theWgt) { //  . . . . . . . . (GameTool)
    int logy = 0, info = 0, nx = 0, rx = 0, why = 0;
    GameTool tuly = null;
    while (true) {
      why++; // why = 1
      if (Logging) if (theWgt != null) rx = theWgt.GetRefNum();
      if (theWgt==null) break;
      info = GetToolInfo(theWgt);
      // if (info==0) info = theWgt.GetInfo(); // futile
      // if ((info&0x4000FFFC) != 0x40004454) break;
      why++; // why = 2
      if (info==0) break;
      nx = (info>>24)&31;
      why++; // why = 3
      tuly = WgTooTb[nx&31];
      if (tuly == null) {
        break;} //~if
      why = 0;
      break;} //~while
    if (Logging) if (GoodTool) if ((why>0)
        || (rx>0) && (((logy&0xFFFF) == rx) || ((logy>>16) == rx)))
      System.out.println("(GeWgToo #" + rx + ") [" + nx
          + SeeHex("] x",info," = ") + why);
    return tuly;} //~GetWgtsTool

 /**
  * TooledShoPoSiz displays widget's position & size in the tool panel.
  * Position comes from the dragable widget, but size varies by tool type.
  *
  * @param   whom  the widget to show its data
  * @param   dont  false calls TooledShoDat (if needed) to set up ActivGmTool,
  *                true prevents recursion
  */
  public void TooledShoPoSiz(GameWgt whom, boolean dont) {
    int zx, rx = 0, posn = 0, info = 0, why = 0;
    String msg, logy = "";
    GameTool tuly = GetWgtsTool(whom);
    if (whom==null) return;
    if (tuly==null) return;
    if (!dont) if (ActivGmTool != tuly) info = TooledShoDat(whom);
    info = tuly.GetWgtsSize(whom);
    Sho2flds(info,"T=\nW=",TallField,WideField);
    posn = whom.GetPosn(); // use coords on draggable group wgt
    Sho2flds(posn,"V=\nH=",TopField,LeftField);} //~TooledShoPoSiz

 /**
  * TooledShoDat displays widget's [optional] data in the tool panel, and
  * returns the proper color for the color swatch by calling the correct
  * subclass implementation. Always displays shared data.
  * Caller does the shared text fields.
  *
  * @param   whom  the widget to show its data
  *
  * @return        this widget's color
  */
  public int TooledShoDat(GameWgt whom) {
    int colo = -2, info = GetToolInfo(whom), rx = 0, zx = 0, tye = 0;
    boolean aBit = false;
    GameWgt nuly = null;
    GameTool tuly = GetWgtsTool(whom);
    if (tuly != null) {
      if (ActivGmTool != null) {
        zx = (ActvGmTooInf>>24)&31;
        if (ActivGmTool != tuly) ActivGmTool.HidWgtsDat();} //~if
      nuly = tuly.FnFlgWgt(whom); // 'Flgz'
      if (nuly != null) tye = nuly.GetType();
      if (TypChkBx != null) {
        aBit = (TypChkBx.CheckIt(false) == ((tye&512)==0));
        aBit = TypChkBx.CheckIt(aBit);
        TypChkBx.HideSho(true);} //~if
      if (BumpChkBx != null) {
        aBit = (BumpChkBx.CheckIt(false) == ((tye&256)==0));
        aBit = BumpChkBx.CheckIt(aBit);
        BumpChkBx.HideSho(true);} //~if
      if (ClikChkBx != null) {
        aBit = (ClikChkBx.CheckIt(false) == ((tye&1024)==0));
        aBit = ClikChkBx.CheckIt(aBit);
        ClikChkBx.HideSho(true);} //~if
      if (DragChkBx != null) {
        aBit = (DragChkBx.CheckIt(false) == ((tye&2048)==0));
        aBit = DragChkBx.CheckIt(aBit);
        DragChkBx.HideSho(true);} //~if
      if (RollChkBx != null) {
        aBit = (RollChkBx.CheckIt(false) == ((tye&4096)==0));
        aBit = RollChkBx.CheckIt(aBit);
        RollChkBx.HideSho(true);} //~if
      ActvGmTooInf = info;
      ActivGmTool = tuly;
      TooledShoPoSiz(whom,true);
      colo = tuly.ShoWgtsDat(info,whom,nuly);} //~if
    if (Logging) if (GoodTool) {
      if (whom != null) rx = whom.GetRefNum();
      System.out.println("  (TooShoDat #" + rx + ") t=" + tye
          + SeeHex(" i=x",info,SeeHex(" => ",colo," (" + zx + ")")));} //~if
    return colo;} //~TooledShoDat

 /**
  * TooledSetPoSiz sets widget's position & size from the tool panel.
  * Position sets the dragable widget, but size varies by tool type.
  *
  * @param   whom  the widget to show its data
  */
  public void TooledSetPoSiz(GameWgt whom) {
    int info = GetToolInfo(whom), cx = ChgdBitSet, rx = 0, vert = 0, horz = 0;
    String msg = "";
    GameTool tuly = GetWgtsTool(whom);
    if (whom==null) return;
    if (tuly == null) return;
    if (Logging) if (GoodTool) {
      rx = whom.GetRefNum();
      msg = "  (TooSePoSiz #" + rx + SeeHex(") t=x",info,"");} //~if
    if ((info&0x8000)==0) { // never update tool location (but say so ;-)
      if (info>0) ChgdBitSet = ChgdBitSet&~0x100000;} //~if // b4=V
    else if (TopField != null) if (LeftField != null) {
      vert = ParseInFld(TopField);
      horz = ParseInFld(LeftField);
      if (Logging) if (GoodTool)
        msg = msg + LogVH(") Po = ",(vert<<16)|horz&0xFFFF,"");
      whom.SetPosn(vert,horz);
      ChgdBitSet = (ChgdBitSet&~16)|1;} //~if
    if (info>0) if (TallField != null) if (WideField != null) {
      vert = ParseInFld(TallField);
      horz = ParseInFld(WideField);
      if (Logging) if (GoodTool)
        msg = msg + LogVH(") Sz = ",(vert<<16)|horz&0xFFFF,"");
      tuly.SetWgtsSize(whom,vert,horz);
      if ((info&0x400) !=0) RefreshPan = true;
      if ((info&0x8000)==0) ChgdBitSet = ChgdBitSet&~0x200000;
        else ChgdBitSet = (ChgdBitSet&~32)|1;} //~if // b0=file, b5=H
    if (Logging) if (GoodTool)
      System.out.println(msg + SeeHex(" ch=x",ChgdBitSet,SeeHex("/x",cx,"")));
    if (SRchanged) ReportChBitS();} //~TooledSetPoSiz

 /**
  * SetTooledDat sets a widget's color and optional data from the tool panel
  * by calling the correct subclass implementation.
  *
  * @param   colo    the color swatch color to be copied to the widget
  * @param   whom    the widget to copy the color and data to
  */
  public void SetTooledDat(int colo, GameWgt whom) {
    int tye, zx, info = GetToolInfo(whom), vert = 0, horz = 0;
    GameWgt nuly = null;
    GameTool tuly = GetWgtsTool(whom);
    if (tuly == null) return;
    if (ActivGmTool != tuly) return;
    nuly = tuly.FnFlgWgt(whom); // 'Flgz'
    if (nuly != null) if ((ChgdBitSet&0x80008) !=0) {
      tye = nuly.GetType();
      zx = tye&255;
      if (TypChkBx != null) if (TypChkBx.CheckIt(false)) zx = zx|512;
      if (BumpChkBx != null) if (BumpChkBx.CheckIt(false)) zx = zx|256;
      if (ClikChkBx != null) if (ClikChkBx.CheckIt(false)) zx = zx|1024;
      if (DragChkBx != null) if (DragChkBx.CheckIt(false)) zx = zx|2048;
      if (RollChkBx != null) if (RollChkBx.CheckIt(false)) zx = zx|4096;
      nuly.SetFlags(zx);
      if ((info&0x8000) !=0) ChgdBitSet = (ChgdBitSet&~8)|1; // b0=file
        else ChgdBitSet = ChgdBitSet&~0x80000;} //~if // b3=flags
    tuly.SetWgtsDat(info,colo,whom);
    TooledSetPoSiz(whom);
    if (SRchanged) ReportChBitS();} //~SetTooledDat

 /**
  * TooledExport safely calls ExporTool to construct the specific text
  * for an exported widget line. These vars should be set appropriately:
  *
  *   xToolTyx   should be the exported widget type
  *   xToolInfx  should be the exported widget's Ginfo, preset to whom's
  *   xTooLinex  should be "the rest of the line"
  *   xToolInit  may have a variable name; if it needs initialization, it
  *                should be replaced with Java code starting with "\n    "
  *
  * @param   info  the ToolType previously returned by GetToolInfo
  * @param   whom  the widget to export
  *
  * @return         <0 use as exported info, instead of xToolInfx
  *                 >= 0xFFFF if there's nothing to export
  *                 =2 to use velocity from 'Flgz' widget as info
  *                 =1 to use info from child widget, otherwise from drug
  *                 =3 to get color from child widget, otherwise from drug
  */
  public int TooledExport(int info, GameWgt whom) {
    int nx = (info>>24)&31, rx = 99999, tye = -1, rez = 0;
    GameTool tuly = GetWgtsTool(whom);
    if (Logging) if (GoodImex||GoodTool) TooTmpStr = "";
    if (whom != null) {
      if (tuly == null) if (nx>0) tuly = WgTooTb[nx&31];
      rx = whom.GetRefNum();
      tye = whom.GetType()&255;} //~if
    if (tuly != null) rez = tuly.ExporTool(info,whom,tye);
    if (rez<0) xToolInfx = rez;
    if (Logging) if (GoodImex||GoodTool) {
      System.out.println("(ToolExp #" + rx + SeeHex(") i=x",info,TooTmpStr)
          + " t=" + tye + " => " + rez + IffyStr(rez<0xFFFF,
          SeeHex(" x",xToolInfx," t=") + xToolTyx + " '" + xTooLinex + "'",""));
      TooTmpStr = "";} //~if
    return rez;} //~TooledExport

 /**
  * TooledImport safely calls ImporTool to get the proper theTool widget
  * for a given import line.
  *
  * @param   tye    the type code parsed from word 2 of the line
  * @param   aLine  the import line
  *
  * @return          ToolType, +0x8000 if it's a draggable widget, not tool
  */
  public GameWgt TooledImport(int tye, String aLine) {
    GameWgt whom = null;
    GameTool tuly = ToolList;
    while (true) {
      if (tuly == null) break;
      whom = tuly.ImporTool(tye,aLine);
      if (whom != null) break;
      tuly = tuly.PreTool;} //~while
    return whom;} //~TooledImport

 /**
  * ToolClick changes the value of a checkbox or radio button
  * under the control of a tool. Somebody else has already captured
  * clicks on tools and game widgets and text input fields.
  *
  * @param   whom    the widget that was clicked
  * @param   vert    the vertical click position in this widget (unused)
  * @param   horz    the horizontal click position in this widget (unused)
  *
  * @return          true if the click was handled
  */
  public boolean ToolClick(GameWgt whom, int vert, int horz) {
    boolean tuly, rez = false;
    int zx = 0, info = 0;
    GameTool nuly = null;
    if (whom==null) return false;
    if (ActivWgt==null) return false; // nothing selected
    info = GetToolInfo(ActivWgt);
    if (info==0) return false; // ditto
    nuly = GetWgtsTool(ActivWgt);
    tuly = (info&0x8000)==0;
    if ((whom==TypChkBx)||(whom==ClikChkBx)||(whom==DragChkBx)
        ||(whom==BumpChkBx)||(whom==RollChkBx)) {
      rez = whom.CheckIt(true);
      if (tuly) ChgdBitSet = ChgdBitSet|0x80000; // b3=flags
        else ChgdBitSet = ChgdBitSet|8;
      rez = true;} //~if
    else if (nuly != null) rez = nuly.ClikTooWgt(whom,info,tuly);
    if (SRchanged) ReportChBitS();
    return rez;} //~ToolClick

 /**
  * ToolFocus is called from GmMkr->FocusEv to decide what kind of widget
  * was clicked on, so to do the right thing with it. The high 16 bits of the
  * return value is either the ref+ of a user game widget (draggable group),
  * or =0 if it's a tool, or else the negative (ASCII) of a letter defining
  * what else it might be; this value can be fed directly to SelectMe.
  * The low 16 bits are from the corresponding bits of this widget's ToolInfo
  * (or =0 if not displayable on the tool panel).
  *
  * @param   whom  the widget to determine what it is
  * @param   scro  the name list scroll (=WgScroSkp), in case needed
  *
  * @return        a composite value, the ref+ of the user game widget,
  *                and the ToolInfo low 16
  */
  public int ToolFocus(GameWgt whom, int scro) { // . . . . . . . . (GameTool)
    int seen = GetToolInfo(whom), info = 0, rez = 0, rx = 0, nx = 0,
        zx = 0, why = 0;
    GameWgt nuly = whom;
    TxBitSel = 0;
    while (true) {
      rez = -0x7A0000; // 'z'
      if (whom==null) break; // probly Blur=deselect
      rx = whom.GetRefNum();
      rez = seen&0xFFFF;
      if (rez==0) { // not a displayable wgt.. what is it?..
        why++; // why = 1
        if (ListofWgts==null) zx--;
        else if (UwgtNamList==null) zx = -2;
        else if (UwgtNamList.FindElemt(rx) >= 0) { // clicked name list,
          zx = whom.GetVposn();
          if (WgtNameHi==16) zx = zx>>4; // .. find its user wgt..
          else if (WgtNameHi>1) zx = zx/WgtNameHi;
          nx = ListofWgts.GetIxData(zx+scro,0);
          if (GoodTool) zx = nx*100+zx;
          nuly = GetIxWgt(nx);
          if (nuly != null) {
            rez = ToolFocus(nuly,scro);
            why = -1;
            break;} //~if
          else zx = zx+100000;} //~if
        else zx = -3;
        rez = -0x750000; // 'u'
        if ((whom.GetType()&255)==4) nuly = GetIxWgt(whom.GetParent());
        if (nuly==null) break;
        info = nuly.GetInfo();
        rez = -0x710000; // 'q'
        if ((info&0xE000C000) != 0x40004000) break; // not text-based ID
        nx = info&0xFFFF;
        if (info==0x436F4671) nx = 81; // 'CoFq' = color swatch => 'Q'
        else if (info==0x6E6F4A53) nx = 78; // 'noJS' => 'N'
        else if (info==0x42696C64) nx = 66; // 'Bild' => 'B'
        else if ((info&0xFFD0)==0x4640) { // 'F*' = known input field..
          rez = info&15; // 'F*': '*' must correspond to ChgdBitSet bit+ **
          TxBitSel = 1<<rez;       // ..or else need lookup table
          if (ToolActive) TxBitSel = TxBitSel<<16;
          nx = nx>>8;
          why = 4;} //~if
        else if ((info&0x7FFF0000)==0x52640000) nx = 82; // = 'Rd_*'
        else if ((info&0x7FFFC0C0)==0x436B4040) nx = 75; // = 'Ck**' => 'K'
        else if ((nx&0xC0C0)==0x4040) nx = nx>>8;
        // else if ((nx==0x4F4B)||(nx==0x436E)||(nx==0x4772) // 'OK'/'Cn'/'Gr'
        //   || (nx==0x4A53)) nx = nx>>8; // 'JS'
          else nx = 63; // '?'
        why++; // why = 2/6
        if (nx>90) {
          why++; // why = 3/7
          rez = -0x780000;} //~if // 'x'
        else if (nx>64) rez = -nx<<16;
          else rez = -0x780000;} //~if // 'x'
      else if (rez >= 0x8000) {
        why--; // why = -1
        rez = (rx<<16)+rez;} //~if
      break;} //~while // rez=0 should be impossible..
    if (Logging) if (GoodTool) System.out.println("  (TooFoc #" + rx
        + ") == " + (rez>>16) + SeeHex("/x",rez&0xFFFF,SeeHex(" t=x",seen,
        SeeHex(" i=x",info,SeeHex(" (x",nx,") = " + why + "/"
        + zx)))) );
    return rez;} //~ToolFocus

 /**
  * ChoseRPS is a RPS icon selector that works more or less like ChoseFont.
  *
  * These values: 0=none, 2=rock, 1=paper, 3=sci, 4=cloud, 7=all (6 shows cloud)
  * in the 2nd parameter doit select that icon to display.
  *
  * If you give it a RPS sprite widget or a radio button group and a
  * negative number, it will return the number associated with that widget;
  * if you give it the number, it will set that radio button or widget.
  *
  * If you give it a RPS sprite widget (tool or deployed) and a number greater
  * than 8, the low 3 bits (3 or 4) selects that one layer to set this animation
  * for, and the next bit (+8) makes it one-shot, and the next four bits is the
  * actual number of frames (scissors=2, cloud=6) and the next six bits is the
  * fractional frame rate, frames*16/100ms, then this call starts the animation
  * on that layer (step size is assumed =32). The two animations of interest
  * at this time are:
  *
  *    i = ChoseRPS(wgt,0x323); // scissors: snip, snip, ~1 sec cycle
  *    i = ChoseRPS(wgt,0x86C); // exploding wgt cloud, ~1.2 sec total
  *
  *    i = ChoseRPS(wgt,0x034); // show small cloud frame
  *    i = ChoseRPS(wgt,0x081); // hide paper, no effect on others..
  *                             // .. 0x82: rock, 0x83: scissors
  *
  *    i = ChoseRPS(wgt,0xCC9933); // change cloud color to gray-red (frex)
  *          // rec'd animation color sequence: FF0, C93, A75, CA8, DDD
  *
  * If you give it a RPS sprite widget (tool or deployed) and name=-32 it
  * will return the animation frame numbers of each of the four icons in
  * the four bytes of the return value, low byte is the cloud, high byte is
  * paper (should always be 0). If name=-31 it returns the cloud color.
  *
  * @param  whom    the RPS sprite widget
  * @param  doit    either a setting number or (negative) to ask
  *
  * @return         the current selection number
  */
  public int ChoseRPS(GameWgt whom, int doit) { // doit<0 rtns current setting;
      // whom = dragable RPS tool/wgt, or else its radio group
      // icon info: (frm rate)*16:6, (+ frms)-1:6, (step sz):6, (pix offset):14
    int prio = doit, didit = -1, colo, bitz, zx, nx, rx = 0,
        info = 0, rez = 0, tye = 0, why = 0;
    boolean oops = false;
    GameWgt nuly = null;
    String msg = "", sho = "";
    while (true) {
      why++; // why = 1
      if (whom==null) break; // why = 1
      why++; // why = 2
      rx = whom.GetRefNum();
      info = whom.GetInfo();
      tye = whom.GetType()&255;
      colo = whom.GetColor();
      if (tye==6) { // got the sprite....................................... t=6
        if (Logging) if (GoodRip) { // verify this..
          why = 16;
          nuly = GetIxWgt(whom.GetParent());
          if (nuly==null) break;
          why++; // why = 17
          if (nuly != SpriteList)
            if ((nuly.GetInfo()&-0x01000000) != 0x51000000) break;} //~if // 'Q'
        if (colo >= 0) {
          whom.SetColor(-4);
          oops = GoodRip;} //~if
        why = 6;
        if (doit>0xFFFF) { // change cloud color..........................colo
          why = 12;
          nuly = GetIxWgt(whom.GetIxData(3,0)); // get its icon
          if (nuly==null) break;
          nx = nuly.GetRefNum();
          why++; // why = 13
          if (nuly.GetType() !=2) break;
          nuly.SetColor(doit);
          why = -why; // why = -13
          break;} //~if // (change cloud color)
        else if (doit>8) { // animate/show/hide this icon.............sho/anim
          doit--;
          if ((doit&4) !=0) break;
          nuly = GetIxWgt(whom.GetIxData(doit&3,0)); // get this particular icon
          why++; // why = 7
          if (nuly==null) break;
          why++; // why = 8
          nx = nuly.GetRefNum();
          why++; // why = 9
          if (nuly.GetType()==2) while (true) { // set icon animation..
            why++; // why = 10
            zx = doit&0xF0; // this is the + of frames-1 = frame+
            if (zx==0) break;
            if ((doit&8) !=0) bitz = 0x3F0; // one-shot uses +frames = 64
              else bitz = zx;  // frame rate in doit&0x3F00..
            bitz = ((((doit&-256)<<2)+bitz)<<16)+(zx<<1)+0x7C000;
            if ((nuly.GetInfo()&1024) !=0) // becoming visible..
              didit = doit&3; // so also set radio btns (if visi)
            nuly.SetInfo(bitz);
            why = -why; // why = -10
            break;} //~while
          if (Logging) if (GoodRip)
            msg = msg + "\n  #" + nx + SeeHex(" => x",bitz," " + zx);
          break;} //~if // (animate/show/hide this icon)
        else if (doit >= 0) {
          doit = doit&7;
          // whom.SetInfo((doit<<16)|info&0xFFF0FFFF);
          rez = doit+0x52645F30;} //~if
        if (doit<8) for (nx=0; nx <= 8; nx++) {
          nuly = GetIxWgt(whom.GetIxData(nx,0)); // each icon in this sprite..
          if (nuly==null) break; // normal exit
          if (nuly.GetType() !=2) continue; // (ants)
          colo = nuly.GetColor();
          if (colo<0) continue;
          zx = (0x1243>>((colo&3)<<2))&7;
          colo = (zx<<28)|colo&0xFFFFFF;
          bitz = nuly.GetInfo();
          info = bitz&-0x4000;
          if (bitz<0) if (bitz+16>0) info = 0;
          if (doit+30<0) {
            rez = (rez<<8)|(bitz>>5)&255;
            if (zx==4) if (doit+32==0) rez = colo&0xFFFFFF;} //~if
          else if (doit<0) {
            if (bitz>0) if ((bitz&1024) !=0) { // not visi, don't count it..
              if (rez==7) if (zx==4) rez--; // =6
              continue;} //~if
            if (Logging) if (GoodRip) if (zx>0) {
              sho = sho + " " + CharAt(zx,"?PRSK???");
              if (zx==4) sho = sho + ((bitz>>5)&7);} //~if
            if (rez>0) rez = 7;
              else rez = zx;} //~if
          else if (doit != zx) {
            if (doit>4) { // turn them all on..
              if (doit==6) if (zx==4) info = info+96;} //~if // show sm cloud (+4)
            else info = info|1024; // turn all else off
            didit = doit;} //~if // (to set radio btns, if visi)
          else if ((bitz&1024) !=0) {
            if (zx==4) info = info|64; // show bigger cloud if alone
            didit = doit;} //~if
          else if (zx==4) {
            info = info|64;
            if (info != bitz) didit = doit;} //~if
          else if (info != bitz) didit = doit;
          if (Logging) if (GoodRip)          // (logs after (Cho line)
            msg = msg + "\n  %_" + nx + ": #" + nuly.GetRefNum()
                + SeeHex(IffyStr(didit<0," = x"," ! = x"),info,
                SeeHex("/",bitz,SeeHex(" c=",colo," " + zx)));
          if (doit<0) continue;
          if (didit<0) continue;
          nuly.SetInfo(info);} //~for // (each icon in this sprite)
        if (false) if (Logging) if (GoodRip) if (doit<0)
          ReportStatus("SEE:" + IffyStr(StrLength(sho)>0,sho," (none)"));
        why = -why; // why = -6
        break;} //~if // (got the sprite)
      // why++; // why = 3
      if ((info&-256) == 0x52644200) { // 'RdB*' = set/get rad btns........ rbtn
        why = -why; // why = -2
        if (doit<8) for (nx=0; nx <= 4; nx++) {
          nuly = GetIxWgt(whom.GetIxData(nx,0)); // radio btn macro group
          if (nuly==null) break;
          info = nuly.GetInfo(); // this btn's name, 'Rd_1'..'Rd_4'
          if ((info&-16) != 0x52645F30) continue; // name must = 'Rd_0'..'Rd_4'
          zx = info&7;
          nuly = GetIxWgt(nuly.GetIxData(1,0)); // the button icon
          if (nuly==null) continue;
          bitz = nuly.GetInfo(); // either =12 (sel'd) or =0 (not)
          if (doit<0) { // query setting..
            if (bitz==0) continue; // not set, don't count it
            if (rez>0) {
              rez = 7;
              break;} //~if
            else rez = zx;} //~if
          else if (doit <= 4) { // set sel'd only.. (otherwise all on)
            if ((doit==zx) == (bitz>0)) continue; // already OK
            bitz = info; // (for log)
            info = 0;
            if (doit==zx) info = doit+0x52645F30;} //~if // => 'Rd_1'..'Rd_4'
          if (Logging) if (GoodRip) {
            zx = nuly.GetRefNum();
            msg = msg + "\n  %_" + nx + ": #" + zx // (logs after (Cho line)
                + SeeHex(" = x",info,SeeHex("/",bitz," (r)"));} //~if
          if (doit<0) continue;
          rez = nuly.RadioBtn(info);} //~for
        else if (whom != null) {
          if (Logging) if (GoodRip)
            msg = msg + SeeHex("    = x",info,"");
          rez = whom.RadioBtn(doit);} //~if
        if (doit<0) break;
        if (Logging) if (GoodRip)
          if (RPS_Rad != null) LogWgtTree(RPS_Rad,1);
        break;} //~if
      why++; // why = 3      // 'QxDT'/'QxDW' .............................. t=5
      if ((info&0x7FF0FFFC) == 0x51704454) {
        why++; // why = 4
        nuly = GetIxWgt(whom.GetIxData(0,0)); // get the token sprite wgt
        if (nuly==null) break;
        why++; // why = 5
        if ((nuly.GetType()&255) !=6) break;
        rez = ChoseRPS(nuly,doit);
        if (doit >= 0) if (doit<8)
          // doit = doit&7;
          whom.SetInfo((doit<<16)|info&0xFFF0FFFF);
        why = -why; // why = -5
        break;} //~if
      break;} //~while // else why = 3
    if (why<0) if (didit >= 0) if (RPS_Rad != null) { // set radio btns, if visi..
      why++; // why = -4/-9   // didit>0: changed icon(s), update radio if visi..
      if (RPS_Rad.IsVisible()) {
        zx = ChoseRPS(RPS_Rad,didit);
        why++;} //~if // why = -3/-8
      else didit = -100-didit;} //~if
    if (Logging || (why>0) || oops) if (GoodRip) {
      System.out.println("(ChosRPS #" + rx + SeeHex(IffyStr(didit<0,") d=",
          ") ! d="),doit,SeeHex("/",prio,
          SeeHex(" c=",colo," !" + didit
          + SeeHex(" x",info,HexIfMore(" = ",rez,"/")))))
          + why
          + msg); }
    return rez;} //~ChoseRPS

public void ClearTools() {
  int nx;
  ChgdBitSet = 0;     TxBitSel = 0;
  KnownFonts = 0;     CashTooInfo = 0;     CashTooMo = 0;
  ToolActive = false; GamesAIdlg = false;
  RefreshPan = false; StartupDone = GoodStart;
  GameBoard = null;   GameNameWgt = null;  WgtNamEntry = null;
  TopField = null;    LeftField = null;    TallField = null;   WideField = null;
  TypChkBx = null;    ClikChkBx = null;    BumpChkBx = null;   DragChkBx = null;
  RollChkBx = null;   NullWgt = null;      AIdlgChkBx = null;  FontRad = null;
  ListofWgts = null;  ActivWgt = null;     VeloEntryG = null;  Rev_Date = null;
  RcInfoFld = null;   RPS_ToolWgt = null;  RestartBtn = null;  RPS_Rad = null;
  CashTooWgt = null;  StatReport = null;   UwgtNamList = null;
  KitchenInv = null;  KitchenToolWgt = null;
  LastTool = 0;       ActvGmTooInf = 0;    xToolInfx = 0;      xToolTyx = 0;
  TooTmpStr = "";     xTooLinex = "";      xToolInit = "";
  ToolList = null;    ActivGmTool = null;
  for (nx=0; nx <= 31; nx++) {
    WgTooTb[nx] = null;
    ToolTypTb[nx] = 0;} //~for
  if (Logging) System.out.println("(ClrTool) $&");} //~ClearTools

// Game Engine Code . . . . . . . . . . . . . . . . . . . . . . . . . . . .

 /**
  * Create a new widget with specified properties, or copy an existing widget,
  * or apply the properties to an existing widget.
  *
  * If the specified type is negative and the given widget is not null, then
  * that widget is mined for the type and all the parameter values given as 0.
  * In particular, a reference to the same data array is used in the copy,
  * unless a non-null data is supplied.
  *
  * If the type (without flags) is greater than 7, then the only way to
  * create a widget of that type is to create it yourself, then (optionally)
  * call NewGameWgt to populate the base class instance variables in it.
  *
  * @param  tye    the type of the new widget, or -1 to clone whom
  * @param  top    the vertical component of the top-left corner
  * @param  left   the horizontal component of the top-left corner
  * @param  tall   the vertical height
  * @param  wide   the horizontal width
  * @param  prnt   the parent reference number, or 0 if none
  * @param  color  the widget color, or -1 if none
  * @param  info   the information value
  * @param  data   an array to use for its Gdata, or null if none
  * @param  whom   a widget to copy or update, or null if none
  *
  * @return        the created widget, or null if it failed
  */
  public GameWgt NewGameWgt(int tye, int top, int left, int tall, int wide,
      int prnt, int color, int info, int slm, int[] data, GameWgt whom) {
    int kx = 0, zx, nx = tye&255, rnum = -1;
    GameWgt[] theList = AllWgts;
    if (tye<0) if (whom != null) { // clone an existing wgt..
      tye = whom.GetType();
      nx = tye&255;
      zx = whom.GetPosn();
      if (top==0) top = zx>>16;
      if (left==0) left = SignExtend(zx);
      zx = whom.GetSize();
      if (tall==0) tall = zx>>16;
      if (wide==0) wide = zx&0xFFFF;
      if (color==0) color = whom.GetColor();
      if (info==0) info = whom.GetInfo();
      if (slm==0) slm = whom.GetSelim();
      if (prnt==0) prnt = whom.GetParent();
      if (data == null) data = whom.GetData();
      if (nx<8) whom = null;} //~if
    if (whom==null) switch (nx) {       // ** need equivalent JS code **
      case 0: // static rect
      case 1: // pix image
      case 5: // wgt list
        whom = new GameWgt();
        break;
      case 2: // 1-color icon
        whom = new GameIcon();
        break;
      case 3: // font
        whom = new GameFont();
        break;
      case 4: // text line
        whom = new GameTxLn();
        break;
      case 6: // sprite
        whom = new GameSprit();
        break;
      case 7: // scrolled (text)
        whom = new GameScro();
        break;} //~switch
    if (whom==null) return null;
    if (theList == null) return null;
    rnum = whom.GetRefNum();
    if (rnum<0) return null;
    if (rnum >= MaxWgts) return null;
    AllWgts[rnum] = whom;
    whom.SetWgType(tye); // only works first time
    whom.SetPosn(top,left);
    whom.SetSize(tall,wide);
    whom.SetSelim(slm);
    whom.SetInfo(info);
    whom.SetParent(prnt);
    whom.SetColor(color);
    kx = whom.GetType();
    if (kx==tye) kx = 0;
      else whom.SetFlags(tye);
    if (Logging) if (StartupDone) {
      myTmpStr = "";
      System.out.println("..nuGmWgt #" + rnum + "=" + tye
          + IffyStr(kx>0,"/" + kx + " "," ") + top + "/" + left + " " + tall
          + "/" + wide + SeeHex(" c=",color,whom.FmtInfo(" i=",nx))
          + " p=" + prnt+ " " + (data != null) + myTmpStr);
      myTmpStr = "";} //~if
    if (tye>0) whom.SetData(data);
    return whom;} //~NewGameWgt

  /**
  * Recursively copy a widget with all its sub-widgets. This recognizes tool
  * widgets as a special case, so that these differences happen:
  *   (a) the parent is changed to Sprite list,
  *   (b) the top coordinates are globalized,
  *   (c) an unparented text (tool name) is preserved, and
  *   (d) the ID info is changed to `**DW`
  *
  * @param  whom   a widget to copy
  * @param  prnt   a parent reference number, or 0 to copy it from whom
  *
  * @return        the copied widget group, or null if it failed
  */
  public GameWgt CloneWgt(GameWgt whom, int prnt) { // rec'ly copy, except tool
    // tool=T: (a) change parent to Sprite list, (b) globalize top coords,
    // .. (c) preserve unparented text (tool name), and (d) change ID to `xxDW`
    // keep=T: preserve Gdata unchanged, otherwise assume it's wgts & clone them
    int deep = (prnt&(-0x10000))+0x10000, prep = prnt&0xFFFF, prio = prep,
        here, thar, info, zx, nx = 0, kx = 0, yx = 0,
        top = 0, left = 0, tye = 0, seen = 0, lxx = 0, why = 0;
    boolean tool = false, keep = false, done = false;
    int[] data = null;
    GameWgt nuly = null, whar = SpriteList;
    if (false) if (Logging) if (prnt==0) {
      if (GoodKchn) TooTmpStr = "\n  [kch]";
        else TooTmpStr = "";
      done = true;} //~if
    prnt = prep;
    while (true) {
      why++; // why = 1
      if (whom==null) break;
      nx = whom.GetRefNum();
      info = whom.GetInfo();
      // kx = whom.GetParent();
      tye = whom.GetType();
      yx = tye&255;
      if (yx==2) info = 0;
      if (deep>0x20020000) deep = deep-0x20000000; // below 1st-level tool
      if (yx<4) keep = true;  // special-case text parent inside tool..
      else if (yx==4) if (deep>0x20000000) { // if (kx != prnt) // prnt always !=
        if (info==0) keep = true; // no font: keep same tool name, else copy it..
        else if (prnt>0) if (info==TextFontNo) if (tye>yx) {
          prio = 0; // (-> nGmWg) this text is list label, so keep parent
          why = why+8;
          prnt = 0;}} //~if                   // IzTool // * * ?
      if (yx==5) if ((info&0xFFFF) == 0x4454) tool = true; // 'DT' = Drag Tool
      lxx = whom.GetIxData(-1,-1); // gets Gdata size
      why++; // why = 2
      if (lxx>0) if (!keep) if ((yx&-4)==4) // clone data array..
        data = new int[lxx];
      why++; // why = 3 => 5
      if (tool) {
        why++; // why = 4 => 6
        top = whom.AddPrntsPos(whom.GetPosn());
        left = SignExtend(top);
        top = top>>16;
        if (whar != null)
          prio = whar.GetRefNum(); // chg tool parent 2B sprite list
        deep = deep|0x20000000;
        info = info|7;} //~if
      else whar = null;
      nuly = NewGameWgt(-1,top,left,0,0,prio,0,info,0,data,whom); // (in CloneW)
      why = why+2; // why = 5/6
      if (nuly==null) break;
      prio = nx;
      nx = nuly.GetRefNum();
      kx = nuly.GetType();
      if ((kx&255)==2) {
        if (false) if (Logging) if (GoodKchn) TooTmpStr = TooTmpStr + " #"
            + nx + HexIfMore(" i=0/",info,"");
        nuly.SetInfo(0);} //~if // always turn animation off
      if (kx==tye) kx = 0;
        else nuly.SetFlags(tye);
      if (yx==4) if (prnt==0) if (deep>0x20000000) { // give it same parent,
        prnt = nuly.GetParent();
        whar = GetIxWgt(prnt); // ..but only if whom is there..
        if (whar != null) if (whar.FindElemt(prio)<0) {
          why = why|16;
          whar = null;}} //~if
      if (whar != null) zx = whar.AddElemt(nx); // gotta dele if cancel'd // ***
      if (data != null) // now clone array contents..
        for (lxx=lxx-1; lxx >= 0; lxx += -1) {
          zx = whom.GetIxData(lxx,0);
          if (zx>0) if (yx>4) {
            whar = GetIxWgt(zx); // don't clone fontless text inside tool..
            if (whar != null) if (((whar.GetType()&255) !=4)
                || (whar.GetInfo() !=0) || (deep<0x20000000)) {
              whar = CloneWgt(whar,nx+deep);
              if (whar != null) zx = whar.GetRefNum();}} //~if
          if (lxx >= 0)
            if (lxx < data.length)
              data[lxx] = zx;} //~for // (lxx)
      why = -why;
      break;} //~while
    if (Logging || (why>0)) if (StartupDone)
      System.out.println("  (ClonWgt #" + nx + "/" + prio + SeeHex(") t=x",tye,
          IffyStr(kx>0,SeeHex("/",kx," p=")," p=")) + prnt
          + "/" + prep + " = " + why + "/" + (deep>>16));
          // + IffyStr(StrLength(TooTmpStr)>8,TooTmpStr,"")
    if (false) if (done) TooTmpStr = "";
    return nuly;} //~CloneWgt

  /**
  * One of the features of the GameMaker tool palette is a color swatch
  * representing the designated color of whatever tool or user game widget
  * is currently selected, or else a checkerboard gray if no color is set.
  * SwatchGetColor returns the currently visible swatch color (from the
  * widget you specify) as a number, 0x00RRGGBB, or else -1 if none.
  *
  * @param  whom    the swatch color widget to get this color from
  *
  * @return         the current swatch color, 0x00RRGGBB or -1 if none
  */
  public int SwatchGetColor(GameWgt whom) {
    int rez = -1, why = 0;
    GameWgt nuly = null;
    while (true) {
      why++; // why = 1
      if (whom==null) break;
      rez = whom.GetColor();
      why++; // why = 2
      nuly = GetIxWgt(whom.GetIxData(0,0));
      if (nuly==null) break;
      why++; // why = 3
      if (nuly.GetInfo()>0) break;
      why = 0;
      rez = -1;
      break;} //~while
    if (Logging) System.out.println(SeeHex("(SwaGetCo c=",rez," = " + why));
    return rez;} //~SwatchGetColor

  /**
  * One of the features of the GameMaker tool palette is a color swatch
  * representing the designated color of whatever tool or user game widget
  * is currently selected, or else a checkerboard gray if no color is set.
  * SetSwatchColor sets the swatch color widget you specify to a numeric
  * color value, 0x00RRGGBB, or else -1 if none.
  *
  * @param  whom    the swatch color widget to set to this swatch color
  * @param  info    the color to set it to, 0x00RRGGBB or -1 if none
  */
  public void SetSwatchColor(GameWgt whom, int info) {
    GameWgt nuly = null;
    int why = 0;
    if (whom==null) why--;
    else {
      nuly = GetIxWgt(whom.GetIxData(0,0));
      if (info<0) {
        if (nuly != null) nuly.SetInfo(0); // why = 0
          else why++; // why = 1
        whom.SetColor(0xFFFFFF);} //~if
      else {
        ///if (info==0xFEFFFF) info = 0xFFFFFF;
        ///else if (info==0x10000) info = 0;
        why = 2;
        if (nuly != null) nuly.SetInfo(32);
          else why++; // why = 3
        whom.SetColor(info);}} //~else
    if (Logging)
      System.out.println(SeeHex("(SetSwaCo c=",info," = " + why));} //~SetSwatchColor

  /**
  * ChoseFont is a multi-purpose font selector.
  *
  * If you pass it null for a widget, it converts one form of font selection
  * -- such as the negative of an existing font RefNum, or the numeric (ASCII)
  * value of the 1-letter font name -- into the other. The letter names are:
  *
  *   Bold=66, Head=72, Italic=73, Monosp=77, Nano=78, Text=84, tinY=89
  *
  * If you give it a GameTxLn widget or a radio button group or the text
  * tool or a user text widget on the game board and a negative number, it
  * will return the numeric font name associated with that widget; if you
  * give it the 1-letter name of a font, it will set that radio button or
  * widget to that font.
  *
  * @param  whom    the widget clicked on
  * @param  name    either a font name (number) or (negative) RefNum
  *
  * @return         the chosen/converted font name/RefNum
  */
  public int ChoseFont(GameWgt whom, int name) { // whom drag'le tool/wgt
    int zx, nx = 0, info = 0, rez = 0, why = 0; // ..or else the radio group;
    // ..rtns current setting; whom=null converts name <=> number
    GameWgt nuly = null; // ..rx is (int)('M'/'T'/'B'/'H'/'I') sets, 0 asks
    String msg = "";
    while (true) {
      why++; // why = 1
      if (whom==null) { // convert name <=> number..
        if (name<0) { // from -font wgt + to 1-char name..
          if (name+MonoFontNo==0) rez = 77; // 'M'
          else if (name+TextFontNo==0) rez = 84; // 'T'
          else if (name+ItalFontNo==0) rez = 73; // 'I'
          else if (name+BoldFontNo==0) rez = 66; // 'B'
          else if (name+HeadFontNo==0) rez = 72; // 'H'
          else if (name+TinyFontNo==0) rez = 89; // 'Y'
          else if (name+NanoFontNo==0) rez = 78; // 'N'
          else if (name+ScorFontNo==0) rez = 83; // 'S'
            else break;} //~if // why = 1 (rez=0)
        else if (name<8) { // from MOS font + to 1-char name..
          if (name==0) rez = 77; // 'M'
          else if (name==1) rez = 84; // 'T'
          else if (name==2) rez = 73; // 'I'
          else if (name==4) rez = 66; // 'B'
          else if (name==5) rez = 72; // 'H'
          else if (name==3) rez = 89; // 'Y'
          else if (name==6) rez = 78; // 'N'
          else if (name==7) rez = 83; // 'S'
            else break;} //~if // why = 1
        else if (name<65) break; // why = 1
        else if (name>90) break; // convert 1-char name to wgt+..
        else if (name==66) rez = BoldFontNo;
        else if (name==72) rez = HeadFontNo;
        else if (name==77) rez = MonoFontNo;
        else if (name==84) rez = TextFontNo;
        else if (name==73) rez = ItalFontNo;
        else if (name==89) rez = TinyFontNo;
        else if (name==78) rez = NanoFontNo;
        else if (name==83) rez = ScorFontNo;
          else break; // why = 1 (rez=0)
        if (rez>0) why = -why; // why = -1
        break;} //~if
      why++; // why = 2
      nx = whom.GetRefNum();
      info = whom.GetInfo()&0x7FFFFF00;
      if (info == 0x52644200) { // 'RdB_' = set/get radio btns
        why = -why; // why = -2
        rez = whom.RadioBtn(name);
        break;} //~if
      ////* if ((info&0x7FFFFFFC) == 0x54784454)
      if ((GetToolInfo(whom)&512) !=0) // 'TxDT'/'TxDW'
        nuly = GetIxWgt(whom.GetIxData(0,0)); // should be the actual text
      else if ((whom.GetType()&255)==4) nuly = whom;
      if (nuly==null) break; // why = 2
      why++; // why = 3
      if ((nuly.GetType()&255) !=4) break;
      why++; // why = 4
      if (name >= 0) { // set it, to what?
        name = name&255;
        info = 0;
        if (name<8) name = ChoseFont(null,name);
        if (name>64) info = ChoseFont(null,name);
        if (info==0) break; // why = 4
        if (name>64) rez = name;
        nuly.SetInfo(info);
        why = -why; // why = -4
        break;} //~if
      why++; // why = 5: otherwise read out the font wgt+, make 1-char name..
      rez--;
      info = nuly.GetInfo();
      if (info <= 0) break; // why = 5
      rez = ChoseFont(null,-info);
      if (name+2==0) // name=-2 rtns MOS font +..
        for (info=0; info <= 8; info++) if (info>7) {
          why++; // why = 6
          rez = -rez;
          break;} //~if
        else if ((int)(CharAt(info,"MTIYBHNS"))==rez) {
          rez = info;
          break;} //~if
      if (why<6) why = 0;
      break;} //~while
    if (Logging || (why>0)) if (StartupDone) System.out.println("  (ChosFnt #" + nx
        + SeeHex(") n=",name," = ")
        + HexIfMore("",rez,"/") + why );
    return rez;} //~ChoseFont

  /**
  * An internal way to adjust one dimension of a newly created widget from
  * its default (created) size to conform to a user-specified size of its
  * containing widget.
  *
  * For a nominal size of (frex) 64, if the macro specifies 62, then the
  * returned value is the actual container size reduced by -2 (the difference)
  * unless the macro default is not close to the nominal (like being at the
  * other end of the created widget).
  *
  * @param  why    a reference number, for diagnostic log
  * @param  thar   the default size in the macro
  * @param  norm   the nominal size, against which to measure the default
  * @param  act    the actual size supplied by the container
  *
  * @return        the calculated adjusted size = act-(norm-thar)
  */
  private int AdjuDim(int why, int thar, int norm, int act, boolean log) {
    int zx = norm>>2;                                // frm BuildThisM
    zx = norm-(zx>>1)-zx;
    if (thar>zx) thar = act-(norm-thar); // if (thar<act)
    return thar;} //~AdjuDim

  /**
  * An internal way to construct a 32-bit integer representation of a 4-char
  * string ID code. The code is enclosed in reverse-single-quotes '`' in the
  * input line, and if not there or invalid, 0 is returned.
  *
  * @param  aLine  the string to find the code in
  *
  * @return        the calculated (integer) ID code
  */
  private int EncodeID(String aLine) { // frm ImportGamLis,BuildThisMac
    // if (NthOfset(0,"`",aLine)<0) return 0; // no ID
    String xID = NthItemOf('`',2,aLine);
    int mID = 0, kx = StrLength(xID), nx, zx;
    myTmpStr = "";
    if (kx>2) if (kx<5) mID = GameWgt. Pack42i(xID,true,true);
    if (mID<0x410000) mID = 0;
    if (mID>0) myTmpStr = xID;
    return mID;} //~EncodeID

  /**
  * Widgets are specified by the user text, each as ten or more integer numbers
  * on a single line, where the second is a type number (plus flags). Type
  * numbers > 224 are deemed to be macros, where multiple widgets are created
  * under a single group. BuildThisMacro adds as sub-widgets to an existing
  * group, whatever the specified macro calls for.
  *
  * @param  whom   the parent group widget to build this macro under
  * @param  aLine  the text line containing parameters this macro can use
  * @param  why    an error code base (the highest used in the caller)
  *                  so that returned error codes (if any) will not collide
  * @param  log    true to log (overrides startup silencer)
  *
  * @return        an error code > why, or else =0 if no error
  */
  private int BuildThisMacro(GameWgt whom, String aLine, int why, boolean log) {
    char delim = '\'';                      // frm ImportGamLis, why = 15
    int ix, kx, nx, zx, tye, top, left, tall, wide, color, info,
        here, thar, mID = 0, whar = 0, prnt = 0, more = 0, ctx = 0,
        nmx = 0, dimz = 0, lxx = 0;
    boolean seen = false;
    String aWord, aStr = "", theMacs = "", xID = "", msg = "";
    GameWgt nuly;
    GameEvent evh = UserEvHandler;
    GameTool tuly = ToolList;
    int[] wgtmx = WgtMacros; // uses WgtMacros, ChkBoxz,ChkBitz, Antz
    int[] wgts = null;
    int[] data = null;
    while (true) { // once thru..
      why = 20;                                       // 254 labelled text entry
      if (whom==null) break;                          // 253 checkbox
      why++; // why = 21                              // 252 pushbutton
      prnt = whom.GetRefNum();                        // 251 color swatch
      dimz = whom.GetSize();                          // 250 marching ants
      tye = SafeParseInt(NthItemOf(' ',2,aLine))&255; // 249 radio button group
      msg = msg + " M=" + tye;                        // 240 input/alert dlog
      if (tye<200) break;
      if (NthOfset(0,"`",aLine)>0) { // got a macro ID..
        mID = EncodeID(aLine);
        xID = myTmpStr;
        if (mID>0)
            if ((whom.GetType()&0x3F00) !=0) if (whom.GetInfo()==0) {
          if (Logging) if (GoodStart||log)
            System.out.println("  (BTM/mID+0) #" + prnt + SeeHex(" = x",mID,""));
          whom.SetInfo(mID);}} //~if // NOT: mID = 0; (might be inner wgts)
      // why++; // why = 22
      while (true) { // loop until found..
        why = 22;
        if (whar<0) break;
        if (whar > wgtmx.length-2)
          break;
        nx = wgtmx[whar];
        lxx = wgtmx[whar+1];
        msg = msg + " @" + whar + ": " + nx + "+" + lxx;
        whar = whar+2;
        if (nx != tye) {
          whar = lxx*7+whar;
          continue;} //~if // only cont, otherwise break
        if (lxx<0) {
          if (tye<222) if (tuly != null) {
            lxx = tuly.ToolMacro(lxx,mID,whom);
            whar = (lxx&0xFF00)*8-0x01000000;
            if (lxx>0) lxx = lxx&255;} //~if
          if (evh != null) {
            lxx = evh.UserMacro(lxx,mID,whom);
            whar = (lxx&0xFF00)*8-0x01000000;
            if (lxx>0) lxx = lxx&255;}} //~if
        if (lxx <= 0) break;
        why++; // why = 23
        wgts = new int[lxx];
        if (wgts == null) break;
        whom.SetData(wgts);
        why++; // why = 24
        why = 25;
        // nx = 255-nx; // switch (nx*8+lxx) { case
        for (zx=0; zx <= lxx-1; zx++) { // each gen'd child wgt in this macro..
          why++; // why = 26
          if (whar>0) {
            if (whar > wgtmx.length-8)
              break; // Wgt_Macros = {253,4, 0,3,1,12,12,0xFFFFFF,0, // chekbx..
                // 254,3, 4,3,0,12,16,0x0,8,            // labelled text entry..
                //   0,0,8,20,32,0xFFFFFF,0, 1540,3,12,12,28,0x0,-77,
            tye = wgtmx[whar]; // get data from the table..
            top = wgtmx[whar+1];
            left = wgtmx[whar+2];
            tall = wgtmx[whar+3];
            wide = wgtmx[whar+4];
            color = wgtmx[whar+5];
            info = wgtmx[whar+6];} //~if
          else if (evh != null) { // get data from user macro..
            ix = whar&0xFFFFF;
            tye = evh.UserMacro(ix,mID,whom); // +0
            ix++;
            top = evh.UserMacro(ix,mID,whom); // +1
            ix++;
            left = evh.UserMacro(ix,mID,whom); // +2
            ix++;
            tall = evh.UserMacro(ix,mID,whom); // +3
            ix++;
            wide = evh.UserMacro(ix,mID,whom); // +4
            ix++;
            color = evh.UserMacro(ix,mID,whom); // +5
            ix++;
            info = evh.UserMacro(ix,mID,whom); // +6
            whar++;} //~if
          else break;
          whar = whar+7;
          seen = false;
          data = null;
          aWord = "";
          // aStr = "";
          here = tye&255; // nx is the outer container ty
          if (here==4) if (info+64<0) info = ChoseFont(null,-info);
          if (here>200) { // queue up this macro for later processing..
            thar = NthOfset(0,String.valueOf(delim),aLine);
            ix = info;
            kx = StrLength(xID);
            if (kx<3) aWord = "";
              else aWord = " `" + Substring(0,kx-2,xID);
            if (info==1) aWord = "'OK'" + IffyStr(kx<3,"",aWord + "OK`");
            else if (info==2)
              aWord = "'Cancel'" + IffyStr(kx<3,"",aWord + "Cn`");
            else if (thar>0) aWord = RestOf(thar,aLine);
              else aWord = " `" + xID + "`";
            if ((nx&254)==240) { // nx/dimz: doing in this (outer) container
              if (here==252) if (info>0) if (info<3) if (mID>0) {
                if (info==1) info = (mID&(-0x10000)) | 0x4F4B; // "OK"
                else if (info==2) info = (mID&(-0x10000)) | 0x436E;} //~if // "Cn"
              if (dimz != 0x4000F0) { // macro nominal is 64/240..
                here = dimz>>16; // adjust coords to match (outer) container..
                top = AdjuDim(0,top,64,here,log);
                tall = AdjuDim(1,tall,64,here,log);
                here = dimz&0xFFFF;
                left = AdjuDim(2,left,240,here,log);
                wide = AdjuDim(3,wide,240,here,log);}} //~if
            aWord = "" + NextRefNo + " " + tye + " " + top + " " + left
                + " " + tall + " " + wide + " " + color + " " + ix // =info
                + " " + prnt + " 0 " + aWord;
            theMacs = theMacs + aWord + "\n";
            if (Logging) if (GoodStart||log)
              System.out.println("  [BTMm] '" + aWord + "'");
            tye = (tye&-256)|5;
            aWord = "";
            nmx--;} //~if
          else if (info+8==0) { // -8: radio button..
            if (color==0) data = RadBtz; // black radio btn overlay
            else if (color==0xFFFFFF) data = RadBBk;} //~if // white bkgnd
          else if (info+7==0) data = ChkBoxz; // -7: black box for checkbox
          else if (info+6==0) { // -6: red checkmark
            data = ChkBitz;
            info = 12;} //~if // (unchecked)
          else if (info+5==0) // -5: checkerboard "no color" pattern
            data = NoCoGr;
          else if (info+4==0) data = DragArrows; // -4: drag-arrow icon
          else if (info+2==0) { // -2: marching ants pattern..
            data = Antz;  // info: 6b(fr.frate),6b(+frs),6b(stpsz),14b(img.offs)
            info = ((top+left)&7)+0x40804000;} //~if
          // else if (nx==250) {} // create marching ants (max 32x32)..
          else if ((nx|4)==253) { // checkbox/radio (tall=16, color=-1)..
            if (here==4) aWord = NthItemOf(delim,2,aLine);} //~if
          // nx/dimz: doing in this container (nx=wgtmx[whar-2])..
          else if (nx==254) { // labelled text entry (tall=16, color=-1)..
            if (dimz>0x200000) { // tall>31 stacks label over entry panel..
              if (zx>0) { // entry panel; label is on top (zx=0 is lbl)..
                if (dimz>0x280000) top = top+24;
                  else top = top+20;} //~if
              wide = dimz&0xFFFF;
              left = 0;
              more = 0;} //~if
            if (here==0) { // back field..
              if (mID == 0x4E616D65) info = mID+1; // 'Name'
              wide = (dimz&0xFFFF)-left;} //~if
            if (here==4) {
              if (zx>0) { // input text
                if (dimz>0x200000) {
                  wide = (dimz&0xFFFF)-8;
                  left = 4;} //~if
                else if (more>0) left = more+4;
                aWord = NthItemOf(delim,4,aLine);} //~if
              else if (info<0) { // skip the label..
                why--; // why = 25
                more = -1;
                continue;} //~if
              else aWord = NthItemOf(delim,2,aLine);} //~if
            else if (more<0) {
              wide = dimz&0xFFFF;
              left = 0;} //~if
            else if (more>0) left = more;} //~if
          else if (nx==252) { // pushbutton (tall=22, color=0xCCCCCC)..
            if (here==4) { // here = tye&255; // (4: this is button label)
              ix = info&0xFFFF;
              if (info==1) aWord = "OK";
              else if (info==2) aWord = "Cancel";
              else if (ix == 0x4F4B) aWord = "OK";
              else if (ix == 0x436E) aWord = "Cancel";
                else aWord = NthItemOf(delim,2,aLine);} //~if
            else if (more>0) { // assumes nominal 32 wide in template..
              left = AdjuDim(4,left,31,more,log);
              wide = AdjuDim(5,wide,31,more,log);} //~if
            seen = true;} //~if
          else if ((nx&254)==240) { // input/alert dlog (tall=64, color=wh)..
            if (tye==4) aWord = NthItemOf(delim,2,aLine);    // (nom.wide=240)
            if (dimz != 0x4000F0) { // macro nominal is 64/240..
              here = dimz>>16; // match posns & sizes to container dims..
              top = AdjuDim(6,top,64,here,log);
              tall = AdjuDim(7,tall,64,here,log);
              here = dimz&0xFFFF;
              left = AdjuDim(8,left,240,here,log);
              wide = AdjuDim(9,wide,240,here,log);}} //~if
          if (info<0) info = 0;
          kx = StrLength(aWord);
          if (mID>0) if ((tye&0x3F00) !=0) if ((info&-4)==0) {
            ix = info;
            if (info==0) {
              if ((mID&255)==0) mID = mID+46; // oops, partly used..
              if ((mID&0xFF00)==0) mID = mID+0x2E00;
              info = mID;
              mID = 0;} //~if
            else if (kx>0) {
              if (info==1) {
                info = (mID&(-0x10000)) | 0x4F4B; // "OK"
                mID = mID&-256;} //~if
              else if (info==2) {
                info = (mID&(-0x10000)) | 0x436E; // "Cn"
                mID = mID&~0xFF00;} //~if
              if ((mID&0xFFFF)==0) mID = 0;} //~if
            if (Logging) if (GoodStart||log) System.out.println("  (BTM/mID+" + ix
                + ") #" + NextRefNo + SeeHex(" = x",info,SeeHex("/",mID,""))
                + " " + kx + " " + nx);
            seen = true;} //~if
          aStr = " t=" + tye + " i=" + info + " '" + aWord + "'"
              + IffyStr(nx==251,"",IffyStr(nx==253,""," lf=" + left
              + IffyStr(nx==252," wi=" + wide,IffyStr((nx&-2)==240," tp="
                + top + " ta=" + tall + " wi=" + wide,""))));
          if (Logging) if (GoodStart||log) {
            if (seen) System.out.println("  ++ " + aStr);
              else msg = msg + aStr;} //~if             // (in BuildThisMac)..
          nuly = NewGameWgt(tye,top,left,tall,wide,prnt,color,info,0,data,null);
          why++; // why = 27
          if (nuly==null) break;
          why++; // why = 28
          ix = nuly.GetRefNum();
          if (ctx<0) break;
          if (ctx < wgts.length)
            wgts[ctx] = ix;
          ctx++;
          why++; // why = 29
          if (whar<0) if (evh != null)  // +7..
            ix = evh.UserMacro((whar-1)&0xFFFFF,mID,nuly); // ignore result
          if (data == null) if (kx>0) {
            // here = nuly.GetSize(); // = tall,wide
            nuly.SetSize(0,kx*10); // temp'ly, cuz GetTextWi looks
            thar = nuly.PutTextLn(aWord); // thar = nuly.GetTextWide();
            nuly.SetSize(tall,wide);
            // more = nuly.GetPosn()&0xFFFF; // =left
            if (thar>0) {
              if (nx>252) {
                thar = thar+4;
                if ((tye&0x600) !=0) if ((mID&0x3FFF) != 0x658) // `xxFx`
                  if (thar <= wide) thar = 0;
                if (thar>0) {
                  nuly.SetSize(0,thar);
                  more = left+thar;
                  if (whom != null) {
                    kx = whom.GetSize();
                    if ((kx&0xFFFF)<more) {
                      msg = msg + " ^" + more; // (shown on exit line)
                      whom.SetSize(kx>>16,more);}}}} //~if
              else if (nx==252) if (whom != null) {
                kx = dimz&0xFFFF;
                ix = thar+6; // = a reasonable text width
                more = ix+8; // = the calc'd button width
                if (more<kx) { // adjust start of text to center it..
                  here = kx-more; // this many pixels to divide
                  more = more+here; // more is now actual button width
                  here = (here>>1)+left;
                  nuly.SetPosn(top,here);} //~if
                else whom.SetSize(0,more);
                if (ix>wide) nuly.SetSize(0,ix); }}} //~if
          why = 25;} //~for // (zx: each gen'd child wgt in this macro)
        if (why>25) break;
        why--; // why = 24
        if (whom==null) break;
        if (mID>0) if (whom.GetInfo()==0) {
          if (Logging) if (GoodStart||log)
            System.out.println("  (BTM/mID+9) #" + prnt + SeeHex(" = x",mID,""));
          whom.SetInfo(mID);} //~if
        if (Logging) if (GoodStart||log) whom.LogMe(".. ");
        if (wgts != null) zx = wgts.length;
        for (zx=zx-1; zx >= 0; zx += -1)
          if (zx < wgts.length) {
            nuly = GetIxWgt(wgts[zx]);
            if (Logging) if (GoodStart||log)
              if (nuly != null) nuly.LogMe("  ..");} //~if
        why = 0;                                // ..(zx: each listed wgt)
        break;} //~while // (loop until found)
      break;} //~while // (once thru)
    if (why==0) why = nmx;
    if (why<0) {
      myTmpStr = theMacs;
      theMacs = " ==> ..\n" + theMacs;} //~if
    if (Logging) if (GoodStart||log)                // why =
      System.out.println("[BTMx #" + prnt + "]" + msg + " = " + why + theMacs);
    return why;} //~BuildThisMacro

  /**
  * A search tool to find uniquely identified widgets based on their Ginfo
  * values or a combination of type and info. The whom parameter is the
  * widget to start this search, and is returned if it matches the criteria.
  * Otherwise FindSubWgt searches its sub-widgets (if any) recursively.
  *
  * If the search number is larger than 16 bits, and it is equal to Ginfo
  * of the widget, then that widget is returned. Otherwise the upper half
  * of the search number is compared to the low 14 bits of Ginfo, and the
  * lower half of the search number is compared to the type+plus flags for
  * each respective widget. The usual search values are either a 4-char ID
  * value, or else a GameTxLn with specified flags and a particular font +,
  * so to pick out the parameters of a draggable widget or tool.
  *
  * Null is returned if the specified widget is not found.
  *
  * The code is not very robust, so sometimes it fails when the sought
  * widget is actually there. It probably needs to be rewritten, but
  * it seems to be good enough for now.
  *
  * @param  whom  the unique string to search for
  * @param  data  the unique value to search for
  * @param  deep  the current search depth, and if greater than 999,
  *                 a unique logging identifier
  *
  * @return       the widget found that matches data
  */
  public GameWgt FindSubWgt(GameWgt whom, int data, int deep) {
    // [this is too slow for searches through deeply nested wide trees]
    int fust = deep, nx, kx = 0, zx = 0, info = 0, tye = 0,  // rec'ly search
        logy = 0, seen = 0, why = 0;  // data = type+flgs+(info<<16) to find
    GameWgt nuly = null;
    while (true) { // once through, but recursive..
      why++; // why = 1
      if (whom==null) break; // only if caller did it
      info = whom.GetInfo();
      kx = whom.GetRefNum();
      why++; // why = 2
      if (deep>999) if ((deep&7)==0) fust = deep%1000; // only if (Logging)
      if (fust==0) { // if (CashFoundWgt != null)
        if (CashFindWgt==whom) if (CashFindWhy==data) {
          nuly = CashFoundWgt;
          fust--;} //~if
        if (deep>999) if (fust==0) FindLog = "";} //~if
      if (deep>999) {
        tye = whom.GetType();
        myTmpStr = "#" + kx + " ";
        if (NthOfset(0,myTmpStr,FindLog)<0) FindLog = FindLog
            + IffyStr(fust>0,"","(" + deep + ") ") + myTmpStr;
        else if (fust <= 0) FindLog = FindLog + "(" + deep + ") ";
        myTmpStr = "";
        if (fust<0) FindLog = FindLog + "$=" + SafeWgtNum(nuly) + "/"
            + SafeWgtNum(CashFindWgt) + SeeHex(" = x",CashFindWhy," ");} //~if
      if (fust<0) break; // same as prior
      why++; // why = 3
      nuly = whom;
      if (info>0x10000) if (info==data) break; // full-word success
      if (tye==0) tye = whom.GetType();
      info = (info&0x3FFF)<<16;
      why++; // why = 4
      if (tye+info == data) break; // split-word success
      why++; // why = 5
      nuly = null;
      if (((tye-5)&255)>2) break; // failed and no (more) recursion
      why++; // why = 6
      for (nx=0; nx <= 999; nx++) {
        zx = whom.GetIxData(nx,0);
        if (deep>999) {
          kx = zx;
          myTmpStr = "#" + zx + " ";
          if (kx>0) if (NthOfset(0,myTmpStr,FindLog)>0) {
            myTmpStr = myTmpStr + "= ";
            kx = -kx;} //~if
          myTmpStr = "+" + nx + " " + myTmpStr;
          FindLog = FindLog + myTmpStr;
          myTmpStr = "";
          if (kx<0) continue;} //~if
        else if (zx<0) continue;
        if (zx==0) {
          if (seen>0) break; // why = 6 (two consecutive zeroes)
          seen++;
          continue;} //~if
        seen = 0;
        kx = 0;
        nuly = GetIxWgt(zx);
        if (nuly==null) continue;
        nuly = FindSubWgt(nuly,data,deep+1);
        if (nuly==null) {
          if (deep>999) FindLog = FindLog + "=0 ";
          continue;} //~if
        kx = nuly.GetRefNum();
        why++; // why = 7
        break;} //~for // found it
      if (deep>999) {
        if (fust==0) FindLog = FindLog + "! ";
          else FindLog = FindLog + "(+) ";} //~if
      break;} //~while
    if (fust==0) if (why>2) if (whom != null) {
      zx = whom.GetRefNum();
      if (zx>0) {
        if (deep>999) FindLog = FindLog + " $C=" + zx
            + SeeHex("/x",data,"=#") + SafeWgtNum(nuly) + " ";
        CashFindWgt = whom;
        CashFindWhy = data;
        CashFoundWgt = nuly;}} //~if
    if (deep>999) {
      if (nuly==null) info = data;
      System.out.println(IffyStr(nuly==null,"(un) f","(*) F")  // ") Found #"
          + "ound #" + kx + SeeHex(" x",info," ") + deep + " = " + why);
      if (StrLength(FindLog)>0) {
        nx = 0;
        if (fust <= 0) nx--;
        else if (nuly==null) nx++;
        else if (NthOfset(0,"$",FindLog)>0) nx++;
        if (nx !=0) System.out.println("  .. " + FindLog);
        if (nx<0) FindLog = "";}} //~if
    return nuly;} //~FindSubWgt

  /**
  * Widgets specified by macro do not always get their dimensions adjusted
  * properly by AdjuDim. This is one of several ad-hoc fixups.
  *
  * The caller gives a negative number to adjust the child widget width
  * down from its parent, then this is added to the group widget size to
  * give a smaller (positive) number to set the size of the child widget.
  *
  * @param  whom   the parent group widget to look under
  * @param  plus   how to adjust it
  */
  private void Widen1stWgt(GameWgt whom, int plus) {
    int here, info, wide = plus, thar = 0, more = 0,
        tye = 0, nx = 0, rx = 0, cx = 0, zx = 0, why = 0;
    String msg = "";
    GameWgt nuly = null;
    while (true) {
      why++; // why = 1
      if (whom==null) break;
      rx = whom.GetRefNum();
      tye = whom.GetType()&255;
      why++; // why = 2
      if (plus>0) break;
      why++; // why = 3
      wide = whom.GetWide()+plus;
      if (wide<8) break;
      why++; // why = 4
      if (tye !=5) break;
      why++; // why = 5
      nuly = GetIxWgt(whom.GetIxData(0,0));
      if (nuly==null) break;
      why++; // why = 6
      nx = nuly.GetRefNum();
      more = nuly.GetSize();
      zx = nuly.GetType()&255;
      wide = wide-nuly.GetHposn();
      if (((more&0xFFFF)+16<wide)) nuly.SetSize(more>>16,wide);
      why++; // why = 7
      if (zx !=5) break;
      why++; // why = 8
      for (here=1; here <= 8; here++) {
        whom = GetIxWgt(nuly.GetIxData(here,0));
        if (whom==null) break;
        if ((whom.GetType()&0x1C00)==0) continue;
        cx = whom.GetRefNum();
        more = whom.GetSize();
        thar = wide-whom.GetHposn();
        whom.SetSize(more>>16,thar);
        why = 0;
        break;} //~for // otherwise why = 8
      break;} //~while
    if (Logging) if (StartupDone) System.out.println("(Widn1stWgt #" + rx
        + " t=" + tye + " / #" + nx + " t=" + zx + IffyStr(plus<0,") ",") +")
        + plus + " => " + wide + " => " + thar + LogVH(" / ",more," #")
        + cx + " = " + why);} //~Widen1stWgt

  /**
  * There are two ways to display marching ants around the edge of a widget.
  * AntsIcon is used for sprites with specially created animated icons (see
  * MkSpriteAnts) that trace around the odd shape of the sprite.
  *
  * @param  sho   true to display the ants, false to hide them
  * @param  whom  the group or sprite widget to turn the ants on or off for
  *
  * @return       true if successful
  */
  public boolean AntsIcon(boolean sho, GameWgt whom) { // = Antsify for icons
    int why = 0;          // much simpler, cuz it's all here & right size..
    String msg = "";
    GameWgt nuly = null;
    while (true) {
      why++; // why = 1
      if (whom==null) break;
      msg = "w=" + whom.toString();
      why++; // why = 2
      if (((whom.GetType()-5)&255)>2) break;
      why++; // why = 3
      nuly = FindSubWgt(whom,0x416E7449,0); // 'AntI'
      if (nuly==null) break;
      why++; // why = 4
      if (nuly != whom) {
        msg = msg + " -> " + nuly.toString();
        whom = nuly;} //~if
      if (whom==null) break;
      why++; // why = 5
      whom.HideSho(sho);
      if (sho) mAntsHost = whom;
      else if (whom==mAntsHost) {
        mAntsHost = null;
        msg = msg + " -mHo";} //~if
      why = 0;
      break;} //~while
    if (Logging) if (StartupDone)
      System.out.println("  (AntzIco/" + sho + ") " + msg + " = "
        + why );
    return why==0;} //~AntsIcon // T: did it

  /**
  * There are two ways to display marching ants around the edge of a widget.
  * Antsify is used for group rectangles which can contain the predefined
  * mAntsWgt group as a sub-widget. Each icon in the group is good for one
  * vertical or horizontal line of ants, up to 32 pixels tall or up to 256
  * pixels wide, and they are mostly pre-positioned for all edges except the
  * bottom and right. Antsify adjusts the positions of those two edges and
  * hides (or shows) the rest as needed for the rectangle size.
  *
  * If the group contains a group with ID 'AntI' it is assumed that AntsIcon
  * can do it.
  *
  * @param  sho   true to display the ants, false to hide them
  * @param  whom  the group widget to turn the ants on or off for
  */
  public void Antsify(boolean sho, GameWgt whom) { // 256x1K max for now
    int fx, hz, nx, px, rx = 0, zx = 0, sz = 0, why = 0;
    String msg = "";
    GameWgt nuly = null;
    if (whom != null) while (true) {
      rx = whom.GetRefNum();
      nuly = FindSubWgt(whom,0x416E7449,0); // 'AntI'
      if (nuly==null) break;
      if (whom==GameBoard) {
        px = nuly.GetParent();
        if (px != rx) {
          if (GetToolInfo(GetIxWgt(px))>0) break; // oops, reached too deep
          if (whom.FindElemt(px)<0) break;}} //~if // ditto
      sho = AntsIcon(sho,nuly);
      return;} //~while
    if (mAntsWgt != null) zx = mAntsWgt.GetRefNum();
    if (!sho) while (true) { // terminating ants..
      why--; // why = -1
      if (mAntsHost != null) {
        if (zx>0) {
          nx = mAntsHost.FindElemt(zx);
          if (nx >= 0) mAntsHost.DeleElemt(nx);
          else if (AntsIcon(false,mAntsHost)) why = -32;
            else why = -31;} //~if
        if (why+1<0) break;
        msg = "oH=" + mAntsHost.toString();
        mAntsHost = null;
        why--;} //~if // why = -2
      if (mAntsWgt != null) {
        if (GameWgtsList != null) px = GameWgtsList.AddElemt(zx);
        mAntsWgt.SetParent(0);
        mAntsWgt.HideSho(false);
        why = why-2; // why = -3/-4
        break;} //~if
      if (whom==null) break;
      if (whom==mAntsWgt) break;
      why = why-4; // why = -5..-8
      msg = msg + " w=" + whom.toString();
      if (mAntsWgt==null) { // only at startup..
        why = why-4; // why = -9..-12
        nuly = FindSubWgt(whom,0x416E747A,0); // 'Antz'
        if (Logging) if (nuly==null)
          nuly = FindSubWgt(whom,0x416E747A,1000);
        if (nuly==null) break;
        msg = msg + " a=" + nuly.toString();
        why = why-4; // why = -13..-16
        mAntsWgt = nuly;
        if (nuly != whom) mAntsHost = whom;} //~if // unlikely
      if (mAntsWgt != null) Antsify(false,mAntsWgt);
        else why = 4-why; // why = +9..20
      break;} //~while // why = -16..+20 // (terminating ants)
    else while (true) {
      why++; // why = 1
      if (whom==null) break;
      if (whom==GameWgtsList) break; // (+1, don't)
      if (whom==theAlertDlg) break;
      if (whom==theInputDlg) break;
      msg = msg + "w=" + whom.toString();
      why++; // why = 2
      if (mAntsHost != null) {
        msg = msg + " oH=" + mAntsHost.toString();
        if (mAntsHost==whom) break;
        Antsify(false,mAntsHost);} //~if
      why++; // why = 3
      if (zx==0) break;
      if (mAntsWgt==null) break;
      msg = msg + " a=" + mAntsWgt.toString();
      why++; // why = 4
      if (((whom.GetType()-5)&255)>2) break; // not a valid group type
      why++; // why = 5
      if (GameWgtsList==null) break;
      nx = GameWgtsList.FindElemt(zx);
      if (nx >= 0) GameWgtsList.DeleElemt(nx); // "(DelElt @"
      px = whom.AddElemt(zx);
      sz = whom.GetSize();
      mAntsHost = whom;
      mAntsWgt.SetParent(whom.GetRefNum());
      mAntsWgt.HideSho(true);
      mAntsWgt.SetSize(sz,0); // now adjust edges to match client..
      for (nx=0; nx <= 99; nx++) {
        zx = mAntsWgt.GetIxData(nx,0);
        if (zx==0) break;
        nuly = GetIxWgt(zx);
        if (nuly==null) break;
        fx = 0;
        px = nuly.GetPosn();
        hz = nuly.GetSize(); // (soon) =0 if this is top or left edge,
        if (Logging) if (MaxLogWgts>0)
          myTmpStr = LogVH(IffyStr(px<0x30000000,": ",": ^"),px&0x0FFFFFFF,
              LogVH(" + ",hz,""));
        if (px !=0) { // top/left (corner) edges never change..
          hz = (hz&2)-(hz&0x20000); // <0 if botm edge, >0 if right edge
          fx = 8;
          if (hz>0) px = (px&(-0x10000))|(sz-1)&0xFFFF;
          else if (hz<0) px = px&0xFFFF|(sz-0x10000)&(-0x10000);
            else fx = 0;
          if (px>0x30000000) {
            px = px-0x40000000;
            if (((sz-px)&0x80008000) !=0) px = px|0x40000000;
              else fx++;} //~if
          else if (((sz-px)&0x80008000) !=0) {
            px = px|0x40000000;
            fx++;}} //~if
        if (Logging) if (StartupDone) if (MaxLogWgts>0) {
          System.out.println("  (Antzi) " + nx + myTmpStr + IffyStr(fx==0,"",
              IffyStr(px<0x40000000," => "," ^") + ((px>>16)&1023)
              + "/" + (px&0xFFFF)));
          myTmpStr = "";} //~if
        if (fx>0) nuly.SetPosn(px>>16,px&0xFFFF);} //~for
      why = 0;
      break;} //~while
    if (Logging) {
      System.out.println("  (Antzify/" + sho + ") " + msg + " = " + why);}} //~Antsify

  /**
  * There are two ways to display marching ants around the edge of a widget.
  * AntsIcon is used for sprites with specially created animated icons that
  * trace around the odd shape of the sprite. MkSpriteAnts creates those
  * icons.
  *
  * The first step is to work through all the visible sub-widgets and
  * accumulate their opaque pixels into an array of pixels.
  *
  * Then the resulting blob of opaque pixels is examined from all sides to
  * find the edges, in a second icon-ready array. This will be the icon
  * data of the white outline.
  *
  * Then the outline is traced and the number of pixels counted,
  * so to divide them into an integral number of 6- to 10-pixel white/black
  * ants (nominally four on then four off).
  *
  * The outline is then traced again (nominally eight times) to turn off
  * the white segments (the black is overlaid on the solid white outline,
  * so the transparent pixels show white), but advanced one more step each
  * iteration, so when the sequence is animated, the ants appear to march
  * around the composite sprite body.
  *
  * @param  why    (for the log) who called it (used: 1,2,3)
  * @param  whom   the sprite widget to create the ants for
  * @param  doit   true to display the created ants, false to hide them
  * @param  logz   true to log its progress
  *
  * @return       true if successful
  */
  public boolean MkSpriteAnts(int why, GameWgt whom, boolean doit,
      boolean logz) {
    int info = GetToolInfo(whom), posn, vert, horz, yx = why&0xEE,
        ix, kx, nx, rx = 0, zx = 0, lxx = 0, prio = 0, bitz = 0, // why = 0;
        tall = 0, wide = 0, size = 0, more = 0, colo = 0, tye = 0;
    boolean nudat = false;
    String msg = "", txt = "";         // why=x10: StartUp/sprite&RPS
    int[] data = null; // (blob)       // why=x50: ChosKitch/new inventory
    int[] pix = null; // (ants)        // why=x30: BestCircData
    int[] tmp = null;                  // why=x40: RPS/SetWgtsInfo
    GameWgt nuly = null, awgt = null;
    if (yx==4) nudat = true; // T: create new data arrays
    else if (yx==64) nudat = true; // only RPS/Kitch
    why = why<<4;
    while (true) { // >15*brk, 0*cont
      why++; // why = +1
      if (whom==null) break;
      rx = whom.GetRefNum();
      tye = whom.GetType();
      yx = tye&255;
      why++; // why = +2
      if (yx==5) {
        why++; // why = +3
        if (info==0) break;
        why++; // why = +4
        nuly = GetIxWgt(whom.GetIxData(0,0));
        if (nuly==null) break;
        yx = nuly.GetType()&255;
        why++; // why = +5
        if (yx !=6) break;
        whom = nuly;} //~if       // why = +2..
      else if (yx != 6) break; // not valid sprite or group (tool) type
        else why = why+4;
      if ((info&0xD00)==256) { // spr, !=c !=r, (could be RPS or Kitch)
        // nuly = GetIxWgt(whom.GetIxData(0,0)); // already got it
        if (Logging) if (StartupDone || logz)
          System.out.println("  (MkSpAnt #" + rx + SeeHex(") t=x",tye," ")
              + doit + " => #" + SafeWgtNum(nuly));
        if (nuly==null) break;
        doit = MkSpriteAnts(why&-16,nuly,doit,logz); // = entry why<<4
        // if (doit) doit = AntsIcon(true,nuly); // already did it, if can
        return doit;} //~if
      why++; // why = +7
      lxx = whom.GetIxData(-1,-1); // = + icons inside this sprite
      size = whom.GetSize(); // sprite size
      tall = size>>16;
      wide = size&0xFFFF;
      if (((tall-1)&0xFFFF)>99) break; // unreasonably tall
      if (((wide-1)&0xFFFF)>99) break; // unreasonable width
      if (tall > WinHi) break;
      if (wide > WinWi) break;
      why++; // why = +8        // ..can't be bigger than window
      if (wide<2) wide++;
      wide++; // (anticipatory)
      data = new int[wide]; // create a blob to accumulate the opaque pixels
      if (data == null) break; // (it will later become the outline)
      why++; // why = +9
      for (zx=wide-1; zx >= 0; zx += -1)
        if (zx >= 0) if (zx < data.length)
          data[zx] = 0;     // not use RectFil..
      nuly = null; // 1st: accumulate the opaque pixels (5*brk, 9*cont).......
      for (nx=0; nx <= lxx; nx++) {
        nuly = null;
        yx = whom.GetIxData(nx,0); // (nuly != null, except if nx=0)
        if (yx <= 0) break; // OK, end (normal exit)
        nuly = GetIxWgt(yx);
        if (nuly==null) break;
        colo = nuly.GetColor();
        info = nuly.GetInfo();
        more = nuly.GetType();
        posn = nuly.GetPosn();
        size = nuly.GetSize();
        vert = posn>>16;
        horz = size&0xFFFF;
        zx = size>>16;
        if (Logging) {
          if (StrLength(msg)==0) msg = " [";
          msg = msg + " #" + yx + LogVH("(",size,SeeHex(")t=",more,
              SeeHex("/c=",colo,"")));} //~if
        more = more&255;
        if (more==5) { // gotta test this before looping back on coords..
          if (Logging) msg = msg + SeeHex(" i=x",info,SeeHex(" w=",why,""));
          if (info==0x416E7449) break;} //~if // 'AntI'
        if (colo<0) continue; // ignore transparent
        // if (colo != 0x10000) continue; // ignore existing ants..
        // if (colo != 0xFEFFFF) continue;
        if (zx <= 0) continue;
        if (horz==0) continue;
        if (vert >= tall) continue;
        if (vert+32 <= 0) continue;
        posn = SignExtend(posn);
        if (posn >= wide) continue;
        if (posn+horz <= 0) continue;
        if (more==0) { // rect..
          if (Logging) msg = msg + " (R)" + zx + "|" + vert + "/" + posn;
          if (info==0x466C677A) continue; // 'Flgz'
          if (vert<0) {
            zx = zx+vert; // diminish v-size by pixels off the top
            if (zx <= 0) continue;
            vert = 0;} //~if
          bitz = -1;
          if (zx<32) if (vert+zx<32) bitz = bitz+(1<<zx); // trim v-size at btm
          if (vert>0) bitz = bitz<<vert; // shift pix down by v-posn
          zx = 0;
          if (posn>0) zx = posn;
          for (zx=zx; zx <= horz+posn-1; zx++) {
            if (zx<0) continue; // (can't)
            if (zx >= data.length) break;
            data[zx] = data[zx]|bitz;} //~for
          } //~if
        else if (more==2) { // icon..
          zx = 0;
          if (posn<0) { // posn=H.offx
            zx = -posn;
            posn = 0;} //~if
          info = info&0x3FFF;
          if (info>0) {
            horz = horz+info; // horz=wi, now end
            zx = zx+info;} //~if // zx is start
          if (Logging) msg = msg + " (I)" + zx + ".." + (horz-1) + "/" + posn;
          for (zx=zx; zx <= horz-1; zx++) {
            bitz = nuly.GetIxData(zx,0);
            if (vert<0) bitz = bitz>>(-vert);
            else if (vert>0) bitz = bitz<<vert;
            ix = zx+posn;
            if (ix<0) continue; // (probly can't)
            if (ix >= data.length) break;
            data[ix] = data[ix]|bitz;} //~for
          } //~if
        else if (more==4) { // we don't do text yet // ***
          if (Logging) msg = msg + " (T)";
          } //~if
        else nuly = null;
        if (nuly==null) break;} //~for // (1st: accumulate the opaque pixels)
      why++; // why = +10
      if (nuly==null) break; // invalid data, otherwise it's the 'AntI' group
      if (info != 0x416E7449) break; // not 'AntI'
      why++; // why = +11
      whom = GetIxWgt(nuly.GetIxData(0,0)); // this will soon be the outline
      if (whom==null) break; // ants wgts must already be there
      awgt = GetIxWgt(nuly.GetIxData(1,0)); // this will become the ants
      why++; // why = +12
      if (awgt==null) break;
      zx = wide*10+2; // max likely ants
      lxx = 0;
      if (!nudat) {
        pix = awgt.GetData();
        lxx = SafeArySz(pix);} //~if
      if (lxx<zx) {
        lxx = zx;
        if (lxx<4) lxx = 8;
        pix = new int[lxx];
        awgt.SetData(pix);} //~if // (grew to 10x)
      tall = -1; // soon: actual width (data is the solid blob)
      if (pix != null) { // AntTrail builds the outline.........................
        for (nx=wide; nx >= 0; nx += -1) {
          zx = 0; // fill at the end, if needed
          if (nx<0) break;
          if (nx < data.length)
          zx = data[nx];
          if (nx < pix.length)
            pix[nx] = zx;
          // if (zx==0) if (prio==0) continue; // no pix [yet]
          if (zx !=0) if (tall<0) tall = nx+2;} //~for // data[nx] is last !=0,
        if (tall>0) wide = tall; // if (tall<wide)
        size = 0;
        whom.SetInfo(0); // no animation here
        whom.SetData(data); // will become outline (keep my copy)
        // whom = null;
        // whom = GetIxWgt(nuly.GetIxData(1,0)); // this will become the ants
        why++; // why = +14
        if (false) { // data is known to be good
          if (awgt==null) break; // ants wgts must already be there
          why++; // why = +15
          // awgt.SetData(pix); // already did it
          size = SafeArySz(data);
          zx = 10;
          /// if (bitz>20) zx = 8;
          /// else if (bitz>3) zx = SmallAnts[bitz&31]&15;
          /// if (zx==0) zx = wide+2; else
          zx = wide+2;
          if (size<zx) { // sb wide*8, one each, 4 pix on then 4 off
            size = zx;
            if (size<4) size = 8;
            data = new int[size];}} //~if
        // if (bitz>5)  // bitz=104, prio = cx*64+rx=2*64+16
        info = DoAnTrail(wide,pix,data,logz);
        /// info = (wide+0x10040)<<14; // 2 steps, 1.0 frame rate
        pix = null;
        // anim'n in Ginfo: 6b(frac.frate),6b(+frs-1),6b(stpsz-1),14b(img.offs)
        // awgt.SetData(pix); // ants (already saved it)
        awgt.SetInfo(info);} //~if // (AntTrail builds the outline)
      why = why-0x10000;
      break;} //~while
    if (Logging) if (StartupDone || logz) {
      if (StrLength(msg)>0) msg = msg + " ]";
      System.out.println("(MkSprAnts #" + rx + SeeHex(") x",tye," +") + lxx
          + IffyStr((why>0)&&(why<9),"","/" + size) + " w=" + wide + msg
          + " " + logz + "/" + GoodRip // + " " + prio + " n=" + bitz
          + "/" + GoodKchn + SeeHex(" = x",why,""));} //~if            // why =
    if (why>0) return false;
    doit = AntsIcon(doit&&logz,nuly);
    return doit;} //~MkSpriteAnts

  /**
  * KnownCircleSz returns the nearest circle diameter, 0<whom<33.
  *
  * @param  whom  the requested circle diameter, in pixels
  *
  * @return       the implemented diameter closest to whom
  */
  public static int KnownCircleSz(int whom) {
    if (whom>32) whom = 32; // rtns nearest supported size
    else if (whom <= 0) whom = 1;
    return whom;} //~KnownCircleSz

  /**
  * Given a valid circle diameter, KnownCircleData returns an array
  * of pixels for a circle of that diameter, and (for ants) sets
  * SharedRez to the correct animation parameters for this circle size.
  *
  * @param  size  the requested circle diameter, in pixels
  * @param  whom  =0 for solid circle, =1 for outline only, =2 if ants
  *
  * @return       the pixel array for this diameter
  */
  public int[] KnownCircleData(int size, int whom) {
    int sz = size, nx = size*3, rx = size&1, zx, ix = 0, kx = 0, why = 0;
    boolean dont = false;
    int[] data = null;
    // animation in Ginfo: 6b(frac.frate),6b(+frs-1),6b(stpsz),14b(img.offs)
    SharedRez = 0; // default no animation
    while (true) {
      why++; // why = 1
      if (size <= 0) break;
      why++; // why = 2
      if (size>MaxCircle) break; // MaxCircle=32
      why++; // why = 3
      if ((whom<0)||(whom>2)) whom = 2; // why = -128;
      nx = nx+whom;
      why++; // why = 4
      if (KnownCirclz == null) break;
      data = KnownCirclz[nx];
      why++; // why = 5
      if (data == null) {
        why++; // why = 6
        if (whom<2) {
          data = new int[MaxCircle+1];
          if (data == null) break;} //~if
        why++; // why = 7
        if (whom==1) { // hollow..
          IconPixCopy(0,-1,0,0,0,0,data,data); // pre-clears whole array
          zx = size>>1;
          // if (size>3) {
          DoArcQ(size,0,data);   // stop,slft,dtop,dlft,tall,wide..
          IconPixCopy(rx,0,zx+rx,0,zx,zx+rx,data,data); // rx=1 if odd
          IconFlip(2,zx+rx,zx+rx,data); // 2: T<->B  4: L<->R
          IconPixCopy(0,rx,0,zx+rx,size,zx,data,data);
          IconFlip(4,size,zx+rx,data);} //~if
          // else if (size==3) { //
        else if (whom==0) { // solid..
          IconPixCopy(0,0,0,0,0,0,KnownCircleData(size,1),data);
          zx = size>>1;
          if (size>2) { // (GoodCirc: solid circ omits bukt fill)
            if (GoodCirc) {
              PokeIconPix(12,true,zx,zx,data,true);} //~if
            else zx = BucketFill(zx,zx,data);}} //~if
        else { // if (whom==2) { ants..
          kx = CirclePerims[(size-1)&31]; // CirPer = {8,8,12,...,84,88};
          sz++; // sz = size+1 now; 0<size<33
          zx = 0;
          if (kx>20) zx = sz<<3; // (8 pix/ant)
          else if (kx>3) zx = (SmallAnts[kx&31]&15)*sz;
          if (zx<8) zx = 8; // alloc minimum 8 pix
          data = new int[zx+2];
          if (data == null) break; // why = 7
          IconPixCopy(0,0,0,0,0,size,KnownCircleData(size,1),data);
          why++; // why = 8
          if (KnownCirclz[nx-1] == null) { // hollow failed
            data = null;
            break;} //~if
          // DoAnTrail(wide,pix,outln,logy).. // lxx,base,
          /// if (size>4) zx = size>>1; // DoAnt wants base=zx = cx*64+rx
          ///   else zx = kx>>2;
          ix = DoAnTrail(sz,data,new int[zx+2],BitPoker);} //~else // rtns anim
        KnownCirclz[nx] = data; } //~if
      else dont = true;
      if (whom==2) { // anim: 6b(frac.frate),6b(+frs-1),6b(stpsz),14b(img.offs)
        if (dont) {
          kx = CirclePerims[(size-1)&31];
          sz++;} //~if
        if (((kx+4)|8)==24) kx = kx>>1; // 12=>6, 20=>10
          else kx = 8;
        zx = (((kx-1)*64+sz-1)<<14)+0x40000000; // (32: x40978000
        SharedRez = zx;} //~if // 6b(fr.frate),6b(+frs-1),6b(stpsz),14b(im.of)
      why = 0;
      break;} //~while
    return data;} //~KnownCircleData

  /**
  * Given a widget that is either a circle icon or a group over a circle
  * icon, and a preferred diameter, choose the best circle size and update
  * the widget (and its sub-widget, if so) to be that size, with a data array
  * to match the selected size.
  *
  * BestCircData is recursive and private, it is expected that clients
  * will call CircleSize and not BestCircData directly.
  *
  * @param  whom   the widget to update
  * @param  tall   the desired diameter
  * @param  ins    if >0, the size of a space to center the widget in
  *
  * @return        the chosen size
  */
  private int BestCircData(int why, GameWgt whom, int tall, int ins) {
    boolean antzi = ins<0, uppy = tall<0, // tall: desired size (<0 force),
        hollo = ins>63; // ins&63 is square to center it in, =0 if no indent
    int prio = tall, info = KnownCircleSz(Math.abs(tall)), // info is best fit
        cx = 0, kx = 0, rx = 0, yx = 0, zx = 0, nx = 0, diam = 0;
    boolean tool = false, drug = false, awh = false, abk = false;
    GameWgt nuly;                                    // only frm CircleSiz
    int[] data = null;
    while (true) {    // only colors != B&W // rtn diam
      // why = why*10; // why = +0 // (caller did)
      if (whom != null) {
        nx = whom.GetRefNum();
        kx = whom.GetType()&255;
        cx = whom.GetColor();
        zx = whom.GetInfo();
        rx = GetToolInfo(whom);
        if (cx==0x10000) abk = true;
        if (cx==0xFEFFFF) awh = true;
        // yx = info;
        // if (yx>9) yx = yx+7; // (not many places to put this..)
        if ((rx&1024) !=0) { // 'C*DT' or 'B*DT' or '..DW'
          tool = ((rx&0x8000)==0);
          whom.SetInfo(Diam2wID(info,zx));
          drug = true;} //~if // T: we also need ants, so small circles get inset
        else rx = zx;} //~if
      if (whom==null) break;
      ins = ins&63; // ins>0 is size of container to center it in
      if (drug) if (info<4) if (ins<info+2) ins = info+2;
      tall = info; // new nominal
      why++; // why = +1
      if (kx==6) { // recurse to icon inside Ball sprite..
        nuly = GetIxWgt(whom.GetIxData(0,0));
        if (nuly==null) break;
        yx = nuly.GetType()&255;
        if (yx !=2) break;
        yx = BestCircData(10,nuly,tall,0);
        if (yx>0) whom.SetSize(yx,yx);
        why = -why; // why = -1
        break;} //~if // (kx=6)
      why++; // why = +2
      if (kx==5) { // recurse group for icon and for ants sublist..
        for (kx=0; kx <= 99; kx++) {
          nuly = GetIxWgt(whom.GetIxData(kx,0));
          if (nuly==null) break;
          rx = nuly.GetRefNum();
          cx = nuly.GetColor();
          yx = nuly.GetType()&255;
          if (yx !=2) if (yx !=5) continue;
          rx = tall;
          if (rx<4) if (awh||abk||(yx==5))
            rx = rx+2; // rx: the icon size (incl ants)
          if ((yx==5)&&drug) rx = rx-64; // -64: create ants
          else if (antzi) if (yx==2) {
            if (cx<0x800000) rx = rx+64; // +64: make it hollow
              else rx = rx-64;} //~if
          yx = BestCircData(20,nuly,tall,rx);
          if (yx>diam) diam = yx;} //~for
        if (drug) if (diam>0) { // both tools & draggable user game wgts..
          if (tool) { // only 'C*DT' or 'B*DT', not *DW'
            yx = 0; // ..the draggable group has (large) offset inside container
            why++; // why = +3
            if (ins>diam) yx = (ins-diam)>>1;
            whom.SetPosn(yx,yx);} //~if
          whom.SetSize(diam,diam);} //~if
        why = -why; // why = -2/-3
        break;} //~if // (kx=5)
      why = why+2; // why = +4
      if (kx !=2) break;
      zx = info;
      if (zx<4) zx = zx+2; // zx: the icon size (ants outside smaller circles)
      if (!uppy) {         // ..see 1st group in  CirclePerims
        diam = whom.GetSize();
        if ((diam&0xFFFF) != zx) uppy = true;
        else if ((diam>>16) != zx) uppy = true;} //~if
      diam = zx;
      if (uppy) whom.SetSize(zx,zx); // set container to ants diam
      if (antzi) zx = zx-64;
      else if (!hollo) {
        zx = info;
        diam = zx;} //~if
      else zx = zx+64;
      why++; // why = +5
      data = KnownCircleData(diam,(zx>>6)&3);
      whom.SetInfo(SharedRez); // set animation info (>0 only if ants)
      whom.SetData(data);
      why = -why;
      break;} //~while
    if (Logging || (why>0)) if (StartupDone)
      System.out.println("  (BtrCirDat #" + nx + ") " + zx + " " + tall
          + "/" + prio + "/" + kx + IffyStr(why>0," = ",
            SeeHex(" i=x",info," = ")) + diam + "/" + why);
    return diam;} //~BestCircData

  /**
  * Given a widget that is either a circle icon or a group over a circle
  * icon, and a preferred diameter, choose the best circle size and update
  * the widget (and its sub-widget, if so) to be that size, with a data array
  * to match the selected size.
  *
  * CircleSize is the public access to BestCircData, which does the work.
  *
  * @param  whom   the widget to update
  * @param  rx     the desired diameter
  */
  public void CircleSize(GameWgt whom, int rx) { // whom sb draggable tool/wgt
    int prio = rx, seen = rx&-256, tye, info,
        zx, nx = 0, yx = 0, tall = 0, wide = 0, why = 0;
    boolean doit = false; // true: also set container size
    GameTool tuly = ToolList;
    GameWgt nuly = whom;
    if (seen==0) seen = GetToolInfo(whom);
      else rx = rx&255;
    while (true) {                // calls BestCircDat to force new radius rx
      why++; // why = 1
      if (whom==null) break;
      why++; // why = 2
      nx = whom.GetRefNum();
      tye = whom.GetType()&255;
      info = whom.GetInfo();
      // seen = GetToolInfo(whom);
      if ((tye|4)==6) { // can be either sprite or icon
        if (tye==6) doit = true;
        if (rx <= 0) rx = whom.GetTall();
        yx = 0;} //~if
      ////* else if ((info&0xFFFC) == 0x4454)
      else if (seen !=0) { // 'DT'/'DW' (in Maker)..
        ////* yx = (1-(info&1))*32;
        // if (rx <= 0) rx = ID2diam(info);
        if (rx <= 0) rx = whom.GetTall();
        if (rx <= 0) rx = 16; // default, if none spec'd
        yx = (~seen>>10)&32; // offset: =32 if tool, =0 in game board..
        if (yx==0) doit = true;} //~if // game board only, also resize container
      else break;
      why++; // why = 3
      if (((tye-5)&255)<3) // should be always, unless icon
        nuly = GetIxWgt(whom.GetIxData(0,0)); // nuly is now the icon
      why++; // why = 4
      if (nuly==null) break;
      if (nuly==whom) {
        whom = GetIxWgt(nuly.GetParent());
        if (GetToolInfo(whom)==0) whom = null;} //~if
      why++; // why = 5
      if ((nuly.GetType()&255) !=2) break;
      why++; // why = 6
      if (rx <= 0) break;
      /// info = nuly.GetColor();
      /// if (info==0xFEFFFF) nuly.SetColor(0xFFFFFF); // BCD needs this..
      /// else if (info==0x10000) nuly.SetColor(0);
      if (rx>32) rx = 32;
      why++; // why = +7
      if (whom != null) {
        why++; // why = 8
        zx = BestCircData(30,whom,-rx,yx);
        if (false) if (doit) if (yx==0) if (rx>0) whom.SetSize(rx,rx);} //~if
      else zx = BestCircData(40,nuly,-rx,yx); // (BCD sets size)
      why = -why;
      break;} //~while
    if (Logging || (why>0)) if (StartupDone)
      System.out.println("  (CircSiz #" + nx + ") r=" + rx + "/" + prio
          + " i=" + yx + " = " + why);} //~CircleSize

  /**
  * Given a rock-paper-scissor widget, this loads up its icon pixel arrays.
  * If the "name" parameter <0, get its value from saved TooTmpStr
  *
  * @param  whom   the widget to initialize
  * @param  name   the icon widget(s) to show (same as ChoseRPS)
  */
  public void LoadRPSpix(GameWgt whom, int name) { // whom sb draggable tool/wgt
    int colo, tye, nx, kx = 0, rx = 0, info = 0, kolo = 0, seen = 0, why = 0;
    int[] data;
    GameWgt nuly = null, prio = whom;
    while (true) {
      why++; // why = 1
      if (whom==null) break;
      why++; // why = 2
      rx = whom.GetRefNum();
      kolo = whom.GetColor();
      tye = whom.GetType()&255;
      if (tye==5) { // hopefully dragable wgt..
        info = whom.GetInfo();
        if ((info&0x7FF0FFFC) != 0x51704454) break;
        whom = GetIxWgt(whom.GetIxData(0,0)); // get the token sprite wgt
        if (name <= 0) name = (info>>16)&7;
        why = 10;
        continue;} //~if
      why++; // why = 3
      if (tye !=6) break;
      whom.SetColor(-2);
      if (name<0) {
        seen = NthOfset(0,"(Q",TooTmpStr);
        if (seen>0) seen = (int)(CharAt(seen+2,TooTmpStr));
        if (seen==112) seen++;
        if (seen>95) if (seen<122) name = seen&7;} //~if
      why++; // why = 4
      for (nx=0; nx <= 8; nx++) {
        nuly = GetIxWgt(whom.GetIxData(nx,0)); // each icon this sprite..
        if (nuly==null) {
          if (nx<4) why = nx+4;
            else why = -why; // success (normal exit)
          break;} //~if
        if (nuly.GetType() !=2) continue;
        colo = nuly.GetColor();
        if (colo >= 0) {
          info = 0;
          switch (colo&3) {
          case 0: // color black = scissors (+3)..
            data = QxSiPix;
            if (nuly != null) nuly.SetData(data);
            if (name<5) if (name !=3) info = 256;
            break;
          case 1: // color lite gray = cloud (explosion)..
            data = QxQiPix;
            if (nuly != null) nuly.SetData(data);
            if (name>5) info = 96; // start out showing middle frame
            else if (name<4) info = 256;
            break;
          case 2: // color dark gray = rock (+2)..
            data = QxRiPix;
            if (nuly != null) nuly.SetData(data);
            if (name<5) if (name !=2) info = 256;
            break;
          case 3: // color white = paper (+1)..
            data = QxPiPix;
            if (nuly != null) nuly.SetData(data);
            if (name<5) if (name !=1) info = 256;
            break;} //~switch
          nuly.SetInfo(info);} //~if
        if (Logging) if (GoodRip) if (colo<0 )
          System.out.println("   ... " + nx + ": #" + SafeWgtNum(nuly)
              + SeeHex(" c=",colo,"/" + info));} //~for // (each icon this sprite)
      // if (why<0) if (MkSpriteAnts(9,prio,false,GoodRip)) break; // can't
      break;} //~while
    if (Logging || (why>0)) if ((why>0) || GoodRip)
      System.out.println("  (LdRPSpx #" + rx + SeeHex(") c=",kolo,
          " n=") + name + "/" + seen + " = " + why);} //~LoadRPSpix

  /**
  * Floats Alert & Input dialogs to top of sprite list.
  */
  public void FloatDlogs() {
    int inx, whom, why = 0;
    GameWgt nuly = SpriteList;
    if (nuly==null) return;
    whom = SafeWgtNum(theInputDlg);
    if (whom>0) while (true) {
      inx = nuly.FindElemt(whom);
      why = inx<<16;
      if (inx<0) break;
      nuly.DeleElemt(inx);
      inx = nuly.AddElemt(whom);
      break;} //~while
    else why = -0x60000;
    whom = SafeWgtNum(theAlertDlg);
    if (whom>0) while (true) {
      inx = nuly.FindElemt(whom);
      why = why+inx+100;
      if (inx<0) break;
      nuly.DeleElemt(inx);
      inx = nuly.AddElemt(whom);
      why = why+inx+100;
      break;} //~while
    else why = why+7;
    Floatem = 0;
    if (Logging) if (StartupDone)
      System.out.println(LogVH("(FlotDlgs) ",why,""));} //~FloatDlogs

  /**
  * SetInputText is a utility to set up an input macro panel with its
  * prompt and predefined input text. Use GetInputTxt to capture the
  * input text after the user has clicked "OK" or otherwise accepted it.
  *
  * @param  whom    the group widget over a pair of GameTxLn widgets
  * @param  prompt  the prompt text, or "" to leave it unchanged
  * @param  data    the predefined input text
  * @param  doit    true to set the predefined input
  *
  * @return         true if successful
  */
  public boolean SetInputText(GameWgt whom, String prompt,
      String data, boolean doit) { // doit=T: set data value
    int lxx = StrLength(data), zx = StrLength(prompt), nx = 0, why = 0;
    GameWgt nuly = null;
    String frm = "";
    while (true) {
      why++; // why = 1
      if (whom==null) break;
      nx = whom.GetRefNum();
      why++; // why = 2
      if (MonoFontNo <= 0) break; // needed font not created..
      if (BoldFontNo <= 0) break;
      why++; // why = 3
      if (zx>0) {
        nuly = FindSubWgt(whom,(BoldFontNo<<16)+4,0);
        if (nuly==null) break; // can't find prompt
        zx = nuly.PutTextLn(prompt);
        if (nuly.GetWide()<zx) nuly.SetSize(0,zx);} //~if
      // else if (lxx>0) if (Logging) prompt = data; // (for log)
      if ((lxx>0)||doit) {
        why++; // why = 4
        nuly = FindSubWgt(whom,(MonoFontNo<<16)+0x0604,0);
        if (nuly==null) break; // can't find data field
        zx = nuly.PutTextLn(data);
        if (nuly.GetWide()<zx) nuly.SetSize(0,zx);
        if (doit) DlgInputData = nuly;} //~if
      why = 0;
      break;} //~while
    if (DlgInputData != null) if (Tab2fields[0]==0) // actual text wgt..
      Tab2fields[0] = DlgInputData.GetIxData(0,0);
    if (Logging || (why>0)) {
      // if (why==4) prompt = data;
      System.out.println("  (SetInText #" + nx + IffyStr(why==0," OK",
          " failed = " + why) + ") '" + prompt + "' '" + data
          + "'");} //~if
    return why==0;} //~SetInputText

  /**
  * GetInputTxt is a utility to capture the input text after the user has
  * clicked "OK" or partial input any time before then. Use SetInputText
  * to set up the input panel with its prompt and predefined input text.
  *
  * @param  whom    the group widget over at least a GameTxLn input widget
  *
  * @return         the text from that widget
  */
  public String GetInputTxt(GameWgt whom) {
    int nx = 0, why = 0;
    GameWgt nuly = null;
    String msg = "", frm = "";
    while (true) {
      why++; // why = 1
      if (whom==null) break;
      nx = whom.GetRefNum();
      why++; // why = 2
      if (MonoFontNo <= 0) break; // needed font not created..
      why++; // why = 3
      nuly = FindSubWgt(whom,(MonoFontNo<<16)+0x0604,0);
      if (nuly==null) break; // can't find data field
      msg = nuly.GetTextLn();
      why = 0;
      break;} //~while
    if (Logging || (why>0)) {
      System.out.println("  (GetInText #" + nx + IffyStr(why==0,"",
          " failed = " + why) + ") = '" + msg + "'" + frm);} //~if
    return msg;} //~GetInputTxt

  /**
  * Calls GetInputTxt to capture the input text, then converts it to a number.
  *
  * @param  whom    the group widget over at least a GameTxLn input widget
  *
  * @return         the numeric (integer) value from that widget's text
  */
  public int ParseInFld(GameWgt whom) {
    return SafeParseInt(GetInputTxt(whom));} //~ParseInFld

  /**
  * Open the Input Dialog (macro) widget with a specified prompt and
  * predefined input text.
  *
  * @param  prompt  the prompt text
  * @param  data    the predefined input text
  *
  * @return         true if successful, false if it failed
  */
  public boolean OpenInputDlog(String prompt, String data) {
    GameEvent evh = UserEvHandler;
    GameWgt whom = theInputDlg, nuly = null;
    int nx = 0, whar = 0, why = 0;
    boolean rez = false;
    while (true) {
      why++; // why = 1
      if (DlogIsOpen>0) break; // already have something open
      why++; // why = 2
      if (whom==null) break; // no such dlg created
      nx = whom.GetRefNum();
      why++; // why = 3
      if (!SetInputText(whom,prompt,data,true)) break; // can't set field(s)
      why++; // why = 4
      nuly = DlgInputData;
      if (nuly==null) break; // didn't set it??
      why++; // why = 5
      if (Floatem>0) FloatDlogs();
      whom.HideSho(true);
      if (evh != null) {
        evh.BlurEv(null); // turns ants off
        evh.FocusEv(nuly,3,nuly.GetWide());
        if (evh.GetRefWgt(-1) != nuly) break;} //~if // didn't take focus
      DlogIsOpen = nx;
      if (evh != null) evh.OpCloEvDlog(nx);
      rez = true;
      why = 0;
      break;} //~while
    if (why>4) if (whom != null) {
      whom.HideSho(false);
      if (evh != null) evh.OpCloEvDlog(0);
      DlogIsOpen = 0;} //~if
    if (Logging) System.out.println("(OpInpDlog) #" + nx + " '" + prompt
        + "' do=" + DlogIsOpen + " = " + why);
    return rez;} //~OpenInputDlog

  /**
  * Open the Alert (macro) widget with a specified message.
  * The Cancel button is shown only if the message contains '?'
  *
  * @param  msg    the message text
  */
  public boolean GameAlert(String msg) {
    GameWgt whom = theAlertDlg, nuly = null;
    int rx = 0, sz = 0, whar = 0, why = 0;
    boolean rez = false, dont = NthOfset(0,"?",msg)<0;
    GameEvent evh = UserEvHandler;
    while (true) {
      why++; // why = 1
      if (DlogIsOpen>0) break; // already have something open
      why++; // why = 2
      if (whom==null) break; // no such dlg created
      rx = whom.GetRefNum();
      why++; // why = 3
      if (!SetInputText(whom,msg,"",false)) break; // can't set msg field
      why++; // why = 4
      nuly = GetIxWgt(whom.GetIxData(2,0)); // should be Cancel btn
      if (nuly != null) {
        why++; // why = 5
        sz = nuly.GetSize();
        if (sz==0x14003C) { // = 20*65536+60
          why++; // why = 6
          whar = nuly.GetHposn();
          if (whar>0) {
            why++; // why = 7
            if (whar>333) {
              why++; // why = 8
              if (dont) whar = -whar;
                else whar = 9;} //~if
            else if (dont) whar = whar+400;
              else whar = -whar;
            if (whar>0) nuly.SetHposn(whar);}}} //~if
      if (Floatem>0) FloatDlogs();
      whom.HideSho(true);
      DlogIsOpen = rx;
      if (evh != null) evh.OpCloEvDlog(rx);
      rez = true;
      why = -why; // -4/../-9
      break;} //~while
    if (Logging) System.out.println("(GmAlert) #" + rx + " '" + msg + "' do="
        + DlogIsOpen + " !? " + whar + LogVH(" ",sz," = ") + why + "/" + rez);
    return rez;} //~GameAlert

  /**
  * Gets the current or most recent input text (see OpenInputDlog).
  *
  * @return         the most recent text value from the Input Dialog
  */
  public String DlgEvData() { // gets current or most recent input text
    GameWgt whom = DlgInputData;
    if (whom==null) return "";
    return whom.GetTextLn();} //~DlgEvData

  /**
  * Determines if this ClickEvt was directed to an open Alert or Input Dialog,
  * and returns 1 if the OK button was clicked, 2 if the Cancel button was
  * clicked, otherwise 0 if the dialog is still open and used this event,
  * or -1 if it's not an Alert or Input Dialog.
  *
  * @param  whom  The widget that was clicked
  * @param  vert  The vertical offset into this widget where it
  *                 was clicked
  * @param  horz  The horizontal offset into this widget
  *
  * @return       1 if OK, 2 if Cancel, -1 if not Alert or Input, otherwise 0
  */
  public int IsDlgEvent(GameWgt whom, int vert, int horz) {
    int prio = DlogIsOpen, rez = 0, zx = 0, nx = 0, info = 0, why = 0;
    boolean seen = false, impy = false, logy = Logging;
    String msg = ")";
    GameEvent evh = UserEvHandler;
    GameWgt nuly = null;
    while (true) {
      if (DlogIsOpen<0) DlogIsOpen = 0;
      why++; // why = 1
      if (whom==null) break; // huh? MouseDisp screwed up
      why++; // why = 2
      nx = whom.GetRefNum();
      info = whom.GetInfo();
      if (logy) msg = whom.GetIDtxt();
      if (vert<0) { // called from GameEvent base when closing..
        if (whom==theInputDlg) nuly = whom;
        else if (whom==theAlertDlg) nuly = whom;
        zx = horz&0x7FFF;
        seen = true;} //~if
      else zx = info&0xFFFF;
      if (logy) {
        if (StrLength(msg)>0) msg = ") `" + msg + SeeHex("` x",zx,"");
        else if ((info&1023) == info) msg = ") i=" + info;
          else msg = SeeHex(") x",info,"");
        if (seen) msg = msg + " (GE)";} //~if
      if (DlogIsOpen==0) break; // gotta have something open
      why++; // why = 3
      if (nuly==null) nuly = GetIxWgt(whom.GetParent());
      if (nuly==null) break; // no parent: can't be mine
      why++; // why = 4
      if (evh == null) break;
      why++; // why = 5
      if ((info&0x08000000) !=0) impy = true; // input ('I'), not alert ('A')
      if (((info>>24)&0x77) != 0x41) break; // (not 'A' or 'I')
      why++; // why = 6: now known to be either Alert or Input..
      if (zx==0x4F4B) rez++; // "OK" (rez = 1)
      else if (zx==0x436E) rez = 2; // "Cn"
      else if (whom==DlgInputData) evh.FocusEv(whom,vert,horz);
      else if (whom==theInputDlg) evh.FocusEv(whom,vert,horz);
      else if (whom==theAlertDlg) evh.FocusEv(whom,vert,horz);
        else break; // no such dlg
      why = -why; // why = -6
      if (rez==0) break; // still open & active
      why--; // why = -7
      if (impy) {                                       // closed here..
        if (logy) msg = msg + " I/" + InputWhy;
        if (theInputDlg != null) theInputDlg.HideSho(false);} //~if
      else if (theAlertDlg != null) theAlertDlg.HideSho(false);
      if (evh != null) {
        if (seen) zx = 0; // GotEsc/Ent already saw this
          else zx = -DlogIsOpen; // so GotEsc/Ent knows
        evh.OpCloEvDlog(zx);} //~if
      DlogIsOpen = 0;
      break;} //~while
    if (why>0) rez = -1;
    if (logy) System.out.println("(IsDlgEvt/#" + nx + IffyStr(rez<0,"",
        IffyStr(rez>1,") (Cn",IffyStr(rez>0,") (OK",") (op"))) + msg
        + "' do=" + DlogIsOpen + "/" + prio + " = " + rez + "/"
        + why );
    return rez;} //~IsDlgEvent // 1: OK, 2: Cancel, 0: still open, -1: not mine

  private int[] FontBits(int info) {
    if (info==0) return MonoFnt;
    if (info==1) return TextFnt;
    if (info==2) return ItalFnt;
    if (info==3) return TinyFnt;
    if (info==4) return BoldFnt;
    if (info==5) return HeadFnt;
    if (info==6) return NanoFnt;
    if (info==7) return ScorFnt;
    if (info==0x4D) return MonoFnt;
    if (info==0x54) return TextFnt;
    if (info==0x49) return ItalFnt;
    if (info==0x59) return TinyFnt;
    if (info==0x42) return BoldFnt;
    if (info==0x48) return HeadFnt;
    if (info==0x4E) return NanoFnt;
    if (info==0x53) return ScorFnt;
    return null;} //~FontBits

  private int FontLn4Name(int rx, int info, String theList) {
    int zx, nx = -info, rez = info, why = 0;
    boolean logy = false;
    String aStr = "";
    if (Logging) if (StartupDone||GoodStart) {
      aStr = ".";
      logy = true;} //~if
    while (true) {
      why++; // why = 1
      if (info>0) break;
      why++; // why = 2
      rez = 0;
      if (info+64 >= 0) break;
      if (info+90<0) break;
      why++; // why = 3
      // nx = -info;
      aStr = "" + ((char)nx);
      nx = NthOfset(0,"[Font " + aStr + "]",theList);
      if (logy) aStr = " '" + aStr + "' @ " + nx;
      if (nx<0) { // can't find it..
        logy = true;
        break;} //~if
      why++; // why = 4
      if (nx>8) for (nx=nx-1; nx >= 0; nx += -1) {
        zx = (int)(CharAt(nx,theList));
        if (logy) aStr = aStr + " ." + nx + "=" + zx;
        if (zx>31) if (zx != 34) {
          if (logy) if (StrLength(aStr)>333) break; // ??
          continue;} //~if
        rez = SafeParseInt(Substring(nx+1,5,theList));
        if (logy) {
          zx = NthOfset(0," .",aStr);
          if (zx>0) aStr = Substring(0,zx,aStr);
          aStr = aStr + "/" + nx;} //~if
        break;} //~for
      if (rez==0) logy = true; // didn't find it
        else why = 0;
      break;} //~while
    if (logy) System.out.println("   (fnt #" + rx + ") i=" + rez + "/" + info
        + aStr + " = " + why );
    return rez;} //~FontLn4Name

  /**
  * A text list of widgets is retrieved from the user's GameList,
  * but if it doesn't seem credible, GameEngine just quits.
  *
  * @param  die   =true to quit if invalid, false rtns ""
  * @param  evh   reference to the user's GameEvent object
  *
  * @return       the list of widgets if good, else ""
  */
  private String ValidGmList(boolean die, GameEvent evh) { //
    int nx = 0, zx = 0, why = 0;
    String aLine = "", theList = "";
    while (true) {
      why++; // why = 1
      if (evh == null) break;
      why++; // why = 2
      theList = evh.GameList( theWindow );
      if (SafeParseInt(theList) !=1) break; // fails also if empty
      why++; // why = 3
      if (Countem("\n",theList)<2) break;
      why++; // why = 4
      zx = NthOfset(0,"\n",theList);
      if (zx<22) break;
      why++; // why = 5
      aLine = Substring(0,zx,theList);
      if ((SafeParseInt(NthItemOf(' ',2,aLine))&255) !=5) break;
      why++; // why = 6
      if (SafeParseInt(NthItemOf(' ',5,aLine)) <= 0) break; // tall
      why++; // why = 7
      if (SafeParseInt(NthItemOf(' ',6,aLine)) <= 0) break; // wide
      why++; // why = 8
      if (SafeParseInt(NthItemOf(' ',10,aLine)) <= 0) break; // data size
      why++; // why = 9
      if (SafeParseInt(NthItemOf(' ',11,aLine)) <= 0) break; // data[0]
      why = 0;
      nx = NthOfset(0,"{",aLine);
      zx = NthOfset(0,"}",aLine)-nx-1;
      if ((nx|zx)>0) aLine = Substring(nx+1,zx,aLine);
      else if (NthOfset(1,"\"",aLine)>0)
        aLine = NthItemOf('\"',2,aLine);
      else aLine = "";
      break;} //~while
    if (why>0) {
      System.out.println("** Invalid GameList() **"
          + IffyStr(GoodNew," $& = "," = ") + why);
      if (die) System.exit(9);
      theList = "";} //~if
    else if (Logging) System.out.println("(ValdGmLst/" + die + ") +" + zx
        + IffyStr(NoReload," NR"," !nr") + IffyStr(GoodNew," $& '"," '")
        + aLine + "'"); // aLine=name, why = 0!
    return theList;} //~ValidGmList

  /**
  * A text list of widgets is retrieved from the user's GameList, and
  * a complete set of game widgets is constructed from it.
  */
  private void ImportGamList() { // <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
    GameEvent evh = UserEvHandler;
    char xCh; // theAlertDlg = null, theInputDlg = null, mAntsWgt = null;
    int lino, kx, zx, nx = 0, mID = 0, lxx = 0, tye = 0, top = 0, left = 0,
        tall = 0, wide = 0, prnt = 0, color = 0, info = 0, fin = 0,
        logy = 0, why = 0;
    boolean logz = false;
    GameWgt whom;
    String aStr, aWord, aLine = "", xID = "", theMacs = "", theList = "";
    int[] data = null;
    if (evh != null) theList = ValidGmList(true,evh); // logs & quits if fails
    lino = NthOfset(0,"\nGameTitle = \"",theList);
    if (lino>0) // got JS title + pwd..
      Game_Title = NthItemOf('"',2,Substring(lino+8,99,theList));
    lino = NthOfset(0,"\nWgtMacros..\n",theList);
    if (lino>0) { // got new macros..
      theMacs = RestOf(lino+8,theList);
      lino = NthOfset(0,"\n\n",theMacs);
      if (lino>0) theMacs = Substring(0,lino,theMacs);
      while (true) {
        lino = NthOfset(0,"\n",theMacs);
        if (lino<0) break;
        if ((((int)(CharAt(0,NthItemOf(' ',1,theMacs)))-48)&255)>9)
          theMacs = RestOf(lino+1,theMacs);
        if (SafeParseInt(theMacs)>222) break;} //~while
      kx = Countem(",",theMacs);
      if (SafeParseInt(theMacs)>222) if (kx>0) while (true) {
        data = new int[kx+2];
        if (data==null) break;
        for (lino=kx+1; lino >= 0; lino += -1) {
          aWord = NthItemOf(',',lino+1,theMacs);
          if (NthOfset(0,"\n",aWord) >= 0) {
            zx = Countem("\n",aWord);
            aWord = NthItemOf('\n',zx+1,aWord);} //~if
          aWord = NthItemOf(' ',1,aWord);
          if (StrLength(aLine)>80)
            aLine = Substring(0,72," " + aWord + aLine) + "..";
          zx = SafeParseInt(aWord);
          if (lino<0) break;
          if (lino < Wgt_Macros.length)
              if (Wgt_Macros[lino] != zx) {
            if (Logging) System.out.println("**!!** " + lino + " " + zx + " = "
                + Wgt_Macros[lino] + " ??..\n  "+aLine);
            nx++;} //~if
          if (data==null) break;
          if (lino < data.length)
             data[lino] = zx;} //~for
        if (nx>0) {
          if (Logging) System.out.println("**!!** " + nx + " changes");} //~if
        if (Logging||(nx>0)) if (GoodStart)
          System.out.println("[IGL+Mx] " + kx + " '" + Substring(0,42,theMacs)
              );
        WgtMacros = data;
        break;} //~while
      data = null;
      theMacs = "";} //~if // (got new macros)
    if (why !=0) {}
    else if (StrLength(theList)==0) why--; // why = -1
    else for (fin=1; fin <= 2222; fin++) { // each line of input............................
      data = null;
      aLine = NthItemOf('\n',fin,theList);
      if (StrLength(aLine)==0) break; // normal exit
      why++; // why = 1
      if (Logging) if (GoodStart)
        System.out.println("[IGL] " + fin + ": '" + aLine + "'");
      if (SafeParseInt(aLine) != fin) break;
      if (fin==2) { // it's a go, initialize the stuff we import..
        theImpoList = "\n" + theList;
        WantsRollover = false;
        WantsDrag = false;
        MonoFontNo = 0;
        TextFontNo = 0;
        ItalFontNo = 0;
        TinyFontNo = 0;
        BoldFontNo = 0;
        HeadFontNo = 0;
        NanoFontNo = 0;
        ScorFontNo = 0;
        Bumpt = false;} //~if
      why++; // why = 2
      mID = 0;
      if (NthOfset(0,"`",aLine)>0) { // got a [macro] ID..
        mID = EncodeID(aLine);
        xID = myTmpStr;} //~if
      for (nx=2; nx <= 10; nx++) { // each item of this line..
        aWord = NthItemOf(' ',nx,aLine);
        if (StrLength(aWord)==0) break; // why = 2
        why++; // why = 3
        zx = (int)(CharAt(0,aWord));
        if (zx != (int)('-')) if (((zx-48)&255)>9) break; // why = 3
          // +0 t=1  0/0 256/512 c=0xFFFF99 i=1 p=0   d=0 'Game Win'
        tye = top;
        top = left;
        left = tall;
        tall = wide;
        wide = color;
        color = info;
        info = prnt;
        prnt = lxx;
        lxx = SafeParseInt(aWord);
        if (Logging) if (GoodStart) if (fin<33)
          System.out.println("... " + fin + "/" + nx + " = " + lxx);
        if (nx==10) why++; // why = 4
          else why--;} //~for // (each item of this line)
      zx = tye&255;
      if ((tye&256) !=0) Bumpt = true;
      if (info==0) if (mID>0) if ((zx==5)||(zx==0)) // if (tye>255)
         info = mID; // NOT: mID = 0; (might be inner wgts)
      if (Logging) if (GoodStart) if (fin<33) System.out.println(" => " + fin
          + ": " + tye + " " + top + " " + left + " " + tall + " " + wide + " "
          + prnt + SeeHex(" ",color,SeeHex(" ",info," +")) + lxx + " = " + why);
      if (why<4) break;
      if (lxx>0) { // we have data..
        data = new int[lxx];
        if (data==null) break; // why = 4
        why++; // why = 5
        for (nx=0; nx <= lxx-1; nx++) { // each child wgt + ..
          aWord = NthItemOf(' ',nx+11,aLine);
          if (StrLength(aWord)==0) break; // why = 5
          why++; // why = 6
          zx = (int)(CharAt(0,aWord));
          if (zx != (int)('-')) if (((zx-48)&255)>9) break; // why = 6
          zx = SafeParseInt(aWord);
          if (Logging) if (GoodStart) if (fin<33)
            System.out.println(" ++ " + fin + " " + nx + "/" + lxx + " = " + zx);
          if (data != null) if (nx < data.length)
            if (nx >= 0) data[nx] = zx;
          if (nx+1 == lxx) why++; // why = 7
            else why--;}} //~if //~for (nx: each child wgt +)
      aWord = "";
      if (NthOfset(0,"'",aLine)>0) { // we have quoted text..
        aWord = NthItemOf('\'',2,aLine);
        if (NthOfset(0,"`",aWord)>0) aWord = ReplacAll("'","`",aWord);
        if (StrLength(aWord)>0) if (lxx==0) if ((tye&255)==4) {
          lxx = StrLength(aWord)+4;
          data = new int[lxx>>2];
          if (data==null) break; // why = 7
          for (nx=0; nx <= lxx; nx += 4) { // each char of text wgt's data..
            zx = GameWgt. Pack42i(Substring(nx,4,aWord),GoodEnd,false);
            kx = nx>>2;
            if (data==null) break;
            if (kx<0) break;
            if (kx >= data.length) break;
            data[kx] = zx;
            if (Logging) if (GoodStart) if (fin<33)
              System.out.println(" ++ " + fin + " " + nx + "/" + lxx // + " = " + zx
                  + " => [" + kx + SeeHex("] x", zx,IffyStr(nx>0,""," '"
                  + aWord + "'")));} //~for // (each char of text wgt's data)
          aWord = "";}} //~if
      why++; // why = 8
      zx = tye&255;
      if (zx>222) { // queue up this macro for later processing..
        theMacs = theMacs + aLine + "\n";
        tye = (tye&-256)|5;} //~if
      else if (zx==4) {
        if (info+64<0) info = FontLn4Name(fin,info,theList);
        zx = 4;} //~if
      else if (zx==3) if (data==null)  // if (info&15 == info)
        data = FontBits(info);
        //  NewGameWgt(int tye, int top, int left, int tall, int wide,
        //    int prnt, int color, int info, int slm, int[] data, GameWgt whom)
      why++; // why = 9
      if (fin==1) if (NextRefNo==0) {
        whom = NewGameWgt(1,top,left,tall,wide,0,color,1,0,null,null);
        if (whom==null) break; // why = 9
        GameRoot = whom;} //~if // (unused)
      why++; // why = 10                             // (in ImportGamL)..
      whom = NewGameWgt(tye,top,left,tall,wide,prnt,color,info,0,data,null);
        // ("..nuGmWgt #" + rnum + "=" + tye + " " + top + "/" + left + " " + tall
        //    + "/" + wide + " " + info + "/" + color + " " + data + " " + prnt
      // => ..nuGmWgt +0=1 0/0 256/512 1/16777113 false 0
      if (whom==null) break; // why = 10
      if (fin>1) {
        if ((tye&4096) !=0) WantsRollover = true;
        if ((tye&2048) !=0) WantsDrag = true;} //~if
      nx = whom.GetRefNum();
      if (zx==3) {
        if (info==0) {
          MonoFontNo = nx;
          KnownFonts = KnownFonts|0x2000;} //~if
        else if (info==1) {
          TextFontNo = nx;
          KnownFonts = KnownFonts|0x100000;} //~if
        else if (info==2) {
          ItalFontNo = nx;
          KnownFonts = KnownFonts|0x200;} //~if
        else if (info==3) {
          TinyFontNo = nx;
          KnownFonts = KnownFonts|0x2000000;} //~if
        else if (info==4) {
          BoldFontNo = nx;
          KnownFonts = KnownFonts|2;} //~if
        else if (info==5) {
          HeadFontNo = nx;
          KnownFonts = KnownFonts|0x100;} //~if
        else if (info==6) {
          NanoFontNo = nx;
          KnownFonts = KnownFonts|0x4000;} //~if
        else if (info==7) {
          ScorFontNo = nx;
          KnownFonts = KnownFonts|0x80000;}} //~if
      if (Logging) whom.LogMe(".. ");
      why++; // why = 11
      if (nx != fin) break;
      why = 0;} //~for // (fin: each line of input)
    if (why==0) for (lino=1; lino <= 2222; lino++) { // each macro line.......................
      data = null;
      aLine = NthItemOf('\n',lino,theMacs);
      if (StrLength(aLine)==0) break; // normal exit
      why = 12;
      logz = false;
      nx = SafeParseInt(aLine);
      if (logy>0) if (((logy>>16)==nx)||((logy&0xFFFF)==nx)) logz = true;
      if (Logging) if (logz || GoodStart)
        System.out.println("[IGLmax] " + lino + ": '" + aLine + "'");
      whom = GetIxWgt(nx);
      if (whom==null) break;
      why++; // why = 13
      if (whom.GetRefNum() != nx) break;
      why++; // why = 14
      if ((whom.GetType()&255) != 5) break;
      why++; // why = 15
      why = BuildThisMacro(whom,aLine,why,logz);
      if (why>0) break;
      if (why==0) continue;
      theMacs = theMacs + myTmpStr;
      myTmpStr = "";} //~for // (lino: each macro line)
    if (why==0) {
      GameWgtsList = FindListWgt("[List of item lists]");
      SpriteList = FindListWgt("[Sprite list]");
      theAlertDlg = FindListWgt("[Std Alert Dialog]");
      theInputDlg = FindListWgt("[Std Input Dialog]");
      if (theInputDlg != null) InputDlgNo = theInputDlg.GetRefNum();
      mAntsWgt = FindListWgt("[Deployable Marching Ants]");
      if (mAntsWgt != null) Antsify(false,mAntsWgt);
      if (theInputDlg != null) {
        Widen1stWgt(theInputDlg,-64);
        theInputDlg.HideSho(false);
        Floatem = 2;} //~if
      if (theAlertDlg != null) {
        theAlertDlg.HideSho(false);
        Floatem++;}} //~if
    if (Logging) if ((why>0)||GoodStart)
      System.out.println("[IGL] = " + why + " f=" + Floatem + " #=" + fin
          + " m=" + lino + IffyStr(WantsDrag," wD "," -d ")
          + IffyStr(WantsRollover,"wR\n","-r\n") + theMacs);} //~ImportGamList

  /**
  * All known widgets are logged to the system console. Short prefixes are
  * repeated every line, longer prefixes are replaced with ".. " after the
  * first line.
  *
  * @param  why   a prefix string inserted in front of first/each log line
  */
  public static void LogAlWgts(String why) { // caller ensures if (Logging)
    int lino = StrLength(why);
    boolean didit = false;
    GameWgt whom = AllWgts[0];
    if (whom != null) {
      whom.LogMe(why);
      if (lino>4) why = ".. ";
      for (lino=1; lino <= MaxWgts; lino++) {
        whom = GetIxWgt(lino);
        if (whom==null) {
          System.out.println("<AllWgts[" + lino + "]=null>");
          if (didit) break;
          didit = true;
          continue;} //~if
        whom.LogMe(why);}} //~if //~for (lino)
    else System.out.println("<AllWgts[0]=null>");} //~LogAlWgts

  /**
  * Determines if the ASCII code of a character is a valid letter or digit.
  * Only the original ASCII character set is supported. '_' is a letter.
  *
  * @param  whom  the ASCII code of a character
  *
  * @return       true if it's a letter or digit
  */
  public boolean IzLetter(int whom) {
    if (whom<48) return false;
    if (whom<58) return true;
    if (whom<65) return false;
    if (whom<90) return true;
    if (whom<97) if (whom !=95) return false;
    if (whom>122) return false;
    return true;} //~IzLetter

  // special-case class vars, GameList; maybe reduce Head font by usage
  // remove 'myGame.', 'JavaGame.', 'super.', '!= null'; cvt '=null' to =0
  // JSwgtMeths cvt 'xxx.zzz(' => 'zzz(xxx,'; OrideGame cvt 'zzz(' => 'o_zzz('

  // 0:Gtype, 1:Gcolor, 2:Gtop,Gleft, 3:Gtall,Gwide,
  // 4:Ginfo, 5:Gselim, 6:Parent, 7:Gdata;

  /**
  * Constructs an array representation of the widget list for JavaScript.
  *
  * @param  theText  the widget list as returned by GameList
  *
  * @return          the text of an array of numbers for the JS GameEngine
  */
  private String MakeJSwgts(String theText) { // only from 2JS
    // for English wgts, assumes GameMaker.txt has already been written
    int lxx = Countem("\n",theText)+myPL, base = 9990000, why = 16, sofar = 8;
    boolean doit, etx, logy = Logging && (lxx<22);
    int lino, ix, kx, nx, zx, top, left, tall, wide, info, whom,
        here, thar = 0, narys = 0, tye = 0, later = 0, moor = 0;
    boolean nequo = false;
    int[] pix = null;
    int[] anfo = new int[256];
    int[][] TmInAr = new int[256][];
    String ary = "", soon = "", prio = "", aLine = "",
        aWord, aStr, xStr, colo, prnt, fin = NthItemOf('\n',lxx,theText),
        data = NthItemOf('\n',1,theText) + "\n      = \""; // = the result str
    GameWgt nuly = null;
    while (true) {
      info = GetGameValue(11); // = InputDlgNo
      nuly = GetIxWgt(info); // = theInputDlg
      info = GetGameValue(0); // = MonoFontNo
      nuly = FindSubWgt(nuly,(info<<16)+0x0604,0); // if (nuly != null)
      for (lino=1; lino <= lxx-1; lino++) { // each line of wgt list..
        xStr = ",\"";
        etx = false;
        doit = false;
        aLine = NthItemOf('\n',lino+1,theText);
        if (StrLength(aLine)==0) break; // why = 16: poor form (unexp'd blnk ln)
        if (NthOfset(0,"\"\"",aLine)>0) { // normal exit..
          why = 0;
          break;} //~if
        aLine = NthItemOf('\"',2,aLine); // remove line from enclosing quotes
        zx = SafeParseInt(aLine);
        if (zx != lino) {
          why++; // why = 17
          if (!logy) break;} //~if // why = 17: invalid line +
        // here = (lino-1)*8;
        tye = SafeParseInt(NthItemOf(' ',2,aLine));
        whom = tye&255;
        if (whom>7) {
          why = 18;
          if (!logy) break;} //~if // why = 18: no macros or private wgt types
        top = SafeParseInt(NthItemOf(' ',3,aLine));
        left = SafeParseInt(NthItemOf(' ',4,aLine));
        tall = SafeParseInt(NthItemOf(' ',5,aLine));
        wide = SafeParseInt(NthItemOf(' ',6,aLine));
        if (etx) {
          logy = Logging;
          if (whom !=4) etx = false;
          else if ((top|left)<256) etx = false;
          else if (left>300) if (wide>200) left = left-300;} //~if
        colo = NthItemOf(' ',7,aLine);
        aWord = NthItemOf(' ',8,aLine);
        prnt = NthItemOf(' ',9,aLine);
        info = SafeParseInt(aWord);
        nx = SafeParseInt(NthItemOf(' ',10,aLine));
        if (StrLength(prnt)==0) why = 20; // why = 20: line is too short
        if (nx<0) why = 21; // why = 21: invalid negative data length
        tall = (tall<<16)+wide;
        top = (top<<16)+left;
        if (Logging) if (logy)
          System.out.println(IffyStr(etx,"  ..Mk.t+ ","  ..Mk.. ")
              + lino + ": " + tye + LogVH(" ",top,LogVH(" ",tall," c="))
              + colo + " i=" + aWord + " '" + prnt + "' " + nx
              + " '" + Substring(0,25,aLine) + "..'");
        if (why>16) break; // why = 17/18/20/21
        if (lino==1) { // special-case 1st line..
          data = data + "1," + colo + HexIfMore(",0,",tall,",1,0,0,0,\"");
          colo = "-1";} //~if
        if (whom==5) { // sprite list, trim excess zeros..
          zx = nx; // (for log)
          if (nx>8) for (kx=nx+10; kx >= 11; kx += -1) {
            aStr = NthItemOf(' ',kx,aLine);
            if (StrLength(aStr)==1) if (CharAt(0,aStr) == '0') continue;
            if (kx<18) nx = 8;
            else if (kx<nx+5) nx = kx-8;
            break;} //~for
          if (Logging) if (logy)
            System.out.println("    ..Mk+5 #" + lino + " " + nx + "/" + zx);
          aWord = CvInt2Str(nx);} //~if
        else if (whom==4) {
          if (info<0) { // replace font line + for font name..
            info = FontLn4Name(lino,info,theText); // =0 if can't find it (see log)
            if (info >= 0) aWord = CvInt2Str(info);} //~if
          aStr = NthItemOf('\'',2,aLine);
          here = NthOfset(0,"'" + aStr + "'",aLine);
          if (here>0) aLine = Substring(0,here,aLine);
          else for (thar=1; thar <= 16; thar++) { // this shouldn't happen, so OK if slo..
            nx = StrLength(aLine);
            aLine = ReplacAll(" ","  ",aLine);
            if ((int)(CharAt(0,aLine))<33) aLine = RestOf(1,aLine);
            zx = StrLength(aLine);
            if (zx<nx) {nx = zx;
              continue;} //~if
            zx = Countem(" ",aLine)+myPL;
            if (zx==10) break;
              // if (StrLength(aLine)>nx) continue; break;} //~if
            if (zx>10) {
              if (zx>11) nx = nx-(zx-10)*2;
                else nx--;
              aLine = Substring(0,nx,aLine);
              continue;} //~if
            if (nx>0) if ((int)(CharAt(nx-1,aLine))>32)
              aLine = aLine + " ";
            aLine = aLine + "0 ";} //~for
          aStr = ReplacAll("'","`",aStr);
          nx = StrLength(aStr);
          for (zx=0; zx <= nx; zx += 4) {
            info = 0;
            if (zx<nx) info = GameWgt. Pack42i(Substring(zx,4,aStr),
                GoodEnd,false);
            aLine = aLine + HexIfMore(" ",info,"");} //~for
          aLine = aLine + " 0 '" + aStr + "'";
          if (nx>2) {
            soon = soon + aLine + "\n";
            later++; // counts the lines in soon..
            nx = (nx>>2)+1;   // ..so we can find this line to process it..
            nx = -(nx*256+later);} //~if
          else nx = 1; }
        else if (whom==6) { // sprite..
          xStr = ",\"";}
        else if (whom==3) { // font..
          soon = soon + aLine + "\n";
          later++;
          doit = true;
          nx = -later;} //~if
        else if (whom==2) { // icon..
          pix = null;
          aStr = "";
          kx = 0;
          if (info+3 !=0) while (true) { // not a circle..
            zx = 0;
            if (zx <= 0) break; // no parent
            aStr = NthItemOf('\n',zx+1,theText);
            aStr = NthItemOf('\"',2,aStr); // remove from enclosing quotes
            if (SafeParseInt(aStr) == zx) { // else something went wrong
              aWord = "0123456789." + NthItemOf('<',2,NthItemOf('>',1,aStr));
              nx = SafeParseInt(NthItemOf(' ',10,aStr))+10;
              for (nx=nx; nx >= 11; nx += -1) {
                if (logy) aStr = aStr + " " + nx;
                if (SafeParseInt(NthItemOf(' ',nx,aStr)) != lino) continue;
                kx = (int)(CharAt(nx,aWord));
                if (logy) aStr = aStr + "=" + kx + " '" + aWord + "'";
                kx = kx-96;
                if (kx<0) kx = 0;
                else if (kx>26) kx = 0;
                break;} //~for
              } //~if
            zx = 0;
            if (false) for (nx=7; nx >= 1; nx += -1) {
              aWord = NthItemOf(' ',nx,aStr);
              zx = StrLength(aWord)+zx+1;} //~for
            break;} //~while // (not a circle)
          else { // known circle..
            if (Logging) aStr = "(C)"; // for log
            zx = NthOfset(0,aWord,aLine); // aWord is info
            pix = KnownCircleData(wide,0);} //~else // (known circle)
          // aLine = Substring(0,zx,aLine) + "0 " + prnt + " ";
          info = 0;
          ix = 0;
          zx = 0;
          if (pix != null) if (TmInAr != null) for (nx=1; nx <= narys; nx++)
            if (nx<TmInAr.length) if (TmInAr[nx&255] == pix) {
              info = nx;
              if (anfo == null) break;
              if (nx < anfo.length)
              zx = anfo[nx]&1023;
              break;} //~if
          if (zx==0) {
            zx = SafeArySz(pix);
            for (nx=zx-1; nx >= 0; nx += -1) { // trim 0s off end of shared (pixel) array..
              if (SafeAryElt(nx,pix,0) !=0) {
                if (nx<7) zx = 8;
                else if (nx+4<zx) zx = nx+2;
                break;} //~if
              if (nx==0) zx = 0;} //~for // shouldn't be zx=0
            if (zx==0) info = 0;
            else if (info==0) if (narys<255) if (TmInAr != null)
                if (anfo != null) {
              narys++;
              info = narys;
              if (info>0) if (info<256) {
                TmInAr[info&255] = pix;
                ix = (kx<<10)+zx; // zx is size, kx is table+ (letter)
                if (info < anfo.length)
                  anfo[info] = ix;}}} //~if
          aWord = "0"; // (new info)
          if (zx>0) nx = zx*256+info+0x02000000;
            else nx = 0x01000400; // default 4 zeros if nothing
          if (Logging) if (logy)
            System.out.println("{Mk+K #" + lino + "} " + prnt + " " + (pix != null)
                + " " + kx + "+" + zx + " [" + info + SeeHex("] (=x",ix,
                SeeHex(") => x",nx,"\n   '")) + aStr + "'");} //~if // (whom=2)
        if (nx>0x01000000) // shared data or all zeros..                <2JS>
          aStr = HexIfMore(Substring(0,(nx>>24)+2,"$$$$"),nx,"");
        else if (nx>0) // nx>0 is + data elts; moor counts total added to ary..
          aStr = HexIfMore("$$",(nx<<16)+moor,"");
        else if (doit) aStr = "$" + CvInt2Str(nx); // it's a font (last)
        else if (nx<0) aStr = "$$" + CvInt2Str(nx);
          else aStr = "0";
        if ((lino&3)==0) xStr = IffyStr(StrLength(xStr)>2,xStr,xStr + " //")
            + " #" + lino + " @ " + (lino*8) + "..";
        data = data + "\n      + " + Quot2 + tye + "," + colo
            + HexIfMore(",",top,HexIfMore(",",tall,",")) + aWord + ",0,"
            + prnt + "," + aStr + xStr;
        sofar = sofar+8; // counts the ints in data as gen'd
        if (nx<0x1000000) for (zx=1; zx <= nx; zx++) {
          if ((moor&15)==0) // add this many ints to ary (count in moor)..
            ary = IffyStr(moor>0,ary + "\"\n      + \"","\n      + \"");
          ary = ary + NthItemOf(' ',zx+10,aLine) + ",";
          moor++;}} //~for // (each line of wgt list)
      if (why>0) break; // why = 16: poor form (no end marker by line +255)
      if (Logging) if (logy) System.out.println("  (MkJSwgt) " + sofar
          + " +" + moor + " -" + later + ": ---->\n" + soon + "<----\n"
          + theText + " --->\n" + data + "<--- ===>\n" + ary + "<=== [" + narys
          + "]" );
      later = 0; // counts line length being added to ary
      whom = 0; // counts line length being added to data
      tall = StrLength(data)-111;
      while (true) { // each '$' data ref.....................................
        pix = null;
        aWord = "";
        here = 0;
        lino = 0;
        thar = 0;
        zx = 0;
        for (nx=4; nx >= 0; nx += -1) {
          zx = nx;
          if (nx==0) break; // normal exit
          here = NthOfset(0,Substring(0,zx+1,",$$$$"),data);
          if (here>0) break;} //~for // do larger runs first
        nx = 0;
        if (here>0) {
          aWord = NthItemOf(',',1,Substring(here+zx+1,10,data));
          nx = SafeParseInt(aWord);
          if (nx>0x200000) {
            lino = nx&255; // where the data is
            thar = (nx>>8)&0xFFFF;}} //~if
        else if (logy) aWord = "";
        if (Logging) if (logy) System.out.println("...(MkJSw) " + here + "/" + zx
            + " '" + aWord + IffyStr(nx>0x200000,"' (#" + lino + "+"
              + thar + ") +","' +") + sofar );
        if (zx==0) break; // normal exit
        if (here <= 0) break; // ditto, but got in on previous line
        here++;
        // aWord = NthItemOf(',',1,Substring(here+zx,10,data));
        zx = StrLength(aWord)+zx;
        // nx = SafeParseInt(aWord);
        thar = nx; // the data size
        if ((nx&-0x1000) !=0) {
          if (nx<0) thar = -nx;
          lino = thar&255; // where the data is
          thar = (thar>>8)&0xFFFF;} //~if
        if (nx<0) { // we have array data stashed in soon..
          aLine = NthItemOf('\n',lino,soon);
          tye = SafeParseInt(NthItemOf(' ',2,aLine))&255;
          if (tye==3) {
            info = SafeParseInt(NthItemOf(' ',8,aLine));
            pix = FontBits(info);
            thar = SafeArySz(pix);} //~if
          aStr = HexIfMore("",(thar<<16)+sofar+moor,"");} //~if
        else if (nx>0x02000000) { // we have shared array data..
          if (narys>0) {
            for (nx=narys; nx >= 1; nx += -1) {
              doit = true; // T: start new line
              pix = null;
              whom = 0;
              if (nx<0) break;
              if (TmInAr == null) break;
              if (anfo == null) break;
              if (nx >= anfo.length) break;
              thar = anfo[nx]; // =size+name*1K
              if (thar <= 0) continue;
              anfo[nx] = (sofar<<16)+thar; // sofar=offs; thar: tb+:6, sz:10
              if (nx < TmInAr.length)
                pix = TmInAr[nx];
              if (pix == null) continue;
              top = sofar;
              kx = (thar>>10)&31;
              thar = thar&0x3FF; // now it's the size
              colo = "..";
              if (kx>0) if (kx<27) colo = " ("
                  + Substring(kx,1," ABCDEFGHIJKLMNOPQRSTUVWXYZ") + ")";
              if (Logging) if (logy)
                System.out.println("  ...(" + nx + ") #" + kx + "+" + thar
                    + " @ " + top + colo);
              for (kx=0; kx <= thar; kx++) {
                info = SafeAryElt(kx,pix,0);
                aWord = HexIfMore("",info,",");
                left = StrLength(aWord);
                if (left+whom>77) doit = true; // T: start new line
                if (doit) { // start new line/ary..
                  if (top==sofar) doit = false; // top=sofar at front of ary
                  else if (top==0) doit = false; // top=0 after start 2nd line
                  data = data + IffyStr(nequo,Quot2,"")
                      + IffyStr(doit," // @ " + top + colo,"")
                      + IffyStr(kx<thar,"\n      + " + Quot2,"");
                  colo = "";
                  nequo = false;
                  if (kx==thar) break; // normal exit
                  nequo = true;
                  if (top==sofar) whom = 20;
                    else whom = 9;
                  if (doit) top = 0;} //~if
                doit = false;
                whom = whom+left;
                data = data + aWord;
                nequo = true;
                sofar++;}} //~for
            if (Logging) if (logy) System.out.println("\n+++ " + sofar + " "
                + narys + " ----> .." + RestOf(tall,data) + "<----\n");
            narys = -narys;} //~if
          top = sofar;
          left = 0;
          if (anfo != null) if (lino>0)
            if (lino < anfo.length)
              left = anfo[lino];
          if (left>4096) { // frex anfo[1]=x06441, [2]=x421 (both clock)
            thar = left&1023; // the data size
            top = left>>16;} //~if // top = existing offset (sofar) in data
          aStr = HexIfMore("",(thar<<16)+top,"");
          nx = 0;} //~if
        else if (nx>0x01000000) { // unshared all zeros (text lines)..
          if (thar==0) continue; // no data (can't)
          aStr = HexIfMore("",(thar<<16)+sofar,"");
          aWord = IffyStr(nequo,Quot2,"") + "\n      + " + Quot2;
          for (nx=0; nx <= thar-1; nx++) aWord = aWord + "0,"; // usually 32 ints
          aWord = aWord + "\" // @ "
              + sofar + "..";
          data = data + aWord;
          nequo = false;
          sofar = sofar+thar;
          nx = 0;} //~if
        else aStr = HexIfMore("",nx+sofar,"");
        data = Substring(0,here,data) + aStr + RestOf(here+zx,data);
        if (nx >= 0) continue; // data is already in ary
        if (tye==3) {
          if ((Qlog&128) !=0) continue;
          if (pix==null) {
            why = 22;
            break;}} //~if // why = 22: no such font
        doit = true;
        prio = "";
        for (nx=0; nx <= thar; nx++) {
          info = 0;
          if (tye !=3)
            info = SafeParseInt(NthItemOf(' ',nx+11,aLine));
          else if (pix != null) if (nx >= 0)
            if (nx < pix.length)
              info = pix[nx];
          aStr = HexIfMore("",info,",");
          zx = StrLength(aStr);
          if ((zx+later>80)||doit) {
            ary = ary + Quot2 + prio + "\n      + \"";
            later = 11;
            // if (whom>60) doit = true;
            if (doit) {
              prio = " // @ " + (sofar+moor) + "..";
              later = StrLength(prio)+later;
              whom = 0;} //~if
            else prio = "";
            doit = false;} //~if
          ary = ary + aStr;
          later = later+zx;
          moor++;}} //~while //~for (nx) // (each '$' data ref)
      data = data + IffyStr(nequo,Quot2,"") + ary + "0\";\n";
      why = 0;
      break;} //~while // (once thru from top)
    if (why>0) data = CvInt2Str(why) + data; // sho err msg
    return data + fin;} //~MakeJSwgts

  /**
  * Converts the Java code of your game into JavaScript and stores
  * the result in class variable JSinfo, and also writes to fiName.js
  * if successful. If there's an error, the error description is stored
  * in class variable JSinfo (length less than 99).
  *
  * @param  theText  the Java code
  * @param  fiName   the game name, should = the class name
  *
  * @return          true if successful
  */
  public boolean toJS(String theText, String fiName) {
    // Phases: [why]
      // delete the main... [1]
      // convert 'Lscore = myGame.FindListWgt("{Lscore}");' to ..const.. [2]
      // make wgt array... [3]
      // convert ass't to rtn... [4]
      // remove excess blank lns [8]
      // remove comments [8]
      // replace var/fun types [9]
      // fix array alloc'ns [..]
      // unsupported method calls... [10]
      // GameEvent overrides [11]
      // GameWgt method call fixups... [12]
      // we know how to do these kinds of things: (null fixes)... [13]
      // Generated text too short [15]
    // char xCh;
    int here, thar, whar, sz, zx, nx, kx = Countem("\n",theText)+myPL,
        lxx = kx, ofx = StrLength(theText), nCh = 0, lino = 0, why = 0;
    boolean dont = false, logy = false, logz = false;
    String aStr, xStr, aLine, aWord = "", ary = "",
        JWM = JSwgtMeths,
        OrGM = OrideGameMeths,
        CDM = CantDoMeths,
        JvTy = JavaTypes;
    if (Logging) if (lxx<777) logy = true;
    if (lxx==0) if (StrLength(fiName)>0) {
      theText = ReadWholeTextFile(fiName + ".java");
      lxx = Countem("\n",theText)+myPL;} //~if
    here = NthOfset(0,"public " + fiName + "()",theText);
    whar = 0;
    if (here>0) if (here+99>ofx) {
      theText = Substring(0,here,theText) + "\nGameStart();\n";
      whar = ofx-here;
      ofx = here+16;} //~if
    if (false) if (Dev_Log) {
      if (ofx<999) xStr = theText;
      else if (SafeCompare(fiName,"Pong")==0) xStr = theText;
        else xStr = Substring(0,999,theText);
      System.out.println("++ 2JS  '" + fiName + "' +" + ofx + " -"  // ++ 2JS
          + whar + " " + logz + " @ " + here + "....\n" + xStr
          + "...." );}
    while (true) {
      why++; // why = 1
      here = NthOfset(0,"public class ",theText);
      thar = NthOfset(0," extends GameEvent {",theText);
      if (here<0) break; // why = 1: can't find class name
      lino = Countem("\n",Substring(0,here+1,theText))+myPL;
      if (thar<here) break;
      xStr = NthItemOf(' ',1,Substring(here+12,thar-12-here,theText));
      if (StrLength(xStr)==0) break; // why = 1: can't find class name
      if (SafeCompare(xStr,fiName) !=0) break; // ..or != fiName
      theText = Replace("\r",here,thar+20-here,theText);
      fiName = fiName + ".js";
      here = NthOfset(0,"void main()",theText); // int main(String args[]) ??
      thar = 0;
      if (here>0) for (here=here-1; here >= 0; here += -1) { // delete the main..................
        zx = (int)(CharAt(here,theText));
        if (zx>31) continue;
        if (zx <= 0) break;
        if (thar>0) { // ..by including it in its own Jdoc block-comment..
          aLine = Substring(here,thar-here,theText); // (soon to be del'd)
          zx = NthOfset(0,"*/",aLine);
          if (false) System.out.println("  --main " + here + "/" + thar
              + ": " + zx + " '" + aLine + "'");
          if (zx<0) break;
          theText = Replace("\n\r",here,thar-here,theText);
          break;} //~if
        else thar = here;} //~for
      why++; // why = 2
      while (true) { // convert 'Lscore = myGame.FindListWgt("{Lscore}");' to ..
        here = NthOfset(0,"myGame.FindListWgt(\"",theText);        // ..const..
        if (here<0) break; // normal exit
        lino = Countem("\n",Substring(0,here+1,theText))+myPL;
        aStr = NthItemOf(';',1,Substring(here,99,theText));
        sz = StrLength(aStr);
        nx = NthOfset(0,"\"",aStr);
        thar = 0;
        if (nx>0) {
          aWord = NthItemOf('\"',2,aStr);
          thar = NthOfset(0,aWord,theText);} //~if
        if (false) System.out.println("  wgt=const " + here + " +" + sz
            + " " + nx + "/" + thar + " '" + aStr + "'");
        if (nx<0) break; // why = 2: invalid search string
        if (thar>99) {
          aLine = Substring(thar-99,99,theText);
          nx = Countem("\n",aLine)+myPL;
          aLine = NthItemOf('\n',nx,aLine)
              + NthItemOf('\n',1,Substring(thar,99,theText));
          aLine = NthItemOf('\"',2,aLine);
          if (NthOfset(0,aWord,aLine)>0) thar = SafeParseInt(aLine);
            else thar = 0;} //~if
        else thar = 0; // didn't find it (presumed OK)
        theText = Replace(CvInt2Str(thar*8),here,sz,theText);} //~while // (convert
      here = NthOfset(0,"<$..>",theText);
      thar = NthOfset(0,"<..$>",theText);
      if (Logging) System.out.println("** 2JS  '" + fiName        // ** 2JS
          + "' +" + lxx + "/" + kx + " (" + ofx + ") " + here + "/"
          + thar );
      why++; // why = 3
      if (here<333) break; // why = 3: Can't find widget list
      if (thar<here) break;
      aStr = MakeJSwgts(Substring(here,thar-here,theText)); // make wgt array...
      nx = SafeParseInt(aStr);
      if (nx>0) if (nx<32) { // it has its own error message..
        why = nx;
        break;} //~if // why = 16+..
      why++; // why = 4
      nx = NthOfset(0,"=",aStr); // convert ass't to rtn..........
      if (nx<4) break; // why = 4: Failed wgt list conversion
      if (nx>88) break;
      aStr = "\n    myGame = whom;\n    return " + RestOf(nx+1,aStr);
      why++; // why = 5
      nx = NthOfset(1,";",aStr);
      if (nx<44) {
        aWord = aStr;
        break;} //~if // why = 5: Invalid wgt list end
      aStr = Substring(0,nx+1,aStr);
      if (NthOfset(0," myGame = whom;",aStr)<0)
        aStr = "\n    myGame = whom;" + aStr;
      why++; // why = 6
      ofx = here-255;
      for (nx=here-1; nx >= ofx; nx += -1) { // ..inside the braces..
        if (CharAt(nx,theText) != '{') continue;
        here = nx+1;
        why++; // why = 7 (OK)
        break;} //~for
      if (why==6) {
        aWord = Substring(ofx,here+8-ofx,theText);
        break;} //~if // why = 6: GameList front corrupted
      ofx = thar+255;
      for (nx=thar+5; nx <= ofx; nx++) {
        if (CharAt(nx,theText) != '}') continue;
        thar = nx-here;
        why++; // why = 8 (OK)
        break;} //~for
      if (why==7) {
        aWord = Substring(thar,ofx-thar,theText);
        break;} //~if // why = 7: GameList back corrupted
      theText = Replace(aStr,here,thar,theText);
      lxx = Countem("\n",aStr)+lxx+myPL; // probly got bigger
      // why++; // why = 8
      ofx = StrLength(theText);
      if (NthOfset(0,"\n\n\n\n",theText) >= 0) while (true) {
        theText = ReplacAll("\n\n","\n\n\n",theText); // remove excess blank lns
        sz = StrLength(theText);
        if (sz<ofx) {
          lxx = lxx-ofx+sz; // reduce the line count
          ofx = sz;} //~if
        else break;} //~while // (remove excess blank lns)
      kx = 0;
      zx = 0;
      for (lino=1; lino <= 22222; lino++) { // remove comments (1*brk, 5*cont) ..............2JS
        aLine = NthItemOf('\n',lino,theText);
        sz = StrLength(aLine);
        if (sz==0) if (lino>33) {
          if (kx+8<lino) break; // why = 8: too many blank lines in a row..
          continue;} //~if  // ..already got rid of them, this must be an error
        kx = lino;
        ofx = 0;
        here = 0;
        if (sz>0) {
          if (!dont) {
            thar = NthOfset(0,"//",aLine);
            if (thar<0) here = NthOfset(0,"/*",aLine);} //~if
          else thar = NthOfset(0,"*/",aLine);    // prior comments already gone..
          if (here >= 0) ofx = NthOfset(0,aLine,theText);} //~if
        if (Logging) if (logy||logz)
          System.out.println("  .. " + lino + IffyStr(dont,": /* ",IffyStr(thar<0,
                IffyStr(sz>30,": _ ",": (" + lxx + ") "),": // "))
              + here + "/" + thar + " @ " + ofx + " +" + sz + " '"
              + Substring(0,50,aLine) + IffyStr(sz>50,"..' ","' ") + zx);
        if (sz==0) continue;
        if (here<0) continue; // only if no comments on this line
        zx = 0;
        aStr = "\r";
        if (dont) { // thar = NthOfset(0,"*/",aLine);
          if (thar >= 0) {
            aLine = RestOf(thar+2,aLine);
            if (StrLength(NthItemOf(' ',1,aLine))>0) aStr = aLine;
            dont = false;} //~if
          // theText = PutLineOf(aStr,lino,theText);
          theText = Replace(aStr,ofx,sz,theText);
          zx = -2;
          continue;} //~if
        aWord = NthItemOf(' ',1,aLine);
        if (StrLength(aWord)==0) continue;
        here = NthOfset(0,Quot2,aLine);
        // thar = NthOfset(0,"//",aLine);
        if (here >= 0) if (here<thar) thar = -4;
        if ((int)(CharAt(0,aWord)) == 47) { // '/'
          nCh = (int)(CharAt(1,aWord));
          zx++; // zx = 1
          if (nCh == 42) { // '*'
            dont = true;
            zx++;} //~if // zx = 2
          else if (nCh == 47) { // comment, is it unused o'ride meth?..
            thar = NthOfset(0,"(",aLine);
            if (thar>16)
                if (SafeCompare(NthItemOf(' ',2,aLine),"public")==0) {
              aStr = Substring(0,thar+1,aLine);
              xStr = "_";
              zx = Countem(" ",aStr)+1; // might be doubled-up spaces..
              while (zx>0) {
                xStr = NthItemOf(' ',zx,aStr);
                if (StrLength(xStr)>0) break;
                zx--;
                if (zx==0) xStr = "_";} //~while // so the search will fail
              aStr = "_" + xStr;
              zx = NthOfset(0,aStr,OrGM); // zx = >100/-1
              if (zx>0) aStr = "function o" // yup
                  + NthItemOf('\n',1,Substring(zx,99,OrGM));
              else aStr = "\r";}} //~if
          else if (thar>0) { // thar = NthOfset(0,"//",aLine);
            aStr = Substring(0,thar,aLine);
            zx = StrLength(aStr)+3;} //~if
          else continue; // zx = 1
          theText = Replace(aStr,ofx,sz,theText);} //~if
        else if (thar>0) {
          aStr = Substring(0,thar,aLine);
          theText = Replace(aStr,ofx,sz,theText);
          if (false) System.out.println("  .. " + lino + ": //" + thar
              + " - '" + RestOf(thar,aLine) + "'");}} //~for // (remove comments)
      // if (kx+8<lxx) break; // why = 8: too many blank lines in a row
      lxx = kx+8;
      why++; // why = 9
      dont = false;
      for (lino=1; lino <= lxx; lino++) { // replace var/fun types (no errors this segment)..2JS
        aLine = NthItemOf('\n',lino,theText);           // 0*brk, 4*cont
        nx = 0;
        aWord = NthItemOf(' ',1,aLine);
        zx = StrLength(aWord);
        if (zx==0) continue; // blank line (1 of 4)
        if (dont) { // prior line had no semicolon..
          if (NthOfset(0,";",aLine)>0) dont = false;
          else if (NthOfset(0,"{",aLine)>0) dont = false;
          if (Logging) if (logz) System.out.println("    , " + lino + "/" + nx
               + ": '" + aWord + "' " + dont);
          continue;} //~if // (prior line had no semicolon) (2 of 4)
        if (NthOfset(0,";",aLine)<0) if (NthOfset(0,"{",aLine)<0)
            // prior line was OK, so '}' alone is also OK..
          if (NthOfset(0,"}",aLine)<0) dont = true;
        sz = StrLength(aLine);
        ofx = NthOfset(0,aLine,theText); // safe, all type names already gone
        while (true) { // try each 1st word & delete it if type..
          if (Logging) if (logz) System.out.println("    . " + lino + "/" + nx
               + ": '" + aWord + "' " + zx  + " " + dont);
          if (zx==0) break;
          aStr = " " + aWord + " ";
          if (NthOfset(0,aStr,JvTy)<0) break; // normal exit
          here = NthOfset(0,aWord,aLine);
          aLine = Replace("",here,zx+1,aLine);
          aWord = NthItemOf(' ',1,aLine);
          zx = StrLength(aWord);
          nx++;} //~while // (try each 1st word & delete it if type)
        if (nx==0) continue; // = + type/prolog words (3 of 4)
        if (zx==0) continue; // =Length(type/prolog word) (4 of 4)
        here = NthOfset(0,"(",aLine);
        thar = NthOfset(0,"=",aLine);
        aStr = "var ";
        if (here>0) if ((thar<0)||(thar>here)) {
          aStr = "function ";
          thar = -2;} //~if
        whar = NthOfset(0,aWord,aLine);
        aLine = Replace(aStr,whar,0,aLine);
        nx = StrLength(aLine);
        if (Logging) if (logy||logz)
          System.out.println("  :: " + lino + " => '" + Substring(0,55,aLine)
              + IffyStr(nx>55,"..'(" + nx + ") ","' ") + dont);
        theText = Replace(aLine,ofx,sz,theText);
        sz = nx;
        if (here>0) if (thar<0) for (thar=0; thar <= 33; thar++) { // de-type the params..
          nx = 0;
          if (thar>0) {
            aLine = NthItemOf('\n',lino+thar,theText);
            ofx = NthOfset(0,aLine,theText); // safe if needed
            sz = StrLength(aLine);} //~if
          else here = NthOfset(0,"(",aLine);
          if (here<0) break; // can't
          whar = NthOfset(0,")",aLine);
          kx = Countem(" ",JvTy); // slightly high in Jv
          if (whar==0) kx = 0;
          else if (whar>0)
            if (StrLength(NthItemOf(' ',1,Substring(0,whar,aLine)))==0) kx = 0;
          if (kx>0) for (kx=kx; kx >= 1; kx += -1) { // all known types..
            aWord = NthItemOf(' ',kx,JvTy) + " ";
            zx = StrLength(aWord);
            if (zx<2) continue;
            while (true) { // every use of this type..
              here = NthOfset(0,aWord,aLine);
              if (here<0) break;
              if (whar>0) if (here>whar) break;
              if (here>0) {
                nCh = (int)(CharAt(here-1,aLine));
                if (IzLetter(nCh)) {
                  aLine = Replace("$",here+1,0,aLine);
                  continue;}} //~if
              aLine = Replace("",here,zx,aLine);
              nx++;} //~while // (every use of this type)
            aLine = ReplacAll("","$",aLine);} //~for // (kx)
          if (nx>0) {
            nx = StrLength(aLine);
            if (Logging) if (logy||logz) System.out.println("  ..." + lino
                + " => '" + Substring(0,55,aLine) + IffyStr(nx>55,"..'","'"));
            theText = Replace(aLine,ofx,sz,theText); } //~if
          if (whar >= 0) break;}} //~for // (thar: de-type the params) (lino)
      while (true) { // fix array alloc'ns (no errors)........................
        ofx = NthOfset(0," new ",theText);
        if (ofx<0) break;
        theText = Replace("$",ofx+2,1,theText);
        ofx = ofx+5;
        aLine = NthItemOf('\n',1,Substring(ofx,99,theText));
        here = NthOfset(0,"[",aLine);
        if (here <= 0) continue;
        thar = NthOfset(0,"]",aLine);
        if (thar<here+2) continue;
        aStr = Substring(here+1,thar-1-here,aLine);
        if (StrLength(NthItemOf(' ',1,aStr))==0) continue;
        aWord = NthItemOf(' ',1,Substring(0,here,aLine));
        if (StrLength(aWord)==0) continue;
        if (NthOfset(0,aWord,JvTy)<0) continue; // not a known type
        sz = thar+1;
        aWord = "Array(" + aStr + ")";
        nx = StrLength(aWord);
        if (Logging) if (logy||logz) System.out.println("(new) @ " + ofx
            + " '" + Substring(0,sz,aLine) + "' => " + aWord);
        theText = Replace(aWord,ofx,sz,theText);} //~while // (fix array alloc'ns)
      theText = ReplacAll("new","n$w",theText);
      why++; // why = 10
      if (Logging) if (logz)  System.out.println("  @ why = 10...\n" + theText);
      dont = false;
      lino = Countem(" ",CDM);
      for (lino=lino; lino >= 1; lino += -1) { // unsupported method calls....................2JS
        aWord = NthItemOf(' ',lino,CDM);
        if (StrLength(aWord)<2) continue;
        if (NthOfset(0,aWord,theText)<0) continue;
        dont = true;
        break;} //~for
      if (dont) {
        lino = 0;
        break;} //~if // why = 10: Can't do this method
      why++; // why = 11
      lino = NthOfset(0,"(int ",theText);
      if (lino<0) lino = NthOfset(0," int ",theText);
      if (lino>0) {
        lino = Countem("\n",Substring(0,lino+2,theText))+myPL;
        aWord = NthItemOf('\n',lino,theText);
        break;} //~if // why = 11: Var declarations must begin line
      for (lino=1; lino <= 33; lino++) { // GameEvent overrides (no errors this segment)........
        aWord = NthItemOf(' ',lino,OrGM);
        lxx = StrLength(aWord);
        if (lxx==1) break; // normal exit
        dont = true; // really: "doit" = false if it's not really mine
        here = NthOfset(0,aWord,theText);
        if (here<0) continue;
        aStr = Replace("$",1,0,aWord);
        nCh = (int)(CharAt(here+lxx,theText));
        theText = Replace(aStr,here,lxx,theText);
        if (here==0) dont = false;
        else if (IzLetter(nCh)) dont = false;
        else if (IzLetter((int)(CharAt(here-1,theText)))) dont = false;
        if (dont) for (thar=here-1; thar >= 0; thar += -1) {
          nCh = (int)(CharAt(thar,theText));
          if (nCh <= 32) continue;
          if (nCh == 46) break; // '.'
          theText = Replace("o_",here,0,theText);
          break;} //~for // OK either way
        if (Logging) if (logy||logz)
          System.out.println("  ~~ '" + aWord + "' " + dont + " @ " + here);
        theText = ReplacAll(aWord,aStr,theText);} //~for
      why++; // why = 12
      lino = Countem(" ",JWM);
      for (lino=lino; lino >= 1; lino += -1) { // GameWgt method call fixups.......
        xStr = NthItemOf(' ',lino,JWM);
        if (StrLength(xStr)<2) continue; // Jv macro counts spaces, not words
        nx = lino;
        aWord = "." + xStr + "(";
        while (true) { // every place it's called
          here = NthOfset(0,aWord,theText);
          if (here<0) break;
          lxx = StrLength(aWord);
          for (nx=0; nx <= 9; nx++) {
            nCh = (int)(CharAt(here+lxx+nx,theText));
            if (nCh <= 32) continue;
            break;} //~for
          if (SafeCompare(xStr,"toString")==0) { // no obj to o'ride, rename it..
            theText = Replace("_",here+3,0,theText);
            lxx++;} //~if
          nx = 0;
          dont = (nCh != 41); // ')'
          for (thar=here-1; thar >= 0; thar += -1) {
            nCh = (int)(CharAt(thar,theText));
            if (IzLetter(nCh)) continue;
            nx = here-1-thar;
            if (nx <= 0) break;
            aStr = NthItemOf(' ',1,Substring(thar+1,nx,theText));
            if (StrLength(aStr)==0) {
              nx = 0;
              break;} //~if
            if (dont) aStr = aStr + ",";
            if (Logging) if (logy||logz) System.out.println("  -> '"
                + aWord + aStr + "' +" + nx + " @ " + here);
            theText = Replace("",thar+1,nx+1,Replace(aStr,here+lxx,0,theText));
            break;}} //~while //~for (thar) (every place it's called)
        if (nx <= 0) if (Logging) System.out.println("  ? '" + aWord
            + "' +" + nx + " @ " + here);} //~for // (lino: GameWgt method call..)
      if (nx <= 0) {
        if (here>0) {
          lino = Countem("\n",Substring(0,here+1,theText))+myPL;
          if (StrLength(aWord)==0) aWord = NthItemOf('\n',lino,theText);} //~if
        else lino = 0;
        if (!logz) break;} //~if // why = 12: invalid GameWgt object in meth call
      why++; // why = 13
      while (true) { // we know how to do these kinds of things: (null fixes)...
        dont = false;                 // 'if (xxx != null)' => 'if (xxx)'
        aWord = "null";               // '(xxx == null)' => '(xxx == 0)'
        here = NthOfset(0,aWord,theText); // 'xxx = null' => 'xxx = 0'
        lino = 0;                     // ',null,' => ',0,' // '(null)' => '(0)'
        if (here<0) {
          why = 0; // normal exit, no errors
          break;} //~if
        lino = Countem("\n",Substring(0,here+1,theText))+myPL;
        if (IzLetter((int)(CharAt(here+4,theText)))) dont = true;
        else if (here>0) if (IzLetter((int)(CharAt(here-1,theText))))
          dont = true;
        if (dont) {
          theText = Replace("$",here+1,0,theText);
          continue;} //~if
        lino = Countem("\n",Substring(0,here+1,theText))+myPL;
        aStr = Substring(here+4,99,theText);
        nCh = (int)(CharAt(0,NthItemOf(' ',1,aStr)));
        if (nCh == 41) dont = true; // ')'
        else if (nCh != 44) // ','
          if (nCh != 59) break; // ';' why = 13: unrecognized op after 'null'
        if (here<8) break;
        lxx = here-99;
        if (lxx <= 0) lxx = 1;
        for (thar=here-1; thar >= lxx; thar += -1) {
          nCh = (int)(CharAt(thar,theText));
          if (nCh <= 32) if (thar>lxx) continue;
          if (nCh == 44) break; // ','
          if (nCh == 40) break; // '('
          if (nCh != 61) { // '='
            nCh = 46; // '.' can't do this one
            break;} //~if
          if (dont) nCh = (int)(CharAt(thar-1,theText));
          if (nCh == 33) { // '!' dont=true
            aLine = Substring(lxx,thar-1-lxx,theText);
            while (true) {
              lxx = StrLength(aLine);
              nx = NthOfset(0,"if",aLine);
              if (nx <= 0) break;
              kx = NthOfset(1,"if",aLine);
              if (kx>0) {
                aLine = RestOf(kx-1,aLine);
                continue;} //~if
              zx = NthOfset(0,"(",aLine);
              if (zx<0) break;
              kx = NthOfset(1,"(",aLine);
              if (kx <= 0) break;
              aLine = RestOf(kx,aLine);} //~while
            if (nx>0) if (zx>nx) if (zx<lxx) {
              aLine = RestOf(zx+1,aLine);
              if (StrLength(NthItemOf(' ',1,aLine))>0)
                  if (StrLength(NthItemOf(' ',2,aLine))==0) {
                theText = Replace("",thar-1,here+5-thar,theText);
                break;}}} //~if
          if (nCh == 33) nCh = 36; // '$' failed the 'if' test, gotta put 0
          if (nCh == 46) nCh = 60; // '`' shouldn't hap
          break;} //~for
        if (nCh != 33) if (nCh != 46) // '.' is err, already deleted '!'
          theText = Replace("0",here,4,theText);
        if (nCh != 46) continue; // '.'
        why++; // why = 14
        break;} //~while // why = 14: unrecognized op before 'null' (null fixes)
      if (why>0) {
        if (logz) System.out.println("** 2JS error = " + why);
          else break;} //~if
      theText = ReplacAll("console.log","System.out.println",
          ReplacAll("","myGame.",ReplacAll("","JavaGame.",
          ReplacAll("","super.",theText))));
      theText = ReplacAll(" ","\r",ReplacAll("","\n\r",
          ReplacAll("","\r\n",ReplacAll("null","n$ull",theText))));
      lxx = StrLength(theText);
      while (true) {
        theText = ReplacAll("\n\n","\n\n\n",ReplacAll("\n"," \n",theText));
        nx = StrLength(theText);
        if (nx<lxx) lxx = nx;
          else break;} //~while
      theText = "<script type='text/javascript'> // " + fiName + " -- "
          + TimeStamp(false) + "\n" + theText + "\n</script>\n";
      why = 0;
      break;} //~while // (once thru from top)
    // lxx = StrLength(theText);
    if (why==0) if (lxx<111) why = 15; // Generated text too short
    JSinfo = "Error #" + why;
    if (why>0) switch (why) {
      case 1: // why = 1:
        JSinfo = "Can't find class name";
        break;
      case 3: // why = 3:
        JSinfo = "Can't find widget list";
        break;
      case 4: // why = 4:
        JSinfo = "Failed wgt list conversion";
        break;
      case 5: // why = 5:
        JSinfo = "Invalid wgt list end";
        break;
      case 6: // why = 6:
        JSinfo = "GameList front corrupted";
        break;
      case 7: // why = 7:
        JSinfo = "GameList back corrupted";
        break;
      case 10: // why = 10:
        JSinfo = "Can't do this method";
        break;
      case 11: // why = 11:
        JSinfo = "Var declarations must begin line";
        break;
      case 12: // why = 12:
        JSinfo = "Invalid GameWgt object in method call";
        break;
      case 13: // why = 13:
        JSinfo = "Unrecognized op after 'null'";
        break;
      case 14: // why = 14:
        JSinfo = "Unrecognized op before 'null'";
        break;
      case 15: // why = 15:
        JSinfo = "Generated text too short";
        break;
      case 16: // why = 16:
        JSinfo = "Bad widget list form";
        break;
      case 17: // why = 17:
        JSinfo = "Invalid line number in widget list";
        break;
      case 18: // why = 18:
        JSinfo = "Macros and private widget types not supported";
        break;
      case 20: // why = 20:
        JSinfo = "Widget spec line is too short";
        break;
      case 21: // why = 21:
        JSinfo = "Invalid negative data length";
        break;
      case 22: // why = 22:
        JSinfo = "No such font";
        break;} //~switch
    if (why>0) aLine = "\n  -- " + JSinfo;
      else aLine = "";
    if (Logging||(why>0)) System.out.println("** 2JS +"
        + StrLength(theText) + IffyStr(why==1,"/" + ofx,"")
        + " @ " + lino + " '" + aWord + "' = " + why + aLine);
    if (why>0)
      return false;
    JSinfo = theText;
    if ((why==0)||logz) WriteWholeTextFile(fiName,theText);
    return true;} //~toJS

  /**
  * Converts the plain-text user name or JS code into a form suitable for
  * upload over the internet. All non-letters are converted to the form
  * %xx where xx is the hexadecimal code for that character.
  *
  * @param  theText  the raw text
  *
  * @return          the same text as encoded
  */
  private String NetEncode(String theText) { // convert/remove unwanted chars
    char aCh;        // ..suitable for a JS cookie or posting to server.
    int info, nx, lxx = StrLength(theText)-1;
    String nuly = "", aStr = "";
    for (nx=0; nx <= lxx; nx++) { // (omit in BeenHere..)
      aCh = CharAt(nx,theText);
      info = (int)(aCh);
      if (info>126) continue;
      if (info==13) info = 10;
      else if (info<32) if (info !=10) continue;
      if (info>96) if (info<123) info = 0; // keep lower case
      if (info>64) if (info<91) info = 0; // keep caps
      if (info>47) if (info<58) info = 0; // keep digits
      if (info==0) aStr = "" + aCh; // auto-converts to String
        else aStr = SeeHex("%",info,"");
      nuly = nuly + aStr;} //~for
    return nuly;} //~NetEncode

  /**
  * Uploads a particular user's (JS) game code to a game server.
  *
  * @param  whom  the user name and password as name_password
  * @param  data  the JS program as translated by toJS() including html header
  *
  * @return       an error code, or =0 if none
  */
  private int ServerUpload(String whom, String data) { // rtns error +
    int here, thar, why = 0;
    data = "?cmd=18&name=" + NetEncode(whom) + "&data=" + NetEncode(data);
    HttpURLConnection httpCon = null;
    PrintWriter px;  BufferedReader br;  try {
      why++; // why = 1
      URL whar = new URL("http://ittybittycomputers.com/Java/Hack.php?cmd=18");
      why++; // why = 2
      httpCon = (HttpURLConnection) whar.openConnection();
      why++; // why = 3
      httpCon.setDoOutput(true);
      why++; // why = 4
      httpCon.setRequestMethod("POST");
      why++; // why = 5
      httpCon.setRequestProperty("User-Agent","Java client");
      why++; // why = 6
      httpCon.setRequestProperty("Content-Type",
          "application/x-www-form-urlencoded; charset=utf-8");
      why++; // why = 7
      px = new PrintWriter(httpCon.getOutputStream());
      why++; // why = 8
      px.write(data);
      why++; // why = 9
      px.flush();
      why++; // why = 10
      px.close();
      why = httpCon.getResponseCode();
      if (why==200) { // this is doing it the hard way..
        why = 11;
        br = new BufferedReader(new InputStreamReader(httpCon.getInputStream()));
        why++; // why = 12
        data = "";
        for (here=1; here <= 33; here++) {
          whom = br.readLine();
          if (here>11) if (StrLength(whom)==0) break;
          data = data + whom + "\n";} //~for
        why++; // why = 13
        br.close();
        why++; // why = 14
        here = NthOfset(0,"<span>",data);
        thar = NthOfset(0,"</span>",data);
        whom = data;
        if (here >= 0) if (thar>here) {
          why++; // why = 15
          whom = Substring(here+6,thar-6-here,data);
          if ((StrLength(whom)&-2) ==2) why = 0;
            else whom = data;}} //~if
      else whom = "HTTP error code = " + why;}
    catch (Exception ex) {whom = "{oops} = " + why;}
    try {if (httpCon != null) httpCon.disconnect();}
    catch (Exception ex) {}
    if (Dev_Log||Logging)
      System.out.println("** ServerUplo = " + why + " '" + whom + "'");
    if (why>0) {
      if (why<20) JSinfo = "Uploader error #" + why;
        else JSinfo = Substring(0,96,whom);} //~if
    return why;} //~ServerUpload

  /**
  * (Temporary) uploader test.
  */
  void JvUpLdTest() {
    String xStr = "<!doctype html public '-//w3c//dtd html 4.0 transitional//en'>"
        + "\n<html><head><title>Upload Test</title></head><body>\n"
        + "<center><h1>Hello World*</h1></center></body></html>";
    int why = ServerUpload("Test_public",xStr);
    System.out.println("** JavaUploadTest = " + why);} //~JvUpLdTest

/**
 * Subclass to handle events when creating a new game. // . . . . . . (GameMk)
 */
  private class GameMakerEv extends GameEvent { // 2021 July 13

    GameWgt GdChkBx, ColoBx, RxWgt, CxWgt, CxAntz, ToolHiLi, UsWgHiLi,
        OKbtn, BildBtn, JSbtn, CantJS, ToolList, ToolName, ToolPanel,
        WgLsScroBar, WgLsScroThm, WgLsScrUpBt, WgLsScrDnBt,
        ActWgNmWgt, AntsChkBx, UndoBtn, TextInWgt;
    boolean GmMkBooTmp, FreshFaster; // FF=T: omit MkWgLst // InChanged,
    int UwgtNamID, UwgtNamPos, ChosenBtn, NextWgtName, GamePosn, WgLsTall,
        nUsrWgts, nWgsVisi, WgScroSkp, WgScrThmPo, WgScrThmSz, wScrStpSz,
        WgScrBarSz, WgScrBtnSz, UsHiLiWgNo, WgScrState, WgLsOfx, GameSz;
    String GamesName, ValidUserName, PendingName, UserNames,
        AddUserWgts, WgtMap, WgtList, MkTmpStr;

  /// [more pseudo-global vars near UserEvHandler]

  final static String
      TakenGameNames = " JavaGame GameEvent GameWgt ";

 /**
  * Return the class name for this object.
  *
  * @return     The reference String for this object
  */
  public String toString() { // . . . . . . . . . . . . . . . . . . . (GameMk)
    String msg = "(GameMakerEv)" + IffyStr(StrLength(GamesName)==0,"",
        " GamesName = '" + GamesName + IffyStr(StrLength(PendingName)==0,"",
        "' => '" + PendingName) + "' " + RevDate);
    if (Logging) if (GoodNew) System.out.println("  (toStr) $& '" + msg + "'");
    return msg;} //~toString

 /**
  * Clear instance vars...
  */
  public void ClearVars() { //  . . . . . . . . . . . . . . . . . . . (GameMk)
    super.ClearVars();
    NextWgtName = 0;   ChosenBtn = 0;     GameSz = 0;     UsHiLiWgNo = 0;
    UwgtNamID = 0;     UwgtNamPos = 0;    GamePosn = 0;   WgScrBtnSz = 0;
    WgScrThmPo = 0;    WgScroSkp = 0;     nUsrWgts = 0;   WgScrBarSz = 0;
    WgScrThmSz = 0;    WgLsTall = 0;      WgLsOfx = 0;    nWgsVisi = 0;
    WgScrState = 0;    wScrStpSz = 0;     // ToolNmLstScro = 0;
    ToolActive = false;   FreshFaster = false;
    GdChkBx = null; // InChanged = false; // NullWgt = null;
    // TypChkBx = null;  ClikChkBx = null; DragChkBx = null;
    // BumpChkBx = null; RollChkBx = null; AIdlgChkBx = null;
    ColoBx = null;     RxWgt = null;      CxWgt = null;
    CxAntz = null;     OKbtn = null;      JSbtn = null;
    CantJS = null;     BildBtn = null;    UndoBtn = null;
    AntsChkBx = null;  ToolHiLi = null;   UsWgHiLi = null;
    ActWgNmWgt = null; WgLsScroBar = null;
    WgLsScroThm = null; WgLsScrUpBt = null; WgLsScrDnBt = null;
    // GameBoard = null; ListofWgts = null; ActivWgt = null;
    // VeloEntry = null; UwgtNamList = null; WgtNamEntry = null;
    // GameNameWgt = null;
    ToolPanel = null;   ToolName = null;    ToolList = null;
    TextInWgt = null;   ValidUserName = "";
    UserNames = "";  // AddUserWgts = "";  GamesName = "";
    MkTmpStr = "";      WgtList = "";    WgtMap = "";
    if (Logging) System.out.println("(GmMk.ClrVars)");} //~ClearVars

  /**
  * NameIsTaken compares a proposed variable name against those already
  * used and returns true if it is taken.
  *
  * @param  aWord   the proposed variable name
  *
  * @return         true if it's taken
  */
  public boolean NameIsTaken(String aWord) {
    String data = " my" + GamesName + TakenGameNames + UserNames;
    // if (gm2) data = data + " " + GamesName + " ";
    if (NthOfset(0," " + aWord + " ",data)<0) return false;
    return true;} //~NameIsTaken

  /**
  * ChooseBtn exposes one of the four response buttons and hides the others.
  *
  * @param  whom    the chosen button 1..4
  */
  public void ChooseBtn(int whom) {
    boolean doit;
    int why = 0;
    GameWgt nuly = null;
    ChosenBtn = 0;
    if (CantJS != null) {
      doit = (whom==5);
      if (doit) nuly = CantJS;
      CantJS.HideSho(doit);} //~if
    else why = 16;
    if (JSbtn != null) { // JSbtn=4
      doit = (whom==4);
      if (doit) nuly = JSbtn;
      JSbtn.HideSho(doit);} //~if
    else why = why+8;
    if (BildBtn != null) { // BildBtn=3
      doit = (whom==3);
      if (doit) nuly = BildBtn;
      BildBtn.HideSho(doit);} //~if
    else why = why+4;
    if (UndoBtn != null) { // not installed
      doit = (whom==2);
      if (doit) nuly = UndoBtn;
      UndoBtn.HideSho(doit);} //~if
    // else why = why+2;
    if (OKbtn != null) {
      doit = (whom==1);
      if (doit) nuly = OKbtn;
      OKbtn.HideSho(doit);} //~if
    else why++;
    if (why==0) {
      if (whom >= 0) if (whom<5) ChosenBtn = whom;
      if (whom>0) {
        if (nuly != null) DefaultKeyFocus(nuly);
        if (Logging) if ((DrawLogger&1)==0) DrawLogger++;} //~if
      else DefaultKeyFocus(GameBoard);} //~if
    if (Logging)
      System.out.println("ChoseBtn " + whom + " DF=#" + SafeWgtNum(DefaultFocus)
          + " = " + why );
    } //~ChooseBtn

  /**
  * NewData is informed that the user has made some change to the game which
  * should be saved, and displays the OK button to do it.
  *
  * @param  doit    true to force the condition
  */
  public void NewData(boolean doit) {
    // if (InChanged) if (!doit) return;
    // if (Logging) DrawLogger++; // ChBtn did it
    // InChanged = true;
    unSavedWgts = true;
    WuzChanged = true;
    ChooseBtn(1); // OK
    WgtList = "";
    JSinfo = "";} //~NewData // set in toJS, used in MakeUserJS

  /**
  * Return true if there's a saved game file by this name,
  * and cache it in SavedGameData.
  *
  * @param  name    the saved game name
  *
  * @return         true if the file is there and loaded
  */
  public boolean SavedFiExists(String name) {
    char xCh;
    int nx, zx, why = 0;
    String aStr, aLine, data = ReadWholeTextFile(name + ".txt");
    if (StrLength(data)==0) while (true) { // try to extract frm .java..
      why--; // why = -1
      data = ReadWholeTextFile(name + ".java");
      nx = NthOfset(0,"<$..>",data);
      if (nx<0) break; // if nothing came, then offset fails
      why--; // why = -2
      nx = nx+5;
      zx = NthOfset(0,"/// <..$>",data)-nx;
      if (zx<0) break;
      why--; // why = -3
      aStr = Substring(nx,zx,data);
      data = "";
      for (nx=1; nx <= 9999; nx++) { // got it, get it into proper form..
        aLine = NthItemOf('\n',nx,aStr);
        zx = NthOfset(0,Quot2,aLine);
        if (zx<0) {
          if (zx<3) continue;
          break;} //~if // normal exit
        aLine = RestOf(zx+1,aLine);
        for (zx=StrLength(aLine)-1; zx >= 0; zx += -1) {
          xCh = CharAt(zx,aLine);
          if (xCh <= ' ') continue;
          if (xCh == '\"') aLine = Substring(0,zx,aLine);
            else aLine = "";
          break;} //~for
        if (StrLength(aLine)>0) data = data + aLine;
          else break;} //~for
      // why--; // why = -4
      data = ReplacAll("\n","\\n",data); // ** gotta deal w/'"'&'\'
      why = 10;
      break;} //~while
    if (why<0) why = why+10; // why = 7/8/9 (no .jv or invalid)
    else while (true) { // got .tx (why=0) or .jv (why=10)..
      why++; // why = +1
      nx = NthOfset(0,"{" + name + "}",data);
      if (nx<0) break; // if nothing came, then offset fails
      why++; // why = +2
      zx = NthOfset(0,"\n",data);
      if (nx>zx) break;
      why++; // why = +3
      zx = NthOfset(0,"{",data);
      if (zx<nx) break;
      why++; // why = +4
      if (NthOfset(0,"[List of item lists]",data)<0) break;
      why++; // why = +5
      SavedGameData = data;
      why = -why; // why = -5/-15 (OK)
      break;} //~while
    if (Logging) if (GoodNew) System.out.println("  (SavFiEx) $& '" + name
        + "' " + nx + "/" + zx + " = " + why);
    return why<0;} //~SavedFiExists

  /**
  * InvalidJava tests if a name is valid Java for a variable name
  * and returns true if not. Reserved words are disallowed.
  *
  * @param  aWord   the proposed variable name
  *
  * @return         true if it's not valid
  */
  public boolean InvalidJava(String aWord) {
    int nx, xCh, lxx = StrLength(aWord)-1;
    String JvRWds = JavaResWds;
    if (lxx >= 0) if (NthOfset(0," "+aWord+" ",JvRWds)<0) for (nx=0; nx <= lxx+1; nx++) {
      if (nx>lxx) return false;
      xCh = (int)(CharAt(nx,aWord));
      if (xCh>122) break; // 'z'
      if (xCh>96) continue; // 'a'-1
      if (xCh==95) if (nx>0) if (nx<lxx) continue; // '_'
      if (xCh>90) break; // 'Z'
      if (xCh>64) continue; // 'A'-1
      if (nx==0) break; // no digit in 1st place
      if (xCh>57) break; // '9'
      if (xCh<48) break;} //~for // '0'
    return true;} //~InvalidJava

  /**
  * Whenever a tool or user widget is selected, ShoInfoFrom is called
  * to display it attributes.
  *
  * @param  whom    the widget clicked on
  */
  public void ShoInfoFrom(GameWgt whom) { // whom: tool or user wgt (or DW)
    int logy = SeenLog&0x20000, colo, zx, nx = 0, info = 0, tye = 0, why = 0;
    boolean user = false, aBit = false;
    String name = "";
    GameWgt more = null, nuly = FontRad;
    while (true) {
      why++; // why = +1
      if (whom==null) break;
      nx = whom.GetRefNum();
      info = GetToolInfo(whom);
      colo = TooledShoDat(whom);
      SetSwatchColor(ColoBx,colo);
      name = " "; ///** should this now be a tool activity?..
      if (whom != NullWgt) { // look for wgt name field..
        nuly = FindSubWgt(whom,(TextFontNo<<16)+0x0404,0);
        if (Logging) if (nuly==null) // log same search..
          nuly = FindSubWgt(whom,(TextFontNo<<16)+0x0404,9000);
        if (nuly != null) name = nuly.GetTextLn();
        if (StrLength(name)==0) name = "untitled";} //~if
      if (WgtNamEntry != null) {            // look for entry field..
        nuly = FindSubWgt(WgtNamEntry,(MonoFontNo<<16)+0x0604,0);
        if (Logging) if (nuly==null)
          nuly = FindSubWgt(WgtNamEntry,(MonoFontNo<<16)+0x0604,10000);
        if (nuly != null) {
          zx = nuly.PutTextLn(name);
          why = why+4;}} //~if                     // can't use find!?
      // nuly = FindSubWgt(whom,0x54794E6D,11000); // look for tool name 'TyNm'
      ///** should this now be a tool activity?..
      if (whom != NullWgt) { // look for tool name..
        nuly = FindSubWgt(whom,4,0); // look for tool name, ty=4, fnt=0
        if (Logging) if (nuly==null) // log same search if failed..
          nuly = FindSubWgt(whom,4,11000);
        if (nuly != null) name = nuly.GetTextLn();} //~if
      if (ToolName != null) if (StrLength(name)>0) {
        zx = ToolName.PutTextLn(name);
        why = why+8;} //~if
        // ShoCoords(whom,true)
        // TooledShoPoSiz(whom,false);} //~if // TooledShoDat did it
      why = -why;
      break;} //~while
    RefreshPan = false;
    if (Logging||(logy==0)) {
      System.out.println("(ShoInFro #" + nx + ") " + user + " = " + why);
      SeenLog = SeenLog|0x20000;}} //~ShoInfoFrom

  /**
  * Whenever you click on the OK button or deselect a game widget,
  * CapturInfo2Wgt is called to copy the displayed attributes to the
  * previously selected widget group.
  *
  * @param  whom    the widget clicked on
  */
  public void CapturInfo2Wgt(GameWgt whom) { // set wgt = panel..
    int zx, rx = 0, colo = 0, info = 0, seen = 0, why = 0; // nx = 0,
    boolean didit = false;
    String name = "", aStr = "", zStr;
    GameWgt velo, more, nuly = FontRad;
    while (true) {
      why++; // why = 1
      if (whom==null) break;
      rx = whom.GetRefNum();
      seen = GetToolInfo(whom);
      nuly = FindSubWgt(whom,0x466C677A,0); // 'Flgz'
      if (seen !=0) { // tool or draggable..
        colo = SwatchGetColor(ColoBx);
        if (colo==0x10000) colo = 0; // (this only matters for icon-based wgts)
        else if (colo==0xFEFFFF) colo = 0xFFFFFF;
        // if (doit) TooledSetPoSiz(whom); // now in GTD
        SetTooledDat(colo,whom);
        if ((seen&0x8000) !=0) ChgdBitSet = (ChgdBitSet&~2)|1; // b0=file
          else ChgdBitSet = ChgdBitSet&~0x20000; // b1=color
        NewData(false);
        why = why|32;} //~if
      why++; // why = +4
      if (WgtNamEntry != null) { // look for entry field..
        why++; // why = +5
        name = GetInputTxt(WgtNamEntry); // name as typed
        // nuly = FindSubWgt(WgtNamEntry,(MonoFontNo<<16)+0x0604,0);
        // if (nuly != null) {
        //   why++; // why = +6
        //   name = nuly.GetTextLn();
        nuly = FindSubWgt(whom,(TextFontNo<<16)+0x0404,0); // (name field)
        if (nuly != null) aStr = nuly.GetTextLn(); // aStr is prior
        if (StrLength(name)==0) {
          NextWgtName++;
          name = "untitled." + NextWgtName;} //~if
        else if (SafeCompare(name,aStr)==0) didit = true; // unchanged
        else while (NameIsTaken(name)) { // name as typed, aStr is prior
          NextWgtName++;
          zx = Countem(".",name);
          if (InvalidJava(Substring(0,1,name))) name = "untitled";
          else if (zx>0) { // remove final number, if any..
            zStr = NthItemOf('.',zx+1,name);
            if (SafeCompare(CvInt2Str(SafeParseInt(zStr)),zStr)==0)
              name = Substring(0,zx,name);} //~if
          name = name + "." + NextWgtName;} //~while
        if (NuNmMode==0) if (DlogIsOpen==0) if (whom==GameBoard)
            if ((seen&0x8000) !=0) {
          if (InvalidJava(name)) {
            didit = GameAlert(Quot2 + name + "\" is not a valid game name");
            return;} //~if
          zx = 0;
          if (!didit) if (StrLength(SavedGameData)>0)
              if (SafeCompare(PendingName,name)==0) {
            zx = NthOfset(0,name,SavedGameData);
            if (zx>0) if (NthOfset(0,"\n",SavedGameData)<zx) zx = -zx;} //~if
          if (Logging) if (GoodNew)
            System.out.println(IffyStr(didit," (nm=)","  " + name
                + " = '" + PendingName + "' " + zx));
          if (didit) didit = false; // (user typed, but no change)
          else if (zx>0) why = 12; // new SGD already loaded
          else if (SavedFiExists(name)) { // saves it in SavedGameData
            if ((ChgdBitSet&0xFFFA)==0) // no other unsaved changes..
              didit = GameAlert("Reload saved " + name + " game?");
            else didit = GameAlert("Discard changes & reload " + name + "?");
            why = -11;
            if (didit) {
              PendingName = name;
              why++; // why = -10
              InputWhy = 10;
              NuNmMode--;} //~if
            break;} //~if
          NameChanged = true;} //~if
        if (nuly != null) {
          zx = nuly.PutTextLn(name);
          if ((seen&0x8000) !=0) ChgdBitSet = (ChgdBitSet&~4)|1; // b0=file
          else if (seen !=0) ChgdBitSet = ChgdBitSet&~0x40000; // b2=name
          if (whom != GameBoard) NewData(false);} //~if
        else why++; // why = +6
        if (whom==GameBoard) {
          GamesName = name;} //~if
        else why = why+2;} //~if // why = +7/8
      why = -why; // why = -4/-7/-8 (OK)
      break;} //~while
    if (SRchanged) ReportChBitS();
    if (Logging) {
      zx = -why;
      if (zx>8) zx = zx&15;
      System.out.println("(CapWgtIn2w #" + rx + SeeHex(") x",seen,
          IffyStr(zx>8," $& = "," = ")) + why);}} //~CapturInfo2Wgt

// These methods generate user Java code..

  /**
  * When creating an exported widget list, it is important to ignore the
  * "meta-widgets" which carry extra information needed for selection and
  * dragging, but are not part of the user data. IzzitMeta returns true
  * for those widgets.
  *
  * @param  whom    the widget being considered for export
  * @param  prnt    the widget's presumed parent
  *
  * @return         true if it's a meta-widget
  */
  private boolean IzzitMeta(GameWgt whom, int prnt) {
    int tye, info, rx = 0, logy = 0;
    if (whom==null) return false;
    tye = whom.GetType()&255;
    info = whom.GetInfo();
    if (info != 0x466C677A) if (info != 0x416E7449) // 'Flgz'/'AntI'/'Antz'
      if (info != 0x416E747A) if (whom.GetParent() == prnt)
        if ((tye !=4)||(info>0)) return false;
    return true;} //~IzzitMeta

  /**
  * When preparing to export a widget list, it is necessary to pre-scan all
  * candidates to find all fonts that might be in use, and to build a list
  * of exportable widgets so they can be renumbered in the export file.
  * Prescan calls itself recursively from the game list root. The listing
  * is made in instance variable WgtMap.
  *
  * @param  whom  the root group widget to export
  * @param  bitz  one bit for each known used font
  * @param  omit  <0 (draggable group head) to search this level but not list
  *
  * @return       updated bitz
  */
  private int Prescan(GameWgt whom, int bitz, int omit, int tuly) {
    int prio = omit, ix, zx, nx = 0,        // omit<0 not map, >0 not list
        lxx = 0, tye = 0, info = 0, posn = 0, size = 0, why = 0;
    boolean seen = false, gone = false;
    String aWord;
    GameWgt nuly = null;
    if (whom != null) {
      why++; // why = 1
      nx = whom.GetRefNum();
      tye = whom.GetType()&255;
      posn = whom.GetPosn();
      size = whom.GetPosn();
      // info = whom.GetInfo(); // (unused)
      info = GetToolInfo(whom);
      if (GamePosn>0) if (GameSz>0) if ((posn<0)||(posn+size>GameSz)) {
        zx = whom.AddPrntsPos(posn);
        if (((zx+size-GamePosn)&0x80008000) !=0) gone = true;
        else if (((GamePosn+GameSz-zx)&0x80008000) !=0) gone = true;} //~if
      if (!gone) if (tye==5) if ((info&0x8000) !=0) { // = '__DW'
        tuly = info&~0x8000;
        if ((info&512) !=0) omit--; // text 'TxDW'
        else if ((info&0x900)==0x900) omit = 8; // circle+sprite 'B*DW'
        aWord = "\n" + nx + ",";
        if (NthOfset(0,aWord,WgtMap)<0) WgtMap = aWord + WgtMap;} //~if
      lxx = whom.GetIxData(-1,-1);
      if (Logging) if (GoodImex) {
        System.out.println("  (Pr+NeedFnt #" + nx + ") t=" + tye + " +" + lxx
            + SeeHex(" i=x",info,SeeHex("/",tuly," o=")) + omit
            + "/" + prio + IffyStr(gone," -gone-",""));
        seen = true;} //~if
      if (gone) why = -8;
      else if (tye==6) while (true) { // embedded sprite..
        why++; // why = 2
        zx = tuly>>24;
        if (zx != 0x51) if (zx != 0x4B) if (zx != 0x53) break; // 'Q'/'K'/'S'
        why++; // why = 3
        if (lxx <= 0) break;
        zx = whom.GetParent();
        why++; // why = 4
        if (zx <= 0) break;
        aWord = "\n" + zx + ",";
        why++; // why = 5
        if (NthOfset(0,aWord,WgtMap)<0) break;
        aWord = "\n" + nx + ",";
        if (NthOfset(0,aWord,WgtMap)<0)
          WgtMap = aWord + ",=" + zx + WgtMap;
        for (ix=lxx-1; ix >= 0; ix += -1) {
          nuly = GetIxWgt(whom.GetIxData(ix,0));
          if (nuly==null) continue;
          if (IzzitMeta(nuly,nx)) continue;
          bitz = Prescan(nuly,bitz,omit,tuly);} //~for
        why = -why; // why = -5
        break;} //~while // (embedded sprite)
      else if (tye==4) { // embedded text..
        why = 6;
        if (omit >= 0) {
          aWord = "\n" + nx + ",";
          if (NthOfset(0,aWord,WgtMap)<0) WgtMap = aWord + WgtMap;
          if (omit>0) WgtMap = WgtMap + nx + " ";} //~if
        zx = ChoseFont(whom,-2);
        if (zx >= 0) if (zx <= 8) bitz = (1<<zx)|bitz;
        why = -why;} //~if // why = -6
      else if ((tye&253)==0) { // embedded rect or icon..
        why = 7;
        if (omit >= 0) {
          aWord = "\n" + nx + ",";
          if (NthOfset(0,aWord,WgtMap)<0) WgtMap = aWord
              + IffyStr(tuly>0,SeeHex(",0x",tuly,""),"") + WgtMap;
          if (omit>0) WgtMap = WgtMap + nx + " ";} //~if
        why = -why;} //~if // why = -7    // don't recurse into plain circles..
      else if (((tye-5)&255)<3) if ((info&0xFF00FF00) != 0x43008400) { // 'C*DW'
        for (ix=lxx-1; ix >= 0; ix += -1) {
          nuly = GetIxWgt(whom.GetIxData(ix,0));
          if (nuly==null) continue;
          if (IzzitMeta(nuly,nx)) continue;
          bitz = Prescan(nuly,bitz,omit,tuly);} //~for
        why = -why;}} //~if // why = -1 // else why=0
    if (Logging) if (GoodImex) if (seen)
      System.out.println("  ..(Pr+NF #" + nx + ") = " + bitz + "/" + why); // why =
    return bitz;} //~Prescan

  /**
  * MakeWgtLst builds an exportable widget list in instance variable WgtList
  * and writes it to file "GameMaker.txt".
  *
  * @return         true if it failed
  */
  public boolean MakeWgtLst() { // rtns true if failed
    boolean circ = false;
    int kx, nx, rx, zx, here, thar, whar = 0, GmBd = 0, tye = 0,
        info = 0, posn = 0, size = 0, color = 0, prnt = 0, bitz = 0,
        nsubs = 0, tuly = 0, velo = 0, lxx = 0, why = 0;
    GameWgt nuly = null, more = null;
    String namex = "", fiName = "", inits = "", msg = "", aWord = "",
        aLine, aStr, theList;
    GameWgt whom = GameBoard;
    while (true) {
      why++; // why = 1
      MkTmpStr = "";
      if (whom==null) break; // whom = GameBoard = [Main Game asset list]
      why++; // why = 2
      lxx = whom.GetIxData(-1,-1); // => array size
      if (lxx<2) break;
      why++; // why = 3
      more = FindSubWgt(whom,0x466C677A,0); // 'Flgz'
      if (more==null) break;
      GmBd = whom.GetRefNum();
      WgtMap = "\n" + GmBd + ",2\n\n0 ";
      why++; // why = 4
      // bitz = more.GetPosn(); // unneed, use chkbx directly
      tye = more.GetType()&0xFF00|5;
      size = whom.GetSize();
      color = whom.GetColor();
      if (color<0) color = 0xCCCCFF;
      // else if (color==0) color++;
      // else if (color==0xFFFFFF) color--;
      theList = " 0 0 " + (size>>16) + " " + (size&0xFFFF);
      aLine = " " + tye + theList;
      theList = "1" + aLine + HexIfMore(" ",color," 0 0 4  2 0 0 0")
          + "  '[List of item lists] in win {" + GamesName
          + "}'\n2 5" + theList + " -1 0 1 " + lxx + " ";
      nsubs = lxx; // = + wgts in Main Game
      info = 0;
      if (GamesAIdlg) info = 17;
      info = Prescan(whom,info,0,0)<<1; // adds ref+s to WgtMap
      aLine = " '[Sprite list]' #" + NextWgtName + UserDate;
      thar = 2;
      for (here=0; here <= 7; here++) { // add used fonts to wgt list..
        info = info>>1;
        if (info==0) break;
        if ((info&1)==0) continue;
        thar++;
        switch (here) {
        case 0:
          aWord = "\n" + MonoFontNo + "," + thar + ",M";
          break;
        case 1:
          aWord = "\n" + TextFontNo + "," + thar + ",T";
          break;
        case 2:
          aWord = "\n" + ItalFontNo + "," + thar + ",I";
          break;
        case 3:
          aWord = "\n" + TinyFontNo + "," + thar + ",Y";
          break;
        case 4:
          aWord = "\n" + BoldFontNo + "," + thar + ",B";
          break;
        case 5:
          aWord = "\n" + HeadFontNo + "," + thar + ",H";
          break;
        case 6:
          aWord = "\n" + NanoFontNo + "," + thar + ",N";
          break;
        case 7:
          aWord = "\n" + ScorFontNo + "," + thar + ",S";
          break;} //~switch
        WgtMap = aWord + WgtMap;
        aLine = aLine + "\n" + thar + " 3 0 0 12 12 0x0 " + here
            + " 0 0 '[Font " + NthItemOf(',',3,aWord) + "]'";} //~for
      info = (size&0xFFFF)>>2;
      if (info>32) info = 32;
      if (GamesAIdlg) {
        nuly = theAlertDlg; // = [Std Alert Dialog]
        if (nsubs>0) if (nuly != null) {
          info = nuly.GetRefNum();
          thar++;
          WgtMap = "\n" + info + "," + thar + WgtMap;
          aLine = aLine + "\n" + thar + " 241 " + (size>>18) + " " + info + " 64"
              + " 240 0xFFFFFF 0 2 0  'Hello world!' [Std Alert Dialog] `AlDg`";
          theList = theList + "  " + info; // (replace later w/new)
          nsubs--;} //~if
        nuly = theInputDlg; // = [Std Input Dialog]
        if (nsubs>0) if (nuly != null) {
          info = nuly.GetRefNum();
          thar++;
          WgtMap = "\n" + info + "," + thar + WgtMap;
          aLine = aLine + "\n" + thar + " 2288 " + (size>>17) + " " + info + " 64"
              + " 240 0xFFFFFF 0 2 0  'Prompt:' '(..)' [Std Input Dialog] `IpDg`";
          theList = theList + "  " + info;
          nsubs--;}} //~if
      why++; // why = 5
      if (Logging) if (GoodImex) System.out.println(".. .." + WgtMap + "\n.. ..");
      kx = 0;
      for (here=2; here <= 2222; here++) { // add actual wgts to list..
        why = 0;
        aWord = NthItemOf('\n',here,WgtMap);
        zx = NthOfset(0,"=",aWord);
        if (zx>0) continue; // save these for later
        tuly = 0;
        rx = SafeParseInt(aWord);
        if (rx>0) {
          aStr = "\n" + rx + ",";
          whar = NthOfset(0,aStr,WgtMap);
          if (whar<0) continue; // (probly can't fail)
          whar = StrLength(aStr)+whar; // -> where new wgt+ goes
          nuly = GetIxWgt(rx);
          zx = NthOfset(0,"0x",aWord);
          if (zx>0) tuly = SafeParseInt(RestOf(zx,aWord));} //~if
        else nuly = null;
        circ = false;
        lxx = 0;
        tye = -1;
        if (nuly != null) {
          tye = nuly.GetType();
          info = GetToolInfo(nuly);
          // circ = (info&1024) !=0; // (now unused)
          if (info>0) tuly = 0;} //~if
        if (Logging) if (GoodImex) System.out.println("  ++wgt " + here
            + ": '" + aWord + "' = " + rx + IffyStr(nuly==null,"",
            SeeHex(" x",info," t=") + tye) + IffyStr(kx<0," e" + kx,""));
        if (StrLength(aWord)==0) break; // normal exit
        if (StrLength(NthItemOf(',',2,aWord))>0) continue; // already got this
        // nuly = GetIxWgt(rx);
        if (nuly==null) continue; // (probly can't fail)
        // info = nuly.GetInfo();
        if ((info&-0x8000) == 0x4D478000) continue; // 'MGDW' (game bd already in)
        // aWord = "\n" + rx + ",";
        // whar = NthOfset(0,aWord,WgtMap);
        // if (whar<0) continue; // (probly can't fail)
        // whar = StrLength(aWord)+whar; // -> where new wgt+ goes
        aWord = "";
        aStr = "";
        velo = 0;
        // tye = nuly.GetType();
        kx = tye&255;
        posn = nuly.GetPosn();
        if (posn >= 0x40000000) posn = posn-0x40000000; // export it as unhid
        size = nuly.GetSize();
        color = nuly.GetColor();
        prnt = nuly.GetParent();
        xToolInfx = nuly.GetInfo();
        xTooLinex = "";
        xToolInit = "";
        xToolTyx = tye;
        more = FindSubWgt(nuly,(TextFontNo<<16)+0x0404,0); // sb wgt name
        if (more != null) {
          aWord = more.GetTextLn();
          zx = StrLength(aWord);
          if (!InvalidJava(aWord)) if (!NameIsTaken(aWord)) { // <== gen inits..
            why = 256;
            xToolInit = aWord;
            if (false) if (circ) inits = inits + "\n    if (" + aWord
                + " != null) myGame.CircleSize(" + aWord + ",0);";
            if (StrLength(namex)==0) namex = aWord;
              else namex = namex + " " + aWord;
            UserNames = UserNames + aWord + " ";
            aWord = " {" + aWord + "}";
            zx = 0;} //~if
          if (zx>0) aWord = " '" + aWord + "'";} //~if
        more = FindSubWgt(nuly,0x466C677A,0); // 'Flgz'
        if (more != null) {
          why = why|128;
          tye = more.GetType()&0xFF00|kx;
          xToolTyx = tye;
          velo = more.GetPosn(); // = velocity if sprite
          if (Logging) if (GoodImex) aStr = " f=#" + more.GetRefNum()
              + SeeHex(": t=x",tye,LogVH(" v=",velo,""));} //~if
        if (tuly>0) kx = kx-256; // embedded wgt inside tooled dragable..
        else if (kx==0) more = null; // rect inside sprite
        else if (kx==2) { // (circle) icon inside sprite..
          xToolInfx = -3;
          more = nuly;} //~if
        else if (kx==4) { // text inside sprite..
          more = nuly;} //~if
        else if (kx !=5) kx = 8;
        else if ((info&0x8000) !=0) kx = kx-512; // it's a game wgt
          else kx = 9;
        if (kx<0) { // let the tool do this one..
          why = why|64;
          kx = kx&255;
          if (tuly>0) info = tuly;
          info = TooledExport(info,nuly); // <<================= ExporTool(
          if (CharAt(0,xToolInit) == '\n') inits = inits + xToolInit;
          aWord = xTooLinex + aWord;
          tye = xToolTyx;
          if (info >= 0xFFFF) kx = 11; // very large means nothing to export
          else if (info >= 0) {
            why = why|32; // +96
            kx = tye&255;             // info>>2 is + sub-wgts
            if (info>3) lxx = info>>2; // +4K: gets wgt list from token sprite
            info = info&3;
            if (info==2) xToolInfx = velo; // info+2 adds velo
            else if (info>0) {
              more = GetIxWgt(nuly.GetIxData(0,0));
              if (more != null) {
                why = why-64; // +32
                if (info>1) color = more.GetColor();
                  else xToolInfx = more.GetInfo();}}} //~if
          else xToolInfx = info;
          if (lxx>0xFFF) { // get the wgt list from 1st child..
            why = why|16;
            more = GetIxWgt(nuly.GetIxData(0,0));
            lxx = lxx&0xFFF;} //~if
          else if (lxx>0) more = nuly; // so we get the wgt list
            else more = null;} //~if // (let the tool do this one)
        info = xToolInfx;          // GoodImex = GoodBits&256..
        if (Logging) if (GoodImex)
          System.out.println("  ... ex: k=" + kx + SeeHex(" i=x",info," t=") + tye
              + " +" + lxx + " #" + (thar+1) + aStr + " `" + aWord + "`");
        aStr = "";
        if ((kx>6)||((kx&1) !=0)) {// (shouldn't fail, but s'pose..)
          kx = -kx;
          continue;} //~if
        thar++;
        WgtMap = Substring(0,whar,WgtMap) + thar + " " + RestOf(whar,WgtMap);
        if (lxx==0) if (more != null)
          lxx = more.GetIxData(-1,-1); // = array size
        if (lxx<0) lxx = 0;
        aLine = aLine + "\n" + thar + " " + tye + " " + (posn>>16) + " "
            + SignExtend(posn) + " " + (size>>16) + " " + (size&0xFFFF)
            + HexIfMore(" ",color," ");
        // if (kx==4) aLine = aLine + HexIfMore(" ",info,""); // 2-sp cuz fnt
        // else if ((kx|4)==6) aLine = HexIfMore(aLine,info,"");
        // else if (more==null) aLine = HexIfMore(aLine,info,"");
        nx = 0;
        if (kx==2) nx++;
        else if (kx==4) nx++;
        else if (kx==6) nx++;
        else if (more==null) nx++;
        else if (more.IsValidIDtxt()) {
          aWord = " `" + more.GetIDtxt() + "`" + aWord;
          aLine = aLine + "0";} //~if
        else nx++;
        if (nx>0) aLine = HexIfMore(aLine,info,"");
        aLine = aLine + "  " + prnt + " " + lxx;
        if (prnt==GmBd) { // if (NthOfset(0," "+rx+" ",WgtMap)<0) {
          theList = theList + "  " + rx; // rx = old ref+
          nsubs--;} //~if // otherwise omit from list, don't count it
        if (lxx>0) if (kx>4) if (kx<8)
          for (zx=0; zx <= lxx-1; zx++) {
            bitz = more.GetIxData(zx,0); // "  " => new wgt+ in export..
            aLine = aLine + IffyStr(bitz>0,"  " + bitz," 0");} //~for
        aLine = aLine + aWord;
        if (nsubs <= 0) break;} //~for // (add actual wgts to list)
      while (true) { // fix those 2-src lines..
        zx = -9;
        kx = 9999;
        aStr = "?";
        here = NthOfset(0,",,=",WgtMap);
        if (here >= 0) while (true) { // got a 2-src line spec..
          here++;
          aStr = Substring(here+2,5,WgtMap) + " !";
          zx = SafeParseInt(aStr);
          if (zx <= 0) break;
          aStr = "\n" + zx + ",";
          whar = NthOfset(0,aStr,WgtMap);
          if (whar<0) break;
          whar = StrLength(aStr)+whar;
          aStr = Substring(whar,5,WgtMap) + " $";
          kx = SafeParseInt(aStr);
          break;} //~while
        if (Logging) if (GoodImex) {
          if (StrLength(aStr)>0) if (CharAt(0,aStr) < ' ')
            aStr = RestOf(1,aStr) + " ^";
          aStr = NthItemOf('\n',1,aStr);
          System.out.println("  @ " + here + " = " + zx + " -> " + whar
              + " => " + kx + " '" + aStr + "'");} //~if
        if (here <= 0) break;
        WgtMap = Substring(0,here,WgtMap)
            + kx + " " + RestOf(here,WgtMap);} //~while
      for (nx=nsubs; nx >= 1; nx += -1) aLine = " 0" + aLine;
      theList = theList + aLine + "\n";
      if (Logging) if (GoodImex) System.out.println(".. .. +" + nsubs
          + "\n" + theList + "\n.. ..\n" + WgtMap + "\n.. ..\n");
      bitz = 0;
      for (here=thar; here >= 2; here += -1) { // fix wgt+s..                         (MakeWgtLs)
        posn = 0;
        bitz = 0;
        aLine = NthItemOf('\n',here,theList);
        nx = StrLength(aLine);
        if (nx==0) break; // shouldn't happen
        tye = SafeParseInt(NthItemOf(' ',2,aLine))&255;
        lxx = SafeParseInt(NthItemOf(' ',10,aLine));
        if (Logging) if (GoodImex)
          aStr = " ..(fixWg #" + here + ") t=" + tye + " +" + lxx;
        // if (tye>3) if (tye<8) if ((tye==4)||(lxx>0))
        for (thar=8; thar <= lxx+10; thar++) {
          color = 0;
          bitz = 0;
          if (thar !=9) {
            if (tye<4) continue;
            if (tye>4) if (thar<11) continue;
            if (tye==4) if (thar>8) continue;} //~if
          aWord = NthItemOf(' ',thar,aLine);
          color = SafeParseInt(aWord);
          if (color<3) continue; // 1,2 both OK
          whar = NthOfset(0,"\n"+aWord+",",WgtMap);
          if (Logging) if (GoodImex)
            aStr = aStr + " " + thar + "=" + aWord + "@" + whar;
          bitz = thar;
          zx = NthOfset(0,"  "+aWord,aLine);
          size = StrLength(aWord)+2;
          if (whar >= 0) {
            color = SafeParseInt(Substring(whar+size,5,WgtMap));
            whar = 0;
            if (color>0) whar = zx;} //~if
          else { // unmapped wgt, might be ants..
            nuly = GetIxWgt(color);
            if (nuly==null) break;
            if (nuly.GetInfo() != 0x416E7449) break; // 'AntI'
            color = 0; // yes, just plug in 0..
            whar = zx;} //~else
          if (Logging) if (GoodImex) aStr = aStr + "=" + color + "/" + whar;
          if (whar <= 0) break;
          bitz = 0;
          whar++;
          size--;
          aLine = Substring(0,whar,aLine) + color + RestOf(whar+size,aLine);
          posn--;} //~for
        if (posn<0) {
          posn = NthOfset(0,"\n"+Substring(0,8,aLine),theList)+1;
          if (posn>0) theList = Substring(0,posn,theList) + aLine
              + RestOf(posn+nx,theList);} //~if
        if (Logging) if (GoodImex)
          System.out.println(aStr + "\n    @ " + posn + " => " + aLine);
        if (bitz==0) continue;
        posn = here;
        break;} //~for // (fix wgt+s)
      why = 6;
      if (Logging) if (GoodImex)
        System.out.println("(MkWgLst)...\n" + theList + "\n");
      if (bitz>0) if (posn>0) {
        System.out.println("!! oops line " + posn + " wd " + bitz);
        break;} //~if
      if (NthOfset(1," ",Game_Title)>0)
        aLine = "\n\nGameTitle = " + Quot2 + Game_Title + Quot2;
      else aLine = "";
      aLine = theList + aLine;
      WriteWholeTextFile("GameMaker.txt",aLine); // => AddUserWgt
      if (StrLength(GamesName)>0) if (SafeCompare(GamesName,DefaltGmName) !=0)
        WriteWholeTextFile(GamesName + ".txt",aLine);
      MkTmpStr = namex + inits;
      WgtList = theList;
      unSavedWgts = false;
      why = 0;
      break;} //~while
    if (Logging) if (GoodImex)
      System.out.println("(MakeWgtLs) = " + why); // why =
    return why>0;} //~MakeWgtLst

  /**
  * MakeUserJava calls MakeWgtLst to build an exportable widget list,
  * then copies the relevant information to the .java file with the
  * game name. If successful, it also puts up the button to create a JS
  * version of the game.
  */
  private void MakeUserJava() { // ** gotta deal w/'"'&'\'
    int here = 0, thar = 0, whar = 0, posn = 0, seen = 0, lxx = 0, why = 0;
    boolean didit = false, gotit = false;
    String data = "", namex = "", inits = "", aLine, aWord,     // .. +shft
        aStr = GamesName, fiName = aStr + ".java",
        theList = UsrGameJv;
    GameWgt whom = GameBoard;
    while (true) {
      why++; // why = 1
      gotit = false;
      here = NthOfset(0,"\n\n/**",theList); // theList = UsrGameJv;
      thar = NthOfset(0,"extends GameEvent",theList);
      if (here>8) if (thar>here) theList = RestOf(here+2,theList);
      UserNames = "";
      if (InvalidJava(aStr)) { // aStr = GamesName
        didit = GameAlert("'" + aStr + "' is not a valid Java class name");
        break;} //~if
      if (NameIsTaken(aStr)) {
        didit = GameAlert("The name '" + aStr + "' is already taken");
        break;} //~if
      UserNames =  " " + aStr + " ";
      // fiName = aStr + ".java";
      data = ReadWholeTextFile(fiName);                        // in MkUserJav
      why++; // why = 2
      if (StrLength(data)==0) {
        data = ReplacAll(GamesName,"UserGame",theList);
        why = 4;} //~if
      RecentJava = data;
      seen++;
      while (true) {
        if (NthOfset(0,"public class "+aStr+" extends GameEvent",data)<0)
          break;
        here = NthOfset(0,"($..)\n",data);
        lxx++; // =1
        if (here<0) break;
        thar = NthOfset(0,"\n  /// (..$)",data);
        lxx++; // =2
        if (thar<here) break;
        here = thar;
        thar = NthOfset(0,"<$..>\n",data);
        lxx++; // =3
        if (thar<here) break;
        here = thar;
        thar = NthOfset(0,"\n    /// <..$>",data);
        lxx++; // =4
        if (thar<here) break;
        here = thar;
        thar = NthOfset(0,"[$..]\n",data);
        lxx++; // =5
        if (thar<here) break;
        here = thar;
        thar = NthOfset(0,"\n    /// [..$]",data);
        lxx++; // =6
        if (thar<here) break;
        why++; // why = 3/5
        break;} //~while
      if (Logging) if (GoodImex)
        System.out.println("(MkUsJav) = " + why + "/" + lxx   // why =
            + " " + here + "/" + thar);
      if (why==2) {
        didit = GameAlert("File " + fiName
            + " cannot be upgraded, delete/rename it & retry");
        break;} //~if
      else if (why==4) {
        didit = GameAlert("Oops, GameMaker is corrupted, re-install it");
        break;} //~if
      else if (why<5) why = 5;
        // else gotit = true;
      if (MakeWgtLst()) break; // T: failed (logged)
      why++; // why = 6
      namex = MkTmpStr; // created by MkWgLs
      MkTmpStr = "";
      here = NthOfset(0,"\n",namex);
      if (here>0) {
        inits = RestOf(here+1,namex) + "\n";
        namex = Substring(0,here,namex);} //~if // = all wgt names
      theList = WgtList;
      aLine = "      = " + Quot2 + ReplacAll("\\n" + Quot2
            + "\n      + " + Quot2,"\n",theList) + Quot2 + ";\n";
      here = NthOfset(0,"<$..>\n",data);
      if (here<0) break; // (already tested)
      why++; // why = 7
      here = here+5;
      thar = NthOfset(0,"\n    /// <..$>",data)-here;
      if (thar<0) break; // (already tested?)
      here++;
      data = Substring(0,here,data) + aLine + RestOf(here+thar,data);
      why++; // why = 8
      if (StrLength(namex)>0) {
        aLine = "    GameWgt " + ReplacAll(" = null, "," ",namex) + " = null;\n";
        thar = 0;
        posn = 0;
        lxx = StrLength(aLine);
        for (whar=0; whar <= 2222; whar++) { // split into lines..
          here = NthOfset(whar,", ",aLine);
          if (here<0) {
            if (posn==0) break; // no commas on last line
            here = StrLength(aLine);
            if (here<thar+72) break;} //~if // last line is short (normal exit)
          posn++;
          if (here<thar+72) continue;
          if (posn>1) if (whar>0) {
            here = NthOfset(whar-1,", ",aLine);
            posn = -posn;} //~if
          here++;
          aLine = Substring(0,here,aLine) + "\n     " + RestOf(here,aLine);
          if (posn<0) posn = 1;
            else posn = 0;
          thar = here+5;} //~for // (split into lines)
        here = NthOfset(0,"($..)\n",data);
        if (here<0) break; // (already tested)
        why++; // why = 9
        here = here+5;
        thar = NthOfset(0,"\n  /// (..$)",data)-here;
        if (thar<0) break;
        here++;
        data = Substring(0,here,data) + aLine + RestOf(here+thar,data);
        why++; // why = 10
        aLine = "";
        for (whar=1; whar <= 2222; whar++) { // make assignments..
          aWord = NthItemOf(' ',whar,namex);
          if (StrLength(aWord)==0) break; // (normal exit)
          aLine = aLine + "    " + aWord + " = myGame.FindListWgt(\"{"
              + aWord + "}\");\n";} //~for // (make assignments)
        here = NthOfset(0,"[$..]\n",data);
        if (here<0) break; // (already tested)
        why++; // why = 11
        here = here+5;
        thar = NthOfset(0,"\n    /// [..$]",data)-here;
        if (thar<0) break;
        here++;
        data = Substring(0,here,data) + "    if (myGame==null) return;\n"
            + aLine + inits + RestOf(here+thar,data);
        if (Do_JS) { // Do_JS = true
          if (StrLength(GamesName)>0)
            gotit = toJS(data,GamesName);}} //~if
      else JSinfo = "There are no widgets in this game"; // why = 8
      why = -why; // why = -8/-11
      WriteWholeTextFile(fiName,data); // fiName = aStr + ".java";
      if (gotit) {
        RecentJava = data; // now unused
        ChooseBtn(4);} //~if
      else if (Do_JS) ChooseBtn(5);
        else ChooseBtn(0);
      // why = 0;
      break;} //~while
    if (Logging) if (GoodImex)                           // no early exits
      System.out.println("(MkUsrJav) '" + GamesName + "' " + gotit + " = " + why
          + IffyStr(why<6,""," " + here + "/" + thar));} //~MakeUserJava

  /**
  * Picks out a display version of the user name
  *
  * @param  aLine  the username_password
  *
  * @return        the prefix alphabetic name only
  */
  private String NameOnly(String aLine) { // pick out user name.. (+544)
    char aCh;
    int nx, info, lxx = StrLength(aLine)-1;
    nx = NthOfset(0,"=",aLine);
    if (nx >= 0) if (nx<lxx) {
      aLine = RestOf(nx+1,aLine);
      lxx = lxx-1-nx;} //~if
    for (nx=0; nx <= lxx; nx++) {
      aCh = CharAt(nx,aLine);
      info = (int)(aCh);
      if (info>96) if (info<123) continue;
      if (info>64) if (info<91) continue;
      aLine = Substring(0,nx,aLine);
      break;} //~for
    return aLine;} //~NameOnly

  /**
  * Driver code to convert the user's (Java) game code into JavaScript
  * and upload it to the game server. This method is called twice, the
  * first time with no user name (it asks you), then again with your reply.
  * The user name should be the same name you created for Chomp. If is
  * not, then your game will not upload.
  *
  * @param  whom    the user name, or none to ask for it
  */
  private void MakeUserJS(String whom) { // frm (JSbtn), (InputWhy=7)
    String data = RecentJava, title = GamesName, whoz = NameOnly(whom),
        link = "", G_T = Game_Title; // long form: "Uname's GameName Uname_pwd"
    int more = Countem(" ",G_T)+myPL, lxx = StrLength(data);
    boolean didit;
    if (lxx>0) if (more>1) if (StrLength(whom)==0)
        if (NthOfset(0,"'",G_T)>0) { // if (NthOfset(0,title,G_T)>0)
      whom = NthItemOf(' ',more,G_T); // = last word of GameTitle
      if (more>2) title = NthItemOf(' ',2,G_T); // = 2nd word
      whoz = NthItemOf('\'',1,G_T);} //~if // = prog'r name in JS page title
    whom = NthItemOf(' ',1,whom); // Unm+pwn must be 1 word
    if (Logging) if (GoodImex)
      System.out.println("(MkUsrJS) '" + whom + "' / " + title + " + " + lxx
          + "/" + more + " [" + G_T + "]");
    if (lxx==0) return;
    if (StrLength(title)==0) return;
    if (StrLength(whom)==0) {
      whom = ValidUserName;
      if (StrLength(whom)==0) {
        if (OpenInputDlog("Username_password:","")) InputWhy = 7;
        return;} //~if
      whoz = NameOnly(whom);} //~if
    lxx = NthOfset(0,"_",whom);
    if (lxx<0) lxx = StrLength(whom);
    if (lxx>0) {
      more = StrLength(whoz);
      link = Substring(0,lxx,whom);
      if (more<3) if (more<lxx) more = 0;
      if (more>0) title = whoz + "'s " + title;
      lxx = StrLength(JSinfo); // should be already translated
      if (lxx==0) {
        if (toJS(data,GamesName)) {}
        lxx = StrLength(JSinfo);} //~if
      data = JSinfo;} //~if
    else data = "Invalid empty user name";
    if (lxx<99) { // short = error msg..
      didit = GameAlert(data);
      return;} //~if
    if (StrLength(data)<99) return;
    JSinfo = "";
    data = "<!doctype html public '-//w3c//dtd html 4.0 transitional//en'>"
        + "\n<html><head><title>" + title + "</title></head><body>\n"
        + data + "</body></html>";
    lxx = ServerUpload(whom,data);
    if (lxx>0) {
      JSinfo = "Upload error #" + lxx;
      ChooseBtn(5);
      return;} //~if
    ValidUserName = whom;
    link = "http://ittybittycomputers.com/Java/GE.php?name=" + link;
    didit = GameAlert("OK, URL = '" + link + "'");} //~MakeUserJS

  /**
  * This manages visible widget list scroll operations. Called by:
  *   Startup to initialize
  *   SelectM to scroll to a particular wgt
  *   ClickEv to scroll
  *   NewListNam to add new name to list
  *
  * It works with: WgLsScroBar, WgLsScroThm, WgLsScrUpBt, WgLsScrDnBt, UsWgHiLi,
  * and these class variables: nUsrWgts, WgScroSkp, WgScrThmPo, WgScrThmSz,
  *   and these (constant) class vars: WgLsTall, WgLsOfx, nWgsVisi, WgScrBarSz
  *
  * @param  doit    the desired operation:
  *                   0: init
  *                   1: mouse Dn
  *                   2: mouse Up
  *                   3: drag
  *                   4: cancel drag
  *                   5: add or dele wgt
  *                   6:
  *                   7: select
  *                   8: (internal) set thumb to -info
  *                   9: (internal) set scroll to info
  * @param  info    whatever number is needed
  * @param  whom    the widget involved
  *
  */
  public void WgtLstMgr(int doit, int info, GameWgt whom) {
    int tall = WgLsTall, fustx = WgLsOfx, nwgs = nUsrWgts+1, visi = nWgsVisi,
        skip = WgScroSkp, posn = WgScrThmPo, thsz = WgScrThmSz, prio = doit,
        sss = wScrStpSz, scrosz = WgScrBarSz, data = info,
        nx = 0, kx = 0, zx = 0, yx = 0, rx = 0, bitz = 0, why = 0;
    GameWgt nuly, scroB = WgLsScroBar, upar = WgLsScrUpBt, dnar = WgLsScrDnBt,
        UNL = UwgtNamList, thmb = WgLsScroThm;
    boolean izzy = GoodScro;
    // GmMkBooTmp = false; // (unused)
    if (doit>3) izzy = false;
    while (true) {
      if (whom != null) rx = whom.GetRefNum();
      why++; // why = 1
      if (scroB==null) break;
      why++; // why = 2
      if (thmb==null) break;
      why++; // why = 3
      if (upar==null) break;
      why++; // why = 4
      if (dnar==null) break;
      if (doit==0) { // init (frm Startup)                   // 'WgLsMgr/0'
        why++; // why = 5
        if (UNL==null) break; // UNL = UwgtNamList
        why++; // why = 6
        if (info==0) break; // info = ID of [Main Game actual name]
        why++; // why = 7
        fustx = UNL.FindElemt(info); // = start of wgt names in list
        if (fustx==0) break;
        WgLsOfx = fustx;
        why++; // why = 8
        tall = UNL.GetTall();
        if (tall<32) break;
        WgLsTall = tall;
        why++; // why = 9
        visi = tall/WgtNameHi;
        if (visi<2) break;
        nWgsVisi = visi;
        why++; // why = 10
        zx = upar.GetTall();
        WgScrBtnSz = zx;
        upar.SetData(BtnAros8);
        upar.SetInfo(0);
        dnar.SetData(BtnAros8);
        dnar.SetInfo(20);
        if (posn <= 0) WgScrThmPo = zx;
        if (thsz==0) {
          WgScrThmSz = 8;
          thsz = 8;} //~if
        if (sss==0) wScrStpSz = 16;
        UsHiLiWgNo = -1;
        bitz = 13;
        scrosz = tall-(zx<<1);
        if (scrosz<20) break;
        WgScrBarSz = scrosz;
        scroB.HideSho(false); // assume all the rest =0 at startup
        why = 0;
        break;} //~if // (doit=0: init frm Startup)
      if (doit==1) { // frm GmMk/MouseDn, info is V          // 'WgLsMgr/1'
        why = -13;
        doit = 9;
        CluckTime = 0; // multi clicks in scrollbar are not 2-clk!
        if (WgScrState<0) {
          if (WgScrState+2>0) upar.SetInfo(0);
            else dnar.SetInfo(20);} //~if
        if (info<WgScrBtnSz) { // click in up arrow..
          why--; // why = -14
          upar.SetInfo(10);
          WgScrState = -1;
          if (skip>0) info = skip-1;
            else info = 0;} //~if
        else if (info<posn) { // click in page-up..
          why = -15;
          zx = visi-1;
          if (skip>zx) info = skip-zx;
            else info = 0;} //~if
        else if (info+WgScrBtnSz>tall) { // click in down arrow..
          why = -16; // => -17
          dnar.SetInfo(30);
          WgScrState = -3;
          info = skip+1;} //~if
        else if (info>posn+thsz) { // click in page-down..
          why = -17; // => -18
          info = skip+visi-1;} //~if
        else { // click in thumb..
          WgtBeingDrug = whom; // = WgLsScroBar
          WgScrState = skip;
          doit = 1;
          break;}} //~if // (doit=1: frm GmMk/MouseDn)
      else if (doit==2) { // frm GmMk/MouseUp, info is V     // 'WgLsMgr/2'
        CluckTime = 0; // multi clicks in scrollbar are not 2-clk!
        if (WgtBeingDrug==WgLsScroBar) {
          why = -25;
          WgtBeingDrug = null;
          if (info<0) info = 0;
            else info = -info;
          doit = 8;} //~if
        else if (WgScrState<0) {
          if (WgScrState+2>0) upar.SetInfo(0);
            else dnar.SetInfo(20);
          why = -22;} //~if
        else why = -29;
        if (doit<8) break;} //~if // (doit=2: frm GmMk/MouseUp)
      else if (doit==3) { // frm GmMk/Dragging, info is V,H  // 'WgLsMgr/3'
        if (WgtBeingDrug==WgLsScroBar) {
          why = -30;
          zx = SignExtend(info); // = H
          nx = info>>16;
          info = WgScrState; // revert to original if way off
          if (info<0) break;
          doit = 9;
          if (zx+32>0) if (zx<WgScrBtnSz+32) if (nx+32>0) if (nx<tall+32) {
            if (nx>0) {
              info = -nx;
              doit--;} //~if // doit = 8
            else info = 0;}} //~if
        else {
          why = -23;
          break;}} //~if // (doit=3: frm GmMk/Dragging)
      else if (doit==4) { // frm GmMk/CancelDrag             // 'WgLsMgr/4'
        why = -44;
        WgtBeingDrug = null;
        info = WgScrState;
        if (info<0) break;
        doit = 9;} //~if // (doit=4: frm GmMk/CancelDrag)
      else if (doit==5) { // update nUsrWgts: info=1 if new wgt, =-1 if dele
        why = 50;                                            // 'WgLsMgr/5'
        nwgs = nwgs+info;
        if (nwgs <= 0) break; // can't dele game board
        why++; // why = 51
        if (UNL==null) break; // UNL = UwgtNamList;
        if (nwgs<3) why = ~why; // why = -52 (not an error @ start)
          else why++; // why = 52
        if (UNL.GetIxData(nwgs-1+fustx,0) <= 0) break; // no wgt there
        if (why<0) why = -why;
        why++; // why = 53
        nUsrWgts = nwgs-1;
        if (whom != null) { // position this new name per skip..
          rx = whom.GetRefNum(); // = ID of actual name
          if (rx <= 0) zx = -8; // probly can't
            else zx = UNL.FindElemt(rx)-fustx;
          bitz = (zx<<8)&255|bitz; // for log
          if (zx >= 0) if (zx<nwgs) // else not an installed user wgt
            whom.SetPosn((zx-skip)*WgtNameHi+4,4);} //~if
        if (info==0) if (!FreshFaster) {
          bitz = 13;
          if (ActWgNmWgt != null) bitz = bitz|16;} //~if
        izzy = scroB.IsVisible();
        if (nwgs <= visi) {
          if (izzy) scroB.HideSho(false);
          if (WgScroSkp !=0) if (!FreshFaster) {
            if (ActWgNmWgt==null) bitz = bitz|9;
              else bitz = bitz|25;} //~if
          skip = 0;
          WgScroSkp = 0; // ToolNmLstScro = 0;
          izzy = false;
          why = -why; // why = -53
          break;} //~if
        thsz = visi*(scrosz-8)/nwgs;
        if (thsz<8) thsz = 8;
        else if (thsz+8>scrosz) thsz = scrosz-8;
        if (thsz != WgScrThmSz) {
          WgScrThmSz = thsz;
          if (!FreshFaster) bitz = bitz|4;} //~if
        nx = nwgs+1-visi; // = + past 1st frame, = + scroll steps
        if (nx>0) sss = ((scrosz-8-thsz)<<4)/nx; // = scro stp sz in 1/16-pix
          else sss = 16; // (unused but >0 anyway)
        if (sss != wScrStpSz) {
          wScrStpSz = sss;
          bitz++;} //~if
        if (posn+thsz>scrosz+WgScrBtnSz) {
          posn = scrosz-thsz;
          WgScrThmPo = posn;
          bitz++;} //~if
        if (!izzy) scroB.HideSho(true);
        izzy = false;
        why = ~why; // why = -54
        if (ToolActive) break;
        if (ActivWgt==null) break;
        why = 1-why; // why = 55
        whom = ActivWgt; // probly new wgt..
        doit = 7;} //~if // ..so scroll to it // (doit=5: update nUsrWgts)
      if (doit==7) { // select (frm SelectMe), positions & shows hili
        if (why >= 0) if (why<8) why = 70;                   // 'WgLsMgr/7'
        UsHiLiWgNo = -1; // whom is the new ('DW') wgt
        if (UNL==null) break; // otherwise look for name field..
        why++; // why = 71/56
        nuly = FindSubWgt(whom,(TextFontNo<<16)+0x0404,0); // nuly its name
        if (Logging) if (nuly==null)
          nuly = FindSubWgt(whom,(TextFontNo<<16)+0x0404,9000);
        if (nuly==null) break;
        why++; // why = 72
        rx = nuly.GetRefNum(); // = ID of actual name
        if (rx <= 0) break; // probly can't
        why = ~why; // why = -73
        zx = UNL.FindElemt(rx)-fustx;
        bitz = (zx<<16)+bitz; // for log
        if (zx<0) break; // not an installed user wgt (often not error)
        why = 1-why; // why = 74
        if (zx >= nwgs) break; // beyond last installed user wgt
        why++; // why = 75/60 => -76
        UsHiLiWgNo = zx; // tell caller posn OK, so sho
        ActWgNmWgt = nuly;
        // GmMkBooTmp = true; // tell caller we did it (unused)
        if (!FreshFaster) bitz = bitz|16;
        nuly = UsWgHiLi;
        if (nuly != null) if (!nuly.IsVisible()) {
          why = why+2; // why = 77 => -78
          nuly.HideSho(true);} //~if
        why = -why; // why = -75/-77
        if (zx<skip) { // off top, scroll it into view..
          if ((visi>>1)>zx) info = 0;
            else info = zx-(visi>>2);
          doit = 9;} //~if // why = -75/-77 => -76/-78
        else if (zx >= skip+visi) { // off btm, scroll into view..
          info = zx-(visi>>1);
          doit = 9;} //~if
        else break;} //~if // (doit=6: select frm SelectMe)
      if (doit>0) { // doit = 9/8 set scroll from info
        // nx = nwgs+1-visi; // = + past 1st frame, = + scroll steps
        // if (nx>0) nx = ((scrosz-8)<<4)/nx; // = scro stp sz in 1/16-pix
        //   else nx = 0;
        if (doit<8) bitz = bitz|32;
        else if (doit==8) { // set scroll to posn (recursive)
          if (why>8) why = -why;
          else if (why >= 0) why = -80;
          if (nwgs <= visi) break; // nothing scrollable
          if (info<0) { // it's a new thumb position..
            why--; // why = -81;
            if (info+posn==0) break; // already there
            zx = WgScrBtnSz+3;
            why = 1-why; // why = 82
            info = -info;
            if (info<zx) info = 0; // now it's + wgts scrolled off top..
            else if (info+thsz+zx<tall) {
              why--; // why = 81
              if (nx>1) info = (((info-4-WgScrBtnSz)<<5)/sss+1)>>1;
                else break;
              why++;} //~if // why = 82
            else info = nwgs-visi;
            why = -why;}} //~if // why = -82 // (doit=8: set scroll to posn)
        // if (doit>0) { // doit = 9/8 // set skip to wgts off top (recursive)
        if (why>8) why = -why;
        else if (why >= 0) why = -90;
        why--; // why = -91/-83/-67/-62/-48/-46/-24/-14..-19
        zx = info;
        if (info <= 0) {
          posn = WgScrBtnSz;
          info = 0;} //~if
        else if (info+visi >= nwgs) {
          info = nwgs-visi;
          posn = tall-thsz-WgScrBtnSz;} //~if
        else posn = ((info*sss+7)>>4)+WgScrBtnSz;
        if (!FreshFaster) {
          if (WgScroSkp != info) {
            if (ActWgNmWgt==null) bitz = bitz|9;
              else bitz = bitz|25;} //~if
          else if (WgScrThmPo != posn) bitz++;} //~if
        skip = info;
        WgScroSkp = skip; // ToolNmLstScro = skip;
        WgScrThmPo = posn;} //~if // (doit=9: set skip to wgts off top)
      break;} //~while
    if (why>1) if (why<12) if (scroB != null) {
      scroB.HideSho(false);
      WgLsScroBar = null;
      bitz = bitz&-256;} //~if
    if ((bitz&3) !=0) if (thmb != null) // update thumb position per posn..
      thmb.SetVposn(posn);
    if ((bitz&4) !=0) if (thmb != null) // update thumb size per thsz..
      thmb.SetTall(thsz);
    if ((bitz&8) !=0) for (nx=nUsrWgts; nx >= 0; nx += -1) { // reposn all names per skip..
      if (UNL==null) break;
      yx = UNL.GetIxData(nx+fustx,0);
      if (yx==0) continue;
      nuly = GetIxWgt(yx);
      if (nuly==null) break;
      nuly.SetPosn((nx-skip)*WgtNameHi+4,4);} //~for
    nuly = UsWgHiLi; // reposition UsWgHiLi per skip..
    whom = ActWgNmWgt;
    // izzy = false; // soon: T if hili is visi (now bitz&64)
    kx = -8;
    if (nuly != null) if (nuly.IsVisible()) while (true) {
      kx--;
      bitz = bitz|64; // izzy = true;
      if ((bitz&16)==0) break;
      yx = SafeWgtNum(whom); // = ID of actual name, =0 if null
      if (yx <= 0) kx = -2; // yx=0 if whom=null
      else if (UNL==null) kx = -3;
        else kx = UNL.FindElemt(yx)-fustx;
      if (kx>nwgs) kx = -4; // else not an installed user wgt
      if (kx<0) {
        nuly.HideSho(false);
        bitz = bitz&~64;} //~if // izzy = false;
      else if (kx<skip) nuly.SetPosn(-WgtNameHi*2,0); // scrolled off top
        else nuly.SetPosn((kx-skip)*WgtNameHi,0); // shows hili there
      break;} //~while
    if ((bitz&64)==0) ActWgNmWgt = null;
    if (Logging) if (GoodScro || !FreshFaster) { // GoodScro => ScroTmp
      System.out.println("(WgLsMgr/" + IffyStr(prio==doit,"",LogVH("",prio,","))
          + doit + ") #" + rx + LogVH(" i=",info,LogVH("/",data,
          SeeHex(" x",bitz," sc="))) + skip + " " + posn + " [+" + fustx
          + " " + nwgs + "/" + visi + " sz= " + tall + " " + scrosz
          + " " + WgScrBtnSz + thsz + FormFixt(" *",sss<<4) + "=" + sss
          + "\n  .. " + zx + IffyStr((bitz&31)>0," wHiLi #" + yx
            + " = " + kx,"")
          + why );} } //~WgtLstMgr

  /**
  * Where ChooseBtn selects one (or none) of four buttons for the user
  * to click on, DoMyButn does whatever that button is specified to do
  * when the user clicks on it, and returns true if it did it.
  *
  * @param  whom    the widget clicked on
  *
  * @return         true if it did it, false if not one of these 4 buttons
  */
  public boolean DoMyButn(GameWgt whom) {
    boolean doit = false;
    int rx = 0, zx = 0, why = 0;
    GameWgt more = ActivWgt; // might get cleared by Blur
    GameWgt nuly = UsWgHiLi;
    while (true) {
      why++; // why = 1
      if (whom==null) break;
      rx = whom.GetRefNum();
      why++; // why = 2
      if (whom.GetPosn()>0x30000000) break; // not visi
      why++; // why = 3
      if (false) if (nuly != null) if (nuly.GetPosn()>0x30000000)
        break; // User Wgt not sel'd
      why++; // why = 4
      if (whom==OKbtn) {
        if (more != null) {
          WuzChanged = true;
          CapturInfo2Wgt(more);
          if (more==GameBoard) if (NameChanged || unSavedWgts)
            RefreshPan = true;
          if (RefreshPan) ShoInfoFrom(more);
          if (!ToolActive) if (!FreshFaster) {
            doit = MakeWgtLst(); // T: failed
            ChgdBitSet = 0;} //~if
          if (ChgdBitSet !=0) NewData(false);
          else if (doit) ChooseBtn(0);
            else ChooseBtn(3);} //~if // InChanged = false;
        why = ~why;
        if (Logging) System.out.println("(OKbtn) A=#" + SafeWgtNum(more)
            + IffyStr(FreshFaster," FF"," nf"));} //~if
      else if (whom==BildBtn) {
        why = -why;
        if (Logging) System.out.println("(BildBtn) A=#" + SafeWgtNum(more));
        MakeUserJava();} //~if
      else if (whom==JSbtn) {
        why = -why;
        if (Logging) System.out.println("(JSbtn)");
        ChooseBtn(0);
        // GameAlert("Coming to a theater near you: JavaScript");
        MakeUserJS("");} //~if
      else if (whom==CantJS) {
        why = -why;
        if (Logging) System.out.println("(CantJS)");
        ChooseBtn(0);
        zx = StrLength(JSinfo);
        if (zx>0) if (zx<99) doit = GameAlert(JSinfo);} //~if
      else if (whom==UndoBtn) {
        why = -why;
        if (Logging) System.out.println("(UndoBtn)");
        } //~if
      else break;
      BlurEv(null);
      if (why>0) why = 0;
      break;} //~while // if (why >= 0) // only exit comes thru here..
    if (Logging)
      System.out.println("(DoMyBtn #" + rx + ") A=" + SafeWgtNum(ActivWgt)
          + " = " + why );
    if (SRchanged) ReportChBitS();
    return (why <= 0);} //~DoMyButn

  /**
  * When the user clicks on a tool or game widget, SelectMe is called
  * to put that widget's information up on the tool panel. It returns
  * false if it is not a tool or game widget.
  *
  * @param  user    >0 if it's a user game widget, =0 if a tool
  * @param  whom    the widget clicked on
  *
  * @return         true if it's now selected, false if failed
  */
  public boolean SelectMe(int user, GameWgt whom) { // rtns true if did
    boolean prio = FreshFaster; // user=0 if tool, >0 game wgt, <0 in transit
    int info = 0, sx = 0, nx = 0, rx = 0, zx = 0, posn = 0, thar = 0, why = 0;
    boolean velo = false, doit = false; // T: hide VeloEntry/AIdlgChkBx
    GameWgt nwgt = null, nuly = ToolHiLi, whar = UsWgHiLi,
        UNL = UwgtNamList, slst = SpriteList;
    GameTool tuly = ActivGmTool;
    while (true) {
      sx = ActvGmTooInf;
      if ((sx&0x8000) !=0) zx++; // save changed user wgt data..
      else if (sx>0) zx--;
      if (zx !=0) if (ActivWgt != null) if (whom != ActivWgt) {
        FreshFaster = true; // auto-update: don't write file
        if ((ChgdBitSet&0xFFFA) !=0) {
          CapturInfo2Wgt(ActivWgt);
          ChgdBitSet = 1; // b0=file
          why = 128;} //~if
        else ChgdBitSet = ChgdBitSet&1; // b0=file
        NewData(false);
        FreshFaster = prio;               // if (zx<0)
        if (nuly != null) if ((whom==null)||(user !=0)) { // hide tool hili..
          why++; // why = 1
          nuly.HideSho(false);} //~if        // if (zx>0)
        if (whar != null) if ((whom==null)||(user <= 0)) { // hide wgt hili..
          why = why+2;
          if (whar.IsVisible()) whar.HideSho(false);} //~if // got a wgt sel'd
        if ((why&3)==0) why = why|256;
        if (tuly != null) {
          why = why|64;
          tuly.HidWgtsDat();
          ActivGmTool = null;} //~if // only cleared here..
        ActvGmTooInf = 0;
        ActWgNmWgt = null;
        ActivWgt = null;} //~if // only cleared here
      if (why==0) why = 259;
      if (whom==null) break; // why = 1/2/256/259/+64/+128 (1st brk)
      nx = whom.GetRefNum(); // = ID of dragable asset list
      sx = GetToolInfo(whom);
      if (whom==ActivWgt) {
        why = why+48;
        break;} //~if
      why = why+4; // why = 4..7
      ////* info = whom.GetInfo()&0xFFFF; // (unneeded)
      // if ((info&-4) != 0x4454) break; // 'DT' or 'DW'
      zx = nx; // (for log only)
      if (nx <= 0) break; // probly can't
      if ((whom.GetType()&255) !=5) break;
      if (user !=0) {
        why = why+4; // why = 8..11
        if ((sx&0x8000)==0) break; // =0 if tool ['DW']
        WgtLstMgr(7,0,whom); // 'WgLsMgr/7'                      // (SelectM)
        zx = UsHiLiWgNo;
        if (zx<0) // WgLsM failed..
          if (slst==null) break; // no sprite list for transients
            else if (slst.FindElemt(nx)<0) break; // not a transient
        ActivWgt = whom;
        ToolActive = false;
        ShoInfoFrom(whom); // calls TooledShoD
        why = -why;} //~if
      else if (nuly != null) { // should be a tool..
        why = why+32; // why = 36..39
        if (sx==0) break;
        if ((sx&0x8000) !=0) break; // unwant user wgt ['DW']
        rx = whom.GetParent();
        zx = rx;
        if (rx <= 0) break; // probly can't
        why = why+4; // why = 40..43
        nwgt = GetIxWgt(rx); // this is the locator token in list
        if (nwgt==null) break;
        posn = nwgt.GetPosn();
        nuly.SetPosn(posn>>16,posn&0xFFFF); // auto'ly shows it there
        ActivWgt = whom;
        ToolActive = true;
        ShoInfoFrom(whom);
        why = -why;} //~if
      break;} //~while
    if (ActivWgt==null) { // something went wrong..
      if (nuly != null) nuly.HideSho(false);
      if (whar != null) whar.HideSho(false);
      ToolActive = false;
      ShoInfoFrom(NullWgt);
      why = why|512;
      doit = true;} //~if
    else if (why>0) if (whom==ActivWgt) why = -why;
    if (Logging) System.out.println("(SelecMe #" + nx + ") " + user
        + SeeHex(" x",sx," ") + zx + " " + doit
        + " A=" + SafeWgtNum(ActivWgt) + IffyStr(prio," FF"," nf")
        + " = " + why );
    FreshFaster = prio;
    if (SRchanged) ReportChBitS();
    return why<0;} //~SelectMe

  /**
  * When a tool is dragged to the game board, NewListName is called to
  * add its name to the list of user game widgets and selected.
  *
  * @param  whom    the widget to be added to the list
  *
  * @return         true if it succeeded
  */
  public boolean NewListName(GameWgt whom) { // whom must be asset list
    int ix, nx = 0, rx = 0, zx = 0,          // (but not easy to vfy)
        thar = 0, top = 0, why = 0;       // rtns true if there & sel'd
    String name = "";
    int[] data = null;
    GameWgt nuly = null, nwgt = null;
    GameWgt whar = ListofWgts;
    while (true) {
      why++; // why = 1
      if (whar==null) break;
      why++; // why = 2
      if (whom==null) break;
      nx = whom.GetRefNum();
      why++; // why = 3
      if (whar.FindElemt(nx)<0) { // else already there
        why++; // why = 4               // look for name field..
        nwgt = FindSubWgt(whom,(TextFontNo<<16)+0x0404,0);
        if (Logging) if (nwgt==null)
          nwgt = FindSubWgt(whom,(TextFontNo<<16)+0x0404,9000);
        why++; // why = 5
        if (nwgt==null) break;
        name = "";
        rx = nwgt.GetRefNum(); // this is the actual name wgt
        why++; // why = 6
        name = nwgt.GetTextLn();
        if (StrLength(name)==0) break;
        why++; // why = 7
        top = whar.AddElemt(nx)*WgtNameHi; // index (now posn), or -1 if failed
        if (top<0) break;
        whar = UwgtNamList;
        why++; // why = 8
        if (whar==null) break;
        zx = nwgt.GetPosn()&0xFFFFFF; // (unhide it)
        nwgt.SetPosn((zx>>16)+top,SignExtend(zx));
        zx = whar.GetRefNum();
        if (false) {
          why = 13;
          data = new int[3];
          if (data==null) break;
          data[0] = rx;
          data[1] = nx;
          why++; // why = 14                         // (in NewListNam)..
          nuly = NewGameWgt(5,top,0,16,56,UwgtNamID,-1,0,0,data,null);
          if (nuly==null) break;
          thar = nuly.GetRefNum();
          why++; // why = 15
          if (whom.AddElemt(thar)<0) break; // add link to asset list
          why = 8;} //~if
        why++; // why = 9
        if (whar.AddElemt(rx)<0) break; // finally add its name to name list
        nwgt.SetParent(zx); // owned by (so it displays in) the list
        WgtLstMgr(5,1,nwgt); // 'WgLsMgr/5'
        why++;} //~if // why = 10
      else why = 11; // (in case there, but sele failed)
      if (whom==ActivWgt) {
        ActWgNmWgt = null;
        ActivWgt = null;} //~if
      if (SelectMe(nx,whom)) { // (T: succeeded) also displays it
        if (!FreshFaster) Antsify(nx>0,whom);
        why = -why;} //~if
      break;} //~while
    if (Logging) System.out.println("(NewLstNm #" + nx + ") => #" + thar
        + " '" + name + "' " + " " + rx
        + IffyStr(FreshFaster," FF = "," nf = ") + why);
    return why <= 0;} //~NewListName

 /**
  * If this widget is a group child, move it up or down in its group's data.
  * Called from KeyFld, no early return.
  *
  * @param  whom  The widget to be moved
  * @param  info  true if up
  */
  public void MoveWgtUpDn(GameWgt whom, boolean uppy) {
    int rx = 0, px = 0, inx = 0, why = 0;
    GameWgt prnt;
    while (true) {
      why++; // why = 1
      if (whom==null) break;
      rx = whom.GetRefNum();
      px = whom.GetParent();
      why++; // why = 2
      if (px <= 0) break;
      prnt = GetIxWgt(px);
      why++; // why = 3
      if (prnt==null) break;
      why++; // why = 4
      if (((prnt.GetType()-5)&255)>2) break;
      why++; // why = 5
      inx = prnt.FindElemt(rx);
      if (inx<0) break;
      why++; // why = 6
      if (!uppy) { // move down = exchange w/lower in data..
        if (inx==0) {
          why++; // why = 7
          break;} //~if
        inx--;} //~if // why = 6
      else if (prnt.GetIxData(inx+1,0)==0) break; // why = 6
      prnt.XchElemt(inx);
      why = 0;
      break;} //~while
    if (Logging) System.out.println("(MoveWg #" + rx + ") @ " + inx
        + " in #" + px + " " + uppy + " = " + why);} //~MoveWgtUpDn

 /**
  * Make PendingName actual for this game
  * (after user approved or cancelled restart).
  */
  public void SetNuName() {
    int why = 0;
    GameWgt whom = GameBoard;
    while (true) {
      why++; // why = 1
      if (whom==null) break;
      why++; // why = 2
      if (StrLength(PendingName)==0) break;
      why++; // why = 3
      whom = FindSubWgt(whom,(TextFontNo<<16)+0x0404,0); // (name field)
      if (whom==null) break;
      why++; // why = 4
      if (SafeCompare(whom.GetTextLn(),PendingName)==0)
        if (SafeCompare(GamesName,PendingName)==0) break;
      why++; // why = 5
      GamesName = PendingName;
      NameChanged = true;
      if (whom.PutTextLn(PendingName)>0) {
        PendingName = "";
        why = 0;} //~if
      if (why>0) break;
      if (ActivWgt != GameBoard) break;
      why--; // why = -1
      whom = WgtNamEntry;
      if (whom==null) break;
      if (whom.PutTextLn(GamesName)>0) why--; // why = -2
      break;} //~while
    if (Logging) System.out.println("(SetNuNm) '" + GamesName + "' / '"
        + PendingName + "' = " + why);} //~SetNuName

  public int GetValue(int why) { // . . . . . . . . . . . . . . . . . (GameMk)
    if (why+2==0) SetNuName(); // makes GamesName = PendingName
    else if (why<0) return StrLength(AddUserWgts); // GetValue(-1)
    if (why==1) return ChosenBtn;
    // if (why==2) if (InChanged) return 1;
    return 0;} //~GetValue

  // These are enabled by Gtype&512:

 /**
  * KeyFld is called for any widget that has focus when the user
  * types any other key. This override looks for keystrokes that could
  * change data, and marks the game as changed if so. It depends on super
  * to enter those keystrokes into whatever text field has focus.
  *
  * @param  whom  The widget that has focus
  * @param  info  The key that was typed
  */
  public void KeyFld(GameWgt whom, char info) { // frm KeyData        (GameMk)
    if (Logging) System.out.println("  (GmMk.KeyFld) #" + SafeWgtNum(whom)
        + " " + (UserMode) );
    if (DlogIsOpen>0) {
      if (whom != DlgInputData) if (DlgInputData != null)
          if (DlgInputData.IsVisible()) if (whom != DlgInputData) {
        whom = DlgInputData;
        if (FocusWgt != whom) if (DefaultFocus != whom)
          FocusEv(whom,-8,-4);}} //~if
    else if ((whom.GetType()&255) !=4) {
      if ((info == '+')||(info == '-')) {
        if (whom != null) MoveWgtUpDn(whom,info == '+');
        return;} //~if
      // else if (info == '~') JvUpLdTest();
      else if (info == '$') LogAlWgts("");} //~if
    super.KeyFld(whom,info);
    if (info<' ') if (info != '\b') return;
    if (TxBitSel==0) return;
    ChgdBitSet = ChgdBitSet|TxBitSel; // 'F*': '*' must ChgdBitSet bit+ **
    if (SRchanged) ReportChBitS();
    NewData(false);} //~KeyFld

 /**
  * GotDelKey is called when the user types the DelKey. This override
  * deletes any text in the FocusWgt (if it is text) or the input dialog,
  * if it is open.
  *
  * @param  whom  The widget that has focus
  */
  public void GotDelKey(GameWgt whom) { // frm KeyData                (GameMk)
    int zx;
    if (Logging) System.out.println("  (GmMk.DelKy) #" + SafeWgtNum(whom));
    if (DlogIsOpen>0) {
      if (whom != DlgInputData) if (DlgInputData != null)
          if (DlgInputData.IsVisible()) if (whom != DlgInputData) {
        whom = DlgInputData;
        if (FocusWgt != whom) if (DefaultFocus != whom)
          FocusEv(whom,-7,-7);}} //~if
    if (whom != null) zx = whom.PutTextLn("");} //~GotDelKey

 /**
  * GotArrowKey is called for any widget that has focus when the user
  * types one of the arrow keys. This override diverts left/right arrows
  * to the FocusWgt (if it is text) or the input dialog, if it is open.
  *
  * @param  whom  The widget that has focus
  * @param  info  The arrow key that was pressed, one of the constant
  *               values JavaGame.Arro_UP, JavaGame.Arro_DOWN,
  *               JavaGame.Arro_LEFT, or JavaGame.Arro_RIGHT.
  */
  public void GotArrowKey(GameWgt whom, char info) {               // (GameMk)
    int tye = 0, refn = 0, whar = 0, why = 0;
    GameWgt prio = null;
    if (whom==null) return;
    tye = whom.GetType();
    if ((info == Arro_RIGHT)||(info == Arro_LEFT)) {
      if (DlogIsOpen>0) why--;
      else if ((FocusWgt==whom)||(DefaultFocus==whom))
        if ((tye&767)==516) why--;
      if (why<0) {
        KeyFld(whom,info);
        return;}} //~if
    if ((tye&2048)==0) return;
    whar = DrugWgtOfx; // (still valid after mouseUp)
    if (whar<0) return;
    if (info == Arro_DOWN) whar = whar+0x10000;
    else if (info == Arro_UP) whar = whar-0x10000;
    else if (info == Arro_RIGHT) whar++;
    else if (info == Arro_LEFT) whar--;
      else return;
    refn = whom.GetRefNum();
    prio = WgtBeingDrug;
    WgtBeingDrug = whom;
    Dragging(whom,whar>>16,SignExtend(whar));
    if (Logging)
      System.out.println("** DragKey #" + refn + " ** = " + why);
    WgtBeingDrug = prio;} //~GotArrowKey

 /**
  * BlurEv is called for a widget when it loses focus. This override lets
  * super do whatever needs doing, except that it looks for active marching
  * ants and turns them off.
  *
  * @param  whom  The widget losing focus, or null if unknown by caller
  */
  public void BlurEv(GameWgt whom) { // from MsDispch                 (GameMk)
    int nx, rx = SafeWgtNum(whom);
    if (Logging) System.out.println("  (GmMk.BlrEv) " + (mAntsHost != null));
    if (mAntsHost != null) Antsify(false,mAntsHost);
    if (rx>0) if (TabIsAt>0) for (nx=0; nx <= 15; nx++) if (Tab2fields[nx]==rx) {
      TabIsAt = 0;
      break;} //~if //~for
    super.BlurEv(whom);} //~BlurEv

 /**
  * FocusEv is called for any widget that is "Focusable" when the user
  * clicks on it. This override lets super do whatever needs doing,
  * except that it also calls SelectMe and then Antsify activate the
  * marching if appropriate.
  *
  * @param  whom  The widget to be given focus
  * @param  vert  The vertical offset into this widget where
  *                 it was clicked
  * @param  horz  The horizontal offset into this widget
  */
  public void FocusEv(GameWgt whom, int vert, int horz) {          // (GameMk)
    int rx = SafeWgtNum(whom), yx = 0, zx = 0, offx = 0, posn = 0, info = 0,
        seen = 0, why = 0;
    if (whom==null) yx--;                              // from MsDispch
    else if (DlogIsOpen==0) {
      seen = ToolFocus(whom,WgScroSkp); // = GetToolInfo(whom); ////*
      if (whom==DlgInputData) yx = -73;
        else yx = seen>>16;
      if (yx>0) whom = GetIxWgt(yx);
      seen = seen&0xFFFF;} //~if
    else if (whom != DlgInputData) if (DlgInputData != null)
        if (DlgInputData.IsVisible()) {
      whom = DlgInputData;
      yx = SafeWgtNum(whom);} //~if
    if (Logging||(why>2))
      System.out.println("  (GmMkr.FocEv #" + IffyStr(yx>0,"" + yx + "/","")
          + rx + ") " + IffyStr(why>0,LogVH("(",posn,
            LogVH(IffyStr(offx<0,") ",") +"),offx," => ")) + zx
          + " f=" + SafeWgtNum(FocusWgt) + " A=" + SafeWgtNum(ActivWgt)
          + IffyStr(yx>0," T"," " + yx) + SeeHex(" i=x",info,
            IffyStr(FreshFaster," FF = "," nf = "))
          + why,"") );
    if (whom==null) {
      BlurEv(null);
      return;} //~if
    super.FocusEv(whom,vert,horz);
    if (whom == DlgInputData) return;
    rx = yx; // <0 (reason) or >0 (game wgt, make ants) or =0 (tool)
    if ((seen>0) && ((seen&0x8000)==0)) rx = 0; // was: IzTool
    else if (FocusWgt != whom) rx = 0;
    else if (yx<0) rx = yx; // ToolFoc said no, yx = rx/-reason
    else if (!SelectMe(rx,whom)) rx = yx; // F: failed, yx = rx/-reason
    else if (yx<0) rx = yx; // successfully selected but should not do ants
    if (!FreshFaster) Antsify(rx>0,whom);} //~FocusEv

 /**
  * GotTabKey is called when the user types the TabKey.
  *
  * @param  whom  The widget that has focus
  */
  public void GotTabKey(GameWgt whom) {
    int prio = TabIsAt, rx = SafeWgtNum(whom), nx, zx, tx = 0, why = 0;
    if (DlogIsOpen>0) {
      why--; // why = -1
      TabIsAt = 0;
      if (Tab2fields[0]>0) {
        whom = GetIxWgt(Tab2fields[0]);
        if (whom != null) if (whom.IsVisible()) tx = Tab2fields[0];}} //~if
    else for (nx=TabIsAt+1; nx <= TabIsAt+16; nx++) {
      zx = Tab2fields[nx&15];
      if (zx==0) continue;
      whom = GetIxWgt(zx);
      if (whom==null) continue;
      if (!whom.IsVisible()) continue;
      why = nx;
      tx = zx;
      break;} //~for
    if (tx>0) if (whom != null) if (SafeWgtNum(whom)==tx) {
      if ((whom.GetType()&255) !=4)
        whom = GetIxWgt(whom.GetIxData(0,0)); // actual text wgt in this fld
      if (whom != null) {
        nx = whom.GetWide();
        FocusEv(whom,4,nx-4);
        TabIsAt = tx;
        why = why+100;}} //~if
    if (Logging) System.out.println("(GotTbKy #" + SafeWgtNum(whom) + ") tx=#"
        + tx + "/" + TabIsAt + "/" + prio + " = " + why);} //~GotTabKey

 /**
  * GotEscKey is called when the user types the EscapeKey or clicked Cancel.
  *
  * @param  whom  The widget that has focus
  *
  * @return       true if taken (by alert or input dlog)
  */
  public boolean GotEscKey(GameWgt whom) { // from KeyData            (GameMk)
    if (Logging) System.out.println("(GotEscKy #" + SafeWgtNum(whom) + ") df=#"
        + SafeWgtNum(DefaultFocus) + IffyStr(GoodNew&&(InputWhy>8)," $&","")
        + IffyStr(EvsDlogOp==0,""," !? " + EvsDlogOp) + " inWhy=" + InputWhy
        + " => '" + PendingName + "'");
    if (super.GotEscKey(whom)) { // cancelled the dialog
      if (StrLength(PendingName)>0) SetNuName();
      InputWhy = 0;} //~if
    return false;} //~GotEscKey

 /**
  * GotEnterKey is called when the user types the EnterKey or clicked Cancel.
  *
  * @param  whom  The widget that has focus
  *
  * @return       true if taken (by 2-click or alert or input dlog)
  */
  public boolean  GotEnterKey(GameWgt whom) { // from KeyData         (GameMk)
    int kx = EvsDlogOp, prio = InputWhy, // dont = StrLength(PendingName),
        nx = 0, info = 0, seen = 0, why = 0;
    boolean gotit = false;
    String msg = "", data = "";
    GameWgt nuly = null;
    while (true) {
      why++; // why = 1
      if (whom != null) {
        nx = whom.GetRefNum();
        seen = GetToolInfo(whom); ////*
        info = whom.GetInfo();} //~if
      if (kx !=0) { // = DlogIsOpen, except <0 when dlog about to close
        nuly = GetIxWgt(Math.abs(kx));
        data = DlgEvData();} //~if
      if (Logging) msg = "(GotEntKy #" + nx + " df=#" + SafeWgtNum(DefaultFocus)
          + ") A=" + SafeWgtNum(ActivWgt) + " o=#" + SafeWgtNum(nuly)
          + " cb=" + ChosenBtn + IffyStr(GoodNew&&(InputWhy>8)," $&","")
          + IffyStr(kx==0," "," !? " + kx + "/" + InputWhy + " '" + data + "'");
      if (super.GotEnterKey(whom)) {
        if (theInputDlg != null) if (theInputDlg==nuly) {
          whom = nuly;
          gotit = true;} //~if
        if (!gotit) if (theAlertDlg != null) if (theAlertDlg==nuly) {
          whom = nuly;
          gotit = true;} //~if
        if (gotit) // defalt foc was stolen by dlog, restore it to OK btn..
          if (ChosenBtn>0) ChooseBtn(ChosenBtn); // ** better: save/restore
        gotit = false;
        if (whom==null) break;
        why = 10+InputWhy;
        if ((whom==theInputDlg)||(InputWhy>8)) {
          // data = DlgEvData();
          if (Logging) msg = msg + " = '" + data + "'";
          switch (InputWhy) {
          case 1: // got new color..
            if (StrLength(data)>0) {
              SetSwatchColor(ColoBx,SafeParseInt(data));
              if (ToolActive) ChgdBitSet = ChgdBitSet|0x20000;
                else ChgdBitSet = ChgdBitSet|2; // b1=color
              NewData(false);} //~if
            else gotit = GotEscKey(whom); // none, treat it as Cancel
            break;
          case 2: // got new text for txt wgt..
            if (TextInWgt==null) break;
            info = TextInWgt.PutTextLn(data);
            if (ToolActive) ChgdBitSet = ChgdBitSet|0x80000000;
              else ChgdBitSet = ChgdBitSet|0x8000; // b3=flags
            NewData(false);
            TextInWgt = null;
            break;
          case 7: // got new Username_password..
            if (StrLength(data)>0) MakeUserJS(data);
            break;
          case 9: // RestartBtn: user accepted Restart/discard..
            RestartMe(2,true);
            break;
          case 10: // user chose to open saved game..
            AddUserWgts = SavedGameData;
            RestartMe(1,true);
            break;
          case 0: // (none)
            break;} //~switch
          if (StrLength(PendingName)>0) SetNuName();
          InputWhy = 0;} //~if
        else if (whom != theAlertDlg) break;
        // if (whom != null) whom.HideSho(false); // super did it
        // DlogIsOpen = 0;
        why = why+10;} //~if
      else if (whom==ColoBx) {
        why++; // why = 2
        if (whom==null) break;
        why++; // why = 3
        BlurEv(whom);
        info = SwatchGetColor(whom);
        if (info >= 0)
          data = SeeHex("0x",info,"");
        else data = "-1";
        if (OpenInputDlog("New color: (0xRRGGBB)",data)) InputWhy = 1;} //~if
      ////* else if ((info&0x7FFFFFFC) == 0x54784454) { // 'TxDT'/'TxDW'
      else if ((seen&512) !=0) { // text..
        why = 4;
        if (whom != null) nuly = GetIxWgt(whom.GetIxData(0,0));
        if (nuly==null) break;
        why++; // why = 5
        TextInWgt = nuly;
        data = nuly.GetTextLn();
        if (OpenInputDlog("New text:",data)) InputWhy = 2;} //~if
      else if (ChosenBtn==1) {
        why = 31;
        if (OKbtn != null) gotit = OKbtn.IsVisible();
        if (Logging)
          msg = msg + " (OKbtn=1) " + (OKbtn != null) + "/" + gotit;
        if (gotit) {
          if (Logging) msg = msg + " => DBtn";
          FreshFaster = false; // FF=T: omit MkWgLst
          if (DoMyButn(OKbtn)) break;} //~if
        else if (OKbtn != null) OKbtn.LogMe(" ?? ");} //~if
      else if (ChosenBtn==3) {
        why = 33;
        if (BildBtn != null) if (BildBtn.IsVisible())
          if (DoMyButn(BildBtn)) break;} //~if
      else if (ChosenBtn==4) {
        why = 34;
        if (JSbtn != null) if (JSbtn.IsVisible())
          if (DoMyButn(JSbtn)) break;} //~if
      why = -why;
      break;} //~while
    if (Logging) { // + "' do=" + DlogIsOpen = kx in msg
      msg = msg + " DF=#" + SafeWgtNum(DefaultFocus)    // "(GotEntKy #"," !? "
          + " inWhy=" + InputWhy + "/" + prio + " = " + why;
      System.out.println(msg);} //~if
    if (SRchanged) ReportChBitS();
    return false;} //~GotEnterKey

  /**
  * CliKnownStuff tries to find what widget got clicked among the known
  * widgets it manages, such as a tool or a game widget.
  *
  * Use cases: Click on tool or user game widget, selected here.
  *    Click anywhere while dialog is open, handled by super.ClickEvt.
  *    Click on OK or other visible button, handled by DoMyButn.
  *    Click on (tooled) checkbox or radio button, handled by ToolClick.
  *    Grid checkbox handled by ClickEvt.
  *
  * @param  whom    the widget clicked on
  *
  * @return         true if it succeeded
  */
  public boolean CliKnownStuff(GameWgt whom, int vert, int horz) {
    int nx = 0, px = 0, zx = 0, info = 0, why = 0;
    GameWgt nuly = null;
    while (true) {
      why++; // why = 1
      if (whom==null) break;
      why++; // why = 2
      nx = whom.GetRefNum();
      info = GetToolInfo(whom);
      FreshFaster = false; // FF=T: omit MkWgLst
      if (info>0) if ((info&0x8000)==0) { // select this tool..
        if (SelectMe(0,whom)) why = -why;
        Antsify(false,null);
        break;} //~if
      why++; // why = 3
      px = whom.GetParent(); // =5 if user wgt name, else =3 if usr wgt
      if ((whom.GetType()&255)==4) { // clicked on user wgt name?
        if (px==UwgtNamID) { // =5 =[User wgt name list]
          why = 8;
          nuly = ListofWgts; // [User wgts list]
          if (nuly==null) break;
          why++; // why = 9
          nx = whom.GetVposn();
          zx = (nx+vert)/WgtNameHi;
          if (zx<0) break;
          why++; // why = 10
          if (zx>=nWgsVisi) break;
          why++; // why = 11
          nx = nuly.GetIxData(zx+WgScroSkp,0);
          nuly = GetIxWgt(nx);
          if (nuly==null) break;
          nx = nuly.GetRefNum();
          if (SelectMe(nx,nuly)) {
            Antsify(true,nuly);
            CluckTime = 0;
            why = -why;}} //~if
        else if (whom==FocusWgt) why = -why; // whatever already sel'd?
        break;} //~if
      why++; // why = 4
      if (px==0) break; // =35 (=Game asset list) if clk wgt on bd
      why++; // why = 5
      nuly = GetIxWgt(px); // = GameBoard if clk'd user wgt itself
      if (nuly==null) break;
      why++; // why = 6
      zx = nuly.GetParent();
      if (nuly==GameBoard) if (SelectMe(nx,whom)) {
        Antsify(nx>0,whom);
        CluckTime = 0;
        why = 0;} //~if
      break;} //~while
    if (Logging) System.out.println("(CliKnoStf #" + nx + ") p=" + px
        + IffyStr(why+6==0," aw=","/") + zx + " = " + why);
    return why <= 0;} //~CliKnownStuff

  // This event is enabled by Gtype&1024, frm MouseDisp:          // (GameMk)

 /**
  * ClickEvt is called for a widget that is "Clickable" when the user
  * clicks on it (mouse down followed by mouse up on the same widget).
  * This override lets super detect and process double-click and dialog
  * events, and if not that, it tries the various things it knows about,
  * whether one of the known checkboxes, or DoMyButn or CliKnownStuff.
  *
  * @param  whom  The widget that was clicked
  * @param  vert  The vertical offset into this widget where
  *                 it was clicked
  * @param  horz  The horizontal offset into this widget
  *
  * @return       true if taken (by 2-click or alert or input dlog)
  */
  public boolean  ClickEvt(GameWgt whom, int vert, int horz) {     // (GameMk)
    int prio = SafeWgtNum(ActivWgt), info = 0, why = 0;
    String msg = "";   // MouseDisp already called FocusEv -> SetFocus
    boolean gotit = false;
    while (true) {                       // (vert,horz are wgt-relative)
      NuNmMode = 0;
      why++; // why = 1
      if (whom==null) break;
      why++; // why = 2
      if (super.ClickEvt(whom,vert,horz)) break; // (2-click or dlog)
      if (Logging) {
        msg = whom.GetIDtxt();
        if (StrLength(msg)==0) msg = HexIfMore(" ",whom.GetInfo(),"");
          else msg = " '" + msg + "'";
        msg = IffyStr(CluckTime==0," ctm=0 "," ctm>0 ")
            + vert + "/" + horz + msg;} //~if
      // if (CluckTime==0) break; // super converted 2-click to Enter
      why++; // why = 3
      if (DoMyButn(whom)) why = 6; // T: did it
      else if (CliKnownStuff(whom,vert,horz)) why = 7; // T: did it
      else if (ToolClick(whom,vert,horz)) {
        CluckTime = 0;
        why = 22;} //~if
      else if (whom==GdChkBx) {
        why++; // why = 4
        gotit = whom.CheckIt(true);
        CluckTime = 0;} //~if
      else if (whom==RestartBtn) {
        RestartMe(3,false);
        why = 9;} //~if
      else info = whom.GetInfo(); // (why = 3) // PushButn(whom,4);
      if (why>14) NewData(false);
      break;} //~while
    if (Logging) System.out.println("(GameMk ClickEv) #" + SafeWgtNum(whom) + msg
        + " DF#" + SafeWgtNum(DefaultFocus) + " A=" + SafeWgtNum(ActivWgt)
        + "/" + prio + " " + info + " " + gotit + " = " + why);
    return true;} //~ClickEvt

  // These are enabled by Gtype&2048, frm MouseDisp:

 /**
  * MouseDn is called for a widget that is "Dragable" when the user
  * presses the (right) mouse button down on it. This override lets
  * super do whatever needs doing, then does any needed scroll action.
  *
  * @param  whom  The widget that was clicked
  * @param  vert  The vertical offset into this widget where it
  *                 was clicked
  * @param  horz  The horizontal offset into this widget
  */
  public void MouseDn(GameWgt whom, int vert, int horz) {          // (GameMk)
    super.MouseDn(whom,vert,horz);             // 'WgLsMgr/1'..
    if (whom==WgLsScroBar) WgtLstMgr(1,vert,whom);} //~MouseDn

 /**
  * MouseUp is called for a widget that is "Dragable" when the user
  * releases the mouse button over the same widget that was clicked.
  * This override does any needed scroll action, then lets super do
  * whatever else needs doing.
  *
  * @param  whom  The widget that was clicked
  * @param  vert  The vertical offset into this widget where it
  *                 was clicked
  * @param  horz  The horizontal offset into this widget
  */
  public void MouseUp(GameWgt whom, int vert, int horz) {          // (GameMk)
    if (whom==WgLsScroBar) WgtLstMgr(2,vert,whom); // 'WgLsMgr/2'
    super.MouseUp(whom,vert,horz);} //~MouseUp

 /**
  * CancelDrag is called to cancel (terminate without effect) a drag.
  * This override lets super do whatever needs doing, after deleting any
  * transient (dragged tool) from the user widget list(s).
  *
  * @param  whom  The widget being dragged
  */
  public void CancelDrag(GameWgt whom) {                           // (GameMk)
    int ix, nx = 0, px = 0, zx = 0, why = 0;
    GameWgt nuly = null, slst = SpriteList;
    if (whom != null) {
      if (whom==WgLsScroThm) WgtLstMgr(4,0,whom); // 'WgLsMgr/4'
      else if (WgtBeingDrug==whom) {
        nx = whom.GetRefNum();     // look for name field..
        nuly = FindSubWgt(whom,(TextFontNo<<16)+0x0404,0);
        while (true) {
          why++; // why = 1
          if (nuly==null) break;
          zx = nuly.GetRefNum();
          px = nuly.GetParent();
          why++; // why = 2
          if (px==0) break;
          why = ~why; // why = -3: OK if me
          if (px==nx) break;
          whom.HideSho(false);
          why = -why; // why = 3
          if (slst != null) { // if transient, remove it from sprite list..
            ix = slst.FindElemt(nx);
            if (ix >= 0) slst.DeleElemt(ix);} //~if
          nuly = GetIxWgt(px);
          why = -why; // why = 3
          if (nuly==null) break;
          why++; // why = 4
          ix = nuly.FindElemt(zx);
          if (ix<0) break;
          nuly.DeleElemt(ix);
          why = ~why; // why = -5: success
          break;} //~while // why = 0: if no actual name (OK)
        if (Logging) System.out.println("  (CancMyDrag #" + nx + ") n= #" + px
            + "/" + zx + " = " + why);}} //~if
    NewData(false);
    super.CancelDrag(whom);} //~CancelDrag

 /**
  * After a StartDrag has been called for a Dragable widget and the user
  * continues to move the mouse, Dragging is called for the same widget.
  * This override lets super do whatever needs doing, then shows the new
  * coordinates and marks the widget changed.
  *
  * @param  whom  The widget that is being dragged
  * @param  vert  The vertical offset into this widget before it moves
  * @param  horz  The horizontal offset into this widget
  */
  public void Dragging(GameWgt whom, int vert, int horz) { //         (GameMk)
    if (whom != WgLsScroBar) {
      super.Dragging(whom,vert,horz); // always logs
      NewData(false);      // update displayed coords, if valid..
      if (whom != null) if (whom==ActivWgt)
        TooledShoPoSiz(whom,false);} //~if // TSPS doesn't log
    else WgtLstMgr(3,(vert<<16)+horz,whom); // 'WgLsMgr/3'
    } //~Dragging

 /**
  * After a MouseDn is called for a widget that is "Dragable" and the
  * user moves the mouse at least eight pixels with the button still
  * pressed, then it becomes a drag and StartDrag is called. This
  * override looks to see if it's a tool being dragged, and if so,
  * clones it into a game widget; then lets super do whatever needs
  * doing with the new widget.
  *
  * @param  whom  The widget that was clicked
  * @param  vert  The vertical offset into this widget where it
  *                 was clicked
  * @param  horz  The horizontal offset into this widget
  *
  * @return       The widget being dragged
  */
  public GameWgt StartDrag(GameWgt whom, int vert, int horz) {     // (GameMk)
    int nx = SafeWgtNum(whom), prio = 0, info = 0, why = 0;
    String name = "";
    GameWgt nuly = NullWgt;
    // we are allowed to substitute a token for whom (null cancels)..
    while (true) {
      why++; // why = 1
      if (whom==WgLsScroBar) { // scrollbar does its own drag mgmt
        whom = null;
        why = 7;} //~if
      else if (DlogIsOpen>0) whom = null;
      if (whom==null) break;
      FreshFaster = false; // FF=T: omit MkWgLst
      if (Logging) whom.LogMe(" << ");
      why++; // why = 2
      NewData(false);
      nx = whom.GetRefNum();          // clone it only if tool..
      info = GetToolInfo(whom);
      if ((info==0)||((info&0x8000) !=0)) { // not 'DT'
        why = -why; // why = -2
        break;} //~if
      prio = nx;
      nx = 0;
      why++; // why = 3
      whom = CloneWgt(whom,0); // (a) sets new parent to Sprite list,
      if (whom==null) break;   // (b) globalizes wgt coords to match..
      if (Logging) if (GoodKchn) {
        LogWgtTree(whom,1);} //~if
      if (nuly==null) nuly = GameBoard;
      if (nuly != null) nx = GetToolInfo(nuly); // clear old whom frm its cache
      nuly = null;
      why++; // why = 4
      nx = whom.GetRefNum();        // look for wgt name field..
      nuly = FindSubWgt(whom,(TextFontNo<<16)+0x0404,0);
      if (nuly != null) {
        why++; // why = 5
        name = nuly.GetTextLn();
        info = NthOfset(0,".0",name);
        if (info>2) if (StrLength(name) == info+2) {
          why++; // why = 6
          NextWgtName++;
          name = Substring(0,info+1,name) + NextWgtName;
          info = nuly.PutTextLn(name);}} //~if
      if (SelectMe(-nx,whom)) why = -why;
      Antsify(false,null);
      break;} //~while
    if (Logging || (why>0)) {
      System.out.println("(StartDrag #" + nx + "/" + prio + ") '" + name
          + SeeHex("' ",info,"/") + NextWgtName + " = " + why);
      if (whom != null) whom.LogMe(" >> ");} //~if
    if (whom==null) return null;
    return super.StartDrag(whom,vert,horz);} //~StartDrag

 /**
  * After a StartDrag has been called for a Dragable widget and the user
  * releases the mouse button, Dropped is called for the same widget.
  * This override looks to see if it's a tool being dragged, and if so,
  * and if the drop is on the game board, adds it to both the game board
  * and user widget list.
  *
  * @param  whom  The widget that was dragged
  */
  public void Dropped(GameWgt whom) { // (in GameMaker)               (GameMk)
    // (coords already updated by Dragging before call)
    int ix, rx = 0, kx = 0, zx = 0, nx = 0, here = 0, thar = 0, whar = 0,
        posn = 0, more = 0, why = 0;
    GameWgt nuly = null, slst = SpriteList;
    GameWgt GmBd = GameBoard;
    while (true) {
      why++; // why = 1
      if (whom==null) break; // if not in sprite list, everything is OK..
      nx = whom.GetRefNum();
      super.Dropped(whom); // only logs
      why++; // why = 2
      zx = (whom.GetSize()>>1)&0x7FFF7FFF; // = half-size, for bt/rt test
      why++; // why = 3
      if (slst==null) break; // no sprite list for transients
      kx = slst.FindElemt(nx);
      why = ~why; // why = -4
      if (kx<0) break; // not a transient tool clone (OK)
      why = -why; // why = 4
      if (GmBd==null) break; // no game board to transfer it to
      rx = GmBd.GetRefNum();
      why++; // why = 5
      here = GmBd.AddPrntsPos(GmBd.GetPosn()); // absolute game locn
      posn = whom.AddPrntsPos(whom.GetPosn()); // absolute drop locn of wgt
      // thar = AddPair(posn,(vert<<16)+(horz&0xFFFF));
      whar = AddPair(posn,PairNeg(here)); // game-rel drop locn of wgt
      thar = AddPair(whar,zx); // = game-rel middle of droppee
      more = GmBd.GetSize();
      if ((thar&0x80008000) !=0) break; // why = 5: mid off game bd up or left
      why++; // why = 6
      zx = AddPair(more,PairNeg(thar)); // >0: mid is inside
      if ((zx&0x80008000) !=0) break; // off the game board down or right
      why++; // why = 7
      slst.DeleElemt(kx); // kx = SpriteList.FindEl
      kx = GmBd.AddElemt(nx); // = [Main Game asset list]
      if (kx<0) break; // failed to add it
      why++; // why = 8
      if (!NewListName(whom)) break; // T: name for wgt is now in wgt name pane
      whom.SetParent(rx); // OK, committed
      whom.SetPosn(whar>>16,SignExtend(whar));
      ShoInfoFrom(whom); // gotta show also info added by import
      NewData(true);
      why = 0;
      break;} //~while
    if (why>1) CancelDrag(whom);
    if (Logging || (why>0)) System.out.println("(Dropd #" + nx + ") -> MG=" + rx
        + LogVH(" @ ",whar,LogVH(" ",here,LogVH(" < ",thar,LogVH(" (",zx,
          LogVH(" ",posn,") = "))))) + why);} //~Dropped // why =

// These methods read in the saved file "GameMaker.txt" and convert
//   its widget lines into game widgets already on the board..

  /**
  * NewDragWgt creates a draggable game widget corresponding to a line from
  * the saved file "GameMaker.txt" and updates its parameters from the
  * attributes on the input line. Only from LoadUserWgt.
  * ** does not handle anything w/more than one child wgt **
  *
  * @param  aLine   one line of import data to convert to game widget
  * @param  lino    its line line number
  *
  * @return         true if succeeded
  */
  private boolean NewDragWgt(String aLine, int lino) { // create draggable wgt
    int scrn = GameWinTall(),      // ..from AddUserWgt line
        kx, rx = 0, zx = 0, vert = 0, horz = 0, tall = 0, wide = 0,
        tye = 0, posn = 0, info = 0, colo = 0, edge = 0, whar = 0, why = 0;
    boolean doit = false, rez = false;
    String name = "", aWord = "", aStr = "";
    int[] data = null;
    GameTool tuly = null;
    GameWgt whom = null, nuly = null, more = null;
    GameWgt LoW = ListofWgts, GmBd = GameBoard;
    while (true) {
      why++; // why = 1
      if (GmBd==null) break; // (caller checked)
      posn = GmBd.GetPosn();
      if (LoW != null) {
        edge = LoW.GetWide();
        why = 9;} //~if // why = why+8
      else edge = GmBd.GetWide();
      why = -why; // why = -1
      tye = SafeParseInt(NthItemOf(' ',2,aLine)); // type+flags
      whar = tye&255;
      if (whar==3) break; // font (all fonts already in)
      why--; // why = -2
      if ((whar&254)==240) break; // Alert/Input Dlog (ditto)
      why--; // why = -3
      if (whar==2) {
        if (NthOfset(0," [Qx",aLine)>0) break; // RPS icon
        if (NthOfset(0," [Kx",aLine)>0) break;} //~if // Kitchen icon
      why = ~why; // why = 2
      if (NthOfset(0,"{",aLine)>0)
        name = NthItemOf('{',2,NthItemOf('}',1,aLine));
      else if (NthOfset(1,"'",aLine)>0) {
        zx = Countem("'",aLine);
        if (zx>1) name = NthItemOf('\'',zx&-2,aLine);} //~if
      if (StrLength(name)==0) break;
      why++; // why = 3
      whom = TooledImport(whar,aLine); // <<===================== ImporTool
      if (whom==null) break; // not defined tool
      why++; // why = 4  // vert/horz are game-board-rel..
      vert = SafeParseInt(NthItemOf(' ',3,aLine)); // +(posn>>16);
      horz = SafeParseInt(NthItemOf(' ',4,aLine)); // +SignExtend(posn);
      tall = SafeParseInt(NthItemOf(' ',5,aLine));
      wide = SafeParseInt(NthItemOf(' ',6,aLine));
      colo = SafeParseInt(NthItemOf(' ',7,aLine));
      info = SafeParseInt(NthItemOf(' ',8,aLine));
      if (info==0) if (NthOfset(1,"`",aLine)>0)
        info = EncodeID(aLine);
      if (Logging) if (GoodImex||GoodNew) System.out.println("(NuDrgWg (" + scrn
          + "/" + edge + ") " + vert + "/" + horz + " + " + tall + "/" + wide
          + SeeHex(" c=",colo,SeeHex(" i=",info,IffyStr(GoodNew," $&",""))));
      if (vert+3>scrn) break; // off-screen bottom
      if (vert+tall<3) break; // off-screen top
      if (horz+wide<3) break; // off-screen left
      if (horz+3>edge) break; // off-screen right
      whar = GetToolInfo(whom);
      why++; // why = 5        // CloneWg clones the panel tool..
      whom = CloneWgt(whom,0); // <<=============================== CloneWg
      if (whom==null) break;   // ..(a) sets new parent to Sprite list,
      why++; // why = 6        // ..(b) globalizes wgt coords to match..
      rx = whom.GetRefNum();
      if (!SelectMe(-rx,whom)) break; // then SetWgtsInfo plugs in the data..
      why++; // why = 7
      more = FindSubWgt(whom,0x466C677A,0); // 'Flgz'
      if (more != null) more.SetFlags(tye);
      tuly = GetWgtsTool(whom);
      zx = (tall<<16)|wide&0xFFFF;
      if (tuly == null) { // not a tooled wgt..
        whom.SetSize(tall,wide);
        doit = true;} //~if       // =========================== SetWgtsInfo..
      else doit = tuly.SetWgtsInfo(whom,info,zx,colo,aLine,AddUserWgts,more);
      why++; // why = 8
      whom.SetPosn((posn>>16)+vert,SignExtend(posn)+horz); // dropp expects abs
      Dropped(whom); // calls ShoInfoFrom->TooledShoDat->TooledShoPoSiz
      if (doit) SetSwatchColor(ColoBx,colo);
      why++; // why = 10
      ChgdBitSet = ChgdBitSet|254; // b1=color, b2=name, b3=flags,
      NewData(true);               // .. b4=V, b5=H, b6=T, b7=Wi
      if (WgtNamEntry != null) if (SetInputText(WgtNamEntry,"",name,true))
        if (DoMyButn(OKbtn)) ChooseBtn(1); // DMB calls CapInfo2Wg
      why++; // why = 11
      zx = StrLength(theImpoList)-1;
      if (zx<9) break;                 // trim off trailing white..
      for (kx=zx; kx <= zx-8; kx++) if ((int)(CharAt(kx,theImpoList))>31) {
        if (kx==zx) break;
        theImpoList = Substring(0,kx+1,theImpoList);
        break;} //~if
      zx = NthOfset(0," ",aLine);
      theImpoList = theImpoList
          + ("\n" + rx + RestOf(zx,NthItemOf('\n',1,aLine)));
      why = 0;
      rez = true;
      break;} //~while
    if (GoodImex||GoodNew) if (Logging || (why>0)) {
      System.out.println("(NuDrgWg #" + rx + SeeHex(" => x",whar,
          SeeHex("/",tye," @ ")) + vert + "/" + horz + " +" + tall
          + "/" + wide + SeeHex(" c=",colo,SeeHex(" i=",info,
          IffyStr(StrLength(name)>0," '" + name + "'\n   ","\n   ")))
          + (whom != null) + IffyStr(FreshFaster," FF"," nf")
          + IffyStr(GoodNew," $& "," ") + zx + LogVH(" po=",posn,
          LogVH(" s=",scrn," e=")) + edge + "\n  '" + aLine + "' = " + why);
      if (rez) if (whom != null) whom.LogMe("(++) ");} //~if
    return rez;} //~NewDragWgt

  /**
  * LoadUserWgts takes the import file "GameMaker.txt" (currently in
  * instance variable AddUserWgts) and feeds it one line at a time to
  * NewDragWgt to populate the game board as it was when the file was saved.
  * ** does not handle anything w/more than one child wgt **
  */
  public void LoadUserWgts(boolean doit) {
    boolean didit = false, seen = false;
        // prio = FreshFaster;
    GameWgt LoW = ListofWgts, GmBd = GameBoard,
        whom = FindSubWgt(GmBd,0x466C677A,0); // = 'Flgz'
    String aLine = NthItemOf('\n',1,AddUserWgts), aWord = "";
    int colo = 0, tops = 0, lift = 0, more = 0, area = 0, why = 0,
        tall = 0, wide = 0, size = 0, lino,
        nx = NthOfset(0," -- 202",AddUserWgts),
        info = SafeParseInt(NthItemOf(' ',2,aLine)),
        whar = NthOfset(0,"{",aLine);
    while (true) {
      if (whar>0) aWord = RestOf(whar,aLine);
      if (nx>0) if (nx<999)
        aWord = aWord + NthItemOf('\n',1,Substring(nx,33,AddUserWgts));
      why++; // why = 1
      if (GmBd==null) break;
      why++; // why = 2
      if (OKbtn==null) break;
      why++; // why = 3
      if (info<5) break;
      why++; // why = 4
      tall = SafeParseInt(NthItemOf(' ',5,aLine));
      wide = SafeParseInt(NthItemOf(' ',6,aLine));
      size = tall*0x10000+wide;
      lino = NthOfset(0,"\nGameTitle = \"",AddUserWgts);
      if (lino>0) // got JS title + pwd..
        Game_Title = NthItemOf('"',2,Substring(lino+8,99,AddUserWgts));
      else Game_Title = "";
      BlurEv(null); // make sure ants are turned off
      if (whom != null) if (GmBd != null) // whom=GmBd->Flgz
         if ((info>255)||(GmBd.GetSize() != size)) {
        if (tall>4) if (wide>4) {
          if (LoW != null) {
            area = LoW.GetSize(); // = workspace dims
            more = GmBd.GetPosn(); // = default position of game board (>0)
            colo = area-(more+size+0x40004); // (tmp use)
            if (colo<0) { // if ((colo&0x80008000) !=0)
              tops = (area>>16)-tall;
              if (tops>4) {
                if (tops>(more>>16)) tops = tops>>3;
                  else tops = tops>>1;} //~if
              else tops = 4;
              GmBd.SetVposn(tops);} //~if
            if (SignExtend(colo)<0) {
              lift = (area&0xFFFF)-wide;
              if (lift>4) {
                if (lift>(more&0xFFFF)) lift = lift>>3;
                  else lift = lift>>1;} //~if
              else lift = 4;
              GmBd.SetHposn(lift);}} //~if
          GmBd.SetSize(tall,wide);
          whom.SetSize(tall,wide);} //~if
        whom.SetFlags(info);
        didit = true;} //~if
      if (Logging) if (GoodImex||GoodNew) System.out.println("(LdUsWgs/" + doit
          + ") " + lino + LogVH(" ",size,LogVH(" < ",area,LogVH(" + ",more,
          LogVH(" : ",colo," => " + tops + "/" + lift))))
          + " [" + Game_Title + IffyStr(GoodNew,"] $&","]"));
      if (didit) ShoInfoFrom(GmBd);
      GamesAIdlg = NthOfset(0,"[Std Alert Dialog]",AddUserWgts)>0;
      ///** maybe do game tool click here?..
      colo = SafeParseInt(NthItemOf(' ',7,aLine));
      if (colo>0) SetSwatchColor(ColoBx,colo);
      if (WgtNamEntry==null) break; // (last early exit)
      why++; // why = 5
      NuNmMode++;
      FreshFaster = true; // FF=T: omit MkWgLst
      if (whar>0) aLine = NthItemOf('{',2,NthItemOf('}',1,aLine));
        else aLine = "";
      // if (doit&&GoodTst) NameChanged = false; else /// dunno why this ***
      if (StrLength(aLine)>0) if (!InvalidJava(aLine)) {
        GamesName = aLine;
        if (StrLength(Game_Title)==0) Game_Title = aLine;
        NameChanged = (SafeCompare(aLine,DefaltGmName) !=0);
        // GamePosn = GameBoard.Posn();
        GameSz = size; // = tall*0x10000+wide;
        if (SetInputText(WgtNamEntry,"",aLine,true))
          if (DoMyButn(OKbtn)) ChooseBtn(1);} //~if
      if (StrLength(Game_Title)==0) Game_Title = GameTitle;
      didit = false;
      for (lino=2; lino <= 999; lino++) {
        aLine = NthItemOf('\n',lino,AddUserWgts);
        if (StrLength(aLine)==0) break; // normal exit
        if (SafeParseInt(aLine) != lino) break; // abnormal exit
        if (NewDragWgt(aLine,lino)) didit = true;} //~for
      if (didit) WuzChanged = true;
      FreshFaster = false;
      BlurEv(null); // make sure ants are turned off
      lino = NthOfset(0," '[Sprite list]' #",AddUserWgts);
      if (lino>0) lino = SafeParseInt(Substring(lino+18,5,AddUserWgts));
      if (lino>0) NextWgtName = lino;
      unSavedWgts = false;
      WgtLstMgr(5,0,null); // (update all the new wgts) 'WgLsMgr/5'
      lino = SafeWgtNum(GmBd);
      if (lino>0) didit = SelectMe(lino,GmBd);
        // if (false) if (didit) Antsify(true,GameBoard);} //~if
      lino = NthOfset(1," ",Game_Title);
      whom = StatReport;
      if (lino>0) if (whom != null)
        lino = whom.PutTextLn(Substring(0,lino,Game_Title));
      AddUserWgts = ""; // so it won't try again
      NoReload = true;
      why = 0;
      break;} //~while
    if (Logging) if (GoodImex||GoodNew) {
      if (why <= 0) aLine = " '" + GamesName + "' = " + why
          + IffyStr(GoodImex&&GoodNew," ..." + theImpoList,"/F");
      else if (StrLength(aLine)==0) aLine = " '' = " + why;
        else aLine = " = " + why + "\n   '" + aLine + "'";
      if (StrLength(Game_Title)>0)
        aLine = "\n  [" + Game_Title + "]" + aLine;
      System.out.println("(LdUsWgs/" + doit + ") " + info // info = AddUserWgt wd 2
          + SeeHex(" c=",colo,IffyStr(GoodNew," $& "," "))
          + IffyStr(NoReload,"NR ","!nr ") + NameChanged + aLine);} //~if
    NuNmMode = 0;} //~LoadUserWgts

 /**
  * GameEnding is called once when the game window is about to close, so
  * you can do whatever needs to be done at the end. This override tries
  * to save the current game state in the output file "GameMaker.txt"
  * [but it doesn't seem to be working].
  */
  public void GameEnding() { // called when window is closing (GameMk)
    if (unSavedWgts) unSavedWgts = MakeWgtLst();} //~GameEnding

 /**
  * After the initial widgets have been created, StartUp is called to do
  * any other initialization the client game may have.
  *
  * This override searches the input widgets for the named widgets needed
  * to operate GameMaker, does some initilization, then calls NewListName
  * to put the game board into the visible widget list.
  */
  public void  StartUp() { // (GameMaker StartUp)                     (GameMk)
    GameWgt whom = null, nuly = null;
    boolean didit = false, more = false, Kitch = false;
    GameTool tuly = null;
    String aLine = "", aStr = "", zStr;
    int[] data;
    int why = 0, nx = 0, rx, zx, lino, info;
    char xCh;
    whom = FindListWgt("[User wgt name list]");
    if (StrLength(GamesName)>0) DefaltGmName = GamesName;
    if (whom != null) {
      UwgtNamID = whom.GetRefNum();
      UwgtNamPos = whom.AddPrntsPos(whom.GetPosn());
      UwgtNamList = whom;} //~if
    Rev_Date = FindListWgt("[RevDate]");
    GdChkBx = FindListWgt("[Grid checkbox]");
    StatReport = FindListWgt("[Status Report]");
    TypChkBx = FindListWgt("[Typable checkbox]");
    ClikChkBx = FindListWgt("[Clickable checkbox]");
    DragChkBx = FindListWgt("[Dragable checkbox]");
    BumpChkBx = FindListWgt("[Collidable checkbox]");
    RollChkBx = FindListWgt("[Rollable checkbox]");
    AntsChkBx = FindListWgt("[Ants checkbox]");
    if (AntsChkBx != null) AntsChkBx.HideSho(false);
    // AIdlgChkBx = FindListWgt("[Alrt+InDlg checkbox]");
    ColoBx = FindListWgt("[Color Swatch]");
    ToolHiLi = FindListWgt("[Tool icon hilight]");
    if (ToolHiLi != null) ToolHiLi.HideSho(false);
    UsWgHiLi = FindListWgt("[Name list hilight]");
    NullWgt = FindListWgt("[null widget]");
    if (NullWgt != null) {
      info = 0x58780000; // 'Xx__'
      NewGmTool(new GameTool()," ",info,NullWgt);} //~if
    for (lino=1; lino <= 99; lino++) { // each defined tool (1*brk, 4*cont)..
      why = 0;
      more = false;
      Kitch = false;
      didit = false;
      zStr = "";
      aStr = NthItemOf('\n',lino,StartupTools);
      if (StrLength(aStr)==0) break; // normal (only) exit
      info = SafeParseInt(aStr);
      if (info <= 0) continue; // invalid form
      zx = NthOfset(0,"[",aStr);
      if (zx <= 0) continue; // invalid form
      xCh = CharAt(0,NthItemOf(' ',2,aStr));
      if (info==256) {
        if (xCh == 'K') {
          rx = NthOfset(0,"<",aStr);
          if (rx>0) zStr = NthItemOf('>',1,RestOf(rx+1,aStr));
          Kitch = true;} //~if
        else if (xCh == 'Q') more = true;} //~if
      aStr = RestOf(zx,aStr);
      whom = FindListWgt(aStr);
      if (whom==null) continue; // not in startup list (3rd cont, last pre-log)
      rx = whom.GetRefNum();
      nuly = FindSubWgt(whom,(TextFontNo<<16)+0x0404,0);
      if (info >= 0x8000) {
        why = 16;
        GameBoard = whom;
        GameSz = whom.GetSize();
        nx = whom.GetIxData(0,0); // = ref+ of game name (1st in wgt name list)
        didit = true;} //~if
      else if (nuly != null) nuly.HideSho(false);
      if (nuly==null) {
        if (!didit) {
          why = why|32;
          NextWgtName++;
          aStr = "nameless." + NextWgtName;} //~if
        else aStr = GamesName;} //~if
      else aStr = nuly.GetTextLn();
      if (didit) GamesName = aStr;
      if (info==512) {
        zx = ChoseFont(whom,(int)('T'));
        why++;} //~if
      else if (info==256) {
        why = why+4;
        if (more) {
          LoadRPSpix(whom,0);
          nuly = FindListWgt("[RPS tool icon ants]");} //~if
        else nuly = FindListWgt("[Sprite tool icon ants]");} //~if
      else if ((info&1024) !=0) why = why+8;
      info = (whom.GetInfo()&-0x10000)+info;
      switch (xCh) { // 1:BoardTool, 2:RecTool, 3:DiagTool, 4:TextTool,
      case 'M':     // 5:CircleTool, 6:SkyRxTool, 7:BallTool, 8:SpriteTool
        tuly = new BoardTool();
        break; // info = 0x8000
      case 'R':
        tuly = new RecTool();
        break; // info = 2048
      case 'D':
        tuly = new DiagTool();
        break; // info = 2048
      case 'T':
        tuly = new TextTool();
        break; // info = 512
      case 'C':
        tuly = new CircleTool();
        break; // info = 1024
      case 'F':
        tuly = new SkyRxTool();
        break; // info = 0x900
      case 'B':
        tuly = new BallTool();
        break; // info = 0x500
      case 'Q':
        tuly = new RPS_Tool();
        break; // info = 256
      case 'S':
        tuly = new SpriteTool();
        break; // info = 256
      default:
        tuly = null;
        break;} //~switch
      if (Logging) if ((tuly == null) || GoodStart || GoodTool)
       System.out.println("  ..(tool #"
          + rx + ") " + lino
          + " '" + zStr + SeeHex("' x",info," = ") + why); // why =
      if ((info>>24) != (int)(xCh)) {
        System.out.println("Oops " + (int)xCh);
        if (false) continue;} //~if // **
      if (tuly == null) continue;
      NewGmTool(tuly,aStr,info,whom);
      if ((info&0xFF00)==256) {
        if (Logging) if (GoodStart||GoodTool||GoodRip||GoodKchn)
          System.out.println("  ..(sprite) #" + SafeWgtNum(nuly) + " " + lino
              + " " + BitPoker + "|" + more + " #" + SafeWgtNum(whom));
        if (MkSpriteAnts(1,whom, true,
            more&&GoodRip||Kitch&&GoodKchn)) if (nuly != null) {
          why = why+2;
          didit = AntsIcon(false,nuly);
          nuly.HideSho(BitPoker);} //~if // BitPoker=GoodBits&128K
        if (whom != null) {
          whom.HideSho(BitPoker || more || Kitch);
          if (Logging) if (GoodStart) if (GoodKchn) LogWgtTree(whom,1);}} //~if
      if ((info&1024)==0) continue;
      CircleSize(whom,16); // or =0: get it from wgt
      // nuly = FindListWgt("[Circle tool icon ants]");
      nuly = FindSubWgt(whom,0x416E7449,0); // 'AntI'
      if (nuly != null) didit = AntsIcon(false,nuly);
      } //~for // (lino: each defined tool)
    more = false;
    ListofWgts = FindListWgt("[User wgts list]");
    ToolList = FindListWgt("[Tool Palette]");
    ToolPanel = FindListWgt("[Tool panel]");
    WgLsScroBar = FindListWgt("[WgLst Scrollbar]");
    WgLsScroThm = FindListWgt("[WgLst ScroThumb]");
    WgLsScrUpBt = FindListWgt("[WgLst UpBtn]");
    WgLsScrDnBt = FindListWgt("[WgLst DnBtn]");
    WgtLstMgr(0,nx,UwgtNamList); // 'WgLsMgr/0'
    whom = FindListWgt("[Top entry field]");
    if (whom != null) {
      if (Tab2top>0) Tab2fields[Tab2top] = whom.GetIxData(0,0);
      TopField = whom;} //~if
    whom = FindListWgt("[Left entry field]");
    if (whom != null) {
      if (Tab2left>0) Tab2fields[Tab2left] = whom.GetIxData(0,0);
      LeftField = whom;} //~if
    whom = FindListWgt("[Tall entry field]");
    if (whom != null) {
      if (Tab2tall>0) Tab2fields[Tab2tall] = whom.GetIxData(0,0);
      TallField = whom;} //~if
    whom = FindListWgt("[Wide entry field]");
    if (whom != null) {
      if (Tab2wide>0) Tab2fields[Tab2wide] = whom.GetIxData(0,0);
      WideField = whom;} //~if
    whom = FontRad;
    if (whom==null) // should have been done in NewTool..
      whom = FindListWgt("[Font selector radio group]");
    if (whom != null) {
      why = ChoseFont(whom,(int)('T'));
      whom.HideSho(false);
      FontRad = whom;} //~if
    // CxAntz = null;
    // if (true) { // this W/A OK..
    OKbtn = FindListWgt("['OK' (accept) button]");
    // UndoBtn = FindListWgt("['Undo' button]"); // not there now
    BildBtn = FindListWgt("['Build' button]");
    JSbtn = FindListWgt("['->JS' button]");
    CantJS = FindListWgt("['noJS' button]");
    ToolName = FindListWgt("[Current wgt type name]");
    whom = FindListWgt("[Kitchen inventory entry field]");
    if (whom != null) { // hide Kitchen-specific stuff..
      whom.HideSho(false);} //~if
    whom = FindListWgt("[name entry panel]");
    if (whom != null) { // widen name panel..
      if (Tab2name>0) Tab2fields[Tab2name] = whom.GetIxData(0,0);
      WgtNamEntry = whom;
      // didit = SetInputText(WgtNamEntry,"","            ",false);
      whom = FindSubWgt(whom,0x42616B46,0); // = 'BakF' (not 'Namf')
      // more = false;
      if (whom != null) whom.SetSize(0,72);
        else more = true;} //~if
    NewData(true); // ChooseBtn(1);
    unSavedWgts = false;
    didit = NewListName(GameBoard); // also selects & displays it
    why = 0;
    lino = StrLength(AddUserWgts);
    info = StrLength(SavedGameData);
    WuzChanged = false;
    if (NoReload) why--; // <<============================= NoRelo->LdUserWg..
    else if (GameBoard==null) why++;
    else if (lino>0) if ((myPL>0)||(info>0))
      LoadUserWgts(false);
    FloatDlogs();
    if (Logging) System.out.println("(GameMaker StartUp) $& " + didit
        + IffyStr(UwgtNamList==null," -uwnl",
          " +UwNmL=" + UwgtNamID + LogVH(" @ ",UwgtNamPos,""))
        + IffyStr(GdChkBx==null," -ckbx"," +CkBx")
        + IffyStr(ColoBx==null," -cosw"," +ColoSw")
        + IffyStr(OKbtn==null," -nOKbn"," +OKbtn") + " " + more
        + "\n    fnts = " + MonoFontNo + "/" + TextFontNo + "/" + TinyFontNo
        + "/" + BoldFontNo + "/" + ItalFontNo + "/" + HeadFontNo
        + "/" + NanoFontNo + "/" + ScorFontNo + SeeHex(" x",KnownFonts,"")
        + IffyStr(NoReload," NR: "," !nr: ")
        + why + " +" + lino + "/" + info);} //~StartUp

 /**
  * GameList returns a text descriptor of all the specified widgets in
  * a game. This override returns the constant String BaseMkrWgts.
  *
  * @param  whom  A reference to the host JavaGame, which is sent to super
  *
  * @return       BaseMkrWgts
  */
  public String GameList( JavaGame whom ) { //        (GameMk)
    int whar, tye, nx, zx = 0,
        lxx = 0, lino = 0, seen = 0, info = 0, why = 0;
    String data = "", theList = "", BMW = BaseMkrWgts,
        aWord, aLine = super.GameList( whom );
    theList = BMW;
    // data = super.GameList( whom ); // already did
    UserDate = "";
    data = AddUserWgts;
    lxx = StrLength(data);
    if ((lxx==0) && !NoReload) { // frm Restart, may have new game in AUW
      data = ReadWholeTextFile("GameMaker.txt");
      if (NthOfset(0,"[List of item lists]",data)<0) {
        data = "";
        lxx = 0;} //~if
      else lxx = StrLength(data);
      whar = NthOfset(0,"'[Sprite list]'",data);
      nx = NthOfset(0," -- 202",data); // transfer file date to wgt list..
      if (nx>whar) if (nx<whar+33)     // (this only works thru 2029)
        UserDate = NthItemOf('\n',1,Substring(nx,33,data));
      AddUserWgts = data;} //~if
    if (Logging) System.out.println("(GmMkr/GmLst) +" + StrLength(theList)
        + " AUW= " + IffyStr(NoReload,"-","") + lxx
        + " '" + UserDate + IffyStr(NoReload,"' NR","' !nr")
        );
    return theList;} //~GameList

 /**
  * Constructor..
  */
  public GameMakerEv() { // . . . . . . . . . . . . . . . . . . . . . (GameMk)
    ClearVars();
    GamesName = "";
    AddUserWgts = "";
    PendingName = "";
    if (Logging) System.out.println("(new GamMkEv)");}} //~GameMakerEv

  // Required Java class (Java version)..
  private class PaintAction implements ActionListener { @Override
    public String toString() {return "(JavaGame PaintAction)";}  @Override
    public void actionPerformed(ActionEvent evt) {
      if (theWindow != null)
        theWindow.repaint();}} //~PaintAction

// Private variables used by low-level game engine code..

  private static GameMakerEv theMaker = null;
  private boolean BusyPaint = false;

  private PaintAction JavaOften = null; // for periodic events..
  // private BufferedImage theImag = null; // use theBuff for both
  private BufferedImage theBuff = null;

// [Shared] methods used only hereafter..

  /**
  * Starts the game timer.
  *
  * @param  ms    =100, nominally the number of milliseconds
  */
  private static void JstartTimer(int ms) { // =100ms
    if (ms>0) {

      if (TickTock != null) TickTock.start(); // posts "Timr"
        else System.out.println("** Oops, no Timer **");  }} //~JstartTimer

//  [Shared] JavaGame methods..

 /**
  * Start the animation timer.
  *
  * @param why    The initial timer count, >0 when animating
  */
  public void TimerStart(int why) { // Start timer (all vers)
    int tm =   FrameTime;
    if (why==0) return;
    if (TimerTix>0) return;
    if (TimerTix==0) {
      TimerTix = why;
      JstartTimer(tm);
      if (Logging)
        System.out.println("[start timer = " + tm + "ms @ " + why + "]");} //~if
    else if (TimerTix<0) if (why>0) {
      if (Logging) System.out.println("[restart timer @ " + why + "]");
      TimerTix = why;}} //~TimerStart

 /**
  * Test for GameWgt collision.
  * Only sprites can move, so they only can collide, but only if collidable
  * and only with objects marked collidable (+256)
  *
  * @param whom    A sprite or list that wants collision testing
  */
  void Collider(GameWgt whom) { // Collision tester, from Every100m.
    int rx, size, far, more, here, thar, whar = 0, refn = 0, info = 0;
    GameWgt twgt;
    GameEvent evh = UserEvHandler;
    if (evh == null) return;
    if (whom==null) return;
    refn = whom.GetRefNum();
    here = whom.AddPrntsPos(whom.GetPosn()); // abs posn of sprite being tested
    size = whom.GetSize();
    far = PairNeg(AddPair(here,size)); // (neg)bottom-right of same wgt
    info = whom.GetType();
    if ((info&0x81FF)==0x8106) { // ran off the edge, say so
      whom.SetFlags(info&~0x8000);
      if (evh.Collided(whom,null)) return;} //~if
    if ((info&255)==5) for (whar=0; whar <= MaxWgts; whar++) {
      twgt = GetIxWgt(whom.GetIxData(whar,0)); // try all sprites..
      if (twgt==null) return; // normal exit: end of list
      if (twgt != whom) if (twgt != DontBump) // ignore all but Bump+Sprite wgt..
        if ((twgt.GetType()&511)==262) Collider(twgt);} //~for
    for (whar=NextRefNo-1; whar >= 1; whar += -1) { // Bump+Sprite wgt, try all Bumpable wgts
      if (whar==refn) continue; // can't collide with self
      twgt = AllWgts[whar];
      if (twgt==null) continue;
      thar = twgt.AddPrntsPos(twgt.GetPosn());
      if (twgt==whom) continue;
      if (twgt==DontBump) continue;
      if ((twgt.GetType()&256)==0) continue;
      if (twgt != GameWgtsList) {   // is whom below or to right of twgt..
        if (((AddPair(thar,far)^0x80008000)&0x80008000) !=0) continue;
        else if (((AddPair(here,PairNeg(AddPair(thar,twgt.GetSize())))
            ^ 0x80008000)&0x80008000) !=0) continue;} //~if // abv or left of twgt
      else if ((AddPair(twgt.GetSize(),far)&0x80008000) !=0)
        twgt = null; // whom is off botm or right edge
      else if ((here&0x80008000) !=0) twgt = null; // off top or left
        else continue;
      if (evh != null) if (evh.Collided(whom,twgt)) break;} //~for
    // DoDeferredOps();
    if (Logging) if (GoodOptn)
      System.out.println("   -Collr #" + refn + "");} //~Collider

  /**
  * MouseDispatch accepts mouse activity from whatever source, then chooses
  * one or more event handlers to direct it to.
  *
  * @param  msg    a number representing what happened to the mouse:
  *                  =0: rollover/idle, 1: mouse down, 2: drag, 3: mouse up,
  *                  +4: right-click, +8* gives reason/source for log
  */
  private void MouseDispatch(int msg, int vert, int horz) {
    // msg: =0: rollover/idle, 1: mouse down, 2: drag, 3: mouse up, +4: right
    //      // +8* gives reason/source
    // vert,horz are window-relative (or 0,0 if unknown)
    int prio = msg, locn, here, thar, whar = 0, preDr = 0,
        mloc = 0, refn = 0, whif = 0, bitz = 0, info = 0, why = 0;
    boolean clikt = false, okfoc = false, doit = false, txt = false;
    GameWgt whom, ClkWg = null, RoWgt = null, WgBeDrg = null, FocWg = null;
    GameEvent evh = UserEvHandler; ///
    if (!ValidMouse) why = 16;
    if (MouseIsDown) why = why+8;
    if (msg>12) msg = msg-16;
    if (msg==1)
      locn = 0; // (plant brkpt here)
    while (true) {
      why++; // why = +1
      if (evh == null) break; // can't do anything if no handler
      why++; // why = +2
      if (msg+2 ==0) { // frm Every100, somebody wants rollover evts..........
        if (!ValidMouse) break;
        msg = 0;
        if (MouseIsDown) {
          if (!WantsDrag) break;
          msg = 2;} //~if
        else if (!WantsRollover) break;
        horz = MouseInfo.getPointerInfo().getLocation().x
            - getLocationOnScreen().x - SignExtend(EdgeOffs);
        vert = MouseInfo.getPointerInfo().getLocation().y-(EdgeOffs>>16)
            - getLocationOnScreen().y;} //~if
      else if (msg<0) if (ValidMouse) if (evh != null) { // (Java) mouse exit
        WgBeDrg = evh.GetRefWgt(2);
        if (WgBeDrg != null) {
          preDr = WgBeDrg.GetRefNum();
          bitz++; // bitz = +1
          evh.CancelDrag(WgBeDrg); // also sets WBD=null
          WgBeDrg = null;
          doit = true;} //~if
        if (false) if (MouseIsDown) { // maybe better to require esc keypress
          if (WantsDrag) MouseDispatch(8+3,vert,horz);    // ..or else ignore
          MouseIsDown = false;} //~if
        ValidMouse = false;} //~if
      why++; // why = +3
      if (msg<0) break;
      msg = msg&3; // higher bits to log where it came from (now in prio)
      MouseVert = vert; // these are window-relative coordinates
      MouseHorz = horz; // (used for 2-click & drag substitution)
      mloc = (vert<<16)+(horz&0xFFFF);
      // here = (vert<<16)+(horz&0xFFFF);
      ValidMouse = true;
      if (MouseIsDown) if (msg>1) if (evh != null) { // mouse still down
        ClkWg = evh.GetRefWgt(1); // (only set below)
        if (msg==3) MouseIsDown = false;
        if (ClkWg != null) {
          // whom = ClkWg;
          info = ClkWg.GetType();
          if ((info&255)==4) txt = true;
          bitz = bitz|info&0xF00;
          okfoc = ((info&0x2000)==0);
          if ((info&2048) !=0) {
            horz = LocalizePos(ClkWg,vert,horz);
            vert = LocalVert; // set by LocalizePos
            bitz = bitz+2;
            if (MouseIsDown) { // ClickLocn = mloc;
              bitz = bitz+2; // bitz = +4
              if (WgBeDrg==null) if (!KillDrag) {
                WgBeDrg = evh.GetRefWgt(2);
                if (WgBeDrg==null) { // start drag if moved far enough..
                  locn = evh.GetClickLoc(); // (set by FocEv)
                  // thar = AddPair((vert<<16)+horz,PairNeg(locn));
                  thar = ((vert<<16)+horz)-locn; // (faster, less accurate)
                  if (!txt) { // calc iAbs of each half..
                    info = thar&0x80008000; // (slight error in LSB)
                    info = info+info-(info>>15); // =FFFF if <0
                    thar = (thar^info)&0xFFF8FFF8;} //~if // !=0 if moved >8px
                  if (locn >= 0) if (thar !=0) { // (only place drag starts)
                    WgBeDrg = evh.StartDrag(ClkWg,locn>>16,locn&0xFFFF);
                    if (WgBeDrg==null) {
                      evh.BlurEv(null);
                      KillDrag = true;}}}} //~if
              if (WgBeDrg != null) {
                if (preDr==0) preDr = WgBeDrg.GetRefNum();
                horz = LocalizePos(WgBeDrg,MouseVert,MouseHorz);
                vert = LocalVert; // set by LocalizePos
                evh.Dragging(WgBeDrg,vert,horz);}} //~if // (MouseIsDown)
            else if (WgBeDrg != null) {
              if (preDr==0) preDr = WgBeDrg.GetRefNum();
              horz = LocalizePos(WgBeDrg,MouseVert,MouseHorz);
              vert = LocalVert; // set by LocalizePos
              evh.MouseUp(WgBeDrg,vert,horz);} //~if
            else {
              if ((info&1024) !=0) {
                whif = (whif<<4)+7;
                FocWg = evh.GetRefWgt(-1);
                if (okfoc) if (FocWg != ClkWg) {
                  whif = (whif<<4)+8;
                  evh.FocusEv(ClkWg,vert,horz); // also calls SetFocus
                  FocWg = evh.GetRefWgt(-1);} //~if // FocEv may redirect it!
                bitz = bitz|32;
                clikt = evh.ClickEvt(ClkWg,vert,horz);} //~if // 1st of 2 ClkEv
              evh.MouseUp(ClkWg,vert,horz);} //~else // bitz = +2
            clikt = true;
            info = 0;} //~if
          else if (MouseIsDown) info = 0;
            else info = info&1024;
          if (info !=0) {
            thar = ClkWg.AddPrntsPos(ClkWg.GetPosn());
            if (((AddPair(thar,PairNeg(mloc))^0x80008000)&0x80008000) !=0)
              info = 0; // not still inside it..
            else if (((AddPair(mloc,PairNeg(AddPair(thar,ClkWg.GetSize())))
                ^ 0x80008000)&0x80008000) !=0) info = 0;
            if (info !=0) {                                    // (1787)
              whif = (whif<<4)+2;
              FocWg = evh.GetRefWgt(-1);
              horz = LocalizePos(ClkWg,vert,horz);
              vert = LocalVert; // set by LocalizePos
              if (okfoc) if (FocWg != ClkWg) {
                whif = (whif<<4)+1;
                evh.FocusEv(ClkWg,vert,horz); // also calls SetFocus
                FocWg = evh.GetRefWgt(-1);} //~if // FocEv may redirect it!
              bitz = bitz|32;
              clikt = evh.ClickEvt(ClkWg,vert,horz); // 2nd of 2 ClickEv calls
              clikt = true;}}} //~if
        if (!MouseIsDown) {
          WgBeDrg = null; // local only, for the log
          evh.SetRefWgt(5,null); // (hi 2 bits for log)
          ClkWg = null;}} //~if // (mouse still down)
      why++; // why = +4
      if (clikt) break;
      why++; // why = +5
      if (msg==3) {
        MouseIsDown = false; // could be missed, if no handler
        break;} //~if
      why++; // why = +6
      if (evh != null) RoWgt = evh.GetRefWgt(0);
      if (RoWgt != null) {
        info = msg;
        if ((RoWgt.GetType()&4096)==0) info++;
        else if (evh != null) {
          if (info==0) { // otherwise >0 if msdn
            thar = RoWgt.AddPrntsPos(RoWgt.GetPosn());
            if (((AddPair(thar,PairNeg(mloc))^0x80008000)&0x80008000) !=0)
              info--; // not still inside it..
            else if (((AddPair(mloc,PairNeg(AddPair(thar,RoWgt.GetSize())))
                ^ 0x80008000)&0x80008000) !=0) info--;} //~if
          horz = LocalizePos(RoWgt,vert,horz);
          vert = LocalVert; // set by LocalizePos
          bitz = bitz+8;
          if (info !=0) { // not still inside & rollover..
            bitz = bitz+8; // bitz = +16
            evh.RollOut(RoWgt,vert,horz);} //~if
          else evh.RollOver(RoWgt,vert,horz); // bitz = +8
          doit = true;} //~if
        else info++;
        if (info==0) break; // did the rollover notification, so done
        if (evh != null) evh.SetRefWgt(4,null); // (hi 2 bits for log)
        RoWgt = null;} //~if
      why++; // why = +7
      MouseIsDown = msg>0;      // search back-to-front to get deepest 1st..
      for (whar=NextRefNo-1; whar >= 1; whar += -1) {
        whom = GetIxWgt(whar);
        if (whom==null) continue;
        info = whom.GetType(); // bitz = bitz+((info&0xF00)<<4);
        txt = ((info&255)==4);
        okfoc = ((info&0x2000)==0);
        if (info<1024) continue; // no mouse evts wanted here
        if (msg==0) if ((info&4096)==0) continue; // rollover unwanted
        thar = whom.AddPrntsPos(whom.GetPosn());
        if (((AddPair(thar,PairNeg(mloc))^0x80008000)&0x80008000) !=0) continue;
        if (((AddPair(mloc,PairNeg(AddPair(thar,whom.GetSize())))
            ^ 0x80008000)&0x80008000) !=0) continue;
        thar = LocalizePos(whom,vert,horz); // horx
        here = LocalVert; // set by LocalizePos
        if (msg==0) {
          evh.SetRefWgt(8,whom); // (hi 2 bits for log)
          RoWgt = whom;
          bitz = bitz|64;
          evh.RollOver(whom,here,thar);
          doit = true;
          break;} //~if
        ClkWg = whom;
        evh.SetRefWgt(9,whom); // = ClickWgt (hi 2 bits for log)
        if (FocWg==null) {
          whif = (whif<<4)+4;
          FocWg = evh.GetRefWgt(-1);} //~if
        if ((info&2048) !=0) {
          bitz = bitz|128;
          KillDrag = false;
          if (okfoc) if ((FocWg != whom)||txt) { // scro drags but not foc..
            whif = (whif<<4)+3;
            evh.FocusEv(whom,here,thar);
            FocWg = evh.GetRefWgt(-1);} //~if // FocEv may redirect it!
          evh.MouseDn(ClkWg,here,thar); // this is the only msDn call
          clikt = true;} //~if
        else if ((info&1024) !=0) {
          if (okfoc) if ((FocWg != whom)||txt) {
            whif = (whif<<4)+5;
            evh.FocusEv(whom,here,thar);
            FocWg = evh.GetRefWgt(-1);}} //~if // FocEv may redirect it!
        break;} //~for // (whar: search back-to-front)
      break;} //~while
    if (evh != null) {
      if (FocWg==null) {
        whif = (whif<<4)+6;
        FocWg = evh.GetRefWgt(-1);} //~if
      if (ClkWg==null) ClkWg = evh.GetRefWgt(1);} //~if
    whom = FocWg;
    if (false) if (msg>0) if (!clikt) if (evh != null) if ((info&512) !=0)
        if (whom != null) if ((whom.GetType()&0x2000)==0) { // =okfoc
      whif = (whif<<4)+9;
      FocWg = null;
      evh.BlurEv(whom);
      // whom.SetFocus(false,0,0); // BlurEv did it
      clikt = true;} //~if
    if (TimerTix==0) if (doit||clikt||((whif&0x11111111) !=0)) // FocEv called
      TimerStart(8);
    if (Logging) {     // bitz = |info&0xF00 // if (!ValidMouse) why = +16;
      myTmpStr = SeeHex(" x",bitz," = "); // if (MouseIsDown) why = +8;
      if (ClkWg != null)
        myTmpStr = " Cw#" + ClkWg.GetRefNum() + myTmpStr;
      if (FocWg != null)
        myTmpStr = " Fw#" + FocWg.GetRefNum() + myTmpStr;
      if (RoWgt != null)
        myTmpStr = " Rw#" + RoWgt.GetRefNum() + myTmpStr;
      if (WgBeDrg != null)
        myTmpStr = " Dw#" + WgBeDrg.GetRefNum() + myTmpStr;
      if (preDr !=0) myTmpStr = " Pw#" + preDr + myTmpStr;
      System.out.println("(MsDispch) =" + msg + "/" + prio + LogVH(" ",mloc," => ")
          + vert + "/" + horz + " " + clikt + IffyStr(ValidMouse," Vm"," -v")
          + IffyStr(MouseIsDown," mD"," mU") + IffyStr(WantsDrag," wD"," -dr")
          + IffyStr(WantsRollover," wR"," -ro") + IffyStr(okfoc," okF "," -of ")
          + info + myTmpStr + why + SeeHex(" f=",whif,"")); // whif = // why =
      myTmpStr = "";} //~if
    // DoDeferredOps();
    } //~MouseDispatch

 /**
  * Timer-driven animation engine. Timer is started by TimerStart.
  */
  private void Every100ms() { // Timer-driven anim'n engine (all vers'ns)
    int logy = SeenLog&0x10000, info, tall = WinHi, wide = WinWi;
    GameEvent evh = UserEvHandler;
    GameWgt whom;
    if (DrawLogger>0) if (DrawLogger<8) if ((DrawLogger&1) !=0) logy = 0;
    if (TimerTix<0) TimerTix--;
    else if (TimerTix>0) TimerTix++;
    if (evh != null) {
      evh. aFrameNum++;
      evh.DoOften();} //~if
    if (ValidMouse) {
      if (MouseIsDown) {if (WantsDrag) MouseDispatch(-2,0,0);} //~if
      else if (WantsRollover) MouseDispatch(-2,0,0);} //~if
    // else if (MouseIsDown) // (cancel drag)
    whom =  AllWgts[0];
    if (myPL==0) if (Logging||(logy==0))
        if ((MaxLogWgts>0)||(TimerTix<99)||(logy==0)||GoodRip||GoodKchn) {
      if (whom != null) {
        if ((TimerTix>55)||(TimerTix+22<0)) SeenLog = SeenLog|0x10000;
          else wide--;
        myTmpStr = whom.toString();} //~if
      else myTmpStr = "000";
      System.out.println("(100ms) " + (evh != null) + " " + myTmpStr
          + IffyStr(TimerTix<0," "," +") + TimerTix );
      myTmpStr = "";} //~if
    MaxLogWgts--;
    while (true) {
      if (whom==null) break;
      whom.  DrawMe(0,0,tall,wide); // = AllWgts[0] = wPixM
      if (!Bumpt) break;
      whom = GetIxWgt(whom.GetInfo()); // => +1 = [List of item lists]
      if (whom==null) break;
      if (logy==0) whom.LogMe("  [LoL] ");
      whom = GetIxWgt(whom.GetIxData(0,0)); // => +2 = [Sprite list]
      if (whom==null) break;
      if (logy==0) whom.LogMe("  [Spr] ");
      if ((whom.GetType()&255)==5) Collider(whom);
      break;} //~while
    // DoDeferredOps();
    if (DrawLogger>0) if (DrawLogger<8) if ((DrawLogger&1) !=0)
      DrawLogger++;} //~Every100ms

  /**
  * KeyDispatch accepts keystroke activity from whatever source,
  * then directs it to the KeyData event handler.
  * xxx. +4 private public
  *
  * @param  xCh    the key code
  * @param  mods   modifiers (unused, always =0)
  */
  private void KeyDispatch(char xCh, int mods) {
    GameEvent evh = UserEvHandler;
    GameWgt FocWg = null;
    int why = 0;
    if (evh != null) {
      if (Logging) myTmpStr = " +Evh = ";
      FocWg = evh.GetRefWgt(-2);} //~if
    else if (Logging) myTmpStr = " -ev = ";
    if (Logging) {
      if (FocWg != null)
        myTmpStr = " Fw#" + FocWg.GetRefNum() + " +"
            + (FocWg.GetType()&512) + myTmpStr;
      System.out.println("(KyDispch) " + (int)  xCh + " " + mods + myTmpStr + why);
      myTmpStr = "";} //~if
    if (evh != null) if (FocWg != null)
      evh.KeyData(FocWg,xCh);} //~KeyDispatch

 /**
  * Install user event handler.
  *
  * @param whom    A subclass of GameEvent that handles the desired events
  */
  public void InstallMyEvts(GameEvent whom) {
    UserEvHandler = whom;
    if (whom != null) if (Logging) System.out.println("** Installed "
        + whom.toString() + " **");} //~InstallMyEvts

  // Java wants these if 'implements WindowListener'..
  @Override public void windowOpened(WindowEvent evt) {}
  @Override public void windowClosed(WindowEvent evt) {}
  @Override public void windowIconified(WindowEvent evt) {}
  @Override public void windowDeiconified(WindowEvent evt) {}
  @Override public void windowActivated(WindowEvent evt) {}
  @Override public void windowDeactivated(WindowEvent evt) {}

 /**
  * Accepts window-about-to-close notification.
  */
  @Override public void windowClosing(WindowEvent evt) {
    GameEvent evh = UserEvHandler;
    if (evh != null) evh.GameEnding();} //~windowClosing

  // Java may also want this if 'implements KeyListener'..
  @Override public void keyReleased(KeyEvent evt) {} // (+2688)

 /**
  * Accepts keystrokes to input text.
  */
  @Override public void keyTyped(KeyEvent evt) { //        (in Game)
    char aCh = evt.getKeyChar();
    int xCh = (int)aCh;
    if (Logging) if ((Qlog&4) !=0) System.out.println("(Game) key Typed"
       + " " + xCh);
    KeyDispatch(aCh,0);} //~keyTyped

 /**
  * Accepts keystrokes to control action, because Java does not
  * do these as keyTyped.
  */
  @Override public void keyPressed(KeyEvent evt) { //      (in Game)
    int whom = evt.getKeyCode();
    char aCh;
    if (whom== KeyEvent. VK_ENTER) aCh = '\n';
    else if (whom== KeyEvent. VK_TAB)
        aCh = '\t';
    else if (whom== KeyEvent. VK_BACK_SPACE)
        aCh = '\b';
    else if (whom== KeyEvent. VK_RIGHT) aCh = Arro_RIGHT;
    else if (whom== KeyEvent. VK_KP_LEFT) aCh = Arro_LEFT;
    else if (whom== KeyEvent. VK_LEFT) aCh = Arro_LEFT;
    else if (whom== KeyEvent. VK_KP_DOWN) aCh = Arro_DOWN;
    else if (whom== KeyEvent. VK_DOWN) aCh = Arro_DOWN;
    else if (whom== KeyEvent. VK_KP_UP) aCh = Arro_UP;
    else if (whom== KeyEvent. VK_UP) aCh = Arro_UP;
      else return;
    if (aCh > '\n') KeyDispatch(aCh,0);} //~keyPressed

  /**
  * JavaMouse accepts mouse activity from whatever source, translates the
  * coordinates to window-content-relative, then directs it to MouseDispatch.
  *
  * @param  whom    activity kind: 0=enter/roll, 1=press/click, 3=up, -1=exit
  * @param  evt     some kind of Java magic containing the click location
  */
  private void JavaMouse(int whom, MouseEvent evt) {
    Insets edges = getInsets();
    int logz = SeenLog&0x4000, kx = 0, Vx = 0, Hx = 0, Vz = 0, Hz = 0;
    boolean didit = false, rite = false ;
    if (whom<8) // mouseClick is redundant
        if (evt != null) if (edges != null) { // we only implement/o'ride this'n
      if (evt.getButton()==MouseEvent.BUTTON3) whom = whom|4;
      Hz = edges.left;
      Vz = edges.top;
      kx = (Vz<<16)+(Hz&0xFFFF);
      if (kx !=0) if (EdgeOffs==0) EdgeOffs = kx;
      Hx = evt.getX()-Hz; // now window(content)-relative
      Vx = evt.getY()-Vz;} //~if
    if (Logging||(logz==0)) {
      System.out.println("(Game) Got mouse @ " + Vx + "/" + Hx
          + " + " + Vz + "/" + Hz + " => " + whom);
      if (whom==3) SeenLog = SeenLog|0x4000;} //~if
    if (whom<8) MouseDispatch(whom+16,Vx,Hx);} //~JavaMouse

 /**
  * Accepts clicks on screen image to control operation
  */
  @Override public void mouseClicked(MouseEvent evt) { //  (in Game)
    JavaMouse(9,evt);} //~mouseClicked // possibly redundant

  // Java wants these if 'implements MouseListen' ..
  @Override public void mouseExited(MouseEvent evt) {
    JavaMouse(-1,evt);} //~mouseExited
  @Override public void mouseEntered(MouseEvent evt) {
    JavaMouse(0,evt);} //~mouseEntered
  @Override public void mousePressed(MouseEvent evt) {
    JavaMouse(1,evt);} //~mousePressed
  @Override public void mouseReleased(MouseEvent evt) {
    JavaMouse(3,evt);} //~mouseReleased

 /**
  * Converts a RGB pixel array to BufferedImage used in painting.
  * Adapted from example code found on StackExchange.
  *
  * @param  pixels  The pixel array
  * @param  width   Its width
  * @param  height  Its height
  *
  * @return         The BufferedImage result
  */
  public BufferedImage Int2BufImg(int[] pixels, int width, int height)
      throws IllegalArgumentException {
    int lxx = 0;
    int[] myDat = null; // Raster raz = null;  DataBufferInt DBI = nell;
    BufferedImage bufIm = null;
    if (pixels != null) lxx = pixels.length;

    if (lxx==0) return null;
    if (width==WinWi) if (height == WinHi) bufIm = theBuff;
    if (bufIm == null) // (should be never)
      bufIm = new BufferedImage(width,height, BufferedImage. TYPE_INT_RGB);
    if (bufIm == null) return null;
    myDat = ((DataBufferInt) bufIm. getRaster().getDataBuffer()). getData();
    System. arraycopy(pixels,0,myDat,0,lxx);
    return bufIm;} //~Int2BufImg

 /**
  * All the heavy lifting happens here. Called on timer activation.
  */
  @Override public void paint(Graphics graf) { //          (in Game)
    Insets edges;
    if (BusyPaint) return; // Java won't accept local static method vars
    BusyPaint = true;
    PendingDraw = false;
    edges = getInsets();
    // super.paint(graf);
    if (thePixels != null) try {
      theBuff = Int2BufImg(thePixels,WinWi,WinHi);
      if (theBuff != null) if (graf != null) if (edges != null)
        graf.drawImage(theBuff,edges.left,edges.top,null);
      Every100ms();} //~if // calls: AllWgts[0].DrawMe();
    catch (Exception ex) {theBuff = null;}
    BusyPaint = false;} //~paint

  /**
  * StartGame is called from either the user game's main() or else
  * from JavaGame's main(), in either case with the desired window
  * title, size, and base color, plus an instance of the GameEvent
  * subclass that will be implementing the game (or GameMaker) logic.
  * StartGame creates the specified window, builds all the widgets,
  * calls the user StartUp method, then begins processing timer (and
  * mouse and keyboard) events.
  *
  * @param  tall    the window height
  * @param  wide    the window width
  * @param  color   the window background color
  * @param  title   the window title
  * @param  evh     the GameEvent subclass instance
  */
  public void StartGame(int tall, int wide, int color, String title,
      GameEvent evh) {
    int nx = tall*wide;
    String sayso = "= " + tall + "x" + wide;
    GameWgt whom;
    int[] myPix = null;
    boolean soon = true;
    AllWgts = new GameWgt[MaxWgts];
    if (Logging) System.out.println("(StartGame) $& " + sayso);
    WuzChanged = false;
    InstallMyEvts(evh);
    if (nx>0) if (color>0) if (title != "") soon = false;
    if (soon) { // no win info, do import early..
      ImportGamList(); // gets "GameImports" from wherever
      whom = AllWgts[0];
      if (whom != null) {
        nx = whom.GetSize();
        color = whom.GetColor();
        tall = nx>>16;
        wide = nx&0xFFFF;
        nx = 0;
        if (tall>0) if (wide>0) {
          WinHi = tall;
          WinWi = wide;
          nx = tall*wide;
          sayso = " = " + tall + "x" + wide + " !";}} //~if
      else System.out.println("ImportGmLst failed");} //~if
    else  if (nx>0) {
      WinHi = tall;
      WinWi = wide;} //~if
    sayso = sayso + " px=" + nx;
    // theStudent = null;
    if (nx>0) myPix = NewArray(nx); // nx = WinHi*WinWi
    if (myPix == null) {
      System.out.println("new pix or Canvas failed" + sayso);
      System.exit(8);
      return;} //~if // 1st of 2 early retn
    // PixOfx = LeftOfx;
    thePixels = myPix;
    GameWgt.  SetMyScreen(myPix,WinHi,WinWi);
    if (color <= 0) color = 0xFFFFFF; // prefill white if unspec'd
    while (nx>0) {
      nx--;
      if (nx<0) break;
      if (nx < myPix.length)
        myPix[nx] = color;} //~while
     // reduce memory-manager burden by pre-allocating this..
    theBuff = new BufferedImage(WinWi,WinHi, BufferedImage. TYPE_INT_RGB);
    // ** if (StartMe(FrameTime,null)) {} // copy in validation?
    if (StrLength(title)==0) title = "JavaGame";
    setSize(WinWi+18,WinHi+40); // make it larger for insets to come
    setTitle(title); // was: this.setTitle etc..   // did these in const'r..
    setVisible(true);
    if (!soon) ImportGamList();
    if (evh != null) evh.StartUp();
    whom = AllWgts[0];
    MaxLogWgts = Max_Log;
    if (whom==null) {
      System.out.println("*** no AllWgs[0] ***");
      return;} //~if // 2nd of 2 early retn
    whom.SetData(myPix); // root needs -> Screen pixels
    whom.  DrawMe(0,0,WinHi,WinWi);
    StartupDone = true;
    TimerStart(-8);} //~StartGame

 /**
  * RestartMe discards the current GameEngine state and reloads whatever
  * GameEvent state was originally loaded from UserEvHandler (but not
  * GameMaker.txt). If there are unsaved changes and doit=false, it asks
  * for a confirmation (doit=true is the requested confirmation).
  *
  * WuzChanged=true when some other game (besides GameMaker) is active,
  * or when non-empty GameMaker.txt is imported, or the user clicks OK
  * in GameMaker, basically anything other than a clean board.
  *
  * NameChanged=true when the user changed the name of the game,
  * or a changed name came in GameMaker.txt.
  *
  * unSavedWgts=true when the user makes a change that shows the OK button.
  * When OK is clicked, unSavedWgts=false;
  *
  * If AddUserWgts != "" then LoadUserWgts will be called after restart
  * (user typed in another game name, this is its data).
  *
  * @param  doit    =false to request confirmation
  */
  public void RestartMe(int why, boolean doit) { // frm RestartBtn
    GameEvent evh = UserEvHandler;        // why=10: EntKy/OK=10, why=20: ..OK=9
    String msg = "", gName = "", aWord;   // why=30: ClkEv/RstrtBtn
    int lino = 0, info = StrLength(SavedGameData), prio = InputWhy,
        zx, tall, wide, colo;
    boolean dont, asky = unSavedWgts;
    why = why*10;
    while (true) {
      why++; // why = +1
      if (!WuzChanged) if (!doit) break;
      why++; // why = +2
      if (evh == null) break;
      lino = evh.GetValue(-1); // log only
      why++; // why = +3
      gName = evh.toString();
      if (NthOfset(0," GamesName = '",gName)<0) gName = "";
      else if (NthOfset(0,"' => '",gName)<0)
        gName = NthItemOf('\'',2,gName);    // = GamesName
      else gName = NthItemOf('\'',4,gName); // = PendingName
      if (!asky) if (!doit) if (SafeCompare(gName,DefaltGmName)==0)
        asky = true;
      if (Logging) if (GoodNew)
        System.out.println("(RestMe$&" + doit + ") " + unSavedWgts + "/" +
            WuzChanged + " do=" + DlogIsOpen + " '" + gName + "' = '"
            + DefaltGmName + IffyStr(NoReload,"' NR","' !nr") + " => "
            + asky + " = " + why);
      NoReload = true; // seen by: ValidGmL->GameList
      if (NuNmMode==0) if (asky) if (!doit) { // request confirmation..
        if (StrLength(ValidGmList(false,evh))==0) break; // why = +3
        if (NameChanged) if (StrLength(gName)>0) msg = " to " + gName;
        dont = GameAlert("Discard changes" + msg + "?");
        why = 15;
        if (DlogIsOpen>0) {
          InputWhy = 9;
          why = -why;} //~if // why = -15
        break;} //~if
      why++; // why = +4
      msg = ValidGmList(true,evh); // logs & quits if invalid
      if (StrLength(msg)==0) break;
      why++; // why = +5
      msg = NthItemOf('\n',1,msg);
      tall = SafeParseInt(NthItemOf(' ',5,msg));
      wide = SafeParseInt(NthItemOf(' ',6,msg));
      colo = SafeParseInt(NthItemOf(' ',7,msg));
      zx = NthOfset(0,"{",msg);
      if (NthOfset(0,"}",msg)>zx)
        msg = NthItemOf('{',2,NthItemOf('}',1,msg));
      else if (NthOfset(1,Quot2,msg)>0) msg = NthItemOf('\"',2,msg);
        else msg = "";
      if (StrLength(msg)==0) msg = "Game Engine";
      // if (tall <= 0) break; // already checked
      // if (wide <= 0) break;
      why++; // why = +6
      if (Logging) if (GoodNew) if (evh != null)
        System.out.println("  (RstMe$&) AUW=" + evh.GetValue(-1) + " = " + why);
      evh.ClearVars();
      NoReload = true; // seen by: GameList, StartUp, ^U
      TimerTix = 0;   TimeBase = 0;   NextRefNo = 0;
      MouseVert = 0;  MouseHorz = 0;  LocalVert = 0;
      MonoFontNo = 0; TextFontNo = 0; TinyFontNo = 0; BoldFontNo = 0;
      ItalFontNo = 0; HeadFontNo = 0; NanoFontNo = 0; ScorFontNo = 0;
      TimerTix = 0;   TimeBase = 0;   DlogIsOpen = 0;
      InputDlgNo = 0; Floatem = 0;   CashFindWhy = 0; InputWhy = 0;
      WuzChanged = false;  NameChanged = false;
      CashFindWgt = null;  CashFoundWgt = null;
      SpriteList = null;   mAntsHost = null;          mAntsWgt = null;
      theAlertDlg = null;  theInputDlg = null;        DlgInputData = null;
      GameWgtsList = null; GameRoot = null;           DontBump = null;
      ValidMouse = false;  MouseIsDown = false;       KillDrag = false;
      PendingDraw = false; unSavedWgts = false;       Bumpt = false;
      WantsDrag = false;   WantsRollover = false;
      BusyPaint = false;   StartupDone = GoodStart;
      RecentJava = "";     JSinfo = "";            // Game_Title = GameTitle;
      ClearTools();
      // SetNuName(); // sets GamesName // not visi: in GameMk but not in evh
      if (evh != null) {
        // zx = evh.GetValue(-2); // in GameMk -> SetNuName (can't: GameBoard=0)
        zx = evh.GetValue(-1);
        if (zx==0) lino = -lino;} //~if // (log only, = length(AddUserWgts))
      if (StrLength(SavedGameData)==0) info = -info;
        else NoReload = false; // GameList reloads AUW if NoReload=F
      StartGame(tall,wide,colo,msg,evh); // <<======================= StartGam
      SavedGameData = "";        // .. StrtGam calls ImportGmLst,evh.StartUp
      why = -why; // why = -6   // .. StartUp calls LoadUserWgts if NoReload=F
      // NoReload = false;     // .. ImportGmLst calls ValidGmL->GameList
      break;} //~while
    if (Logging) System.out.println("(RestMe$&" + doit + ") " + asky + " "  // why =
        + " inWhy=" + InputWhy + "/" + prio + " do=" + DlogIsOpen + " " + lino
        + "/" + info + IffyStr(NoReload," NR"," !nr") + " = " + why);} //~RestartMe

  /**
  * NewMaker is an access function to create a new GameMakerEv instance
  * when JavaGame is started from its own main().
  *
  * @return       the new GameMakerEv instance
  */
  public GameMakerEv NewMaker() {
    theMaker = new GameMakerEv();
    return theMaker;} //~NewMaker

 /**
  * This is the constructor, which sets everything up.
  */
  public JavaGame() { // outer class constructor..       // (+2788)
    int nx;
    MaxWinSize = Win_Hi*0x10000
        + Win_Wi;
    theWindow = this;
    JavaOften = new PaintAction();
    TimeBase = GetMilliSecs();
    TickTock = new Timer( FrameTime,JavaOften); // (not start it)
    if (Logging) System.out.println("iGame ");
    // KnownCirclz = new int[100][];
    for (nx=0; nx <= 31; nx++) {
      WgTooTb[nx] = null;
      ToolTypTb[nx] = 0;} //~for
    for (nx=0; nx <= 99; nx++) KnownCirclz[nx] = null;
    setDefaultCloseOperation( JFrame. EXIT_ON_CLOSE);
    addMouseListener(this); // if 'implements MouseListen..'
    addKeyListener(this); // if 'implements KeyListen..'
    if (false) setVisible(true);}} //~JavaGame

// Index to Selected Useful JavaGame Methods (BlueJ has none)..
//   // [Predefined] GameEngine Macros & Data..
//   // [Shared] GameEngine working data (static)..
//   CharAt  NthOfset  AddPair   class GameIco
//   UsrGameJv  AllWgts  GetMilliSecs  SafeParseInt
//   ReadWholeTextFile  ReplacAll  ParsFixPt
// Tool Panel Update Manage   class GameTool  ShoWgtsDat
//   class BoardTool  CircleTool  TextTool  BallTool  DiagTool
//   FormInFld3  GetToolInfo  TooledSetPoSiz  TooledExport
//   ToolClick  NewGameWgt  ChoseFont  BuildThisMacro
//   FindSubWgt  Antsify  MkSpriteAnts  CircleSize  SetInputText
//   OpenInputDlog  IsDlgEvent  PaintAction  ImportGamList
// class GameMakerEv  InvalidJava  SetSwatchColor  ShoInfoFrom
//   MakeWgtLst  MakeUserJava  DoMyButn  SelectMe  KeyFld
//   CliKnownStuff  StartDrag  Collided  LoadUserWgts  StartUp
//   // [Shared] methods used only hereafter..
//   Collider  Every100ms  KeyDispatch  JvUpLdTest  toJS
//   windowClosing  JavaMouse  paint  StartGame  JavaGame()

  // (javagame) (BJ)
